@@ -1,6 +1,6 @@
-/* Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
+/* Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  * Modifications Copyright (C) 2022 RasterGrid Kft.
  *
@@ -17,6 +17,8 @@
  * limitations under the License.
  */
 
+#include <vulkan/vk_enum_string_helper.h>
+#include <vulkan/vulkan_core.h>
 #include "best_practices/best_practices_validation.h"
 #include "best_practices/bp_state.h"
 #include "state_tracker/buffer_state.h"
@@ -41,9 +43,11 @@ bool BestPractices::PreCallValidateAllocateMemory(VkDevice device, const VkMemor
                                                   const ErrorObject& error_obj) const {
     bool skip = false;
 
-    if ((Count<vvl::DeviceMemory>() + 1) > kMemoryObjectWarningLimit) {
+    const size_t object_count = Count<vvl::DeviceMemory>() + 1;
+    if (object_count > kMemoryObjectWarningLimit) {
         skip |= LogPerformanceWarning("BestPractices-vkAllocateMemory-too-many-objects", device, error_obj.location,
-                                      "This app has over %" PRIu32 " memory objects.", kMemoryObjectWarningLimit);
+                                      "This app has %zu memory objects, recommended max is %" PRIu32 ".", object_count,
+                                      kMemoryObjectWarningLimit);
     }
 
     if (pAllocateInfo->allocationSize < kMinDeviceAllocationSize) {
@@ -130,7 +134,7 @@ void BestPractices::PreCallRecordFreeMemory(VkDevice device, VkDeviceMemory memo
         }
     }
 
-    ValidationStateTracker::PreCallRecordFreeMemory(device, memory, pAllocator, record_obj);
+    BaseClass::PreCallRecordFreeMemory(device, memory, pAllocator, record_obj);
 }
 
 bool BestPractices::PreCallValidateFreeMemory(VkDevice device, VkDeviceMemory memory, const VkAllocationCallbacks* pAllocator,
@@ -144,7 +148,7 @@ bool BestPractices::PreCallValidateFreeMemory(VkDevice device, VkDeviceMemory me
     for (const auto& item : mem_info->ObjectBindings()) {
         const auto& obj = item.first;
         const LogObjectList objlist(device, obj, mem_info->Handle());
-        skip |= LogWarning(layer_name.c_str(), objlist, error_obj.location, "VK Object %s still has a reference to mem obj %s.",
+        skip |= LogWarning("BestPractices", objlist, error_obj.location, "VK Object %s still has a reference to mem obj %s.",
                            FormatHandle(obj).c_str(), FormatHandle(mem_info->Handle()).c_str());
     }
 
@@ -280,8 +284,8 @@ bool BestPractices::PreCallValidateBindImageMemory2KHR(VkDevice device, uint32_t
     return PreCallValidateBindImageMemory2(device, bindInfoCount, pBindInfos, error_obj);
 }
 
-void BestPractices::PreCallRecordSetDeviceMemoryPriorityEXT(VkDevice device, VkDeviceMemory memory, float priority,
-                                                            const RecordObject& record_obj) {
+void BestPractices::PostCallRecordSetDeviceMemoryPriorityEXT(VkDevice device, VkDeviceMemory memory, float priority,
+                                                             const RecordObject& record_obj) {
     auto mem_info = std::static_pointer_cast<bp_state::DeviceMemory>(Get<vvl::DeviceMemory>(memory));
     mem_info->dynamic_priority.emplace(priority);
 }
@@ -307,31 +311,59 @@ bool BestPractices::ValidateBindMemory(VkDevice device, VkDeviceMemory memory, c
 }
 
 std::shared_ptr<vvl::DeviceMemory> BestPractices::CreateDeviceMemoryState(
-    VkDeviceMemory handle, const VkMemoryAllocateInfo* pAllocateInfo, uint64_t fake_address, const VkMemoryType& memory_type,
+    VkDeviceMemory handle, const VkMemoryAllocateInfo* allocate_info, uint64_t fake_address, const VkMemoryType& memory_type,
     const VkMemoryHeap& memory_heap, std::optional<vvl::DedicatedBinding>&& dedicated_binding, uint32_t physical_device_count) {
     return std::static_pointer_cast<vvl::DeviceMemory>(std::make_shared<bp_state::DeviceMemory>(
-        handle, pAllocateInfo, fake_address, memory_type, memory_heap, std::move(dedicated_binding), physical_device_count));
+        handle, allocate_info, fake_address, memory_type, memory_heap, std::move(dedicated_binding), physical_device_count));
 }
 
 void BestPractices::ManualPostCallRecordBindBufferMemory2(VkDevice device, uint32_t bindInfoCount,
                                                           const VkBindBufferMemoryInfo* pBindInfos,
                                                           const RecordObject& record_obj) {
     if (record_obj.result != VK_SUCCESS && bindInfoCount > 1) {
-        // Details of check found in https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/5527
-        LogWarning(
-            "BestPractices-Partial-Bound-Buffer", device, record_obj.location,
-            "all buffer are now in an indeterminate state because the call failed to return VK_SUCCESS. The best action to take "
-            "is to destroy the buffers instead of trying to rebind");
+        bool found_status = false;
+        for (uint32_t i = 0; i < bindInfoCount; i++) {
+            if (auto* bind_memory_status = vku::FindStructInPNextChain<VkBindMemoryStatus>(pBindInfos[i].pNext)) {
+                found_status = true;
+                if (bind_memory_status->pResult && *bind_memory_status->pResult != VK_SUCCESS) {
+                    LogWarning("BestPractices-Partial-Bound-Buffer-Status", device,
+                               record_obj.location.dot(Field::pBindInfos, i).pNext(vvl::Struct::VkBindMemoryStatus, Field::pResult),
+                               "was %s", string_VkResult(*bind_memory_status->pResult));
+                }
+            }
+        }
+
+        if (!found_status) {
+            // Details of check found in https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/5527
+            LogWarning("BestPractices-Partial-Bound-Buffer", device, record_obj.location,
+                       "all buffer are now in an indeterminate state because the call failed to return VK_SUCCESS. The best action "
+                       "to take "
+                       "is to destroy the buffers instead of trying to rebind");
+        }
     }
 }
 
 void BestPractices::ManualPostCallRecordBindImageMemory2(VkDevice device, uint32_t bindInfoCount,
                                                          const VkBindImageMemoryInfo* pBindInfos, const RecordObject& record_obj) {
     if (record_obj.result != VK_SUCCESS && bindInfoCount > 1) {
-        // Details of check found in https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/5527
-        LogWarning(
-            "BestPractices-Partial-Bound-Image", device, record_obj.location,
-            "all image are now in an indeterminate state because the call failed to return VK_SUCCESS. The best action to take is "
-            "to destroy the images instead of trying to rebind");
+        bool found_status = false;
+        for (uint32_t i = 0; i < bindInfoCount; i++) {
+            if (auto* bind_memory_status = vku::FindStructInPNextChain<VkBindMemoryStatus>(pBindInfos[i].pNext)) {
+                found_status = true;
+                if (bind_memory_status->pResult && *bind_memory_status->pResult != VK_SUCCESS) {
+                    LogWarning("BestPractices-Partial-Bound-Image-Status", device,
+                               record_obj.location.dot(Field::pBindInfos, i).pNext(vvl::Struct::VkBindMemoryStatus, Field::pResult),
+                               "was %s", string_VkResult(*bind_memory_status->pResult));
+                }
+            }
+        }
+
+        if (!found_status) {
+            // Details of check found in https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/5527
+            LogWarning("BestPractices-Partial-Bound-Image", device, record_obj.location,
+                       "all image are now in an indeterminate state because the call failed to return VK_SUCCESS. The best action "
+                       "to take is "
+                       "to destroy the images instead of trying to rebind");
+        }
     }
 }
