@@ -1,6 +1,6 @@
-/* Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
+/* Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  * Modifications Copyright (C) 2022 RasterGrid Kft.
  *
@@ -17,6 +17,7 @@
  * limitations under the License.
  */
 
+#include <vulkan/vk_enum_string_helper.h>
 #include "best_practices/best_practices_validation.h"
 #include "error_message/error_strings.h"
 #include "best_practices/bp_state.h"
@@ -65,10 +66,10 @@ static inline bool RenderPassUsesAttachmentAsImageOnly(const vku::safe_VkRenderP
     }
 
     for (uint32_t subpass = 0; subpass < create_info.subpassCount; subpass++) {
-        const auto& subpassInfo = create_info.pSubpasses[subpass];
+        const auto& subpass_info = create_info.pSubpasses[subpass];
 
-        for (uint32_t i = 0; i < subpassInfo.inputAttachmentCount; i++) {
-            if (subpassInfo.pInputAttachments[i].attachment == attachment) {
+        for (uint32_t i = 0; i < subpass_info.inputAttachmentCount; i++) {
+            if (subpass_info.pInputAttachments[i].attachment == attachment) {
                 return true;
             }
         }
@@ -129,23 +130,22 @@ bool BestPractices::PreCallValidateCreateRenderPass(VkDevice device, const VkRen
 
     if (IsExtEnabled(device_extensions.vk_ext_multisampled_render_to_single_sampled)) {
         for (uint32_t i = 0; i < pCreateInfo->subpassCount; ++i) {
-            if (pCreateInfo->pSubpasses[i].pResolveAttachments) {
-                for (uint32_t j = 0; j < pCreateInfo->pSubpasses[i].colorAttachmentCount; ++j) {
-                    const uint32_t attachment = pCreateInfo->pSubpasses[i].pResolveAttachments[j].attachment;
-                    if (attachment != VK_ATTACHMENT_UNUSED) {
-                        const VkFormat format = pCreateInfo->pAttachments[attachment].format;
-                        VkSubpassResolvePerformanceQueryEXT performance_query = vku::InitStructHelper();
-                        VkFormatProperties2 format_properties2 = vku::InitStructHelper(&performance_query);
-                        DispatchGetPhysicalDeviceFormatProperties2Helper(physical_device, format, &format_properties2);
-                        if (performance_query.optimal == VK_FALSE) {
-                            skip |= LogPerformanceWarning(
-                                "BestPractices-vkCreateRenderPass-SubpassResolve-NonOptimalFormat", device,
-                                create_info_loc.dot(Field::pSubpasses, i).dot(Field::pResolveAttachments, j).dot(Field::attachment),
-                                "(%" PRIu32
-                                ") in the VkRenderPass has the format %s and is used as a resolve attachment, "
-                                "but VkSubpassResolvePerformanceQueryEXT::optimal is VK_FALSE.",
-                                attachment, string_VkFormat(format));
-                        }
+            if (!pCreateInfo->pSubpasses[i].pResolveAttachments) continue;
+            for (uint32_t j = 0; j < pCreateInfo->pSubpasses[i].colorAttachmentCount; ++j) {
+                const uint32_t attachment = pCreateInfo->pSubpasses[i].pResolveAttachments[j].attachment;
+                if (attachment != VK_ATTACHMENT_UNUSED) {
+                    const VkFormat format = pCreateInfo->pAttachments[attachment].format;
+                    VkSubpassResolvePerformanceQueryEXT performance_query = vku::InitStructHelper();
+                    VkFormatProperties2 format_properties2 = vku::InitStructHelper(&performance_query);
+                    DispatchGetPhysicalDeviceFormatProperties2Helper(physical_device, format, &format_properties2);
+                    if (performance_query.optimal == VK_FALSE) {
+                        skip |= LogPerformanceWarning(
+                            "BestPractices-vkCreateRenderPass-SubpassResolve-NonOptimalFormat", device,
+                            create_info_loc.dot(Field::pSubpasses, i).dot(Field::pResolveAttachments, j).dot(Field::attachment),
+                            "(%" PRIu32
+                            ") in the VkRenderPass has the format %s and is used as a resolve attachment, "
+                            "but VkSubpassResolvePerformanceQueryEXT::optimal is VK_FALSE.",
+                            attachment, string_VkFormat(format));
                     }
                 }
             }
@@ -265,11 +265,12 @@ bool BestPractices::ValidateCmdBeginRenderPass(VkCommandBuffer commandBuffer, co
 bool BestPractices::ValidateCmdBeginRendering(VkCommandBuffer commandBuffer, const VkRenderingInfo* pRenderingInfo,
                                               const Location& loc) const {
     bool skip = false;
-    const Location rendering_info = loc.dot(Field::pRenderingInfo);
+    const Location rendering_info_loc = loc.dot(Field::pRenderingInfo);
 
     for (uint32_t i = 0; i < pRenderingInfo->colorAttachmentCount; ++i) {
-        const auto& color_attachment = pRenderingInfo->pColorAttachments[i];
-        const Location color_attachment_info = rendering_info.dot(Field::pColorAttachments, i);
+        const VkRenderingAttachmentInfo& color_attachment = pRenderingInfo->pColorAttachments[i];
+        if (color_attachment.imageView == VK_NULL_HANDLE) continue;
+        const Location color_attachment_loc = rendering_info_loc.dot(Field::pColorAttachments, i);
 
         auto image_view_state = Get<vvl::ImageView>(color_attachment.imageView);
         ASSERT_AND_CONTINUE(image_view_state);
@@ -277,7 +278,7 @@ bool BestPractices::ValidateCmdBeginRendering(VkCommandBuffer commandBuffer, con
         if (VendorCheckEnabled(kBPVendorNVIDIA)) {
             if (color_attachment.loadOp == VK_ATTACHMENT_LOAD_OP_CLEAR) {
                 const VkFormat format = image_view_state->create_info.format;
-                skip |= ValidateClearColor(commandBuffer, format, color_attachment.clearValue.color, color_attachment_info);
+                skip |= ValidateClearColor(commandBuffer, format, color_attachment.clearValue.color, color_attachment_loc);
             }
         }
 
@@ -288,7 +289,7 @@ bool BestPractices::ValidateCmdBeginRendering(VkCommandBuffer commandBuffer, con
                 image_view_state->image_state->create_info.samples != VK_SAMPLE_COUNT_1_BIT) {
                 const LogObjectList objlist(commandBuffer, resolve_image_view_state->Handle(), image_view_state->Handle());
                 skip |= LogWarning("BestPractices-VkRenderingInfo-ResolveModeNone", commandBuffer,
-                                   color_attachment_info.dot(Field::resolveMode),
+                                   color_attachment_loc.dot(Field::resolveMode),
                                    "is VK_RESOLVE_MODE_NONE but resolveImageView is pointed to a valid VkImageView with "
                                    "VK_SAMPLE_COUNT_1_BIT and imageView is pointed to a VkImageView with %s. If "
                                    "VK_RESOLVE_MODE_NONE is set, the resolveImageView value is ignored.",
@@ -300,33 +301,46 @@ bool BestPractices::ValidateCmdBeginRendering(VkCommandBuffer commandBuffer, con
     return skip;
 }
 
+bool BestPractices::PreCallValidateCmdBeginRendering(VkCommandBuffer commandBuffer, const VkRenderingInfo* pRenderingInfo,
+                                                     const ErrorObject& error_obj) const {
+    return ValidateCmdBeginRendering(commandBuffer, pRenderingInfo, error_obj.location);
+}
+
+bool BestPractices::PreCallValidateCmdBeginRenderingKHR(VkCommandBuffer commandBuffer, const VkRenderingInfo* pRenderingInfo,
+                                                        const ErrorObject& error_obj) const {
+    return ValidateCmdBeginRendering(commandBuffer, pRenderingInfo, error_obj.location);
+}
+
 void BestPractices::PreCallRecordCmdEndRenderPass(VkCommandBuffer commandBuffer, const RecordObject& record_obj) {
-    RecordCmdEndRenderingCommon(commandBuffer);
+    BaseClass::PreCallRecordCmdEndRenderPass(commandBuffer, record_obj);
 
-    ValidationStateTracker::PreCallRecordCmdEndRenderPass(commandBuffer, record_obj);
     auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    if (cb_state) {
-        // Add Deferred Queue
-        cb_state->queue_submit_functions.insert(cb_state->queue_submit_functions.end(),
-                                                cb_state->queue_submit_functions_after_render_pass.begin(),
-                                                cb_state->queue_submit_functions_after_render_pass.end());
-        cb_state->queue_submit_functions_after_render_pass.clear();
+    // Using PreCallRecord because logic relies on render pass state not being destroyed yet
+    if (auto rp_state = cb_state->activeRenderPass.get()) {
+        RecordCmdEndRenderingCommon(*cb_state, *rp_state);
     }
+
+    // Add Deferred Queue
+    cb_state->queue_submit_functions.insert(cb_state->queue_submit_functions.end(),
+                                            cb_state->queue_submit_functions_after_render_pass.begin(),
+                                            cb_state->queue_submit_functions_after_render_pass.end());
+    cb_state->queue_submit_functions_after_render_pass.clear();
 }
 
 void BestPractices::PreCallRecordCmdEndRenderPass2(VkCommandBuffer commandBuffer, const VkSubpassEndInfo* pSubpassInfo,
                                                    const RecordObject& record_obj) {
-    RecordCmdEndRenderingCommon(commandBuffer);
-
-    ValidationStateTracker::PreCallRecordCmdEndRenderPass2(commandBuffer, pSubpassInfo, record_obj);
+    BaseClass::PreCallRecordCmdEndRenderPass2(commandBuffer, pSubpassInfo, record_obj);
     auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    if (cb_state) {
-        // Add Deferred Queue
-        cb_state->queue_submit_functions.insert(cb_state->queue_submit_functions.end(),
-                                                cb_state->queue_submit_functions_after_render_pass.begin(),
-                                                cb_state->queue_submit_functions_after_render_pass.end());
-        cb_state->queue_submit_functions_after_render_pass.clear();
+    // Using PreCallRecord because logic relies on render pass state not being destroyed yet
+    if (auto rp_state = cb_state->activeRenderPass.get()) {
+        RecordCmdEndRenderingCommon(*cb_state, *rp_state);
     }
+
+    // Add Deferred Queue
+    cb_state->queue_submit_functions.insert(cb_state->queue_submit_functions.end(),
+                                            cb_state->queue_submit_functions_after_render_pass.begin(),
+                                            cb_state->queue_submit_functions_after_render_pass.end());
+    cb_state->queue_submit_functions_after_render_pass.clear();
 }
 
 void BestPractices::PreCallRecordCmdEndRenderPass2KHR(VkCommandBuffer commandBuffer, const VkSubpassEndInfoKHR* pSubpassInfo,
@@ -335,53 +349,35 @@ void BestPractices::PreCallRecordCmdEndRenderPass2KHR(VkCommandBuffer commandBuf
 }
 
 void BestPractices::PreCallRecordCmdEndRendering(VkCommandBuffer commandBuffer, const RecordObject& record_obj) {
-    RecordCmdEndRenderingCommon(commandBuffer);
-
-    ValidationStateTracker::PreCallRecordCmdEndRendering(commandBuffer, record_obj);
+    BaseClass::PreCallRecordCmdEndRendering(commandBuffer, record_obj);
+    auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
+    // Using PreCallRecord because logic relies on render pass state not being destroyed yet
+    if (auto rp_state = cb_state->activeRenderPass.get()) {
+        RecordCmdEndRenderingCommon(*cb_state, *rp_state);
+    }
 }
 
 void BestPractices::PreCallRecordCmdEndRenderingKHR(VkCommandBuffer commandBuffer, const RecordObject& record_obj) {
     PreCallRecordCmdEndRendering(commandBuffer, record_obj);
 }
 
-void BestPractices::PreCallRecordCmdBeginRenderPass(VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo* pRenderPassBegin,
-                                                    VkSubpassContents contents, const RecordObject& record_obj) {
-    ValidationStateTracker::PreCallRecordCmdBeginRenderPass(commandBuffer, pRenderPassBegin, contents, record_obj);
-    RecordCmdBeginRenderingCommon(commandBuffer);
-    RecordCmdBeginRenderPass(commandBuffer, pRenderPassBegin);
+void BestPractices::PostCallRecordCmdBeginRendering(VkCommandBuffer commandBuffer, const VkRenderingInfo* pRenderingInfo,
+                                                    const RecordObject& record_obj) {
+    BaseClass::PostCallRecordCmdBeginRendering(commandBuffer, pRenderingInfo, record_obj);
+    auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
+    RecordCmdBeginRenderingCommon(*cb_state);
 }
 
-void BestPractices::PreCallRecordCmdBeginRenderPass2(VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo* pRenderPassBegin,
-                                                     const VkSubpassBeginInfo* pSubpassBeginInfo, const RecordObject& record_obj) {
-    ValidationStateTracker::PreCallRecordCmdBeginRenderPass2(commandBuffer, pRenderPassBegin, pSubpassBeginInfo, record_obj);
-    RecordCmdBeginRenderingCommon(commandBuffer);
-    RecordCmdBeginRenderPass(commandBuffer, pRenderPassBegin);
-}
-
-void BestPractices::PreCallRecordCmdBeginRenderPass2KHR(VkCommandBuffer commandBuffer,
-                                                        const VkRenderPassBeginInfo* pRenderPassBegin,
-                                                        const VkSubpassBeginInfo* pSubpassBeginInfo,
-                                                        const RecordObject& record_obj) {
-    PreCallRecordCmdBeginRenderPass2(commandBuffer, pRenderPassBegin, pSubpassBeginInfo, record_obj);
-}
-
-void BestPractices::PreCallRecordCmdBeginRendering(VkCommandBuffer commandBuffer, const VkRenderingInfo* pRenderingInfo,
-                                                   const RecordObject& record_obj) {
-    ValidationStateTracker::PreCallRecordCmdBeginRendering(commandBuffer, pRenderingInfo, record_obj);
-    RecordCmdBeginRenderingCommon(commandBuffer);
-}
-
-void BestPractices::PreCallRecordCmdBeginRenderingKHR(VkCommandBuffer commandBuffer, const VkRenderingInfo* pRenderingInfo,
-                                                      const RecordObject& record_obj) {
-    PreCallRecordCmdBeginRendering(commandBuffer, pRenderingInfo, record_obj);
+void BestPractices::PostCallRecordCmdBeginRenderingKHR(VkCommandBuffer commandBuffer, const VkRenderingInfo* pRenderingInfo,
+                                                       const RecordObject& record_obj) {
+    PostCallRecordCmdBeginRendering(commandBuffer, pRenderingInfo, record_obj);
 }
 
 void BestPractices::PostCallRecordCmdNextSubpass(VkCommandBuffer commandBuffer, VkSubpassContents contents,
                                                  const RecordObject& record_obj) {
-    ValidationStateTracker::PostCallRecordCmdNextSubpass(commandBuffer, contents, record_obj);
-    RecordCmdNextSubpass(commandBuffer);
-
+    BaseClass::PostCallRecordCmdNextSubpass(commandBuffer, contents, record_obj);
     auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
+    RecordCmdNextSubpass(*cb_state);
     auto rp = cb_state->activeRenderPass.get();
     ASSERT_AND_RETURN(rp);
 
@@ -405,11 +401,9 @@ void BestPractices::PostCallRecordCmdNextSubpass(VkCommandBuffer commandBuffer,
     }
 }
 
-void BestPractices::RecordCmdBeginRenderPass(VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo* pRenderPassBegin) {
+void BestPractices::RecordCmdBeginRenderPass(bp_state::CommandBuffer& cb_state, const VkRenderPassBeginInfo* pRenderPassBegin) {
     if (!pRenderPassBegin) return;
 
-    auto cb = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-
     auto rp_state = Get<vvl::RenderPass>(pRenderPassBegin->renderPass);
     ASSERT_AND_RETURN(rp_state);
 
@@ -423,8 +417,8 @@ void BestPractices::RecordCmdBeginRenderPass(VkCommandBuffer commandBuffer, cons
         }
 
         // If renderpass doesn't load attachment, no need to validate image in queue
-        if ((!vkuFormatIsStencilOnly(attachment.format) && attachment.loadOp == VK_ATTACHMENT_LOAD_OP_NONE_KHR) ||
-            (vkuFormatHasStencil(attachment.format) && attachment.stencilLoadOp == VK_ATTACHMENT_LOAD_OP_NONE_KHR)) {
+        if ((!vkuFormatIsStencilOnly(attachment.format) && attachment.loadOp == VK_ATTACHMENT_LOAD_OP_NONE) ||
+            (vkuFormatHasStencil(attachment.format) && attachment.stencilLoadOp == VK_ATTACHMENT_LOAD_OP_NONE)) {
             continue;
         }
 
@@ -453,7 +447,7 @@ void BestPractices::RecordCmdBeginRenderPass(VkCommandBuffer commandBuffer, cons
             }
         }
 
-        QueueValidateImageView(cb->queue_submit_functions, Func::vkCmdBeginRenderPass, image_view.get(), usage);
+        QueueValidateImageView(cb_state.queue_submit_functions, Func::vkCmdBeginRenderPass, image_view.get(), usage);
     }
 
     // Check store ops
@@ -490,18 +484,16 @@ void BestPractices::RecordCmdBeginRenderPass(VkCommandBuffer commandBuffer, cons
             }
         }
 
-        QueueValidateImageView(cb->queue_submit_functions_after_render_pass, Func::vkCmdEndRenderPass, image_view.get(), usage);
+        QueueValidateImageView(cb_state.queue_submit_functions_after_render_pass, Func::vkCmdEndRenderPass, image_view.get(),
+                               usage);
     }
 }
 
-void BestPractices::RecordCmdBeginRenderingCommon(VkCommandBuffer commandBuffer) {
-    auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-
-    auto rp = cb_state->activeRenderPass.get();
+void BestPractices::RecordCmdBeginRenderingCommon(bp_state::CommandBuffer& cb_state) {
+    auto rp = cb_state.activeRenderPass.get();
     ASSERT_AND_RETURN(rp);
 
     if (VendorCheckEnabled(kBPVendorNVIDIA)) {
-        std::shared_ptr<vvl::ImageView> depth_image_view_shared_ptr;
         vvl::ImageView* depth_image_view = nullptr;
         std::optional<VkAttachmentLoadOp> load_op;
 
@@ -509,9 +501,10 @@ void BestPractices::RecordCmdBeginRenderingCommon(VkCommandBuffer commandBuffer)
             const auto depth_attachment = rp->dynamic_rendering_begin_rendering_info.pDepthAttachment;
             if (depth_attachment) {
                 load_op.emplace(depth_attachment->loadOp);
-                depth_image_view_shared_ptr = Get<vvl::ImageView>(depth_attachment->imageView);
-                ASSERT_AND_RETURN(depth_image_view_shared_ptr);
-                depth_image_view = depth_image_view_shared_ptr.get();
+                const auto depth_image_view_shared_ptr = Get<vvl::ImageView>(depth_attachment->imageView);
+                if (depth_image_view_shared_ptr) {
+                    depth_image_view = depth_image_view_shared_ptr.get();
+                }
             }
 
             for (uint32_t i = 0; i < rp->dynamic_rendering_begin_rendering_info.colorAttachmentCount; ++i) {
@@ -532,14 +525,14 @@ void BestPractices::RecordCmdBeginRenderingCommon(VkCommandBuffer commandBuffer)
                         const uint32_t attachment_index = depth_attachment->attachment;
                         if (attachment_index != VK_ATTACHMENT_UNUSED) {
                             load_op.emplace(rp->create_info.pAttachments[attachment_index].loadOp);
-                            depth_image_view = cb_state->active_attachments[attachment_index].image_view;
+                            depth_image_view = cb_state.active_attachments[attachment_index].image_view;
                         }
                     }
                 }
-                for (uint32_t i = 0; i < cb_state->active_render_pass_begin_info.clearValueCount; ++i) {
+                for (uint32_t i = 0; i < cb_state.active_render_pass_begin_info.clearValueCount; ++i) {
                     const auto& attachment = rp->create_info.pAttachments[i];
                     if (attachment.loadOp == VK_ATTACHMENT_LOAD_OP_CLEAR) {
-                        const auto& clear_color = cb_state->active_render_pass_begin_info.pClearValues[i].color;
+                        const auto& clear_color = cb_state.active_render_pass_begin_info.pClearValues[i].color;
                         RecordClearColor(attachment.format, clear_color);
                     }
                 }
@@ -548,46 +541,41 @@ void BestPractices::RecordCmdBeginRenderingCommon(VkCommandBuffer commandBuffer)
         if (depth_image_view && (depth_image_view->create_info.subresourceRange.aspectMask & VK_IMAGE_ASPECT_DEPTH_BIT) != 0U) {
             const VkImage depth_image = depth_image_view->image_state->VkHandle();
             const VkImageSubresourceRange& subresource_range = depth_image_view->create_info.subresourceRange;
-            RecordBindZcullScope(*cb_state, depth_image, subresource_range);
+            RecordBindZcullScope(cb_state, depth_image, subresource_range);
         } else {
-            RecordUnbindZcullScope(*cb_state);
+            RecordUnbindZcullScope(cb_state);
         }
         if (load_op) {
             if (*load_op == VK_ATTACHMENT_LOAD_OP_CLEAR || *load_op == VK_ATTACHMENT_LOAD_OP_DONT_CARE) {
-                RecordResetScopeZcullDirection(*cb_state);
+                RecordResetScopeZcullDirection(cb_state);
             }
         }
     }
-    if (cb_state->activeRenderPass) {
+    if (cb_state.activeRenderPass) {
         // Spec states that after BeginRenderPass all resources should be rebound
-        if (cb_state->activeRenderPass->has_multiview_enabled) {
-            cb_state->UnbindResources();
+        if (cb_state.activeRenderPass->has_multiview_enabled) {
+            cb_state.UnbindResources();
         }
     }
 }
 
-void BestPractices::RecordCmdEndRenderingCommon(VkCommandBuffer commandBuffer) {
-    auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-
-    auto rp = cb_state->activeRenderPass.get();
-    ASSERT_AND_RETURN(rp);
-
+void BestPractices::RecordCmdEndRenderingCommon(bp_state::CommandBuffer& cb_state, const vvl::RenderPass& rp_state) {
     if (VendorCheckEnabled(kBPVendorNVIDIA)) {
         std::optional<VkAttachmentStoreOp> store_op;
 
-        if (rp->use_dynamic_rendering || rp->use_dynamic_rendering_inherited) {
-            const auto depth_attachment = rp->dynamic_rendering_begin_rendering_info.pDepthAttachment;
+        if (rp_state.use_dynamic_rendering || rp_state.use_dynamic_rendering_inherited) {
+            const auto depth_attachment = rp_state.dynamic_rendering_begin_rendering_info.pDepthAttachment;
             if (depth_attachment) {
                 store_op.emplace(depth_attachment->storeOp);
             }
         } else {
-            if (rp->create_info.subpassCount > 0) {
-                const uint32_t last_subpass = rp->create_info.subpassCount - 1;
-                const auto depth_attachment = rp->create_info.pSubpasses[last_subpass].pDepthStencilAttachment;
+            if (rp_state.create_info.subpassCount > 0) {
+                const uint32_t last_subpass = rp_state.create_info.subpassCount - 1;
+                const auto depth_attachment = rp_state.create_info.pSubpasses[last_subpass].pDepthStencilAttachment;
                 if (depth_attachment) {
                     const uint32_t attachment = depth_attachment->attachment;
                     if (attachment != VK_ATTACHMENT_UNUSED) {
-                        store_op.emplace(rp->create_info.pAttachments[attachment].storeOp);
+                        store_op.emplace(rp_state.create_info.pAttachments[attachment].storeOp);
                     }
                 }
             }
@@ -595,19 +583,17 @@ void BestPractices::RecordCmdEndRenderingCommon(VkCommandBuffer commandBuffer) {
 
         if (store_op) {
             if (*store_op == VK_ATTACHMENT_STORE_OP_DONT_CARE || *store_op == VK_ATTACHMENT_STORE_OP_NONE) {
-                RecordResetScopeZcullDirection(*cb_state);
+                RecordResetScopeZcullDirection(cb_state);
             }
         }
 
-        RecordUnbindZcullScope(*cb_state);
+        RecordUnbindZcullScope(cb_state);
     }
 }
 
 bool BestPractices::PreCallValidateCmdBeginRenderPass(VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo* pRenderPassBegin,
                                                       VkSubpassContents contents, const ErrorObject& error_obj) const {
-    bool skip = StateTracker::PreCallValidateCmdBeginRenderPass(commandBuffer, pRenderPassBegin, contents, error_obj);
-    skip |= ValidateCmdBeginRenderPass(commandBuffer, pRenderPassBegin, error_obj.location);
-    return skip;
+    return ValidateCmdBeginRenderPass(commandBuffer, pRenderPassBegin, error_obj.location);
 }
 
 bool BestPractices::PreCallValidateCmdBeginRenderPass2KHR(VkCommandBuffer commandBuffer,
@@ -620,43 +606,26 @@ bool BestPractices::PreCallValidateCmdBeginRenderPass2KHR(VkCommandBuffer comman
 bool BestPractices::PreCallValidateCmdBeginRenderPass2(VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo* pRenderPassBegin,
                                                        const VkSubpassBeginInfo* pSubpassBeginInfo,
                                                        const ErrorObject& error_obj) const {
-    bool skip = StateTracker::PreCallValidateCmdBeginRenderPass2(commandBuffer, pRenderPassBegin, pSubpassBeginInfo, error_obj);
-    skip |= ValidateCmdBeginRenderPass(commandBuffer, pRenderPassBegin, error_obj.location);
-    return skip;
-}
-
-bool BestPractices::PreCallValidateCmdBeginRendering(VkCommandBuffer commandBuffer, const VkRenderingInfo* pRenderingInfo,
-                                                     const ErrorObject& error_obj) const {
-    bool skip = StateTracker::PreCallValidateCmdBeginRendering(commandBuffer, pRenderingInfo, error_obj);
-    skip |= ValidateCmdBeginRendering(commandBuffer, pRenderingInfo, error_obj.location);
-    return skip;
-}
-
-bool BestPractices::PreCallValidateCmdBeginRenderingKHR(VkCommandBuffer commandBuffer, const VkRenderingInfo* pRenderingInfo,
-                                                        const ErrorObject& error_obj) const {
-    return PreCallValidateCmdBeginRendering(commandBuffer, pRenderingInfo, error_obj);
+    return ValidateCmdBeginRenderPass(commandBuffer, pRenderPassBegin, error_obj.location);
 }
 
 void BestPractices::PostCallRecordCmdNextSubpass2KHR(VkCommandBuffer commandBuffer, const VkSubpassBeginInfo* pSubpassBeginInfo,
                                                      const VkSubpassEndInfo* pSubpassEndInfo, const RecordObject& record_obj) {
-    StateTracker::PostCallRecordCmdNextSubpass2(commandBuffer, pSubpassBeginInfo, pSubpassEndInfo, record_obj);
-    RecordCmdNextSubpass(commandBuffer);
+    PostCallRecordCmdNextSubpass2(commandBuffer, pSubpassBeginInfo, pSubpassEndInfo, record_obj);
 }
 
 void BestPractices::PostCallRecordCmdNextSubpass2(VkCommandBuffer commandBuffer, const VkSubpassBeginInfo* pSubpassBeginInfo,
                                                   const VkSubpassEndInfo* pSubpassEndInfo, const RecordObject& record_obj) {
-    StateTracker::PostCallRecordCmdNextSubpass2(commandBuffer, pSubpassBeginInfo, pSubpassEndInfo, record_obj);
-    RecordCmdNextSubpass(commandBuffer);
+    BaseClass::PostCallRecordCmdNextSubpass2(commandBuffer, pSubpassBeginInfo, pSubpassEndInfo, record_obj);
+    auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
+    RecordCmdNextSubpass(*cb_state);
 }
 
-void BestPractices::RecordCmdNextSubpass(VkCommandBuffer commandBuffer) {
-    auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
-    if (cb_state) {
-        if (cb_state->activeRenderPass) {
-            // Spec states that after NextSubpass all resources should be rebound
-            if (cb_state->activeRenderPass->has_multiview_enabled) {
-                cb_state->UnbindResources();
-            }
+void BestPractices::RecordCmdNextSubpass(bp_state::CommandBuffer& cb_state) {
+    if (cb_state.activeRenderPass) {
+        // Spec states that after NextSubpass all resources should be rebound
+        if (cb_state.activeRenderPass->has_multiview_enabled) {
+            cb_state.UnbindResources();
         }
     }
 }
@@ -664,21 +633,25 @@ void BestPractices::RecordCmdNextSubpass(VkCommandBuffer commandBuffer) {
 void BestPractices::PostCallRecordCmdPushConstants(VkCommandBuffer commandBuffer, VkPipelineLayout layout,
                                                    VkShaderStageFlags stageFlags, uint32_t offset, uint32_t size,
                                                    const void* pValues, const RecordObject& record_obj) {
-    StateTracker::PostCallRecordCmdPushConstants(commandBuffer, layout, stageFlags, offset, size, pValues, record_obj);
+    BaseClass::PostCallRecordCmdPushConstants(commandBuffer, layout, stageFlags, offset, size, pValues, record_obj);
+}
+
+void BestPractices::PostCallRecordCmdPushConstants2(VkCommandBuffer commandBuffer, const VkPushConstantsInfo* pPushConstantsInfo,
+                                                    const RecordObject& record_obj) {
+    BaseClass::PostCallRecordCmdPushConstants2(commandBuffer, pPushConstantsInfo, record_obj);
 }
 
 void BestPractices::PostCallRecordCmdPushConstants2KHR(VkCommandBuffer commandBuffer,
                                                        const VkPushConstantsInfoKHR* pPushConstantsInfo,
                                                        const RecordObject& record_obj) {
-    StateTracker::PostCallRecordCmdPushConstants2KHR(commandBuffer, pPushConstantsInfo, record_obj);
+    PostCallRecordCmdPushConstants2(commandBuffer, pPushConstantsInfo, record_obj);
 }
 
-void BestPractices::PostRecordCmdBeginRenderPass(VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo* pRenderPassBegin) {
+void BestPractices::PostRecordCmdBeginRenderPass(bp_state::CommandBuffer& cb_state, const VkRenderPassBeginInfo* pRenderPassBegin) {
     // Reset the renderpass state
-    auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
     // TODO - move this logic to the Render Pass state as cb->has_draw_cmd should stay true for lifetime of command buffer
-    cb_state->has_draw_cmd = false;
-    auto& render_pass_state = cb_state->render_pass_state;
+    cb_state.has_draw_cmd = false;
+    auto& render_pass_state = cb_state.render_pass_state;
     render_pass_state.touchesAttachments.clear();
     render_pass_state.earlyClearAttachments.clear();
     render_pass_state.numDrawCallsDepthOnly = 0;
@@ -689,7 +662,7 @@ void BestPractices::PostRecordCmdBeginRenderPass(VkCommandBuffer commandBuffer,
     // Don't reset state related to pipeline state.
 
     // Reset NV state
-    cb_state->nv = {};
+    cb_state.nv = {};
 
     if (auto rp_state = Get<vvl::RenderPass>(pRenderPassBegin->renderPass)) {
         // track depth / color attachment usage within the renderpass
@@ -699,10 +672,10 @@ void BestPractices::PostRecordCmdBeginRenderPass(VkCommandBuffer commandBuffer,
 
             if (rp_state->create_info.pSubpasses[i].colorAttachmentCount > 0) render_pass_state.colorAttachment = true;
         }
-        if (cb_state->activeRenderPass) {
+        if (cb_state.activeRenderPass) {
             // Spec states that after BeginRenderPass all resources should be rebound
-            if (cb_state->activeRenderPass->has_multiview_enabled) {
-                cb_state->UnbindResources();
+            if (cb_state.activeRenderPass->has_multiview_enabled) {
+                cb_state.UnbindResources();
             }
         }
     }
@@ -710,14 +683,20 @@ void BestPractices::PostRecordCmdBeginRenderPass(VkCommandBuffer commandBuffer,
 
 void BestPractices::PostCallRecordCmdBeginRenderPass(VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo* pRenderPassBegin,
                                                      VkSubpassContents contents, const RecordObject& record_obj) {
-    StateTracker::PostCallRecordCmdBeginRenderPass(commandBuffer, pRenderPassBegin, contents, record_obj);
-    PostRecordCmdBeginRenderPass(commandBuffer, pRenderPassBegin);
+    BaseClass::PostCallRecordCmdBeginRenderPass(commandBuffer, pRenderPassBegin, contents, record_obj);
+    auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
+    PostRecordCmdBeginRenderPass(*cb_state, pRenderPassBegin);
+    RecordCmdBeginRenderingCommon(*cb_state);
+    RecordCmdBeginRenderPass(*cb_state, pRenderPassBegin);
 }
 
 void BestPractices::PostCallRecordCmdBeginRenderPass2(VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo* pRenderPassBegin,
                                                       const VkSubpassBeginInfo* pSubpassBeginInfo, const RecordObject& record_obj) {
-    StateTracker::PostCallRecordCmdBeginRenderPass2(commandBuffer, pRenderPassBegin, pSubpassBeginInfo, record_obj);
-    PostRecordCmdBeginRenderPass(commandBuffer, pRenderPassBegin);
+    BaseClass::PostCallRecordCmdBeginRenderPass2(commandBuffer, pRenderPassBegin, pSubpassBeginInfo, record_obj);
+    auto cb_state = GetWrite<bp_state::CommandBuffer>(commandBuffer);
+    PostRecordCmdBeginRenderPass(*cb_state, pRenderPassBegin);
+    RecordCmdBeginRenderingCommon(*cb_state);
+    RecordCmdBeginRenderPass(*cb_state, pRenderPassBegin);
 }
 
 void BestPractices::PostCallRecordCmdBeginRenderPass2KHR(VkCommandBuffer commandBuffer,
@@ -729,9 +708,7 @@ void BestPractices::PostCallRecordCmdBeginRenderPass2KHR(VkCommandBuffer command
 
 bool BestPractices::PreCallValidateCmdEndRenderPass2(VkCommandBuffer commandBuffer, const VkSubpassEndInfo* pSubpassEndInfo,
                                                      const ErrorObject& error_obj) const {
-    bool skip = false;
-    skip |= StateTracker::PreCallValidateCmdEndRenderPass2(commandBuffer, pSubpassEndInfo, error_obj);
-    skip |= ValidateCmdEndRenderPass(commandBuffer, error_obj.location);
+    bool skip = ValidateCmdEndRenderPass(commandBuffer, error_obj.location);
     if (VendorCheckEnabled(kBPVendorNVIDIA)) {
         const auto cb_state = GetRead<bp_state::CommandBuffer>(commandBuffer);
         skip |= ValidateZcullScope(*cb_state, error_obj.location);
@@ -745,9 +722,7 @@ bool BestPractices::PreCallValidateCmdEndRenderPass2KHR(VkCommandBuffer commandB
 }
 
 bool BestPractices::PreCallValidateCmdEndRenderPass(VkCommandBuffer commandBuffer, const ErrorObject& error_obj) const {
-    bool skip = false;
-    skip |= StateTracker::PreCallValidateCmdEndRenderPass(commandBuffer, error_obj);
-    skip |= ValidateCmdEndRenderPass(commandBuffer, error_obj.location);
+    bool skip = ValidateCmdEndRenderPass(commandBuffer, error_obj.location);
     if (VendorCheckEnabled(kBPVendorNVIDIA)) {
         const auto cb_state = GetRead<bp_state::CommandBuffer>(commandBuffer);
         skip |= ValidateZcullScope(*cb_state, error_obj.location);
@@ -757,7 +732,6 @@ bool BestPractices::PreCallValidateCmdEndRenderPass(VkCommandBuffer commandBuffe
 
 bool BestPractices::PreCallValidateCmdEndRendering(VkCommandBuffer commandBuffer, const ErrorObject& error_obj) const {
     bool skip = false;
-    skip |= StateTracker::PreCallValidateCmdEndRendering(commandBuffer, error_obj);
     if (VendorCheckEnabled(kBPVendorNVIDIA)) {
         const auto cb_state = GetRead<bp_state::CommandBuffer>(commandBuffer);
         skip |= ValidateZcullScope(*cb_state, error_obj.location);
@@ -846,11 +820,12 @@ bool BestPractices::ValidateCmdEndRenderPass(VkCommandBuffer commandBuffer, cons
             if (untouched_aspects) {
                 skip |= LogPerformanceWarning(
                     "BestPractices-vkCmdEndRenderPass-redundant-attachment-on-tile", commandBuffer, loc,
-                    "%s %s: Render pass was ended, but attachment #%u (format: %u, untouched aspects 0x%x) "
+                    "%s %s: Render pass was ended, but attachment #%u (format: %s, untouched aspects %s) "
                     "was never accessed by a pipeline or clear command. "
                     "On tile-based architectures, LOAD_OP_LOAD and STORE_OP_STORE consume bandwidth and should not be part of the "
                     "render pass if the attachments are not intended to be accessed.",
-                    VendorSpecificTag(kBPVendorArm), VendorSpecificTag(kBPVendorIMG), i, attachment.format, untouched_aspects);
+                    VendorSpecificTag(kBPVendorArm), VendorSpecificTag(kBPVendorIMG), i, string_VkFormat(attachment.format),
+                    string_VkImageAspectFlags(untouched_aspects).c_str());
             }
         }
     }
@@ -868,14 +843,14 @@ void BestPractices::RecordAttachmentAccess(bp_state::CommandBuffer& cb_state, ui
     if (itr != rp_state.touchesAttachments.end()) {
         itr->aspects |= aspects;
     } else {
-        rp_state.touchesAttachments.push_back({fb_attachment, aspects});
+        rp_state.touchesAttachments.emplace_back(fb_attachment, aspects);
     }
 }
 
-void BestPractices::RecordAttachmentClearAttachments(bp_state::CommandBuffer& cmd_state, uint32_t fb_attachment,
+void BestPractices::RecordAttachmentClearAttachments(bp_state::CommandBuffer& cb_state, uint32_t fb_attachment,
                                                      uint32_t color_attachment, VkImageAspectFlags aspects, uint32_t rectCount,
                                                      const VkClearRect* pRects) {
-    auto& rp_state = cmd_state.render_pass_state;
+    auto& rp_state = cb_state.render_pass_state;
     // If we observe a full clear before any other access to a frame buffer attachment,
     // we have candidate for redundant clear attachments.
     auto itr =
@@ -887,14 +862,14 @@ void BestPractices::RecordAttachmentClearAttachments(bp_state::CommandBuffer& cm
         new_aspects = aspects & ~itr->aspects;
         itr->aspects |= aspects;
     } else {
-        rp_state.touchesAttachments.push_back({fb_attachment, aspects});
+        rp_state.touchesAttachments.emplace_back(fb_attachment, aspects);
     }
 
     if (new_aspects == 0) {
         return;
     }
 
-    if (cmd_state.IsSeconary()) {
+    if (cb_state.IsSecondary()) {
         // The first command might be a clear, but might not be the first in the render pass, defer any checks until
         // CmdExecuteCommands.
         rp_state.earlyClearAttachments.push_back(
