@@ -1,6 +1,6 @@
-/* Copyright (c) 2015-2017, 2019-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2017, 2019-2023 Valve Corporation
- * Copyright (c) 2015-2017, 2019-2023 LunarG, Inc.
+/* Copyright (c) 2015-2017, 2019-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2017, 2019-2024 Valve Corporation
+ * Copyright (c) 2015-2017, 2019-2024 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,9 +23,7 @@
 #include <cassert>
 #include <limits>
 #include <memory>
-#include <map>
 #include <unordered_map>
-#include <set>
 #include <algorithm>
 #include <iterator>
 #include <type_traits>
@@ -35,9 +33,13 @@
 #ifdef USE_ROBIN_HOOD_HASHING
 #include "robin_hood.h"
 #else
+#include <map>
+#include <set>
 #include <unordered_set>
 #endif
 
+#include <vulkan/utility/vk_concurrent_unordered_map.hpp>
+
 // namespace aliases to allow map and set implementations to easily be swapped out
 namespace vvl {
 
@@ -108,6 +110,14 @@ template <typename T>
 using insert_iterator = std::insert_iterator<T>;
 #endif
 
+#if 1
+template <typename Key, typename T, int BucketsLog2 = 2, typename Hash = std::hash<Key>>
+using concurrent_unordered_map = vku::concurrent::unordered_map<Key, T, BucketsLog2, vvl::unordered_map<Key, T, Hash>>;
+#else
+template <typename Key, typename T, int BucketsLog2 = 2>
+using concurrent_unordered_map = vku::concurrent_unordered_map<Key, T, BucketsLog2, vvl::unordered_map<Key, T>>;
+#endif
+
 }  // namespace vvl
 
 // A vector class with "small string optimization" -- meaning that the class contains a fixed working store for N elements.
@@ -118,7 +128,7 @@ using insert_iterator = std::insert_iterator<T>;
 //       MoveAssignable and MoveConstructable
 // NOTE: Unlike std::vector, iterators are invalidated by move assignment between small_vector objects effectively the
 //       "small string" allocation functions as an incompatible allocator.
-template <typename T, size_t N, typename SizeType = uint8_t>
+template <typename T, size_t N, typename SizeType = uint32_t>
 class small_vector {
   public:
     using value_type = T;
@@ -804,51 +814,6 @@ class small_unordered_map : public small_container<Key, typename vvl::unordered_
 template <typename Key, int N = 1>
 class small_unordered_set : public small_container<Key, Key, vvl::unordered_set<Key>, value_type_helper_set<Key>, N> {};
 
-// For the given data key, look up the layer_data instance from given layer_data_map
-template <typename DATA_T>
-DATA_T *GetLayerDataPtr(void *data_key, small_unordered_map<void *, DATA_T *, 2> &layer_data_map) {
-    /* TODO: We probably should lock here, or have caller lock */
-    DATA_T *&got = layer_data_map[data_key];
-
-    if (got == nullptr) {
-        got = new DATA_T;
-    }
-
-    return got;
-}
-
-template <typename DATA_T>
-void FreeLayerDataPtr(void *data_key, small_unordered_map<void *, DATA_T *, 2> &layer_data_map) {
-    delete layer_data_map[data_key];
-    layer_data_map.erase(data_key);
-}
-
-// For the given data key, look up the layer_data instance from given layer_data_map
-template <typename DATA_T>
-DATA_T *GetLayerDataPtr(void *data_key, std::unordered_map<void *, DATA_T *> &layer_data_map) {
-    DATA_T *debug_data;
-    /* TODO: We probably should lock here, or have caller lock */
-    auto got = layer_data_map.find(data_key);
-
-    if (got == layer_data_map.end()) {
-        debug_data = new DATA_T;
-        layer_data_map[(void *)data_key] = debug_data;
-    } else {
-        debug_data = got->second;
-    }
-
-    return debug_data;
-}
-
-template <typename DATA_T>
-void FreeLayerDataPtr(void *data_key, std::unordered_map<void *, DATA_T *> &layer_data_map) {
-    auto got = layer_data_map.find(data_key);
-    assert(got != layer_data_map.end());
-
-    delete got->second;
-    layer_data_map.erase(got);
-}
-
 namespace vvl {
 
 inline constexpr std::in_place_t in_place{};
@@ -875,8 +840,8 @@ class enumeration {
     iterator end() { return data_ + count_; }
     const_iterator end() const { return data_ + count_; }
 
-    T &operator[](int i) { return data_[i]; }
-    const T &operator[](int i) const { return data_[i]; }
+    T &operator[](size_t i) { return data_[i]; }
+    const T &operator[](size_t i) const { return data_[i]; }
 
     T &front() { return *data_; }
     const T &front() const { return *data_; }
@@ -887,8 +852,7 @@ class enumeration {
     size_t size() const { return count_; }
     bool empty() const { return count_ == 0; }
 
-    pointer data() { return data_; }
-    const_pointer data() const { return data_; }
+    pointer data() const { return data_; }
 
   private:
     pointer data_ = {};
@@ -954,6 +918,14 @@ enumeration<T, IndexedIterator<T>> enumerate(T *begin, T *end) {
     return enumeration<T, IndexedIterator<T>>(begin, end);
 }
 
+template <typename Container>
+enumeration<typename Container::value_type, IndexedIterator<typename Container::value_type, typename Container::size_type>>
+enumerate(Container &container) {
+    return enumeration<typename Container::value_type,
+                       IndexedIterator<typename Container::value_type, typename Container::size_type>>(container.data(),
+                                                                                                       container.size());
+}
+
 template <typename BaseType>
 using base_type =
     typename std::remove_reference<typename std::remove_const<typename std::remove_pointer<BaseType>::type>::type>::type;
@@ -1030,6 +1002,56 @@ bool Contains(const Container &container, const Key &key) {
     return container.find(key) != container.cend();
 }
 
+//
+// if (vvl::Contains(objects_vector, candidate)) { candidate->jump(); }
+//
+template <typename T>
+bool Contains(const std::vector<T> &v, const T &value) {
+    return std::find(v.cbegin(), v.cend(), value) != v.cend();
+}
+// Overload for the case of shared_ptr<const T> and shared_ptr<T>.
+// They are convertible but conversion is not performed during template type deduction.
+template <typename T>
+bool Contains(const std::vector<std::shared_ptr<const T>> &v, const std::shared_ptr<T> &value) {
+    return std::find(v.cbegin(), v.cend(), value) != v.cend();
+}
+
+//
+// if (auto* thing = vvl::Find(map, key)) { thing->jump(); }
+//
+template <typename Container, typename Key = typename Container::key_type, typename Value = typename Container::mapped_type>
+Value *Find(Container &container, const Key &key) {
+    auto it = container.find(key);
+    return (it != container.end()) ? &it->second : nullptr;
+}
+template <typename Container, typename Key = typename Container::key_type, typename Value = typename Container::mapped_type>
+const Value *Find(const Container &container, const Key &key) {
+    auto it = container.find(key);
+    return (it != container.cend()) ? &it->second : nullptr;
+}
+
+//
+// auto& thing = vvl::FindExisting(map, key);
+//
+template <typename Container, typename Key = typename Container::key_type, typename Value = typename Container::mapped_type>
+Value &FindExisting(Container &container, const Key &key) {
+    auto it = container.find(key);
+    assert(it != container.end());
+    return it->second;
+}
+
+template <typename Container, typename Key = typename Container::key_type, typename Value = typename Container::mapped_type>
+const Value &FindExisting(const Container &container, const Key &key) {
+    auto it = container.find(key);
+    assert(it != container.end());
+    return it->second;
+}
+
+template <typename T>
+void Append(std::vector<T> &dst, const std::vector<T> &src) {
+    dst.insert(dst.end(), src.begin(), src.end());
+}
+
 // EraseIf is not implemented as std::erase(std::remove_if(...), ...) for two reasons:
 //   1) Robin Hood containers don't support two-argument erase functions
 //   2) STL remove_if requires the predicate to be const w.r.t the value-type, and std::erase_if doesn't AFAICT
@@ -1047,6 +1069,16 @@ typename Container::size_type EraseIf(Container &c, Predicate &&p) {
     return before_size - c.size();
 }
 
+// Replace with the std version after VVL switches to C++20.
+// https://en.cppreference.com/w/cpp/container/vector/erase2
+template <typename T, typename Pred>
+typename std::vector<T>::size_type erase_if(std::vector<T> &c, Pred pred) {
+    auto it = std::remove_if(c.begin(), c.end(), pred);
+    auto r = c.end() - it;
+    c.erase(it, c.end());
+    return r;
+}
+
 template <typename T>
 constexpr T MaxTypeValue([[maybe_unused]] T) {
     return std::numeric_limits<T>::max();
@@ -1066,6 +1098,10 @@ constexpr auto kI32Max = std::numeric_limits<int32_t>::max();
 // Typesafe INT64_MAX
 constexpr auto kI64Max = std::numeric_limits<int64_t>::max();
 
+// Descriptive names to indicate uninitialized/invalid unsigned index values
+constexpr auto kNoIndex32 = kU32Max;
+constexpr auto kNoIndex64 = kU64Max;
+
 template <typename T>
 T GetQuotientCeil(T numerator, T denominator) {
     denominator = std::max(denominator, T{1});
