@@ -19,7 +19,6 @@
  */
 
 #include <vulkan/vk_enum_string_helper.h>
-#include "generated/chassis.h"
 #include "core_validation.h"
 #include "chassis/chassis_modification_state.h"
 
@@ -28,8 +27,8 @@ bool CoreChecks::PreCallValidateCreateComputePipelines(VkDevice device, VkPipeli
                                                        const VkAllocationCallbacks *pAllocator, VkPipeline *pPipelines,
                                                        const ErrorObject &error_obj, PipelineStates &pipeline_states,
                                                        chassis::CreateComputePipelines &chassis_state) const {
-    bool skip = StateTracker::PreCallValidateCreateComputePipelines(device, pipelineCache, count, pCreateInfos, pAllocator,
-                                                                    pPipelines, error_obj, pipeline_states, chassis_state);
+    bool skip = BaseClass::PreCallValidateCreateComputePipelines(device, pipelineCache, count, pCreateInfos, pAllocator, pPipelines,
+                                                                 error_obj, pipeline_states, chassis_state);
 
     skip |= ValidateDeviceQueueSupport(error_obj.location);
     for (uint32_t i = 0; i < count; i++) {
@@ -41,16 +40,17 @@ bool CoreChecks::PreCallValidateCreateComputePipelines(VkDevice device, VkPipeli
         const auto &stage_state = pipeline->stage_states[0];
         skip |= ValidateShaderStage(stage_state, pipeline, stage_info);
         if (stage_state.pipeline_create_info) {
-            skip |= ValidatePipelineShaderStage(*pipeline, *stage_state.pipeline_create_info, stage_info);
+            skip |= ValidatePipelineShaderStage(*pipeline, *stage_state.pipeline_create_info, pCreateInfos[i].pNext, stage_info);
         }
 
+        skip |= ValidateComputePipelineDerivatives(pipeline_states, i, create_info_loc);
         skip |= ValidatePipelineCacheControlFlags(pipeline->create_flags, create_info_loc.dot(Field::flags),
                                                   "VUID-VkComputePipelineCreateInfo-pipelineCreationCacheControl-02875");
         skip |= ValidatePipelineIndirectBindableFlags(pipeline->create_flags, create_info_loc.dot(Field::flags),
                                                       "VUID-VkComputePipelineCreateInfo-flags-09007");
 
         if (const auto *pipeline_robustness_info =
-                vku::FindStructInPNextChain<VkPipelineRobustnessCreateInfoEXT>(pCreateInfos[i].pNext)) {
+                vku::FindStructInPNextChain<VkPipelineRobustnessCreateInfo>(pCreateInfos[i].pNext)) {
             skip |= ValidatePipelineRobustnessCreateInfo(*pipeline, *pipeline_robustness_info, create_info_loc);
         }
 
@@ -64,3 +64,36 @@ bool CoreChecks::PreCallValidateCreateComputePipelines(VkDevice device, VkPipeli
     }
     return skip;
 }
+
+bool CoreChecks::ValidateComputePipelineDerivatives(PipelineStates &pipeline_states, uint32_t pipe_index,
+                                                    const Location &loc) const {
+    bool skip = false;
+    const auto &pipeline = *pipeline_states[pipe_index].get();
+    // If create derivative bit is set, check that we've specified a base
+    // pipeline correctly, and that the base pipeline was created to allow
+    // derivatives.
+    if (pipeline.create_flags & VK_PIPELINE_CREATE_2_DERIVATIVE_BIT) {
+        std::shared_ptr<const vvl::Pipeline> base_pipeline;
+        const auto &pipeline_ci = pipeline.ComputeCreateInfo();
+        const VkPipeline base_handle = pipeline_ci.basePipelineHandle;
+        const int32_t base_index = pipeline_ci.basePipelineIndex;
+        if (base_index != -1 && base_index < static_cast<int32_t>(pipeline_states.size())) {
+            if (static_cast<uint32_t>(base_index) >= pipe_index) {
+                skip |= LogError("VUID-vkCreateComputePipelines-flags-00695", base_handle, loc,
+                                 "base pipeline (index %" PRId32
+                                 ") must occur earlier in array than derivative pipeline (index %" PRIu32 ").",
+                                 base_index, pipe_index);
+            } else {
+                base_pipeline = pipeline_states[base_index];
+            }
+        } else if (base_handle != VK_NULL_HANDLE) {
+            base_pipeline = Get<vvl::Pipeline>(base_handle);
+        }
+
+        if (base_pipeline && !(base_pipeline->create_flags & VK_PIPELINE_CREATE_2_ALLOW_DERIVATIVES_BIT)) {
+            skip |= LogError("VUID-vkCreateComputePipelines-flags-00696", base_pipeline->Handle(), loc,
+                             "base pipeline does not allow derivatives.");
+        }
+    }
+    return skip;
+}
