@@ -25,7 +25,8 @@ struct GpuVuidsCmdDraw : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDraw-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDraw-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDraw-None-08613";
-        invalid_descriptor = "VUID-vkCmdDraw-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDraw-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDraw-None-10068";
     }
 };
 
@@ -35,7 +36,8 @@ struct GpuVuidsCmdDrawMultiEXT : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawMultiEXT-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawMultiEXT-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawMultiEXT-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawMultiEXT-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawMultiEXT-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawMultiEXT-None-10068";
     }
 };
 
@@ -45,7 +47,8 @@ struct GpuVuidsCmdDrawIndexed : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawIndexed-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawIndexed-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawIndexed-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawIndexed-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawIndexed-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawIndexed-None-10068";
     }
 };
 
@@ -55,7 +58,8 @@ struct GpuVuidsCmdDrawMultiIndexedEXT : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawMultiIndexedEXT-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawMultiIndexedEXT-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawMultiIndexedEXT-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawMultiIndexedEXT-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawMultiIndexedEXT-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawMultiIndexedEXT-None-10068";
     }
 };
 
@@ -65,8 +69,9 @@ struct GpuVuidsCmdDrawIndirect : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawIndirect-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawIndirect-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawIndirect-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawIndirect-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawIndirect-None-08114";
         first_instance_not_zero = "VUID-VkDrawIndirectCommand-firstInstance-00501";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawIndirect-None-10068";
     }
 };
 
@@ -76,8 +81,9 @@ struct GpuVuidsCmdDrawIndexedIndirect : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawIndexedIndirect-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawIndexedIndirect-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawIndexedIndirect-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawIndexedIndirect-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawIndexedIndirect-None-08114";
         first_instance_not_zero = "VUID-VkDrawIndexedIndirectCommand-firstInstance-00554";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawIndexedIndirect-None-10068";
     }
 };
 
@@ -87,7 +93,8 @@ struct GpuVuidsCmdDispatch : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDispatch-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDispatch-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDispatch-None-08613";
-        invalid_descriptor = "VUID-vkCmdDispatch-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDispatch-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDispatch-None-10068";
     }
 };
 
@@ -97,10 +104,11 @@ struct GpuVuidsCmdDispatchIndirect : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDispatchIndirect-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDispatchIndirect-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDispatchIndirect-None-08613";
-        invalid_descriptor = "VUID-vkCmdDispatchIndirect-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDispatchIndirect-None-08114";
         group_exceeds_device_limit_x = "VUID-VkDispatchIndirectCommand-x-00417";
         group_exceeds_device_limit_y = "VUID-VkDispatchIndirectCommand-y-00418";
         group_exceeds_device_limit_z = "VUID-VkDispatchIndirectCommand-z-00419";
+        descriptor_index_oob_10068 = "VUID-vkCmdDispatchIndirect-None-10068";
 
     }
 };
@@ -111,10 +119,9 @@ struct GpuVuidsCmdDrawIndirectCount : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawIndirectCount-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawIndirectCount-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawIndirectCount-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawIndirectCount-None-08114";
-        count_exceeds_bufsize_1 = "VUID-vkCmdDrawIndirectCount-countBuffer-03121";
-        count_exceeds_bufsize = "VUID-vkCmdDrawIndirectCount-countBuffer-03122";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawIndirectCount-None-08114";
         count_exceeds_device_limit = "VUID-vkCmdDrawIndirectCount-countBuffer-02717";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawIndirectCount-None-10068";
     }
 };
 
@@ -124,10 +131,9 @@ struct GpuVuidsCmdDrawIndexedIndirectCount : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawIndexedIndirectCount-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawIndexedIndirectCount-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawIndexedIndirectCount-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawIndexedIndirectCount-None-08114";
-        count_exceeds_bufsize_1 = "VUID-vkCmdDrawIndexedIndirectCount-countBuffer-03153";
-        count_exceeds_bufsize = "VUID-vkCmdDrawIndexedIndirectCount-countBuffer-03154";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawIndexedIndirectCount-None-08114";
         count_exceeds_device_limit = "VUID-vkCmdDrawIndexedIndirectCount-countBuffer-02717";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawIndexedIndirectCount-None-10068";
     }
 };
 
@@ -137,7 +143,8 @@ struct GpuVuidsCmdTraceRaysNV : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdTraceRaysNV-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdTraceRaysNV-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdTraceRaysNV-None-08613";
-        invalid_descriptor = "VUID-vkCmdTraceRaysNV-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdTraceRaysNV-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdTraceRaysNV-None-10068";
     }
 };
 
@@ -147,7 +154,8 @@ struct GpuVuidsCmdTraceRaysKHR : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdTraceRaysKHR-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdTraceRaysKHR-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdTraceRaysKHR-None-08613";
-        invalid_descriptor = "VUID-vkCmdTraceRaysKHR-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdTraceRaysKHR-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdTraceRaysKHR-None-10068";
     }
 };
 
@@ -157,10 +165,8 @@ struct GpuVuidsCmdTraceRaysIndirectKHR : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdTraceRaysIndirectKHR-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdTraceRaysIndirectKHR-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdTraceRaysIndirectKHR-None-08613";
-        invalid_descriptor = "VUID-vkCmdTraceRaysIndirectKHR-None-08114";
-        trace_rays_width_exceeds_device_limit = "VUID-VkTraceRaysIndirectCommandKHR-width-03638";
-        trace_rays_height_exceeds_device_limit = "VUID-VkTraceRaysIndirectCommandKHR-height-03639";
-        trace_rays_depth_exceeds_device_limit = "VUID-VkTraceRaysIndirectCommandKHR-depth-03640";
+        invalid_descriptor_08114 = "VUID-vkCmdTraceRaysIndirectKHR-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdTraceRaysIndirectKHR-None-10068";
     }
 };
 
@@ -170,9 +176,10 @@ struct GpuVuidsCmdTraceRaysIndirect2KHR : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdTraceRaysIndirect2KHR-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdTraceRaysIndirect2KHR-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdTraceRaysIndirect2KHR-None-08613";
-        invalid_descriptor = "VUID-vkCmdTraceRaysIndirect2KHR-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdTraceRaysIndirect2KHR-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdTraceRaysIndirect2KHR-None-10068";
     }
-};
+};  
 
 struct GpuVuidsCmdDrawMeshTasksNV : GpuVuid {
     GpuVuidsCmdDrawMeshTasksNV() : GpuVuid() {
@@ -180,7 +187,8 @@ struct GpuVuidsCmdDrawMeshTasksNV : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawMeshTasksNV-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawMeshTasksNV-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawMeshTasksNV-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawMeshTasksNV-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawMeshTasksNV-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawMeshTasksNV-None-10068";
     }
 };
 
@@ -190,7 +198,8 @@ struct GpuVuidsCmdDrawMeshTasksIndirectNV : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawMeshTasksIndirectNV-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawMeshTasksIndirectNV-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawMeshTasksIndirectNV-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawMeshTasksIndirectNV-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawMeshTasksIndirectNV-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawMeshTasksIndirectNV-None-10068";
         task_group_count_exceeds_max_x = "VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07322";
         task_group_count_exceeds_max_y = "VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07323";
         task_group_count_exceeds_max_z = "VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07324";
@@ -208,9 +217,8 @@ struct GpuVuidsCmdDrawMeshTasksIndirectCountNV : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawMeshTasksIndirectCountNV-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawMeshTasksIndirectCountNV-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawMeshTasksIndirectCountNV-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawMeshTasksIndirectCountNV-None-08114";
-        count_exceeds_bufsize_1 = "VUID-vkCmdDrawMeshTasksIndirectCountNV-countBuffer-02191";
-        count_exceeds_bufsize = "VUID-vkCmdDrawMeshTasksIndirectCountNV-countBuffer-02192";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawMeshTasksIndirectCountNV-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawMeshTasksIndirectCountNV-None-10068";
         count_exceeds_device_limit = "VUID-vkCmdDrawMeshTasksIndirectCountNV-countBuffer-02717";
         task_group_count_exceeds_max_x = "VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07322";
         task_group_count_exceeds_max_y = "VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07323";
@@ -229,7 +237,8 @@ struct GpuVuidsCmdDrawMeshTasksEXT : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawMeshTasksEXT-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawMeshTasksEXT-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawMeshTasksEXT-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawMeshTasksEXT-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawMeshTasksEXT-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawMeshTasksEXT-None-10068";
     }
 };
 
@@ -239,7 +248,8 @@ struct GpuVuidsCmdDrawMeshTasksIndirectEXT : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawMeshTasksIndirectEXT-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawMeshTasksIndirectEXT-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawMeshTasksIndirectEXT-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawMeshTasksIndirectEXT-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawMeshTasksIndirectEXT-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawMeshTasksIndirectEXT-None-10068";
         task_group_count_exceeds_max_x = "VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07322";
         task_group_count_exceeds_max_y = "VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07323";
         task_group_count_exceeds_max_z = "VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07324";
@@ -257,9 +267,8 @@ struct GpuVuidsCmdDrawMeshTasksIndirectCountEXT : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawMeshTasksIndirectCountEXT-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawMeshTasksIndirectCountEXT-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawMeshTasksIndirectCountEXT-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawMeshTasksIndirectCountEXT-None-08114";
-        count_exceeds_bufsize_1 = "VUID-vkCmdDrawMeshTasksIndirectCountEXT-countBuffer-07098";
-        count_exceeds_bufsize = "VUID-vkCmdDrawMeshTasksIndirectCountEXT-countBuffer-07099";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawMeshTasksIndirectCountEXT-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawMeshTasksIndirectCountEXT-None-10068";
         count_exceeds_device_limit = "VUID-vkCmdDrawMeshTasksIndirectCountEXT-countBuffer-02717";
         task_group_count_exceeds_max_x = "VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07322";
         task_group_count_exceeds_max_y = "VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07323";
@@ -278,7 +287,8 @@ struct GpuVuidsCmdDrawIndirectByteCountEXT : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDrawIndirectByteCountEXT-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDrawIndirectByteCountEXT-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDrawIndirectByteCountEXT-None-08613";
-        invalid_descriptor = "VUID-vkCmdDrawIndirectByteCountEXT-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDrawIndirectByteCountEXT-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDrawIndirectByteCountEXT-None-10068";
     }
 };
 
@@ -288,11 +298,25 @@ struct GpuVuidsCmdDispatchBase : GpuVuid {
         storage_access_oob_06936 = "VUID-vkCmdDispatchBase-storageBuffers-06936";
         uniform_access_oob_08612 = "VUID-vkCmdDispatchBase-None-08612";
         storage_access_oob_08613 = "VUID-vkCmdDispatchBase-None-08613";
-        invalid_descriptor = "VUID-vkCmdDispatchBase-None-08114";
+        invalid_descriptor_08114 = "VUID-vkCmdDispatchBase-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdDispatchBase-None-10068";
+    }
+};
+
+struct GpuVuidsCmdExecuteGeneratedCommandsEXT : GpuVuid {
+    GpuVuidsCmdExecuteGeneratedCommandsEXT() : GpuVuid() {
+        uniform_access_oob_06935 = "VUID-vkCmdExecuteGeneratedCommandsEXT-uniformBuffers-06935";
+        storage_access_oob_06936 = "VUID-vkCmdExecuteGeneratedCommandsEXT-storageBuffers-06936";
+        uniform_access_oob_08612 = "VUID-vkCmdExecuteGeneratedCommandsEXT-None-08612";
+        storage_access_oob_08613 = "VUID-vkCmdExecuteGeneratedCommandsEXT-None-08613";
+        invalid_descriptor_08114 = "VUID-vkCmdExecuteGeneratedCommandsEXT-None-08114";
+        descriptor_index_oob_10068 = "VUID-vkCmdExecuteGeneratedCommandsEXT-None-10068";
     }
 };
 
 using Func = vvl::Func;
+
+static const std::map<Func, GpuVuid> &GetGpuVuidsMap() {
 // This LUT is created to allow a static listing of each VUID that is covered by drawdispatch commands
 static const std::map<Func, GpuVuid> gpu_vuid = {
     {Func::vkCmdDraw, GpuVuidsCmdDraw()},
@@ -320,16 +344,20 @@ static const std::map<Func, GpuVuid> gpu_vuid = {
     {Func::vkCmdDrawIndirectByteCountEXT, GpuVuidsCmdDrawIndirectByteCountEXT()},
     {Func::vkCmdDispatchBase, GpuVuidsCmdDispatchBase()},
     {Func::vkCmdDispatchBaseKHR, GpuVuidsCmdDispatchBase()},
+    {Func::vkCmdExecuteGeneratedCommandsEXT, GpuVuidsCmdExecuteGeneratedCommandsEXT()},
     // Used if invalid function is used
     {Func::Empty, GpuVuid()}
 };
+return gpu_vuid;
+}
 
 const GpuVuid &GetGpuVuid(Func command) {
-    if (gpu_vuid.find(command) != gpu_vuid.cend()) {
-        return gpu_vuid.at(command);
+    const auto &gpu_vuids_map = GetGpuVuidsMap();
+    if (gpu_vuids_map.find(command) != gpu_vuids_map.cend()) {
+        return gpu_vuids_map.at(command);
     }
     else {
-        return gpu_vuid.at(Func::Empty);
+        return gpu_vuids_map.at(Func::Empty);
     }
 }
 // clang-format on
