@@ -15,31 +15,36 @@
 
 #include "module.h"
 #include <spirv/unified1/spirv.hpp>
-#include "gpu/shaders/gpu_shaders_constants.h"
+#include "gpu/shaders/gpuav_shaders_constants.h"
 #include "error_message/logging.h"
+#include "error_message/log_message_type.h"
 
 #include "buffer_device_address_pass.h"
-#include "bindless_descriptor_pass.h"
-#include "non_bindless_oob_buffer_pass.h"
-#include "non_bindless_oob_texel_buffer_pass.h"
+#include "descriptor_indexing_oob_pass.h"
+#include "descriptor_class_general_buffer_pass.h"
+#include "descriptor_class_texel_buffer_pass.h"
 #include "ray_query_pass.h"
 #include "debug_printf_pass.h"
+#include "post_process_descriptor_indexing.h"
 
 #include <iostream>
 
-namespace gpu {
+namespace gpuav {
 namespace spirv {
 
-Module::Module(vvl::span<const uint32_t> words, DebugReport* debug_report, const Settings& settings)
+Module::Module(vvl::span<const uint32_t> words, DebugReport* debug_report, const Settings& settings,
+               const std::vector<std::vector<BindingLayout>>& set_index_to_bindings_layout_lut)
     : type_manager_(*this),
-      max_instrumented_count_(settings.max_instrumented_count),
+      max_instrumentations_count_(settings.max_instrumentations_count),
       shader_id_(settings.shader_id),
       output_buffer_descriptor_set_(settings.output_buffer_descriptor_set),
+      support_non_semantic_info_(settings.support_non_semantic_info),
       support_int64_(settings.support_int64),
       support_memory_model_device_scope_(settings.support_memory_model_device_scope),
       has_bindless_descriptors_(settings.has_bindless_descriptors),
       print_debug_info_(settings.print_debug_info),
-      debug_report_(debug_report) {
+      debug_report_(debug_report),
+      set_index_to_bindings_layout_lut_(set_index_to_bindings_layout_lut) {
     uint32_t instruction_count = 0;
     spirv_iterator it = words.begin();
     header_.magic_number = *it++;
@@ -106,12 +111,15 @@ Module::Module(vvl::span<const uint32_t> words, DebugReport* debug_report, const
                 annotations_.emplace_back(std::move(new_inst));
                 break;
 
+            case spv::OpSpecConstantTrue:
+            case spv::OpSpecConstantFalse:
             case spv::OpConstantTrue:
             case spv::OpConstantFalse: {
                 const Type& type = type_manager_.GetTypeBool();
                 type_manager_.AddConstant(std::move(new_inst), type);
                 break;
             }
+            case spv::OpSpecConstant:
             case spv::OpConstant:
             case spv::OpConstantNull:
             case spv::OpConstantComposite: {
@@ -276,8 +284,8 @@ void Module::AddMemberDecoration(uint32_t target_id, uint32_t index, spv::Decora
     annotations_.emplace_back(std::move(new_inst));
 }
 
-bool Module::RunPassBindlessDescriptor() {
-    BindlessDescriptorPass pass(*this);
+bool Module::RunPassDescriptorIndexingOOB() {
+    DescriptorIndexingOOBPass pass(*this);
     const bool changed = pass.Run();
     if (print_debug_info_) {
         pass.PrintDebugInfo();
@@ -285,8 +293,8 @@ bool Module::RunPassBindlessDescriptor() {
     return changed;
 }
 
-bool Module::RunPassNonBindlessOOBBuffer() {
-    NonBindlessOOBBufferPass pass(*this);
+bool Module::RunPassDescriptorClassGeneralBuffer() {
+    DescriptorClassGeneralBufferPass pass(*this);
     const bool changed = pass.Run();
     if (print_debug_info_) {
         pass.PrintDebugInfo();
@@ -294,8 +302,8 @@ bool Module::RunPassNonBindlessOOBBuffer() {
     return changed;
 }
 
-bool Module::RunPassNonBindlessOOBTexelBuffer() {
-    NonBindlessOOBTexelBufferPass pass(*this);
+bool Module::RunPassDescriptorClassTexelBuffer() {
+    DescriptorClassTexelBufferPass pass(*this);
     const bool changed = pass.Run();
     if (print_debug_info_) {
         pass.PrintDebugInfo();
@@ -331,6 +339,15 @@ bool Module::RunPassDebugPrintf(uint32_t binding_slot) {
     return changed;
 }
 
+bool Module::RunPassPostProcessDescriptorIndexing() {
+    PostProcessDescriptorIndexingPass pass(*this);
+    const bool changed = pass.Run();
+    if (print_debug_info_) {
+        pass.PrintDebugInfo();
+    }
+    return changed;
+}
+
 uint32_t Module::TakeNextId() {
     // SPIR-V limit.
     assert(header_.bound < 0x3FFFFF);
@@ -549,7 +566,7 @@ void Module::LinkFunction(const LinkInfo& info) {
                 }
 
                 // Replace LinkConstants
-                if (constant_value == gpuav::glsl::kLinkShaderId) {
+                if (constant_value == glsl::kLinkShaderId) {
                     new_inst->words_[3] = shader_id_;
                 }
             }
@@ -580,6 +597,16 @@ void Module::LinkFunction(const LinkInfo& info) {
                 // It is valid to have duplicated Capabilities
                 capabilities_.emplace_back(std::move(new_inst));
             }
+        } else if (opcode == spv::OpExtInstImport) {
+            const uint32_t new_result_id = TakeNextId();
+            id_swap_map[old_result_id] = new_result_id;
+            new_inst->ReplaceResultId(new_result_id);
+            ext_inst_imports_.emplace_back(std::move(new_inst));
+        } else if (opcode == spv::OpString) {
+            const uint32_t new_result_id = TakeNextId();
+            id_swap_map[old_result_id] = new_result_id;
+            new_inst->ReplaceResultId(new_result_id);
+            debug_source_.emplace_back(std::move(new_inst));
         } else if (opcode == spv::OpExtension) {
             extensions_.emplace_back(std::move(new_inst));
         }
@@ -682,6 +709,9 @@ void Module::PostProcess() {
 
     // The instrumentation code has atomicAdd() to update the output buffer
     // If the incoming code only has VulkanMemoryModel it will need to support device scope
+    //
+    // Found that QueueFamily was added to mostly solve this, if a device doesn't support Device scope we could use QueueFamily, the
+    // issue is that the GLSL we have is static and if we use QueueFamily then we "need" the MemoryModel enabled
     if (HasCapability(spv::CapabilityVulkanMemoryModel)) {
         if (!support_memory_model_device_scope_) {
             InternalError(
@@ -716,4 +746,4 @@ void Module::InternalError(const char* tag, const char* message) {
 }
 
 }  // namespace spirv
-}  // namespace gpu
\ No newline at end of file
+}  // namespace gpuav
\ No newline at end of file
