@@ -15,22 +15,20 @@
 #pragma once
 
 #include <stdint.h>
-#include "pass.h"
+#include "inject_conditional_function_pass.h"
 
 namespace gpuav {
 namespace spirv {
 
-class Module;
-struct Function;
-struct BasicBlock;
-
 // Create a pass to instrument SPV_KHR_ray_query instructions
-class RayQueryPass : public Pass {
+class RayQueryPass : public InjectConditionalFunctionPass {
   public:
-    RayQueryPass(Module& module) : Pass(module, true) {}
+    RayQueryPass(Module& module) : InjectConditionalFunctionPass(module) {}
+    const char* Name() const final { return "RayQueryPass"; }
+    void PrintDebugInfo();
 
   private:
-    bool AnalyzeInstruction(const Function& function, const Instruction& inst) final;
+    bool RequiresInstrumentation(const Function& function, const Instruction& inst) final;
     uint32_t CreateFunctionCall(BasicBlock& block, InstructionIt* inst_it, const InjectionData& injection_data) final;
     void Reset() final;
 
