@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (C) 2015-2024 Google Inc.
+/* Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (C) 2015-2025 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,14 +15,30 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-#include "generated/chassis.h"
-
 #include "object_lifetime_validation.h"
-#include "generated/layer_chassis_dispatch.h"
+#include "chassis/dispatch_object.h"
 
 uint64_t object_track_index = 0;
 
+static std::shared_mutex lifetime_set_mutex;
+static vvl::unordered_set<ObjectLifetimes *> lifetime_set;
+
+ObjectLifetimes::ObjectLifetimes(vvl::dispatch::Device *dev, ObjectLifetimes *instance)
+    : BaseClass(dev, LayerObjectTypeObjectTracker) {
+    WriteLockGuard lock(lifetime_set_mutex);
+    lifetime_set.insert(this);
+}
+
+ObjectLifetimes::ObjectLifetimes(vvl::dispatch::Instance *instance) : BaseClass(instance, LayerObjectTypeObjectTracker) {
+    WriteLockGuard lock(lifetime_set_mutex);
+    lifetime_set.insert(this);
+}
+
+ObjectLifetimes::~ObjectLifetimes() {
+    WriteLockGuard lock(lifetime_set_mutex);
+    lifetime_set.erase(this);
+}
+
 VulkanTypedHandle ObjTrackStateTypedHandle(const ObjTrackState &track_state) {
     // TODO: Unify Typed Handle representation (i.e. VulkanTypedHandle everywhere there are handle/type pairs)
     VulkanTypedHandle typed_handle;
@@ -58,21 +74,23 @@ bool ObjectLifetimes::CheckObjectValidity(uint64_t object_handle, VulkanObjectTy
     }
     // Object not found, look for it in other device object maps
     const ObjectLifetimes *other_lifetimes = nullptr;
-    for (const auto &other_device_data : layer_data_map) {
-        const auto lifetimes = other_device_data.second->GetValidationObject<ObjectLifetimes>();
-        if (lifetimes && lifetimes != this && lifetimes->TracksObject(object_handle, object_type)) {
-            other_lifetimes = lifetimes;
+    {
+        ReadLockGuard lock(lifetime_set_mutex);
+        for (const auto *lifetimes : lifetime_set) {
+            if (lifetimes != this && lifetimes->TracksObject(object_handle, object_type)) {
+                other_lifetimes = lifetimes;
 
-            // Sometimes (calls such as vkRegisterDisplayEventEXT) interact with both the device and physical device
-            if (parent_type == kVulkanObjectTypePhysicalDevice) {
-                auto iter = other_lifetimes->object_map[object_type].find(object_handle);
-                if (iter != other_lifetimes->object_map[object_type].end()) {
-                    if (iter->second->parent_object == HandleToUint64(physical_device)) {
-                        return skip;
+                // Sometimes (calls such as vkRegisterDisplayEventEXT) interact with both the device and physical device
+                if (parent_type == kVulkanObjectTypePhysicalDevice) {
+                    auto iter = other_lifetimes->object_map[object_type].find(object_handle);
+                    if (iter != other_lifetimes->object_map[object_type].end()) {
+                        if (iter->second->parent_object == HandleToUint64(physical_device)) {
+                            return skip;
+                        }
                     }
                 }
+                break;
             }
-            break;
         }
     }
 
@@ -287,25 +305,30 @@ bool ObjectLifetimes::ValidateDescriptorWrite(VkWriteDescriptorSet const *desc,
         case VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER:
         case VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE:
         case VK_DESCRIPTOR_TYPE_STORAGE_IMAGE: {
-            for (uint32_t i = 0; i < desc->descriptorCount; ++i) {
-                skip |= ValidateObject(desc->pImageInfo[i].imageView, kVulkanObjectTypeImageView, true,
-                                       "VUID-VkWriteDescriptorSet-descriptorType-02996",
-                                       "VUID-vkUpdateDescriptorSets-pDescriptorWrites-06239",
-                                       loc.dot(Field::pImageInfo, i).dot(Field::imageView));
-                if (!null_descriptor_enabled && desc->pImageInfo[i].imageView == VK_NULL_HANDLE) {
-                    skip |= LogError("VUID-VkWriteDescriptorSet-descriptorType-02997", desc->dstSet,
-                                     loc.dot(Field::pImageInfo, i).dot(Field::imageView), "is VK_NULL_HANDLE.");
+            if (desc->pImageInfo) {
+                for (uint32_t i = 0; i < desc->descriptorCount; ++i) {
+                    // Only validate image here, we have to validate Sampler state tracker object
+                    skip |= ValidateObject(desc->pImageInfo[i].imageView, kVulkanObjectTypeImageView, true,
+                                           "VUID-VkWriteDescriptorSet-descriptorType-02996",
+                                           "VUID-vkUpdateDescriptorSets-pDescriptorWrites-06239",
+                                           loc.dot(Field::pImageInfo, i).dot(Field::imageView));
+                    if (!null_descriptor_enabled && desc->pImageInfo[i].imageView == VK_NULL_HANDLE) {
+                        skip |= LogError("VUID-VkWriteDescriptorSet-descriptorType-02997", desc->dstSet,
+                                         loc.dot(Field::pImageInfo, i).dot(Field::imageView), "is VK_NULL_HANDLE.");
+                    }
                 }
             }
             break;
         }
         case VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT: {
             // Input attachments can never be null
-            for (uint32_t i = 0; i < desc->descriptorCount; ++i) {
-                skip |= ValidateObject(desc->pImageInfo[i].imageView, kVulkanObjectTypeImageView, false,
-                                       "VUID-VkWriteDescriptorSet-descriptorType-07683",
-                                       "VUID-vkUpdateDescriptorSets-pDescriptorWrites-06239",
-                                       loc.dot(Field::pImageInfo, i).dot(Field::imageView));
+            if (desc->pImageInfo) {
+                for (uint32_t i = 0; i < desc->descriptorCount; ++i) {
+                    skip |= ValidateObject(desc->pImageInfo[i].imageView, kVulkanObjectTypeImageView, false,
+                                           "VUID-VkWriteDescriptorSet-descriptorType-07683",
+                                           "VUID-vkUpdateDescriptorSets-pDescriptorWrites-06239",
+                                           loc.dot(Field::pImageInfo, i).dot(Field::imageView));
+                }
             }
             break;
         }
@@ -313,13 +336,15 @@ bool ObjectLifetimes::ValidateDescriptorWrite(VkWriteDescriptorSet const *desc,
         case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER:
         case VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC:
         case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC: {
-            for (uint32_t i = 0; i < desc->descriptorCount; ++i) {
-                skip |= ValidateObject(
-                    desc->pBufferInfo[i].buffer, kVulkanObjectTypeBuffer, true, "VUID-VkDescriptorBufferInfo-buffer-parameter",
-                    "VUID-vkUpdateDescriptorSets-pDescriptorWrites-06237", loc.dot(Field::pBufferInfo, i).dot(Field::buffer));
-                if (!null_descriptor_enabled && desc->pBufferInfo[i].buffer == VK_NULL_HANDLE) {
-                    skip |= LogError("VUID-VkDescriptorBufferInfo-buffer-02998", desc->dstSet,
-                                     loc.dot(Field::pBufferInfo, i).dot(Field::buffer), "is VK_NULL_HANDLE.");
+            if (desc->pBufferInfo) {
+                for (uint32_t i = 0; i < desc->descriptorCount; ++i) {
+                    skip |= ValidateObject(
+                        desc->pBufferInfo[i].buffer, kVulkanObjectTypeBuffer, true, "VUID-VkDescriptorBufferInfo-buffer-parameter",
+                        "VUID-vkUpdateDescriptorSets-pDescriptorWrites-06237", loc.dot(Field::pBufferInfo, i).dot(Field::buffer));
+                    if (!null_descriptor_enabled && desc->pBufferInfo[i].buffer == VK_NULL_HANDLE) {
+                        skip |= LogError("VUID-VkDescriptorBufferInfo-buffer-02998", desc->dstSet,
+                                         loc.dot(Field::pBufferInfo, i).dot(Field::buffer), "is VK_NULL_HANDLE.");
+                    }
                 }
             }
             break;
@@ -346,8 +371,10 @@ bool ObjectLifetimes::ValidateDescriptorWrite(VkWriteDescriptorSet const *desc,
             }
             break;
         }
-        // TODO - These need to be checked as well
+        // handled in core check because need to know if using immutable samplers or not
         case VK_DESCRIPTOR_TYPE_SAMPLER:
+
+        // TODO - These need to be checked as well
         case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:
         case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_NV:
         case VK_DESCRIPTOR_TYPE_SAMPLE_WEIGHT_IMAGE_QCOM:
@@ -360,14 +387,14 @@ bool ObjectLifetimes::ValidateDescriptorWrite(VkWriteDescriptorSet const *desc,
     return skip;
 }
 
-bool ObjectLifetimes::PreCallValidateCmdPushDescriptorSetKHR(VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint,
-                                                             VkPipelineLayout layout, uint32_t set, uint32_t descriptorWriteCount,
-                                                             const VkWriteDescriptorSet *pDescriptorWrites,
-                                                             const ErrorObject &error_obj) const {
+bool ObjectLifetimes::PreCallValidateCmdPushDescriptorSet(VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint,
+                                                          VkPipelineLayout layout, uint32_t set, uint32_t descriptorWriteCount,
+                                                          const VkWriteDescriptorSet *pDescriptorWrites,
+                                                          const ErrorObject &error_obj) const {
     bool skip = false;
-    // Checked by chassis: commandBuffer: "VUID-vkCmdPushDescriptorSetKHR-commandBuffer-parameter"
-    skip |= ValidateObject(layout, kVulkanObjectTypePipelineLayout, false, "VUID-vkCmdPushDescriptorSetKHR-layout-parameter",
-                           "VUID-vkCmdPushDescriptorSetKHR-commonparent", error_obj.location.dot(Field::layout));
+    // Checked by chassis: commandBuffer: "VUID-vkCmdPushDescriptorSet-commandBuffer-parameter"
+    skip |= ValidateObject(layout, kVulkanObjectTypePipelineLayout, false, "VUID-vkCmdPushDescriptorSet-layout-parameter",
+                           "VUID-vkCmdPushDescriptorSet-commonparent", error_obj.location.dot(Field::layout));
     if (pDescriptorWrites) {
         for (uint32_t index0 = 0; index0 < descriptorWriteCount; ++index0) {
             skip |=
@@ -377,13 +404,21 @@ bool ObjectLifetimes::PreCallValidateCmdPushDescriptorSetKHR(VkCommandBuffer com
     return skip;
 }
 
-bool ObjectLifetimes::PreCallValidateCmdPushDescriptorSet2KHR(VkCommandBuffer commandBuffer,
-                                                              const VkPushDescriptorSetInfoKHR *pPushDescriptorSetInfo,
-                                                              const ErrorObject &error_obj) const {
+bool ObjectLifetimes::PreCallValidateCmdPushDescriptorSetKHR(VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint,
+                                                             VkPipelineLayout layout, uint32_t set, uint32_t descriptorWriteCount,
+                                                             const VkWriteDescriptorSet *pDescriptorWrites,
+                                                             const ErrorObject &error_obj) const {
+    return PreCallValidateCmdPushDescriptorSet(commandBuffer, pipelineBindPoint, layout, set, descriptorWriteCount,
+                                               pDescriptorWrites, error_obj);
+}
+
+bool ObjectLifetimes::PreCallValidateCmdPushDescriptorSet2(VkCommandBuffer commandBuffer,
+                                                           const VkPushDescriptorSetInfo *pPushDescriptorSetInfo,
+                                                           const ErrorObject &error_obj) const {
     bool skip = false;
-    // Checked by chassis: commandBuffer: "VUID-vkCmdPushDescriptorSet2KHR-commandBuffer-parameter"
+    // Checked by chassis: commandBuffer: "VUID-vkCmdPushDescriptorSet2-commandBuffer-parameter"
     skip |= ValidateObject(pPushDescriptorSetInfo->layout, kVulkanObjectTypePipelineLayout, true,
-                           "VUID-VkPushDescriptorSetInfoKHR-layout-parameter", kVUIDUndefined,
+                           "VUID-VkPushDescriptorSetInfo-layout-parameter", kVUIDUndefined,
                            error_obj.location.dot(Field::pPushDescriptorSetInfo).dot(Field::layout));
 
     if (pPushDescriptorSetInfo->pDescriptorWrites) {
@@ -396,6 +431,12 @@ bool ObjectLifetimes::PreCallValidateCmdPushDescriptorSet2KHR(VkCommandBuffer co
     return skip;
 }
 
+bool ObjectLifetimes::PreCallValidateCmdPushDescriptorSet2KHR(VkCommandBuffer commandBuffer,
+                                                              const VkPushDescriptorSetInfoKHR *pPushDescriptorSetInfo,
+                                                              const ErrorObject &error_obj) const {
+    return PreCallValidateCmdPushDescriptorSet2(commandBuffer, pPushDescriptorSetInfo, error_obj);
+}
+
 void ObjectLifetimes::CreateQueue(VkQueue vkObj, const Location &loc) {
     std::shared_ptr<ObjTrackState> p_obj_node = NULL;
     auto queue_item = object_map[kVulkanObjectTypeQueue].find(HandleToUint64(vkObj));
@@ -469,8 +510,8 @@ bool ObjectLifetimes::PreCallValidateDestroyInstance(VkInstance instance, const
                      string_VkDebugReportObjectTypeEXT(debug_object_type), FormatHandle(ObjTrackStateTypedHandle(*node)).c_str());
 
         // Throw errors if any device objects belonging to this instance have not been destroyed
-        auto device_layer_data = GetLayerDataPtr(GetDispatchKey(device), layer_data_map);
-        auto obj_lifetimes_data = device_layer_data->GetValidationObject<ObjectLifetimes>();
+        auto device_data = vvl::dispatch::GetData(device);
+        auto obj_lifetimes_data = static_cast<ObjectLifetimes *>(device_data->GetValidationObject(LayerObjectTypeObjectTracker));
         skip |= obj_lifetimes_data->ReportUndestroyedDeviceObjects(device, error_obj.location);
 
         skip |= ValidateDestroyObject(device, kVulkanObjectTypeDevice, pAllocator, "VUID-vkDestroyInstance-instance-00630",
@@ -527,9 +568,12 @@ bool ObjectLifetimes::PreCallValidateDestroyDevice(VkDevice device, const VkAllo
 
 void ObjectLifetimes::PreCallRecordDestroyDevice(VkDevice device, const VkAllocationCallbacks *pAllocator,
                                                  const RecordObject &record_obj) {
-    auto instance_data = GetLayerDataPtr(GetDispatchKey(physical_device), layer_data_map);
-    auto object_lifetimes = instance_data->GetValidationObject<ObjectLifetimes>();
-    object_lifetimes->RecordDestroyObject(device, kVulkanObjectTypeDevice);
+    auto object_lifetimes = static_cast<ObjectLifetimes *>(dispatch_instance_->GetValidationObject(LayerObjectTypeObjectTracker));
+    // If ObjectTracker was removed (in an early teardown) this might be null, could search in aborted_object_dispatch but if it is
+    // there, no need to record anything else
+    if (object_lifetimes) {
+        object_lifetimes->RecordDestroyObject(device, kVulkanObjectTypeDevice);
+    }
     DestroyLeakedDeviceObjects();
 
     // Clean up Queue's MemRef Linked Lists
@@ -732,8 +776,8 @@ void ObjectLifetimes::PostCallRecordCreateDevice(VkPhysicalDevice physicalDevice
     if (record_obj.result < VK_SUCCESS) return;
     CreateObject(*pDevice, kVulkanObjectTypeDevice, pAllocator, record_obj.location);
 
-    auto device_data = GetLayerDataPtr(GetDispatchKey(*pDevice), layer_data_map);
-    auto object_tracking = device_data->GetValidationObject<ObjectLifetimes>();
+    auto device_data = vvl::dispatch::GetData(*pDevice);
+    auto object_tracking = static_cast<ObjectLifetimes*>(device_data->GetValidationObject(LayerObjectTypeObjectTracker));
 
     const auto *robustness2_features = vku::FindStructInPNextChain<VkPhysicalDeviceRobustness2FeaturesEXT>(pCreateInfo->pNext);
     object_tracking->null_descriptor_enabled = robustness2_features && robustness2_features->nullDescriptor;
@@ -1059,17 +1103,15 @@ bool ObjectLifetimes::PreCallValidateCreateFramebuffer(VkDevice device, const Vk
     bool skip = false;
     // Checked by chassis: device: "VUID-vkCreateFramebuffer-device-parameter"
 
-    if (pCreateInfo) {
-        const Location create_info_loc = error_obj.location.dot(Field::pCreateInfo);
-        skip |= ValidateObject(pCreateInfo->renderPass, kVulkanObjectTypeRenderPass, false,
-                               "VUID-VkFramebufferCreateInfo-renderPass-parameter", "VUID-VkFramebufferCreateInfo-commonparent",
-                               create_info_loc.dot(Field::renderPass));
-        if ((pCreateInfo->flags & VK_FRAMEBUFFER_CREATE_IMAGELESS_BIT) == 0) {
-            for (uint32_t index1 = 0; index1 < pCreateInfo->attachmentCount; ++index1) {
-                skip |= ValidateObject(pCreateInfo->pAttachments[index1], kVulkanObjectTypeImageView, true,
-                                       "VUID-VkFramebufferCreateInfo-flags-02778", "VUID-VkFramebufferCreateInfo-commonparent",
-                                       create_info_loc.dot(Field::pAttachments, index1));
-            }
+    const Location create_info_loc = error_obj.location.dot(Field::pCreateInfo);
+    skip |= ValidateObject(pCreateInfo->renderPass, kVulkanObjectTypeRenderPass, false,
+                           "VUID-VkFramebufferCreateInfo-renderPass-parameter", "VUID-VkFramebufferCreateInfo-commonparent",
+                           create_info_loc.dot(Field::renderPass));
+    if ((pCreateInfo->flags & VK_FRAMEBUFFER_CREATE_IMAGELESS_BIT) == 0) {
+        for (uint32_t index1 = 0; index1 < pCreateInfo->attachmentCount; ++index1) {
+            skip |= ValidateObject(pCreateInfo->pAttachments[index1], kVulkanObjectTypeImageView, true,
+                                   "VUID-VkFramebufferCreateInfo-flags-02778", "VUID-VkFramebufferCreateInfo-commonparent",
+                                   create_info_loc.dot(Field::pAttachments, index1));
         }
     }
 
@@ -1188,20 +1230,18 @@ bool ObjectLifetimes::PreCallValidateCreateDescriptorUpdateTemplate(VkDevice dev
     bool skip = false;
     // Checked by chassis: device: "VUID-vkCreateDescriptorUpdateTemplate-device-parameter"
 
-    if (pCreateInfo) {
-        const Location create_info_loc = error_obj.location.dot(Field::pCreateInfo);
-        if (pCreateInfo->templateType == VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_DESCRIPTOR_SET) {
-            skip |= ValidateObject(pCreateInfo->descriptorSetLayout, kVulkanObjectTypeDescriptorSetLayout, false,
-                                   "VUID-VkDescriptorUpdateTemplateCreateInfo-templateType-00350",
-                                   "VUID-VkDescriptorUpdateTemplateCreateInfo-commonparent",
-                                   create_info_loc.dot(Field::descriptorSetLayout));
-        }
-        if (pCreateInfo->templateType == VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_PUSH_DESCRIPTORS_KHR) {
-            skip |= ValidateObject(pCreateInfo->pipelineLayout, kVulkanObjectTypePipelineLayout, false,
-                                   "VUID-VkDescriptorUpdateTemplateCreateInfo-templateType-00352",
-                                   "VUID-VkDescriptorUpdateTemplateCreateInfo-commonparent",
-                                   create_info_loc.dot(Field::pipelineLayout));
-        }
+    const Location create_info_loc = error_obj.location.dot(Field::pCreateInfo);
+    if (pCreateInfo->templateType == VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_DESCRIPTOR_SET) {
+        skip |= ValidateObject(pCreateInfo->descriptorSetLayout, kVulkanObjectTypeDescriptorSetLayout, false,
+                               "VUID-VkDescriptorUpdateTemplateCreateInfo-templateType-00350",
+                               "VUID-VkDescriptorUpdateTemplateCreateInfo-commonparent",
+                               create_info_loc.dot(Field::descriptorSetLayout));
+    }
+    if (pCreateInfo->templateType == VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_PUSH_DESCRIPTORS) {
+        skip |=
+            ValidateObject(pCreateInfo->pipelineLayout, kVulkanObjectTypePipelineLayout, false,
+                           "VUID-VkDescriptorUpdateTemplateCreateInfo-templateType-00352",
+                           "VUID-VkDescriptorUpdateTemplateCreateInfo-commonparent", create_info_loc.dot(Field::pipelineLayout));
     }
 
     return skip;
@@ -1340,32 +1380,33 @@ bool ObjectLifetimes::PreCallValidateCreateRayTracingPipelinesKHR(VkDevice devic
     return skip;
 }
 
-void ObjectLifetimes::PostCallRecordCreateRayTracingPipelinesKHR(VkDevice device, VkDeferredOperationKHR deferredOperation,
-                                                                 VkPipelineCache pipelineCache, uint32_t createInfoCount,
-                                                                 const VkRayTracingPipelineCreateInfoKHR *pCreateInfos,
-                                                                 const VkAllocationCallbacks *pAllocator, VkPipeline *pPipelines,
-                                                                 const RecordObject &record_obj) {
+void ObjectLifetimes::PostCallRecordCreateRayTracingPipelinesKHR(
+    VkDevice device, VkDeferredOperationKHR deferredOperation, VkPipelineCache pipelineCache, uint32_t createInfoCount,
+    const VkRayTracingPipelineCreateInfoKHR *pCreateInfos, const VkAllocationCallbacks *pAllocator, VkPipeline *pPipelines,
+    const RecordObject &record_obj, PipelineStates &pipeline_states,
+    std::shared_ptr<chassis::CreateRayTracingPipelinesKHR> chassis_state) {
     if (VK_ERROR_VALIDATION_FAILED_EXT == record_obj.result) return;
     if (pPipelines) {
         if (deferredOperation != VK_NULL_HANDLE && record_obj.result == VK_OPERATION_DEFERRED_KHR) {
-            auto register_fn = [this, pAllocator, record_obj](const std::vector<VkPipeline> &pipelines) {
-                for (auto pipe : pipelines) {
-                    if (!pipe) continue;
+            auto register_fn = [this, pAllocator, record_obj, chassis_state](const std::vector<VkPipeline> &pipelines) {
+                // Just need to capture chassis state to maintain pipeline creations parameters alive, see
+                // https://vkdoc.net/chapters/deferred-host-operations#deferred-host-operations-requesting
+                (void)chassis_state;
+                for (VkPipeline pipe : pipelines) {
                     this->CreateObject(pipe, kVulkanObjectTypePipeline, pAllocator, record_obj.location);
                 }
             };
 
-            auto layer_data = GetLayerDataPtr(GetDispatchKey(device), layer_data_map);
-            if (wrap_handles) {
-                deferredOperation = layer_data->Unwrap(deferredOperation);
+            if (dispatch_device_->wrap_handles) {
+                deferredOperation = dispatch_device_->Unwrap(deferredOperation);
             }
             std::vector<std::function<void(const std::vector<VkPipeline> &)>> cleanup_fn;
-            auto find_res = layer_data->deferred_operation_post_check.pop(deferredOperation);
+            auto find_res = dispatch_device_->deferred_operation_post_check.pop(deferredOperation);
             if (find_res->first) {
                 cleanup_fn = std::move(find_res->second);
             }
             cleanup_fn.emplace_back(register_fn);
-            layer_data->deferred_operation_post_check.insert(deferredOperation, cleanup_fn);
+            dispatch_device_->deferred_operation_post_check.insert(deferredOperation, cleanup_fn);
         } else {
             for (uint32_t index = 0; index < createInfoCount; index++) {
                 if (!pPipelines[index]) continue;
@@ -1527,3 +1568,80 @@ void ObjectLifetimes::PreCallRecordDestroyPipeline(VkDevice device, VkPipeline p
 
     linked_graphics_pipeline_map.erase(HandleToUint64(pipeline));
 }
+
+bool ObjectLifetimes::PreCallValidateCreateIndirectExecutionSetEXT(VkDevice device,
+                                                                   const VkIndirectExecutionSetCreateInfoEXT *pCreateInfo,
+                                                                   const VkAllocationCallbacks *pAllocator,
+                                                                   VkIndirectExecutionSetEXT *pIndirectExecutionSet,
+                                                                   const ErrorObject &error_obj) const {
+    bool skip = false;
+    // Checked by chassis: device: "VUID-vkCreateIndirectExecutionSetEXT-device-parameter"
+
+    const Location create_info_loc = error_obj.location.dot(Field::pCreateInfo);
+    const Location info_loc = create_info_loc.dot(Field::info);
+    if (pCreateInfo->type == VK_INDIRECT_EXECUTION_SET_INFO_TYPE_PIPELINES_EXT && pCreateInfo->info.pPipelineInfo) {
+        [[maybe_unused]] const Location pipeline_info_loc = info_loc.dot(Field::pPipelineInfo);
+        skip |= ValidateObject(pCreateInfo->info.pPipelineInfo->initialPipeline, kVulkanObjectTypePipeline, false,
+                               "VUID-VkIndirectExecutionSetPipelineInfoEXT-initialPipeline-parameter", kVUIDUndefined,
+                               pipeline_info_loc.dot(Field::initialPipeline));
+    }
+
+    if (pCreateInfo->type == VK_INDIRECT_EXECUTION_SET_INFO_TYPE_SHADER_OBJECTS_EXT && pCreateInfo->info.pShaderInfo) {
+        const VkIndirectExecutionSetShaderInfoEXT &shader_info = *pCreateInfo->info.pShaderInfo;
+        const Location shader_info_loc = info_loc.dot(Field::pShaderInfo);
+        if (shader_info.pSetLayoutInfos && shader_info.pInitialShaders) {
+            for (uint32_t i = 0; i < shader_info.shaderCount; ++i) {
+                skip |= ValidateObject(shader_info.pInitialShaders[i], kVulkanObjectTypeShaderEXT, false,
+                                       "VUID-VkIndirectExecutionSetShaderInfoEXT-pInitialShaders-parameter", kVUIDUndefined,
+                                       shader_info_loc.dot(Field::pInitialShaders, i));
+
+                const Location set_layout_info_loc = shader_info_loc.dot(Field::pSetLayoutInfos, i);
+                const VkIndirectExecutionSetShaderLayoutInfoEXT &layout_info = shader_info.pSetLayoutInfos[i];
+                if ((layout_info.setLayoutCount > 0) && (layout_info.pSetLayouts)) {
+                    for (uint32_t layout_index = 0; layout_index < layout_info.setLayoutCount; ++layout_index) {
+                        skip |= ValidateObject(layout_info.pSetLayouts[layout_index], kVulkanObjectTypeDescriptorSetLayout, true,
+                                               "VUID-VkIndirectExecutionSetShaderLayoutInfoEXT-pSetLayouts-parameter",
+                                               "UNASSIGNED-VkIndirectExecutionSetShaderLayoutInfoEXT-pSetLayouts-parent",
+                                               set_layout_info_loc.dot(Field::pSetLayouts, layout_index));
+                    }
+                }
+            }
+        }
+    }
+
+    return skip;
+}
+
+bool ObjectLifetimes::PreCallValidateReleaseCapturedPipelineDataKHR(VkDevice device,
+                                                                    const VkReleaseCapturedPipelineDataInfoKHR *pInfo,
+                                                                    const VkAllocationCallbacks *pAllocator,
+                                                                    const ErrorObject &error_obj) const {
+    bool skip = false;
+    // Checked by chassis: device: "VUID-vkReleaseCapturedPipelineDataKHR-device-parameter"
+
+    if (pInfo) {
+        const Location pInfo_loc = error_obj.location.dot(Field::pInfo);
+        skip |= ValidateObject(pInfo->pipeline, kVulkanObjectTypePipeline, false,
+                               "VUID-VkReleaseCapturedPipelineDataInfoKHR-pipeline-parameter",
+                               "UNASSIGNED-VkReleaseCapturedPipelineDataInfoKHR-pipeline-parent", pInfo_loc.dot(Field::pipeline));
+
+        skip |= ValidateDestroyObject(pInfo->pipeline, kVulkanObjectTypePipeline, pAllocator,
+                                      "VUID-vkReleaseCapturedPipelineDataKHR-pipeline-09611",
+                                      "VUID-vkReleaseCapturedPipelineDataKHR-pipeline-09612", pInfo_loc.dot(Field::pipeline));
+    }
+
+    return skip;
+}
+
+void ObjectLifetimes::PostCallRecordCreatePipelineBinariesKHR(VkDevice device, const VkPipelineBinaryCreateInfoKHR *pCreateInfo,
+                                                              const VkAllocationCallbacks *pAllocator,
+                                                              VkPipelineBinaryHandlesInfoKHR *pBinaries,
+                                                              const RecordObject &record_obj) {
+    if (record_obj.result < VK_SUCCESS) return;
+
+    if (pBinaries->pPipelineBinaries) {
+        for (uint32_t index = 0; index < pBinaries->pipelineBinaryCount; index++) {
+            CreateObject(pBinaries->pPipelineBinaries[index], kVulkanObjectTypePipelineBinaryKHR, pAllocator, record_obj.location);
+        }
+    }
+}
