@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (C) 2015-2024 Google Inc.
+/* Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (C) 2015-2025 Google Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -62,7 +62,7 @@ class DeviceMemory : public StateObject {
     void *p_driver_data;                   // Pointer to application's actual memory
     const VkDeviceSize fake_base_address;  // To allow a unified view of allocations, useful to Synchronization Validation
 
-    DeviceMemory(VkDeviceMemory memory, const VkMemoryAllocateInfo *pAllocateInfo, uint64_t fake_address,
+    DeviceMemory(VkDeviceMemory memory, const VkMemoryAllocateInfo *allocate_info, uint64_t fake_address,
                  const VkMemoryType &memory_type, const VkMemoryHeap &memory_heap,
                  std::optional<DedicatedBinding> &&dedicated_binding, uint32_t physical_device_count);
 
@@ -100,8 +100,10 @@ struct MEM_BINDING {
 
 class BindableMemoryTracker {
   public:
+    using BufferRange = sparse_container::range<VkDeviceSize>;
     using MemoryRange = sparse_container::range<VkDeviceSize>;
     using BoundMemoryRange = std::map<VkDeviceMemory, std::vector<MemoryRange>>;
+    using BoundRanges = vvl::unordered_map<VkDeviceMemory, std::vector<std::pair<MemoryRange, BufferRange>>>;
     using DeviceMemoryState = unordered_set<std::shared_ptr<vvl::DeviceMemory>>;
 
     virtual ~BindableMemoryTracker() {}
@@ -113,6 +115,7 @@ class BindableMemoryTracker {
     virtual void BindMemory(StateObject *, std::shared_ptr<vvl::DeviceMemory> &, VkDeviceSize, VkDeviceSize, VkDeviceSize) = 0;
 
     virtual BoundMemoryRange GetBoundMemoryRange(const MemoryRange &) const = 0;
+    virtual BoundRanges GetBoundRanges(const BufferRange &ranges_bounds, const std::vector<BufferRange> &ranges) const = 0;
     virtual DeviceMemoryState GetBoundMemoryStates() const = 0;
 };
 
@@ -130,6 +133,9 @@ class BindableNoMemoryTracker : public BindableMemoryTracker {
     void BindMemory(StateObject *, std::shared_ptr<vvl::DeviceMemory> &, VkDeviceSize, VkDeviceSize, VkDeviceSize) override {}
 
     BoundMemoryRange GetBoundMemoryRange(const MemoryRange &) const override { return BoundMemoryRange{}; }
+    BoundRanges GetBoundRanges(const BufferRange &ranges_bounds, const std::vector<BufferRange> &ranges) const override {
+        return {};
+    }
     DeviceMemoryState GetBoundMemoryStates() const override { return DeviceMemoryState{}; }
 };
 
@@ -149,6 +155,8 @@ class BindableLinearMemoryTracker : public BindableMemoryTracker {
                     VkDeviceSize resource_offset, VkDeviceSize size) override;
 
     BoundMemoryRange GetBoundMemoryRange(const MemoryRange &range) const override;
+    // No need to have this overload for linear memory
+    BoundRanges GetBoundRanges(const BufferRange &ranges_bounds, const std::vector<BufferRange> &ranges) const override;
     DeviceMemoryState GetBoundMemoryStates() const override;
 
   private:
@@ -172,6 +180,13 @@ class BindableSparseMemoryTracker : public BindableMemoryTracker {
                     VkDeviceSize resource_offset, VkDeviceSize size) override;
 
     BoundMemoryRange GetBoundMemoryRange(const MemoryRange &range) const override;
+    // With a list of (VALID) buffer ranges as input, and `ranges_bounds` being a range that contains all of those buffer ranges,
+    // find what tracked VkDeviceMemory maps to what buffer range.
+    // Result is stored in a "VkDeviceMemory -> vector<(memory range, buffer range)>" map.
+    // Pairs in vector<(memory range, buffer range)> are sorted in ascending order with respect to memory ranges
+    // Note: The stored buffer range in the map are all subranges of the ranges listed in `buffer_ranges`,
+    // since an input range could be mapped to multiple VkDeviceMemory
+    BoundRanges GetBoundRanges(const BufferRange &ranges_bounds, const std::vector<BufferRange> &buffer_ranges) const override;
 
     DeviceMemoryState GetBoundMemoryStates() const override;
 
@@ -199,7 +214,11 @@ class BindableMultiplanarMemoryTracker : public BindableMemoryTracker {
                     VkDeviceSize resource_offset, VkDeviceSize size) override;
 
     BoundMemoryRange GetBoundMemoryRange(const MemoryRange &range) const override;
-
+    // No reason to have this function for multi planar memory
+    BoundRanges GetBoundRanges(const BufferRange &ranges_bounds, const std::vector<BufferRange> &ranges) const override {
+        assert(false);
+        return {};
+    }
     DeviceMemoryState GetBoundMemoryStates() const override;
 
   private:
@@ -241,6 +260,7 @@ class Bindable : public StateObject {
                ((external_memory_handle_types & VK_EXTERNAL_MEMORY_HANDLE_TYPE_SCREEN_BUFFER_BIT_QNX) != 0);
     }
 
+    // Will be false if VkBindMemoryStatus had a non-success result
     const vvl::DeviceMemory *MemState() const {
         const MEM_BINDING *binding = Binding();
         return binding ? binding->memory_state.get() : nullptr;
@@ -294,6 +314,11 @@ class Bindable : public StateObject {
         return memory_tracker_->GetBoundMemoryRange(range);
     }
 
+    BindableLinearMemoryTracker::BoundRanges GetBoundRanges(const BindableMemoryTracker::BufferRange &ranges_bounds,
+                                                            const std::vector<BindableMemoryTracker::BufferRange> ranges) const {
+        return memory_tracker_->GetBoundRanges(ranges_bounds, ranges);
+    }
+
     BindableMemoryTracker::DeviceMemoryState GetBoundMemoryStates() const {
         return memory_tracker_->GetBoundMemoryStates();
     }
@@ -316,6 +341,7 @@ class Bindable : public StateObject {
     const bool unprotected;  // can't be used for protected memory
 
     // For when an array of binds don't succeed and the object is in an indeterminate state
+    // This is solved if the app provides VkBindMemoryStatus
     bool indeterminate_state = false;
 
   private:
