@@ -21,8 +21,7 @@
 #include "state_tracker/pipeline_state.h"
 #include "state_tracker/descriptor_sets.h"
 #include "state_tracker/shader_module.h"
-#include <limits>
-#include <string_view>
+#include "generated/dispatch_functions.h"
 
 static VkImageSubresourceRange MakeImageFullRange(const VkImageCreateInfo &create_info) {
     const auto format = create_info.format;
@@ -59,8 +58,8 @@ VkImageSubresourceRange NormalizeSubresourceRange(const VkImageCreateInfo &image
 }
 
 static bool IsDepthSliced(const VkImageCreateInfo &image_create_info, const VkImageViewCreateInfo &create_info) {
-    auto kDepthSlicedFlags = VK_IMAGE_CREATE_2D_ARRAY_COMPATIBLE_BIT | VK_IMAGE_CREATE_2D_VIEW_COMPATIBLE_BIT_EXT;
-    return ((image_create_info.flags & kDepthSlicedFlags) != 0) &&
+    auto depth_slice_flag = VK_IMAGE_CREATE_2D_ARRAY_COMPATIBLE_BIT | VK_IMAGE_CREATE_2D_VIEW_COMPATIBLE_BIT_EXT;
+    return ((image_create_info.flags & depth_slice_flag) != 0) &&
            (create_info.viewType == VK_IMAGE_VIEW_TYPE_2D || create_info.viewType == VK_IMAGE_VIEW_TYPE_2D_ARRAY);
 }
 
@@ -255,7 +254,7 @@ Image::Image(const ValidationStateTracker &dev_data, VkImage img, const VkImageC
 
 void Image::Destroy() {
     // NOTE: due to corner cases in aliased images, the layout_range_map MUST not be cleaned up here.
-    // If it is, bad local entries could be created by vvl::CommandBuffer::GetImageSubresourceLayoutMap()
+    // If it is, bad local entries could be created by vvl::CommandBuffer::GetOrCreateImageLayoutRegistry()
     // If an aliasing image was being destroyed (and layout_range_map was reset()), a nullptr keyed
     // entry could get put into vvl::CommandBuffer::aliased_image_layout_map.
     //
@@ -570,7 +569,7 @@ Swapchain::Swapchain(ValidationStateTracker &dev_data_, const VkSwapchainCreateI
       image_create_info(GetImageCreateInfo(pCreateInfo)),
       dev_data(dev_data_) {}
 
-void Swapchain::PresentImage(uint32_t image_index, uint64_t present_id) {
+void Swapchain::PresentImage(uint32_t image_index, uint64_t present_id, const AcquireFenceSync &acquire_fence_sync) {
     if (image_index >= images.size()) return;
     assert(acquired_images > 0);
     if (!shared_presentable) {
@@ -581,17 +580,31 @@ void Swapchain::PresentImage(uint32_t image_index, uint64_t present_id) {
     } else {
         images[image_index].image_state->layout_locked = true;
     }
+    images[image_index].acquire_fence_sync = acquire_fence_sync;
     if (present_id > max_present_id) {
         max_present_id = present_id;
     }
 }
 
+void Swapchain::ReleaseImage(uint32_t image_index) {
+    if (image_index >= images.size()) return;
+    assert(acquired_images > 0);
+    acquired_images--;
+    images[image_index].acquired = false;
+    images[image_index].acquire_semaphore.reset();
+    images[image_index].acquire_fence.reset();
+}
+
 void Swapchain::AcquireImage(uint32_t image_index, const std::shared_ptr<vvl::Semaphore> &semaphore_state,
                              const std::shared_ptr<vvl::Fence> &fence_state) {
     acquired_images++;
     images[image_index].acquired = true;
     images[image_index].acquire_semaphore = semaphore_state;
     images[image_index].acquire_fence = fence_state;
+    if (fence_state) {
+        fence_state->SetAcquireFenceSync(images[image_index].acquire_fence_sync);
+        images[image_index].acquire_fence_sync = {};
+    }
     if (shared_presentable) {
         images[image_index].image_state->shared_presentable = shared_presentable;
     }
@@ -725,7 +738,7 @@ vvl::span<const vku::safe_VkSurfaceFormat2KHR> Surface::GetFormats(bool get_surf
         } else {
             result.resize(count);
             for (uint32_t surface_format_index = 0; surface_format_index < count; ++surface_format_index) {
-                result.emplace_back(vku::safe_VkSurfaceFormat2KHR(&formats2[surface_format_index]));
+                result.emplace_back(&formats2[surface_format_index]);
             }
         }
     } else {
@@ -743,7 +756,7 @@ vvl::span<const vku::safe_VkSurfaceFormat2KHR> Surface::GetFormats(bool get_surf
             VkSurfaceFormat2KHR format2 = vku::InitStructHelper();
             for (const auto &format : formats) {
                 format2.surfaceFormat = format;
-                result.emplace_back(vku::safe_VkSurfaceFormat2KHR(&format2));
+                result.emplace_back(&format2);
             }
         }
     }
@@ -767,14 +780,17 @@ const Surface::PhysDevCache *Surface::GetPhysDevCache(VkPhysicalDevice phys_dev)
 
 void Surface::UpdateCapabilitiesCache(VkPhysicalDevice phys_dev, const VkSurfaceCapabilitiesKHR &surface_caps) {
     auto guard = Lock();
-    cache_[phys_dev].capabilities = surface_caps;
+    PhysDevCache &cache = cache_[phys_dev];
+    cache.capabilities = surface_caps;
+    cache.last_capability_query_used_present_mode = false;
 }
 
 void Surface::UpdateCapabilitiesCache(VkPhysicalDevice phys_dev, const VkSurfaceCapabilities2KHR &surface_caps,
                                       VkPresentModeKHR present_mode) {
     auto guard = Lock();
     auto &cache = cache_[phys_dev];
-    // Get entry for a given presentation mode
+
+    // Get entry for the given presentation mode
     PresentModeInfo *info = nullptr;
     for (auto &cur_info : cache.present_mode_infos) {
         if (cur_info.present_mode == present_mode) {
@@ -783,10 +799,11 @@ void Surface::UpdateCapabilitiesCache(VkPhysicalDevice phys_dev, const VkSurface
         }
     }
     if (!info) {
-        cache.present_mode_infos.push_back(PresentModeInfo{});
+        cache.present_mode_infos.emplace_back(PresentModeInfo{});
         info = &cache.present_mode_infos.back();
         info->present_mode = present_mode;
     }
+
     // Update entry
     info->surface_capabilities = surface_caps.surfaceCapabilities;
     const auto *present_scaling_caps = vku::FindStructInPNextChain<VkSurfacePresentScalingCapabilitiesEXT>(surface_caps.pNext);
@@ -798,6 +815,14 @@ void Surface::UpdateCapabilitiesCache(VkPhysicalDevice phys_dev, const VkSurface
         info->compatible_present_modes.emplace(compat_modes->pPresentModes,
                                                compat_modes->pPresentModes + compat_modes->presentModeCount);
     }
+    cache.last_capability_query_used_present_mode = true;
+}
+
+bool Surface::IsLastCapabilityQueryUsedPresentMode(VkPhysicalDevice phys_dev) const {
+    if (auto guard = Lock(); auto cache = GetPhysDevCache(phys_dev)) {
+        return cache->last_capability_query_used_present_mode;
+    }
+    return false;
 }
 
 VkSurfaceCapabilitiesKHR Surface::GetSurfaceCapabilities(VkPhysicalDevice phys_dev, const void *surface_info_pnext) const {
