@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (C) 2015-2024 Google Inc.
+/* Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (C) 2015-2025 Google Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  * Modifications Copyright (C) 2022 RasterGrid Kft.
  *
@@ -19,10 +19,10 @@
  */
 #pragma once
 
-#include <utility>
 #include <variant>
 
 #include "state_tracker/device_memory_state.h"
+#include "state_tracker/fence_state.h"
 #include "state_tracker/image_layout_map.h"
 #include "utils/vk_layer_utils.h"
 
@@ -100,10 +100,10 @@ class Image : public Bindable {
     const VkFormatFeatureFlags2KHR format_features;
     // Need to memory requirments for each plane if image is disjoint
     const bool disjoint;  // True if image was created with VK_IMAGE_CREATE_DISJOINT_BIT
-    static constexpr int MAX_PLANES = 3;
-    using MemoryReqs = std::array<VkMemoryRequirements, MAX_PLANES>;
+    static constexpr int kMaxPlanes = 3;
+    using MemoryReqs = std::array<VkMemoryRequirements, kMaxPlanes>;
     const MemoryReqs requirements;
-    std::array<bool, MAX_PLANES> memory_requirements_checked = {};
+    std::array<bool, kMaxPlanes> memory_requirements_checked = {};
 
     const bool sparse_residency;
     using SparseReqs = std::vector<VkSparseImageMemoryRequirements>;
@@ -148,6 +148,9 @@ class Image : public Bindable {
 
     bool IsSwapchainImage() const { return create_from_swapchain != VK_NULL_HANDLE; }
 
+    // TODO - need to understand if VkBindImageMemorySwapchainInfoKHR counts as "bound"
+    bool HasBeenBound() const { return (MemState() != nullptr) || (bind_swapchain); }
+
     inline bool IsImageTypeEqual(const VkImageCreateInfo &other_create_info) const {
         return create_info.imageType == other_create_info.imageType;
     }
@@ -314,6 +317,11 @@ struct SwapchainImage {
     bool acquired = false;
     std::shared_ptr<vvl::Semaphore> acquire_semaphore;
     std::shared_ptr<vvl::Fence> acquire_fence;
+
+    // Each swapchain image keeps information about submissions associated with current present.
+    // When the image is re-acquired later this information can be used to synchronize with
+    // these submissions by using acquire fence.
+    AcquireFenceSync acquire_fence_sync;
 };
 
 // State for VkSwapchainKHR objects.
@@ -347,7 +355,9 @@ class Swapchain : public StateObject {
 
     VkSwapchainKHR VkHandle() const { return handle_.Cast<VkSwapchainKHR>(); }
 
-    void PresentImage(uint32_t image_index, uint64_t present_id);
+    void PresentImage(uint32_t image_index, uint64_t present_id, const AcquireFenceSync &acquire_fence_sync);
+
+    void ReleaseImage(uint32_t image_index);
 
     void AcquireImage(uint32_t image_index, const std::shared_ptr<vvl::Semaphore> &semaphore_state,
                       const std::shared_ptr<vvl::Fence> &fence_state);
@@ -422,6 +432,7 @@ class Surface : public StateObject {
     void UpdateCapabilitiesCache(VkPhysicalDevice phys_dev, const VkSurfaceCapabilities2KHR &surface_caps,
                                  VkPresentModeKHR present_mode);
 
+    bool IsLastCapabilityQueryUsedPresentMode(VkPhysicalDevice phys_dev) const;
     VkSurfaceCapabilitiesKHR GetSurfaceCapabilities(VkPhysicalDevice phys_dev, const void *surface_info_pnext) const;
     VkSurfaceCapabilitiesKHR GetPresentModeSurfaceCapabilities(VkPhysicalDevice phys_dev, VkPresentModeKHR present_mode) const;
     VkSurfacePresentScalingCapabilitiesEXT GetPresentModeScalingCapabilities(VkPhysicalDevice phys_dev,
@@ -454,6 +465,7 @@ class Surface : public StateObject {
         std::optional<std::vector<VkPresentModeKHR>> present_modes;
         std::optional<VkSurfaceCapabilitiesKHR> capabilities;
         std::vector<PresentModeInfo> present_mode_infos;
+        bool last_capability_query_used_present_mode = false;
 
         const PresentModeInfo *GetPresentModeInfo(VkPresentModeKHR present_mode) const;
     };
