@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (C) 2015-2024 Google Inc.
+/* Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (C) 2015-2025 Google Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,6 +17,7 @@
  * limitations under the License.
  */
 #include "state_tracker/pipeline_state.h"
+#include "generated/dynamic_state_helper.h"
 #include "state_tracker/descriptor_sets.h"
 #include "state_tracker/cmd_buffer_state.h"
 #include "state_tracker/render_pass_state.h"
@@ -57,114 +58,119 @@ static vku::safe_VkGraphicsPipelineCreateInfo MakeGraphicsCreateInfo(const VkGra
 
 // static
 std::vector<ShaderStageState> Pipeline::GetStageStates(const ValidationStateTracker &state_data, const Pipeline &pipe_state,
-                                                       spirv::StatelessData *stateless_data,
-                                                       ShaderModuleUniqueIds *shader_unique_id_map) {
+                                                       spirv::StatelessData *stateless_data) {
     std::vector<ShaderStageState> stage_states;
 
-    // stages such as VK_SHADER_STAGE_ALL are find as this code is only looking for exact matches, not bool logic
-    for (const auto &stage : AllVkShaderStageFlags) {
-        bool stage_found = false;
-        // shader stages need to be recorded in pipeline order
+    std::vector<VkShaderStageFlagBits> lookup_in_library_stages = {VK_SHADER_STAGE_VERTEX_BIT,
+                                                                   VK_SHADER_STAGE_FRAGMENT_BIT,
+                                                                   VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT,
+                                                                   VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT,
+                                                                   VK_SHADER_STAGE_GEOMETRY_BIT,
+                                                                   VK_SHADER_STAGE_TASK_BIT_EXT,
+                                                                   VK_SHADER_STAGE_MESH_BIT_EXT};
 
-        for (size_t stage_index = 0; stage_index < pipe_state.shader_stages_ci.size(); ++stage_index) {
-            if (pipe_state.pipeline_type == VK_PIPELINE_BIND_POINT_GRAPHICS &&
-                !pipe_state.OwnsSubState(pipe_state.fragment_shader_state) &&
-                !pipe_state.OwnsSubState(pipe_state.pre_raster_state)) {
-                break;  // pStages are ignored if not using one of these substates
-            }
+    for (size_t stage_index = 0; stage_index < pipe_state.shader_stages_ci.size(); ++stage_index) {
+        if (pipe_state.pipeline_type == VK_PIPELINE_BIND_POINT_GRAPHICS &&
+            !pipe_state.OwnsSubState(pipe_state.fragment_shader_state) && !pipe_state.OwnsSubState(pipe_state.pre_raster_state)) {
+            continue;  // pStages are ignored if not using one of these sub-states
+        }
 
-            const auto &stage_ci = pipe_state.shader_stages_ci[stage_index];
-            if (stage_ci.stage == stage) {
-                auto module_state = state_data.Get<vvl::ShaderModule>(stage_ci.module);
-                if (!module_state && pipe_state.pipeline_cache) {
-                    // Attempt to look up the pipeline cache for shader module data
-                    module_state = pipe_state.pipeline_cache->GetStageModule(pipe_state, stage_index);
+        const auto &stage_ci = pipe_state.shader_stages_ci[stage_index];
+        auto module_state = state_data.Get<vvl::ShaderModule>(stage_ci.module);
+        if (!module_state && pipe_state.pipeline_cache) {
+            // Attempt to look up the pipeline cache for shader module data
+            module_state = pipe_state.pipeline_cache->GetStageModule(pipe_state, stage_index);
+        }
+        if (!module_state) {
+            // See if the module is referenced in a library sub state
+            module_state = pipe_state.GetSubStateShader(stage_ci.stage);
+        }
+
+        if (!module_state || !module_state->spirv) {
+            // If module is null and there is a VkShaderModuleCreateInfo in the pNext chain of the stage info, then this
+            // module is part of a library and the state must be created
+            // This support was also added in VK_KHR_maintenance5
+            if (const auto shader_ci = vku::FindStructInPNextChain<VkShaderModuleCreateInfo>(stage_ci.pNext)) {
+                // don't need to worry about GroupDecoration in GPL
+                auto spirv_module = std::make_shared<spirv::Module>(shader_ci->codeSize, shader_ci->pCode, stateless_data);
+                module_state = std::make_shared<vvl::ShaderModule>(VK_NULL_HANDLE, spirv_module);
+                if (stateless_data) {
+                    stateless_data->pipeline_pnext_module = spirv_module;
                 }
-                if (!module_state) {
-                    // See if the module is referenced in a library sub state
-                    module_state = pipe_state.GetSubStateShader(stage_ci.stage);
-                }
-
-                if (!module_state || !module_state->spirv) {
-                    // If module is null and there is a VkShaderModuleCreateInfo in the pNext chain of the stage info, then this
-                    // module is part of a library and the state must be created
-                    // This support was also added in VK_KHR_maintenance5
-                    const uint32_t unique_shader_id = (shader_unique_id_map) ? (*shader_unique_id_map)[stage] : 0;
-                    if (const auto shader_ci = vku::FindStructInPNextChain<VkShaderModuleCreateInfo>(stage_ci.pNext)) {
-                        // don't need to worry about GroupDecoration in GPL
-                        auto spirv_module = std::make_shared<spirv::Module>(shader_ci->codeSize, shader_ci->pCode, stateless_data);
-                        module_state = std::make_shared<vvl::ShaderModule>(VK_NULL_HANDLE, spirv_module, unique_shader_id);
-                        if (stateless_data) {
-                            stateless_data->pipeline_pnext_module = spirv_module;
-                        }
-                    } else {
-                        // VK_EXT_shader_module_identifier could legally provide a null module handle
-                        module_state = std::make_shared<vvl::ShaderModule>(unique_shader_id);
-                    }
-                }
-
-                stage_states.emplace_back(&stage_ci, nullptr, module_state, module_state->spirv);
-                stage_found = true;
+            } else {
+                // VK_EXT_shader_module_identifier could legally provide a null module handle
+                module_state = std::make_shared<vvl::ShaderModule>();
             }
         }
-        if (!stage_found) {
-            // Check if stage has been supplied by a library
-            std::shared_ptr<const vvl::ShaderModule> module_state = nullptr;
-            const vku::safe_VkPipelineShaderStageCreateInfo *stage_ci = nullptr;
-            switch (stage) {
-                case VK_SHADER_STAGE_VERTEX_BIT:
-                    if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->vertex_shader) {
-                        module_state = pipe_state.pre_raster_state->vertex_shader;
-                        stage_ci = pipe_state.pre_raster_state->vertex_shader_ci;
-                    }
-                    break;
-                case VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT:
-                    if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->tessc_shader) {
-                        module_state = pipe_state.pre_raster_state->tessc_shader;
-                        stage_ci = pipe_state.pre_raster_state->tessc_shader_ci;
-                    }
-                    break;
-                case VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT:
-                    if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->tesse_shader) {
-                        module_state = pipe_state.pre_raster_state->tesse_shader;
-                        stage_ci = pipe_state.pre_raster_state->tesse_shader_ci;
-                    }
-                    break;
-                case VK_SHADER_STAGE_GEOMETRY_BIT:
-                    if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->geometry_shader) {
-                        module_state = pipe_state.pre_raster_state->geometry_shader;
-                        stage_ci = pipe_state.pre_raster_state->geometry_shader_ci;
-                    }
-                    break;
-                case VK_SHADER_STAGE_TASK_BIT_EXT:
-                    if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->task_shader) {
-                        module_state = pipe_state.pre_raster_state->task_shader;
-                        stage_ci = pipe_state.pre_raster_state->task_shader_ci;
-                    }
-                    break;
-                case VK_SHADER_STAGE_MESH_BIT_EXT:
-                    if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->mesh_shader) {
-                        module_state = pipe_state.pre_raster_state->mesh_shader;
-                        stage_ci = pipe_state.pre_raster_state->mesh_shader_ci;
-                    }
-                    break;
-                case VK_SHADER_STAGE_FRAGMENT_BIT:
-                    if (pipe_state.fragment_shader_state && pipe_state.fragment_shader_state->fragment_shader) {
-                        module_state = pipe_state.fragment_shader_state->fragment_shader;
-                        stage_ci = pipe_state.fragment_shader_state->fragment_shader_ci.get();
-                    }
-                    break;
-                default:
-                    // no-op
-                    break;
-            }
-            if (!stage_ci) {
-                continue;
-            }
 
-            stage_states.emplace_back(stage_ci, nullptr, module_state, module_state->spirv);
+        stage_states.emplace_back(&stage_ci, nullptr, module_state, module_state->spirv);
+
+        // If stage was found, do not try to look for it in library
+        auto found_stage = std::find(lookup_in_library_stages.begin(), lookup_in_library_stages.end(), stage_ci.stage);
+        if (found_stage != lookup_in_library_stages.end()) {
+            const size_t last_library_stage_i = lookup_in_library_stages.size() - 1;
+            std::swap(*found_stage, lookup_in_library_stages[last_library_stage_i]);
+            lookup_in_library_stages.resize(last_library_stage_i);
         }
     }
+
+    for (const auto &stage_flag : lookup_in_library_stages) {
+        // Check if stage has been supplied by a library
+        std::shared_ptr<const vvl::ShaderModule> module_state = nullptr;
+        const vku::safe_VkPipelineShaderStageCreateInfo *stage_ci = nullptr;
+        switch (stage_flag) {
+            case VK_SHADER_STAGE_VERTEX_BIT:
+                if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->vertex_shader) {
+                    module_state = pipe_state.pre_raster_state->vertex_shader;
+                    stage_ci = pipe_state.pre_raster_state->vertex_shader_ci;
+                }
+                break;
+            case VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT:
+                if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->tessc_shader) {
+                    module_state = pipe_state.pre_raster_state->tessc_shader;
+                    stage_ci = pipe_state.pre_raster_state->tessc_shader_ci;
+                }
+                break;
+            case VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT:
+                if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->tesse_shader) {
+                    module_state = pipe_state.pre_raster_state->tesse_shader;
+                    stage_ci = pipe_state.pre_raster_state->tesse_shader_ci;
+                }
+                break;
+            case VK_SHADER_STAGE_GEOMETRY_BIT:
+                if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->geometry_shader) {
+                    module_state = pipe_state.pre_raster_state->geometry_shader;
+                    stage_ci = pipe_state.pre_raster_state->geometry_shader_ci;
+                }
+                break;
+            case VK_SHADER_STAGE_TASK_BIT_EXT:
+                if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->task_shader) {
+                    module_state = pipe_state.pre_raster_state->task_shader;
+                    stage_ci = pipe_state.pre_raster_state->task_shader_ci;
+                }
+                break;
+            case VK_SHADER_STAGE_MESH_BIT_EXT:
+                if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->mesh_shader) {
+                    module_state = pipe_state.pre_raster_state->mesh_shader;
+                    stage_ci = pipe_state.pre_raster_state->mesh_shader_ci;
+                }
+                break;
+            case VK_SHADER_STAGE_FRAGMENT_BIT:
+                if (pipe_state.fragment_shader_state && pipe_state.fragment_shader_state->fragment_shader) {
+                    module_state = pipe_state.fragment_shader_state->fragment_shader;
+                    stage_ci = pipe_state.fragment_shader_state->fragment_shader_ci.get();
+                }
+                break;
+            default:
+                // no-op
+                break;
+        }
+        if (!stage_ci) {
+            continue;
+        }
+
+        stage_states.emplace_back(stage_ci, nullptr, module_state, module_state->spirv);
+    }
     return stage_states;
 }
 
@@ -247,7 +253,7 @@ static CBDynamicFlags GetGraphicsDynamicState(Pipeline &pipe_state) {
                 case VK_DYNAMIC_STATE_DEPTH_BIAS_ENABLE:
                 case VK_DYNAMIC_STATE_DEPTH_CLAMP_ENABLE_EXT:
                 case VK_DYNAMIC_STATE_POLYGON_MODE_EXT:
-                case VK_DYNAMIC_STATE_LINE_STIPPLE_KHR:
+                case VK_DYNAMIC_STATE_LINE_STIPPLE:
                 case VK_DYNAMIC_STATE_LINE_RASTERIZATION_MODE_EXT:
                 case VK_DYNAMIC_STATE_LINE_STIPPLE_ENABLE_EXT:
                 case VK_DYNAMIC_STATE_RASTERIZATION_STREAM_EXT:
@@ -255,6 +261,7 @@ static CBDynamicFlags GetGraphicsDynamicState(Pipeline &pipe_state) {
                 case VK_DYNAMIC_STATE_EXTRA_PRIMITIVE_OVERESTIMATION_SIZE_EXT:
                 case VK_DYNAMIC_STATE_DEPTH_CLIP_ENABLE_EXT:
                 case VK_DYNAMIC_STATE_PROVOKING_VERTEX_MODE_EXT:
+                case VK_DYNAMIC_STATE_DEPTH_CLAMP_RANGE_EXT:
                 // VkPipelineTessellationStateCreateInfo
                 case VK_DYNAMIC_STATE_PATCH_CONTROL_POINTS_EXT:
                 case VK_DYNAMIC_STATE_TESSELLATION_DOMAIN_ORIGIN_EXT:
@@ -384,24 +391,24 @@ static CBDynamicFlags GetRayTracingDynamicState(Pipeline &pipe_state) {
 
 static bool UsesPipelineRobustness(const void *pNext, const Pipeline &pipe_state) {
     bool result = false;
-    const auto robustness_info = vku::FindStructInPNextChain<VkPipelineRobustnessCreateInfoEXT>(pNext);
+    const auto robustness_info = vku::FindStructInPNextChain<VkPipelineRobustnessCreateInfo>(pNext);
     if (!robustness_info) {
         return false;
     }
-    result |= (robustness_info->storageBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2_EXT) ||
-              (robustness_info->storageBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_EXT);
-    result |= (robustness_info->uniformBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2_EXT) ||
-              (robustness_info->uniformBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_EXT);
+    result |= (robustness_info->storageBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2) ||
+              (robustness_info->storageBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS);
+    result |= (robustness_info->uniformBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2) ||
+              (robustness_info->uniformBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS);
     if (!result) {
         for (const auto &stage_ci : pipe_state.shader_stages_ci) {
-            const auto stage_robustness_info = vku::FindStructInPNextChain<VkPipelineRobustnessCreateInfoEXT>(stage_ci.pNext);
+            const auto stage_robustness_info = vku::FindStructInPNextChain<VkPipelineRobustnessCreateInfo>(stage_ci.pNext);
             if (stage_robustness_info) {
                 result |=
-                    (stage_robustness_info->storageBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2_EXT) ||
-                    (stage_robustness_info->storageBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_EXT);
+                    (stage_robustness_info->storageBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2) ||
+                    (stage_robustness_info->storageBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS);
                 result |=
-                    (stage_robustness_info->uniformBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2_EXT) ||
-                    (stage_robustness_info->uniformBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_EXT);
+                    (stage_robustness_info->uniformBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2) ||
+                    (stage_robustness_info->uniformBuffers == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS);
             }
         }
     }
@@ -410,19 +417,18 @@ static bool UsesPipelineRobustness(const void *pNext, const Pipeline &pipe_state
 
 static bool UsesPipelineVertexRobustness(const void *pNext, const Pipeline &pipe_state) {
     bool result = false;
-    const auto robustness_info = vku::FindStructInPNextChain<VkPipelineRobustnessCreateInfoEXT>(pNext);
+    const auto robustness_info = vku::FindStructInPNextChain<VkPipelineRobustnessCreateInfo>(pNext);
     if (!robustness_info) {
         return false;
     }
-    result |= (robustness_info->vertexInputs == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2_EXT) ||
-              (robustness_info->vertexInputs == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_EXT);
+    result |= (robustness_info->vertexInputs == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2) ||
+              (robustness_info->vertexInputs == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS);
     if (!result) {
         for (const auto &stage_ci : pipe_state.shader_stages_ci) {
-            const auto stage_robustness_info = vku::FindStructInPNextChain<VkPipelineRobustnessCreateInfoEXT>(stage_ci.pNext);
+            const auto stage_robustness_info = vku::FindStructInPNextChain<VkPipelineRobustnessCreateInfo>(stage_ci.pNext);
             if (stage_robustness_info) {
-                result |=
-                    (stage_robustness_info->vertexInputs == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2_EXT) ||
-                    (stage_robustness_info->vertexInputs == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_EXT);
+                result |= (stage_robustness_info->vertexInputs == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2) ||
+                          (stage_robustness_info->vertexInputs == VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS);
             }
         }
     }
@@ -503,7 +509,7 @@ static VkPrimitiveTopology GetTopologyAtRasterizer(const Pipeline &pipeline) {
 }
 
 static VkPipelineCreateFlags2KHR GetPipelineCreateFlags(const void *pNext, VkPipelineCreateFlags flags) {
-    const auto flags2 = vku::FindStructInPNextChain<VkPipelineCreateFlags2CreateInfoKHR>(pNext);
+    const auto flags2 = vku::FindStructInPNextChain<VkPipelineCreateFlags2CreateInfo>(pNext);
     if (flags2) {
         return flags2->flags;
     }
@@ -682,7 +688,7 @@ std::shared_ptr<const vvl::ShaderModule> Pipeline::GetSubStateShader(VkShaderSta
 Pipeline::Pipeline(const ValidationStateTracker &state_data, const VkGraphicsPipelineCreateInfo *pCreateInfo,
                    std::shared_ptr<const vvl::PipelineCache> &&pipe_cache, std::shared_ptr<const vvl::RenderPass> &&rpstate,
                    std::shared_ptr<const vvl::PipelineLayout> &&layout,
-                   spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages], ShaderModuleUniqueIds *shader_unique_id_map)
+                   spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages])
     : StateObject(static_cast<VkPipeline>(VK_NULL_HANDLE), kVulkanObjectTypePipeline),
       rp_state(rpstate),
       create_info(MakeGraphicsCreateInfo(*pCreateInfo, rpstate, state_data)),
@@ -699,7 +705,7 @@ Pipeline::Pipeline(const ValidationStateTracker &state_data, const VkGraphicsPip
       fragment_shader_state(
           CreateFragmentShaderState(*this, state_data, *pCreateInfo, GraphicsCreateInfo(), rpstate, stateless_data)),
       fragment_output_state(CreateFragmentOutputState(*this, state_data, *pCreateInfo, GraphicsCreateInfo(), rpstate)),
-      stage_states(GetStageStates(state_data, *this, stateless_data, shader_unique_id_map)),
+      stage_states(GetStageStates(state_data, *this, stateless_data)),
       create_info_shaders(GetCreateInfoShaders(*this)),
       linking_shaders(GetLinkingShaders(library_create_info, state_data)),
       active_shaders(create_info_shaders | linking_shaders),
@@ -747,7 +753,7 @@ Pipeline::Pipeline(const ValidationStateTracker &state_data, const VkComputePipe
       create_flags(GetPipelineCreateFlags(ComputeCreateInfo().pNext, ComputeCreateInfo().flags)),
       shader_stages_ci(&ComputeCreateInfo().stage, 1),
       uses_shader_module_id(UsesShaderModuleId(*this)),
-      stage_states(GetStageStates(state_data, *this, stateless_data, nullptr)),
+      stage_states(GetStageStates(state_data, *this, stateless_data)),
       create_info_shaders(GetCreateInfoShaders(*this)),
       active_shaders(create_info_shaders),  // compute has no linking shaders
       active_slots(GetActiveSlots(stage_states)),
@@ -772,7 +778,7 @@ Pipeline::Pipeline(const ValidationStateTracker &state_data, const VkRayTracingP
       shader_stages_ci(RayTracingCreateInfo().pStages, RayTracingCreateInfo().stageCount),
       ray_tracing_library_ci(RayTracingCreateInfo().pLibraryInfo),
       uses_shader_module_id(UsesShaderModuleId(*this)),
-      stage_states(GetStageStates(state_data, *this, stateless_data, nullptr)),
+      stage_states(GetStageStates(state_data, *this, stateless_data)),
       create_info_shaders(GetCreateInfoShaders(*this)),
       active_shaders(create_info_shaders),  // RTX has no linking shaders
       active_slots(GetActiveSlots(stage_states)),
@@ -797,7 +803,7 @@ Pipeline::Pipeline(const ValidationStateTracker &state_data, const VkRayTracingP
       shader_stages_ci(RayTracingCreateInfo().pStages, RayTracingCreateInfo().stageCount),
       ray_tracing_library_ci(RayTracingCreateInfo().pLibraryInfo),
       uses_shader_module_id(UsesShaderModuleId(*this)),
-      stage_states(GetStageStates(state_data, *this, stateless_data, nullptr)),
+      stage_states(GetStageStates(state_data, *this, stateless_data)),
       create_info_shaders(GetCreateInfoShaders(*this)),
       active_shaders(create_info_shaders),  // RTX has no linking shaders
       active_slots(GetActiveSlots(stage_states)),
@@ -815,10 +821,10 @@ Pipeline::Pipeline(const ValidationStateTracker &state_data, const VkRayTracingP
 
 void LastBound::UnbindAndResetPushDescriptorSet(std::shared_ptr<vvl::DescriptorSet> &&ds) {
     if (push_descriptor_set) {
-        for (auto &ps : per_set) {
-            if (ps.bound_descriptor_set == push_descriptor_set) {
-                cb_state.RemoveChild(ps.bound_descriptor_set);
-                ps.bound_descriptor_set.reset();
+        for (auto &ds_slot : ds_slots) {
+            if (ds_slot.ds_state == push_descriptor_set) {
+                cb_state.RemoveChild(ds_slot.ds_state);
+                ds_slot.ds_state.reset();
             }
         }
     }
@@ -834,7 +840,7 @@ void LastBound::Reset() {
         push_descriptor_set->Destroy();
     }
     push_descriptor_set.reset();
-    per_set.clear();
+    ds_slots.clear();
 }
 
 bool LastBound::IsDepthTestEnable() const {
@@ -893,6 +899,19 @@ bool LastBound::IsDepthBiasEnable() const {
     return false;
 }
 
+bool LastBound::IsDepthClampEnable() const {
+    if (!pipeline_state || pipeline_state->IsDynamic(CB_DYNAMIC_STATE_DEPTH_CLAMP_ENABLE_EXT)) {
+        if (cb_state.IsDynamicStateSet(CB_DYNAMIC_STATE_DEPTH_CLAMP_ENABLE_EXT)) {
+            return cb_state.dynamic_state_value.depth_clamp_enable;
+        }
+    } else {
+        if (pipeline_state->RasterizationState()) {
+            return pipeline_state->RasterizationState()->depthClampEnable;
+        }
+    }
+    return false;
+}
+
 bool LastBound::IsStencilTestEnable() const {
     if (!pipeline_state || pipeline_state->IsDynamic(CB_DYNAMIC_STATE_STENCIL_TEST_ENABLE)) {
         if (cb_state.IsDynamicStateSet(CB_DYNAMIC_STATE_STENCIL_TEST_ENABLE)) {
@@ -1104,6 +1123,84 @@ bool LastBound::IsCoverageModulationTableEnable() const {
     return false;
 }
 
+bool LastBound::IsStippledLineEnable() const {
+    if (!pipeline_state || pipeline_state->IsDynamic(CB_DYNAMIC_STATE_LINE_STIPPLE_ENABLE_EXT)) {
+        if (cb_state.IsDynamicStateSet(CB_DYNAMIC_STATE_LINE_STIPPLE_ENABLE_EXT)) {
+            return cb_state.dynamic_state_value.stippled_line_enable;
+        }
+    } else {
+        if (const auto line_state_ci = vku::FindStructInPNextChain<VkPipelineRasterizationLineStateCreateInfo>(
+                pipeline_state->RasterizationStatePNext())) {
+            return line_state_ci->stippledLineEnable;
+        }
+    }
+    return false;
+}
+
+bool LastBound::IsDiscardRectangleEnable() const {
+    if (!pipeline_state || pipeline_state->IsDynamic(CB_DYNAMIC_STATE_DISCARD_RECTANGLE_ENABLE_EXT)) {
+        if (cb_state.IsDynamicStateSet(CB_DYNAMIC_STATE_DISCARD_RECTANGLE_ENABLE_EXT)) {
+            return cb_state.dynamic_state_value.discard_rectangle_enable;
+        }
+    } else {
+        // VK_EXT_discard_rectangles had a special v2 added right away to give it dynamic state
+        // "If the VK_DYNAMIC_STATE_DISCARD_RECTANGLE_ENABLE_EXT dynamic state is not enabled for the pipeline the presence of this
+        // structure in the VkGraphicsPipelineCreateInfo chain, and a discardRectangleCount greater than zero, implicitly enables
+        // discard rectangles in the pipeline"
+        const void *pipeline_pnext = pipeline_state->GetCreateInfoPNext();
+        if (const auto *discard_rectangle_state =
+                vku::FindStructInPNextChain<VkPipelineDiscardRectangleStateCreateInfoEXT>(pipeline_pnext)) {
+            return discard_rectangle_state->discardRectangleCount > 0;
+        }
+    }
+    return false;
+}
+
+bool LastBound::IsShadingRateImageEnable() const {
+    if (!pipeline_state || pipeline_state->IsDynamic(CB_DYNAMIC_STATE_SHADING_RATE_IMAGE_ENABLE_NV)) {
+        if (cb_state.IsDynamicStateSet(CB_DYNAMIC_STATE_SHADING_RATE_IMAGE_ENABLE_NV)) {
+            return cb_state.dynamic_state_value.shading_rate_image_enable;
+        }
+    } else {
+        if (auto viewport_state = pipeline_state->ViewportState()) {
+            if (const auto *shading_rate_image_state =
+                    vku::FindStructInPNextChain<VkPipelineViewportShadingRateImageStateCreateInfoNV>(viewport_state->pNext)) {
+                return shading_rate_image_state->shadingRateImageEnable;
+            }
+        }
+    }
+    return false;
+}
+
+bool LastBound::IsViewportWScalingEnable() const {
+    if (!pipeline_state || pipeline_state->IsDynamic(CB_DYNAMIC_STATE_VIEWPORT_W_SCALING_ENABLE_NV)) {
+        if (cb_state.IsDynamicStateSet(CB_DYNAMIC_STATE_VIEWPORT_W_SCALING_ENABLE_NV)) {
+            return cb_state.dynamic_state_value.viewport_w_scaling_enable;
+        }
+    } else {
+        if (auto viewport_state = pipeline_state->ViewportState()) {
+            if (const auto *viewport_w_scaling_state =
+                    vku::FindStructInPNextChain<VkPipelineViewportWScalingStateCreateInfoNV>(viewport_state->pNext)) {
+                return viewport_w_scaling_state->viewportWScalingEnable;
+            }
+        }
+    }
+    return false;
+}
+
+bool LastBound::IsPrimitiveRestartEnable() const {
+    if (!pipeline_state || pipeline_state->IsDynamic(CB_DYNAMIC_STATE_PRIMITIVE_RESTART_ENABLE)) {
+        if (cb_state.IsDynamicStateSet(CB_DYNAMIC_STATE_PRIMITIVE_RESTART_ENABLE)) {
+            return cb_state.dynamic_state_value.primitive_restart_enable;
+        }
+    } else {
+        if (auto ia_state = pipeline_state->InputAssemblyState()) {
+            return ia_state->primitiveRestartEnable == VK_TRUE;
+        }
+    }
+    return false;
+}
+
 VkCoverageModulationModeNV LastBound::GetCoverageModulationMode() const {
     if (!pipeline_state || pipeline_state->IsDynamic(CB_DYNAMIC_STATE_COVERAGE_MODULATION_MODE_NV)) {
         if (cb_state.IsDynamicStateSet(CB_DYNAMIC_STATE_COVERAGE_MODULATION_MODE_NV)) {
@@ -1150,6 +1247,29 @@ vvl::ShaderObject *LastBound::GetShaderState(ShaderObjectStage stage) const {
     return shader_object_states[static_cast<uint32_t>(stage)];
 }
 
+const vvl::ShaderObject *LastBound::GetShaderStateIfValid(ShaderObjectStage stage) const {
+    if (!shader_object_bound[static_cast<uint32_t>(stage)]) {
+        return nullptr;
+    }
+    return shader_object_states[static_cast<uint32_t>(stage)];
+}
+
+const vvl::ShaderObject *LastBound::GetFirstShader(VkPipelineBindPoint bind_point) const {
+    if (bind_point == VK_PIPELINE_BIND_POINT_COMPUTE) {
+        return GetShaderStateIfValid(ShaderObjectStage::COMPUTE);
+    } else if (bind_point == VK_PIPELINE_BIND_POINT_GRAPHICS) {
+        if (const vvl::ShaderObject *vs = GetShaderStateIfValid(ShaderObjectStage::VERTEX)) {
+            return vs;
+        }
+
+        if (const vvl::ShaderObject *ms = GetShaderStateIfValid(ShaderObjectStage::MESH)) {
+            return ms;
+        }
+    }
+
+    return nullptr;
+}
+
 bool LastBound::HasShaderObjects() const {
     for (uint32_t i = 0; i < kShaderObjectStageCount; ++i) {
         if (GetShader(static_cast<ShaderObjectStage>(i)) != VK_NULL_HANDLE) {
@@ -1159,12 +1279,7 @@ bool LastBound::HasShaderObjects() const {
     return false;
 }
 
-bool LastBound::IsValidShaderBound(ShaderObjectStage stage) const {
-    if (!shader_object_bound[static_cast<uint32_t>(stage)]) {
-        return false;
-    }
-    return shader_object_states[static_cast<uint32_t>(stage)] != nullptr;
-}
+bool LastBound::IsValidShaderBound(ShaderObjectStage stage) const { return GetShaderStateIfValid(stage) != nullptr; }
 
 bool LastBound::IsValidShaderOrNullBound(ShaderObjectStage stage) const {
     return shader_object_bound[static_cast<uint32_t>(stage)];
@@ -1208,42 +1323,101 @@ bool LastBound::IsAnyGraphicsShaderBound() const {
         IsValidShaderBound(ShaderObjectStage::MESH);
 }
 
+VkShaderStageFlags LastBound::GetAllActiveBoundStages() const {
+    if (pipeline_state) {
+        return pipeline_state->active_shaders;
+    }
+    // else shader object
+    VkShaderStageFlags stages = 0;
+    if (IsValidShaderBound(ShaderObjectStage::VERTEX)) {
+        stages |= VK_SHADER_STAGE_VERTEX_BIT;
+    }
+    if (IsValidShaderBound(ShaderObjectStage::TESSELLATION_CONTROL)) {
+        stages |= VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT;
+    }
+    if (IsValidShaderBound(ShaderObjectStage::TESSELLATION_EVALUATION)) {
+        stages |= VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT;
+    }
+    if (IsValidShaderBound(ShaderObjectStage::GEOMETRY)) {
+        stages |= VK_SHADER_STAGE_GEOMETRY_BIT;
+    }
+    if (IsValidShaderBound(ShaderObjectStage::FRAGMENT)) {
+        stages |= VK_SHADER_STAGE_FRAGMENT_BIT;
+    }
+    if (IsValidShaderBound(ShaderObjectStage::COMPUTE)) {
+        stages |= VK_SHADER_STAGE_COMPUTE_BIT;
+    }
+    if (IsValidShaderBound(ShaderObjectStage::TASK)) {
+        stages |= VK_SHADER_STAGE_TASK_BIT_EXT;
+    }
+    if (IsValidShaderBound(ShaderObjectStage::MESH)) {
+        stages |= VK_SHADER_STAGE_MESH_BIT_EXT;
+    }
+    return stages;
+}
+
 bool LastBound::IsBoundSetCompatible(uint32_t set, const vvl::PipelineLayout &pipeline_layout) const {
-    if ((set >= per_set.size()) || (set >= pipeline_layout.set_compat_ids.size())) {
+    if ((set >= ds_slots.size()) || (set >= pipeline_layout.set_compat_ids.size())) {
         return false;
     }
-    return (*(per_set[set].compat_id_for_set) == *(pipeline_layout.set_compat_ids[set]));
+    return (*(ds_slots[set].compat_id_for_set) == *(pipeline_layout.set_compat_ids[set]));
 }
 
 bool LastBound::IsBoundSetCompatible(uint32_t set, const vvl::ShaderObject &shader_object_state) const {
-    if ((set >= per_set.size()) || (set >= shader_object_state.set_compat_ids.size())) {
+    if ((set >= ds_slots.size()) || (set >= shader_object_state.set_compat_ids.size())) {
         return false;
     }
-    return (*(per_set[set].compat_id_for_set) == *(shader_object_state.set_compat_ids[set]));
+    return (*(ds_slots[set].compat_id_for_set) == *(shader_object_state.set_compat_ids[set]));
 };
 
 std::string LastBound::DescribeNonCompatibleSet(uint32_t set, const vvl::PipelineLayout &pipeline_layout) const {
     std::ostringstream ss;
-    if (set >= per_set.size()) {
-        ss << "The set (" << set << ") is out of bounds for the number of sets bound (" << per_set.size() << ")\n";
+    if (set >= ds_slots.size()) {
+        ss << "The set (" << set << ") is out of bounds for the number of sets bound (" << ds_slots.size() << ")\n";
     } else if (set >= pipeline_layout.set_compat_ids.size()) {
         ss << "The set (" << set << ") is out of bounds for the number of sets in the non-compatible VkPipelineLayout ("
            << pipeline_layout.set_compat_ids.size() << ")\n";
     } else {
-        return per_set[set].compat_id_for_set->DescribeDifference(*(pipeline_layout.set_compat_ids[set]));
+        return ds_slots[set].compat_id_for_set->DescribeDifference(*(pipeline_layout.set_compat_ids[set]));
     }
     return ss.str();
 }
 
 std::string LastBound::DescribeNonCompatibleSet(uint32_t set, const vvl::ShaderObject &shader_object_state) const {
     std::ostringstream ss;
-    if (set >= per_set.size()) {
-        ss << "The set (" << set << ") is out of bounds for the number of sets bound (" << per_set.size() << ")\n";
+    if (set >= ds_slots.size()) {
+        ss << "The set (" << set << ") is out of bounds for the number of sets bound (" << ds_slots.size() << ")\n";
     } else if (set >= shader_object_state.set_compat_ids.size()) {
         ss << "The set (" << set << ") is out of bounds for the number of sets in the non-compatible VkDescriptorSetLayout ("
            << shader_object_state.set_compat_ids.size() << ")\n";
     } else {
-        return per_set[set].compat_id_for_set->DescribeDifference(*(shader_object_state.set_compat_ids[set]));
+        return ds_slots[set].compat_id_for_set->DescribeDifference(*(shader_object_state.set_compat_ids[set]));
+    }
+    return ss.str();
+}
+
+bool IsPipelineLayoutSetCompatible(uint32_t set, const vvl::PipelineLayout *a, const vvl::PipelineLayout *b) {
+    if (!a || !b) {
+        return false;
+    }
+    if ((set >= a->set_compat_ids.size()) || (set >= b->set_compat_ids.size())) {
+        return false;
+    }
+    return *(a->set_compat_ids[set]) == *(b->set_compat_ids[set]);
+}
+
+std::string DescribePipelineLayoutSetNonCompatible(uint32_t set, const vvl::PipelineLayout *a, const vvl::PipelineLayout *b) {
+    std::ostringstream ss;
+    if (!a || !b) {
+        ss << "The set (" << set << ") has a null VkPipelineLayout object\n";
+    } else if (set >= a->set_compat_ids.size()) {
+        ss << "The set (" << set << ") is out of bounds for the number of sets in the non-compatible VkDescriptorSetLayout ("
+           << a->set_compat_ids.size() << ")\n";
+    } else if (set >= b->set_compat_ids.size()) {
+        ss << "The set (" << set << ") is out of bounds for the number of sets in the non-compatible VkDescriptorSetLayout ("
+           << b->set_compat_ids.size() << ")\n";
+    } else {
+        return a->set_compat_ids[set]->DescribeDifference(*(b->set_compat_ids[set]));
     }
     return ss.str();
 }
