@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (C) 2015-2024 Google Inc.
+/* Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (C) 2015-2025 Google Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -21,12 +21,12 @@
 
 #include <vulkan/utility/vk_safe_struct.hpp>
 
-#include "utils/hash_vk_types.h"
 #include "state_tracker/pipeline_sub_state.h"
 #include "generated/dynamic_state_helper.h"
 #include "utils/shader_utils.h"
 #include "state_tracker/state_tracker.h"
 #include "state_tracker/shader_stage_state.h"
+#include "utils/vk_layer_utils.h"
 
 // Fwd declarations -- including descriptor_set.h creates an ugly include loop
 namespace vvl {
@@ -131,12 +131,26 @@ class Pipeline : public StateObject {
     const bool uses_pipeline_vertex_robustness;
     bool ignore_color_attachments;
 
+    mutable bool binary_data_released = false;
+
+    // TODO - Because we have hack to create a pipeline at PreCallValidate time (for GPL) we have no proper way to create inherited
+    // state objects of the pipeline This is to make it clear that while currently everyone has to allocate this memory, it is only
+    // ment for GPU-AV
+    struct InstrumentationData {
+        // We create a VkShaderModule that is instrumented and need to delete before leaving the pipeline call
+        std::vector<VkShaderModule> instrumented_shader_module;
+        // TODO - For GPL, this doesn't get passed down from linked shaders
+        bool was_instrumented = false;
+        // When we instrument GPL at link time, we need to hold the new libraries until they are done
+        VkPipeline pre_raster_lib = VK_NULL_HANDLE;
+        VkPipeline frag_out_lib = VK_NULL_HANDLE;
+    } instrumentation_data;
+
     // Executable or legacy pipeline
     Pipeline(const ValidationStateTracker &state_data, const VkGraphicsPipelineCreateInfo *pCreateInfo,
              std::shared_ptr<const vvl::PipelineCache> &&pipe_cache, std::shared_ptr<const vvl::RenderPass> &&rpstate,
              std::shared_ptr<const vvl::PipelineLayout> &&layout,
-             spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages],
-             ShaderModuleUniqueIds *shader_unique_id_map = nullptr);
+             spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages]);
 
     // Compute pipeline
     Pipeline(const ValidationStateTracker &state_data, const VkComputePipelineCreateInfo *pCreateInfo,
@@ -409,8 +423,7 @@ class Pipeline : public StateObject {
     const VkPipelineRenderingCreateInfo *GetPipelineRenderingCreateInfo() const { return rendering_create_info; }
 
     static std::vector<ShaderStageState> GetStageStates(const ValidationStateTracker &state_data, const Pipeline &pipe_state,
-                                                        spirv::StatelessData *stateless_data,
-                                                        ShaderModuleUniqueIds *shader_unique_id_map);
+                                                        spirv::StatelessData *stateless_data);
 
     // Return true if for a given PSO, the given state enum is dynamic, else return false
     bool IsDynamic(const CBDynamicState state) const { return dynamic_state[state]; }
@@ -565,7 +578,7 @@ class Pipeline : public StateObject {
     static bool EnablesRasterizationStates(const CreateInfo &create_info) {
         if (create_info.pDynamicState && create_info.pDynamicState->pDynamicStates) {
             for (uint32_t i = 0; i < create_info.pDynamicState->dynamicStateCount; ++i) {
-                if (create_info.pDynamicState->pDynamicStates[i] == VK_DYNAMIC_STATE_RASTERIZER_DISCARD_ENABLE_EXT) {
+                if (create_info.pDynamicState->pDynamicStates[i] == VK_DYNAMIC_STATE_RASTERIZER_DISCARD_ENABLE) {
                     // If RASTERIZER_DISCARD_ENABLE is dynamic, then we must return true (i.e., rasterization is enabled)
                     // NOTE: create_info must contain pre-raster state, otherwise it is an invalid pipeline and will trigger
                     //       an error outside of this function.
@@ -656,20 +669,23 @@ struct LastBound {
     // We have to track shader_object_bound, because shader_object_states will be nullptr when VK_NULL_HANDLE is used
     bool shader_object_bound[kShaderObjectStageCount]{false};
     vvl::ShaderObject *shader_object_states[kShaderObjectStageCount]{nullptr};
+    // The compatible layout used binding descriptor sets (track location to provide better error message)
     VkPipelineLayout desc_set_pipeline_layout = VK_NULL_HANDLE;
+    vvl::Func desc_set_bound_command = vvl::Func::Empty;  // will be something like vkCmdBindDescriptorSets
     std::shared_ptr<vvl::DescriptorSet> push_descriptor_set;
 
     struct DescriptorBufferBinding {
         uint32_t index = 0;
         VkDeviceSize offset = 0;
     };
-    // Ordered bound set tracking where index is set# that given set is bound to
-    struct PER_SET {
-        std::shared_ptr<vvl::DescriptorSet> bound_descriptor_set;
-        std::optional<DescriptorBufferBinding> bound_descriptor_buffer;
+
+    // Each command buffer has a "slot" to hold a descriptor set binding. This "slot" also might be empty
+    struct DescriptorSetSlot {
+        std::shared_ptr<vvl::DescriptorSet> ds_state;
+        std::optional<DescriptorBufferBinding> descriptor_buffer_binding;
 
         // one dynamic offset per dynamic descriptor bound to this CB
-        std::vector<uint32_t> dynamicOffsets;
+        std::vector<uint32_t> dynamic_offsets;
         PipelineLayoutCompatId compat_id_for_set{0};
 
         // Cache most recently validated descriptor state for ValidateActionState/UpdateDrawState
@@ -678,13 +694,14 @@ struct LastBound {
         uint64_t validated_set_image_layout_change_count{~0ULL};
 
         void Reset() {
-            bound_descriptor_set.reset();
-            bound_descriptor_buffer.reset();
-            dynamicOffsets.clear();
+            ds_state.reset();
+            descriptor_buffer_binding.reset();
+            dynamic_offsets.clear();
         }
     };
 
-    std::vector<PER_SET> per_set;
+    // Ordered bound set tracking where index is set# that given set is bound to
+    std::vector<DescriptorSetSlot> ds_slots;
 
     void Reset();
 
@@ -695,6 +712,7 @@ struct LastBound {
     bool IsDepthBoundTestEnable() const;
     bool IsDepthWriteEnable() const;
     bool IsDepthBiasEnable() const;
+    bool IsDepthClampEnable() const;
     bool IsStencilTestEnable() const;
     VkStencilOpState GetStencilOpStateFront() const;
     VkStencilOpState GetStencilOpStateBack() const;
@@ -710,16 +728,25 @@ struct LastBound {
     bool IsExclusiveScissorEnabled() const;
     bool IsCoverageToColorEnabled() const;
     bool IsCoverageModulationTableEnable() const;
+    bool IsDiscardRectangleEnable() const;
+    bool IsStippledLineEnable() const;
+    bool IsShadingRateImageEnable() const;
+    bool IsViewportWScalingEnable() const;
+    bool IsPrimitiveRestartEnable() const;
     VkCoverageModulationModeNV GetCoverageModulationMode() const;
 
     bool ValidShaderObjectCombination(const VkPipelineBindPoint bind_point, const DeviceFeatures &device_features) const;
     VkShaderEXT GetShader(ShaderObjectStage stage) const;
     vvl::ShaderObject *GetShaderState(ShaderObjectStage stage) const;
+    const vvl::ShaderObject *GetShaderStateIfValid(ShaderObjectStage stage) const;
+    // Return compute shader for compute pipeline, vertex or mesh shader for graphics
+    const vvl::ShaderObject *GetFirstShader(VkPipelineBindPoint bind_point) const;
     bool HasShaderObjects() const;
     bool IsValidShaderBound(ShaderObjectStage stage) const;
     bool IsValidShaderOrNullBound(ShaderObjectStage stage) const;
     std::vector<vvl::ShaderObject *> GetAllBoundGraphicsShaders();
     bool IsAnyGraphicsShaderBound() const;
+    VkShaderStageFlags GetAllActiveBoundStages() const;
 
     bool IsBoundSetCompatible(uint32_t set, const vvl::PipelineLayout &pipeline_layout) const;
     bool IsBoundSetCompatible(uint32_t set, const vvl::ShaderObject &shader_object_state) const;
@@ -729,15 +756,9 @@ struct LastBound {
     const spirv::EntryPoint *GetFragmentEntryPoint() const;
 };
 
-static inline bool IsPipelineLayoutSetCompat(uint32_t set, const vvl::PipelineLayout *a, const vvl::PipelineLayout *b) {
-    if (!a || !b) {
-        return false;
-    }
-    if ((set >= a->set_compat_ids.size()) || (set >= b->set_compat_ids.size())) {
-        return false;
-    }
-    return a->set_compat_ids[set] == b->set_compat_ids[set];
-}
+// Used to compare 2 layouts independently when not tied to the last bound object
+bool IsPipelineLayoutSetCompatible(uint32_t set, const vvl::PipelineLayout *a, const vvl::PipelineLayout *b);
+std::string DescribePipelineLayoutSetNonCompatible(uint32_t set, const vvl::PipelineLayout *a, const vvl::PipelineLayout *b);
 
 enum LvlBindPoint {
     BindPoint_Graphics = VK_PIPELINE_BIND_POINT_GRAPHICS,
@@ -791,6 +812,19 @@ static VkPipelineBindPoint inline ConvertToPipelineBindPoint(VkShaderStageFlagBi
     return VK_PIPELINE_BIND_POINT_MAX_ENUM;
 }
 
+static VkPipelineBindPoint inline ConvertToPipelineBindPoint(VkShaderStageFlags stage) {
+    // Assumes the call has checked stages have not been mixed
+    if (stage & kShaderStageAllGraphics) {
+        return VK_PIPELINE_BIND_POINT_GRAPHICS;
+    } else if (stage & VK_SHADER_STAGE_COMPUTE_BIT) {
+        return VK_PIPELINE_BIND_POINT_COMPUTE;
+    } else if (stage & kShaderStageAllRayTracing) {
+        return VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR;
+    } else {
+        assert(false);
+        return VK_PIPELINE_BIND_POINT_MAX_ENUM;
+    }
+}
 static LvlBindPoint inline ConvertToLvlBindPoint(VkShaderStageFlagBits stage) {
     switch (stage) {
         case VK_SHADER_STAGE_VERTEX_BIT:
