@@ -36,21 +36,27 @@ VertexInputState::VertexInputState(const vvl::Pipeline &p, const vku::safe_VkGra
 
     if (input_state) {
         if (input_state->vertexBindingDescriptionCount) {
-            const uint32_t count = input_state->vertexBindingDescriptionCount;
-            binding_descriptions.reserve(count);
-            binding_to_index_map.reserve(count);
-
-            for (uint32_t i = 0; i < count; i++) {
-                binding_descriptions.emplace_back(input_state->pVertexBindingDescriptions[i]);
-                binding_to_index_map[binding_descriptions.back().binding] = i;
+            for (const auto [i, bd] :
+                 vvl::enumerate(input_state->pVertexBindingDescriptions, input_state->vertexBindingDescriptionCount)) {
+                bindings.emplace(bd->binding, VertexBindingState(i, bd));
+            }
+            const auto *divisor_info = vku::FindStructInPNextChain<VkPipelineVertexInputDivisorStateCreateInfo>(input_state->pNext);
+            if (divisor_info) {
+                for (const auto [i, di] :
+                     vvl::enumerate(divisor_info->pVertexBindingDivisors, divisor_info->vertexBindingDivisorCount)) {
+                    if (auto *binding_state = vvl::Find(bindings, di->binding)) {
+                        binding_state->desc.divisor = di->divisor;
+                    }
+                }
             }
         }
-
-        vertex_attribute_descriptions.reserve(input_state->vertexAttributeDescriptionCount);
-        for (const auto [i, description] :
+        for (const auto [i, ad] :
              vvl::enumerate(input_state->pVertexAttributeDescriptions, input_state->vertexAttributeDescriptionCount)) {
-            vertex_attribute_descriptions.emplace_back(vku::InitStruct<VkVertexInputAttributeDescription2EXT>(
-                nullptr, description->location, description->binding, description->format, description->offset));
+            auto *binding_state = vvl::Find(bindings, ad->binding);
+            if (!binding_state) {
+                continue;
+            }
+            binding_state->locations.emplace(ad->location, VertexAttrState(i, ad));
         }
     }
 }
@@ -77,6 +83,10 @@ PreRasterState::PreRasterState(const vvl::Pipeline &p, const ValidationStateTrac
         all_stages |= stage;
 
         auto module_state = state_data.Get<vvl::ShaderModule>(stage_ci.module);
+        if (!module_state && p.pipeline_cache) {
+            // Attempt to look up the pipeline cache for shader module data
+            module_state = p.pipeline_cache->GetStageModule(p, i);
+        }
         if (!module_state) {
             // If module is null and there is a VkShaderModuleCreateInfo in the pNext chain of the stage info, then this
             // module is part of a library and the state must be created.
@@ -86,7 +96,7 @@ PreRasterState::PreRasterState(const vvl::Pipeline &p, const ValidationStateTrac
                 spirv::StatelessData *stateless_data_stage =
                     (stateless_data && i < kCommonMaxGraphicsShaderStages) ? &stateless_data[i] : nullptr;
                 auto spirv_module = std::make_shared<spirv::Module>(shader_ci->codeSize, shader_ci->pCode, stateless_data_stage);
-                module_state = std::make_shared<vvl::ShaderModule>(VK_NULL_HANDLE, spirv_module, 0);
+                module_state = std::make_shared<vvl::ShaderModule>(VK_NULL_HANDLE, spirv_module);
                 if (stateless_data_stage) {
                     stateless_data_stage->pipeline_pnext_module = spirv_module;
                 }
@@ -186,12 +196,16 @@ std::unique_ptr<const vku::safe_VkPipelineShaderStageCreateInfo> ToShaderStageCI
 }
 
 template <typename CreateInfo>
-void SetFragmentShaderInfoPrivate(FragmentShaderState &fs_state, const ValidationStateTracker &state_data,
-                                  const CreateInfo &create_info,
+void SetFragmentShaderInfoPrivate(const vvl::Pipeline &pipeline_state, FragmentShaderState &fs_state,
+                                  const ValidationStateTracker &state_data, const CreateInfo &create_info,
                                   spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages]) {
     for (uint32_t i = 0; i < create_info.stageCount; ++i) {
         if (create_info.pStages[i].stage == VK_SHADER_STAGE_FRAGMENT_BIT) {
             auto module_state = state_data.Get<vvl::ShaderModule>(create_info.pStages[i].module);
+            if (!module_state && pipeline_state.pipeline_cache) {
+                // Attempt to look up the pipeline cache for shader module data
+                module_state = pipeline_state.pipeline_cache->GetStageModule(pipeline_state, i);
+            }
             if (!module_state) {
                 // If module is null and there is a VkShaderModuleCreateInfo in the pNext chain of the stage info, then this
                 // module is part of a library and the state must be created
@@ -202,7 +216,7 @@ void SetFragmentShaderInfoPrivate(FragmentShaderState &fs_state, const Validatio
                         (stateless_data && i < kCommonMaxGraphicsShaderStages) ? &stateless_data[i] : nullptr;
                     auto spirv_module =
                         std::make_shared<spirv::Module>(shader_ci->codeSize, shader_ci->pCode, stateless_data_stage);
-                    module_state = std::make_shared<vvl::ShaderModule>(VK_NULL_HANDLE, spirv_module, 0);
+                    module_state = std::make_shared<vvl::ShaderModule>(VK_NULL_HANDLE, spirv_module);
                     if (stateless_data_stage) {
                         stateless_data_stage->pipeline_pnext_module = spirv_module;
                     }
@@ -232,17 +246,19 @@ void SetFragmentShaderInfoPrivate(FragmentShaderState &fs_state, const Validatio
 }
 
 // static
-void FragmentShaderState::SetFragmentShaderInfo(FragmentShaderState &fs_state, const ValidationStateTracker &state_data,
+void FragmentShaderState::SetFragmentShaderInfo(const vvl::Pipeline &pipeline_state, FragmentShaderState &fs_state,
+                                                const ValidationStateTracker &state_data,
                                                 const VkGraphicsPipelineCreateInfo &create_info,
                                                 spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages]) {
-    SetFragmentShaderInfoPrivate(fs_state, state_data, create_info, stateless_data);
+    SetFragmentShaderInfoPrivate(pipeline_state, fs_state, state_data, create_info, stateless_data);
 }
 
 // static
-void FragmentShaderState::SetFragmentShaderInfo(FragmentShaderState &fs_state, const ValidationStateTracker &state_data,
+void FragmentShaderState::SetFragmentShaderInfo(const vvl::Pipeline &pipeline_state, FragmentShaderState &fs_state,
+                                                const ValidationStateTracker &state_data,
                                                 const vku::safe_VkGraphicsPipelineCreateInfo &create_info,
                                                 spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages]) {
-    SetFragmentShaderInfoPrivate(fs_state, state_data, create_info, stateless_data);
+    SetFragmentShaderInfoPrivate(pipeline_state, fs_state, state_data, create_info, stateless_data);
 }
 
 FragmentShaderState::FragmentShaderState(const vvl::Pipeline &p, const ValidationStateTracker &dev_data,
