@@ -19,6 +19,7 @@
 
 #include "state_tracker/pipeline_layout_state.h"
 #include <vulkan/utility/vk_safe_struct.hpp>
+#include <vulkan/utility/vk_struct_helper.hpp>
 
 // Graphics pipeline sub-state as defined by VK_KHR_graphics_pipeline_library
 
@@ -52,17 +53,45 @@ struct PipelineSubState {
     VkPipelineLayoutCreateFlags PipelineLayoutCreateFlags() const;
 };
 
+struct VertexAttrState {
+    VertexAttrState(uint32_t index_, const VkVertexInputAttributeDescription *desc_) : index(index_) {
+        desc.location = desc_->location;
+        desc.binding = desc_->binding;
+        desc.format = desc_->format;
+        desc.offset = desc_->offset;
+    }
+
+    VertexAttrState(uint32_t index_, const VkVertexInputAttributeDescription2EXT *desc_) : index(index_), desc(desc_) {}
+
+    uint32_t index;  // Original index into the caller's pVertexAttributeDescriptions
+    vku::safe_VkVertexInputAttributeDescription2EXT desc;
+};
+
+struct VertexBindingState {
+    VertexBindingState(uint32_t index_, const VkVertexInputBindingDescription *desc_) : index(index_) {
+        desc.binding = desc_->binding;
+        desc.stride = desc_->stride;
+        desc.inputRate = desc_->inputRate;
+        desc.divisor = 1;
+    }
+
+    VertexBindingState(uint32_t index_, const VkVertexInputBindingDescription2EXT *desc_) : index(index_), desc(desc_) {}
+
+    // Original index into the caller's pVertexBindingDescriptions
+    uint32_t index;
+    vku::safe_VkVertexInputBindingDescription2EXT desc;
+    // Attributes for this binding, key is the location
+    vvl::unordered_map<uint32_t, VertexAttrState> locations;
+};
+
 struct VertexInputState : public PipelineSubState {
     VertexInputState(const vvl::Pipeline &p, const vku::safe_VkGraphicsPipelineCreateInfo &create_info);
 
     vku::safe_VkPipelineVertexInputStateCreateInfo *input_state = nullptr;
     vku::safe_VkPipelineInputAssemblyStateCreateInfo *input_assembly_state = nullptr;
 
-    std::vector<VkVertexInputBindingDescription> binding_descriptions;
-
-    vvl::unordered_map<uint32_t, uint32_t> binding_to_index_map;
-
-    std::vector<VkVertexInputAttributeDescription2EXT> vertex_attribute_descriptions;
+    // key is binding number
+    vvl::unordered_map<uint32_t, VertexBindingState> bindings;
 
     std::shared_ptr<VertexInputState> FromCreateInfo(const ValidationStateTracker &state,
                                                      const vku::safe_VkGraphicsPipelineCreateInfo &create_info);
@@ -113,20 +142,20 @@ std::unique_ptr<const vku::safe_VkPipelineShaderStageCreateInfo> ToShaderStageCI
 std::unique_ptr<const vku::safe_VkPipelineShaderStageCreateInfo> ToShaderStageCI(const VkPipelineShaderStageCreateInfo &cbs);
 
 struct FragmentShaderState : public PipelineSubState {
-    FragmentShaderState(const vvl::Pipeline &p, const ValidationStateTracker &dev_data, std::shared_ptr<const vvl::RenderPass> rp,
-                        uint32_t subpass, VkPipelineLayout layout);
+    FragmentShaderState(const vvl::Pipeline &pipeline_state, const ValidationStateTracker &dev_data,
+                        std::shared_ptr<const vvl::RenderPass> rp, uint32_t subpass, VkPipelineLayout layout);
 
     template <typename CreateInfo>
-    FragmentShaderState(const vvl::Pipeline &p, const ValidationStateTracker &dev_data, const CreateInfo &create_info,
+    FragmentShaderState(const vvl::Pipeline &pipeline_state, const ValidationStateTracker &dev_data, const CreateInfo &create_info,
                         std::shared_ptr<const vvl::RenderPass> rp, spirv::StatelessData *stateless_data)
-        : FragmentShaderState(p, dev_data, rp, create_info.subpass, create_info.layout) {
+        : FragmentShaderState(pipeline_state, dev_data, rp, create_info.subpass, create_info.layout) {
         if (create_info.pMultisampleState) {
             ms_state = ToSafeMultisampleState(*create_info.pMultisampleState);
         }
         if (create_info.pDepthStencilState) {
             ds_state = ToSafeDepthStencilState(*create_info.pDepthStencilState);
         }
-        FragmentShaderState::SetFragmentShaderInfo(*this, dev_data, create_info, stateless_data);
+        FragmentShaderState::SetFragmentShaderInfo(pipeline_state, *this, dev_data, create_info, stateless_data);
     }
 
     static inline VkShaderStageFlags ValidShaderStages() { return VK_SHADER_STAGE_FRAGMENT_BIT; }
@@ -144,10 +173,11 @@ struct FragmentShaderState : public PipelineSubState {
     std::shared_ptr<const spirv::EntryPoint> fragment_entry_point;
 
   private:
-    static void SetFragmentShaderInfo(FragmentShaderState &fs_state, const ValidationStateTracker &state_data,
-                                      const VkGraphicsPipelineCreateInfo &create_info,
+    static void SetFragmentShaderInfo(const vvl::Pipeline &pipeline_state, FragmentShaderState &fs_state,
+                                      const ValidationStateTracker &state_data, const VkGraphicsPipelineCreateInfo &create_info,
                                       spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages]);
-    static void SetFragmentShaderInfo(FragmentShaderState &fs_state, const ValidationStateTracker &state_data,
+    static void SetFragmentShaderInfo(const vvl::Pipeline &pipeline_state, FragmentShaderState &fs_state,
+                                      const ValidationStateTracker &state_data,
                                       const vku::safe_VkGraphicsPipelineCreateInfo &create_info,
                                       spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages]);
 };
@@ -193,6 +223,11 @@ struct FragmentOutputState : public PipelineSubState {
             sample_location_enabled = IsSampleLocationEnabled(create_info);
         }
 
+        const auto flags2 = vku::FindStructInPNextChain<VkPipelineCreateFlags2CreateInfoKHR>(create_info.pNext);
+        if (flags2) {
+            legacy_dithering_enabled = (flags2->flags & VK_PIPELINE_CREATE_2_ENABLE_LEGACY_DITHERING_BIT_EXT) != 0;
+        }
+
         // TODO
         // auto format_ci = vku::FindStructInPNextChain<VkPipelineRenderingFormatCreateInfoKHR>(gpci->pNext);
     }
@@ -207,6 +242,7 @@ struct FragmentOutputState : public PipelineSubState {
 
     AttachmentStateVector attachment_states;
 
+    bool legacy_dithering_enabled = false;
     bool blend_constants_enabled = false;  // Blend constants enabled for any attachments
     bool sample_location_enabled = false;
 };
