@@ -21,6 +21,8 @@
 #include "vulkan/vulkan.h"
 #include "containers/custom_containers.h"
 #include "utils/vk_layer_utils.h"
+#include "generated/vk_object_types.h"
+#include "error_message/logging.h"
 
 #include <atomic>
 
@@ -47,7 +49,6 @@ class StateObject: public std::enable_shared_from_this<StateObject>, public Type
     // parent types without locking every weak_ptr.
     using NodeMap = unordered_map<VulkanTypedHandle, std::weak_ptr<StateObject>>;
     using NodeList = small_vector<std::shared_ptr<StateObject>, 4, uint32_t>;
-    using IdType = uint32_t;
 
     template <typename Handle>
     StateObject(Handle h, VulkanObjectType t) : TypedHandleWrapper(h, t), destroyed_(false), id_(0) {}
@@ -70,8 +71,8 @@ class StateObject: public std::enable_shared_from_this<StateObject>, public Type
     // Some drivers may reuse vulkan handles, which can confuse some parts of validation that cache state.
     // Add a unique id to help detect this condition. SetId() should only be called by the state tracker during
     // object creation.
-    void SetId(IdType id) { id_ = id; }
-    IdType GetId() const { return id_; }
+    void SetId(uint32_t id) { id_ = id; }
+    uint32_t GetId() const { return id_; }
 
     // returns true if this vulkan object or any it uses have been destroyed
     virtual bool Invalid() const { return Destroyed(); }
@@ -114,7 +115,8 @@ class StateObject: public std::enable_shared_from_this<StateObject>, public Type
     // Set to true when the API-level object is destroyed, but this object may
     // hang around until its shared_ptr refcount goes to zero.
     std::atomic<bool> destroyed_;
-    IdType id_;
+    uint32_t id_;
+
   private:
     ReadLockGuard ReadLockTree() const { return ReadLockGuard(tree_lock_); }
     WriteLockGuard WriteLockTree() { return WriteLockGuard(tree_lock_); }
