@@ -1,5 +1,6 @@
-/* Copyright (c) 2022-2023 The Khronos Group Inc.
- * Copyright (c) 2022-2023 RasterGrid Kft.
+/* Copyright (c) 2022-2024 The Khronos Group Inc.
+ * Copyright (c) 2022-2024 RasterGrid Kft.
+ * Modifications Copyright (C) 2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,7 +18,7 @@
 
 #include "state_tracker/state_object.h"
 #include "utils/hash_util.h"
-#include "generated/vk_safe_struct.h"
+#include <vulkan/utility/vk_safe_struct.hpp>
 #include <memory>
 #include <mutex>
 #include <vector>
@@ -29,6 +30,25 @@ class Image;
 class ImageView;
 class VideoSession;
 
+struct QuantizationMapTexelSize {
+    struct compare {
+        bool operator()(VkExtent2D const &lhs, VkExtent2D const &rhs) const {
+            return lhs.width == rhs.width && lhs.height == rhs.height;
+        }
+    };
+
+    struct hash {
+        std::size_t operator()(VkExtent2D const &texel_size) const {
+            hash_util::HashCombiner hc;
+            hc << texel_size.width << texel_size.height;
+            return hc.Value();
+        }
+    };
+};
+
+using SupportedQuantizationMapTexelSizes =
+    unordered_set<VkExtent2D, QuantizationMapTexelSize::hash, QuantizationMapTexelSize::compare>;
+
 using SupportedVideoProfiles = unordered_set<std::shared_ptr<const class VideoProfileDesc>>;
 
 // The VideoProfileDesc contains the entire video profile description, which includes all
@@ -50,8 +70,10 @@ class VideoProfileDesc : public std::enable_shared_from_this<VideoProfileDesc> {
         union {
             VkVideoDecodeH264ProfileInfoKHR decode_h264;
             VkVideoDecodeH265ProfileInfoKHR decode_h265;
+            VkVideoDecodeAV1ProfileInfoKHR decode_av1;
             VkVideoEncodeH264ProfileInfoKHR encode_h264;
             VkVideoEncodeH265ProfileInfoKHR encode_h265;
+            VkVideoEncodeAV1ProfileInfoKHR encode_av1;
         };
     };
 
@@ -65,8 +87,17 @@ class VideoProfileDesc : public std::enable_shared_from_this<VideoProfileDesc> {
         union {
             VkVideoDecodeH264CapabilitiesKHR decode_h264;
             VkVideoDecodeH265CapabilitiesKHR decode_h265;
+            VkVideoDecodeAV1CapabilitiesKHR decode_av1;
             VkVideoEncodeH264CapabilitiesKHR encode_h264;
             VkVideoEncodeH265CapabilitiesKHR encode_h265;
+            VkVideoEncodeAV1CapabilitiesKHR encode_av1;
+        };
+        union {
+            struct {
+            } decode_ext;
+            struct {
+                VkVideoEncodeQuantizationMapCapabilitiesKHR quantization_map;
+            } encode_ext;
         };
     };
 
@@ -76,10 +107,14 @@ class VideoProfileDesc : public std::enable_shared_from_this<VideoProfileDesc> {
     const Profile &GetProfile() const { return profile_; }
     const Capabilities &GetCapabilities() const { return capabilities_; }
 
+    const SupportedQuantizationMapTexelSizes &GetSupportedQuantDeltaMapTexelSizes() const { return quant_delta_map_texel_sizes_; }
+    const SupportedQuantizationMapTexelSizes &GetSupportedEmphasisMapTexelSizes() const { return emphasis_map_texel_sizes_; }
+
     bool IsDecode() const { return profile_.is_decode; }
     bool IsEncode() const { return profile_.is_encode; }
     VkVideoCodecOperationFlagBitsKHR GetCodecOp() const { return profile_.base.videoCodecOperation; }
     VkVideoDecodeH264PictureLayoutFlagBitsKHR GetH264PictureLayout() const { return profile_.decode_h264.pictureLayout; }
+    bool HasAV1FilmGrainSupport() const { return profile_.decode_av1.filmGrainSupport; }
 
     VkExtent2D GetMaxCodingBlockSize() const {
         switch (profile_.base.videoCodecOperation) {
@@ -93,38 +128,17 @@ class VideoProfileDesc : public std::enable_shared_from_this<VideoProfileDesc> {
                 return {max_ctb_size, max_ctb_size};
             }
 
+            case VK_VIDEO_CODEC_OPERATION_ENCODE_AV1_BIT_KHR: {
+                uint32_t max_sb_size = 128 * static_cast<uint32_t>(log2(capabilities_.encode_av1.superblockSizes));
+                return {max_sb_size, max_sb_size};
+            }
+
             default:
                 assert(false);
                 return {0, 0};
         }
     }
 
-    VkOffset3D GetEffectiveImageOffset(VkOffset2D coded_offset) const {
-        VkOffset3D offset{coded_offset.x, coded_offset.y, 0};
-
-        // Round to picture access granularity
-        const auto gran = capabilities_.base.pictureAccessGranularity;
-        offset.x = (offset.x / gran.width) * gran.width;
-        offset.y = (offset.y / gran.height) * gran.height;
-        return offset;
-    }
-
-    VkExtent3D GetEffectiveImageExtent(VkExtent2D coded_extent) const {
-        VkExtent3D extent{coded_extent.width, coded_extent.height, 0};
-
-        // H.264 decode interlacing with separate planes only accesses half of the coded height
-        if (GetCodecOp() == VK_VIDEO_CODEC_OPERATION_DECODE_H264_BIT_KHR &&
-            GetH264PictureLayout() == VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_INTERLACED_SEPARATE_PLANES_BIT_KHR) {
-            extent.height /= 2;
-        }
-
-        // Round to picture access granularity
-        const auto gran = capabilities_.base.pictureAccessGranularity;
-        extent.width = ((extent.width + gran.width - 1) / gran.width) * gran.width;
-        extent.height = ((extent.height + gran.height - 1) / gran.height) * gran.height;
-        return extent;
-    }
-
     struct compare {
       public:
         bool operator()(VideoProfileDesc const *lhs, VideoProfileDesc const *rhs) const {
@@ -154,6 +168,11 @@ class VideoProfileDesc : public std::enable_shared_from_this<VideoProfileDesc> {
                         match = match && lhs->profile_.decode_h265.stdProfileIdc == rhs->profile_.decode_h265.stdProfileIdc;
                         break;
 
+                    case VK_VIDEO_CODEC_OPERATION_DECODE_AV1_BIT_KHR:
+                        match = match && lhs->profile_.decode_av1.stdProfile == rhs->profile_.decode_av1.stdProfile &&
+                                lhs->profile_.decode_av1.filmGrainSupport == rhs->profile_.decode_av1.filmGrainSupport;
+                        break;
+
                     case VK_VIDEO_CODEC_OPERATION_ENCODE_H264_BIT_KHR:
                         match = match && lhs->profile_.encode_h264.stdProfileIdc == rhs->profile_.encode_h264.stdProfileIdc;
                         break;
@@ -162,6 +181,10 @@ class VideoProfileDesc : public std::enable_shared_from_this<VideoProfileDesc> {
                         match = match && lhs->profile_.encode_h265.stdProfileIdc == rhs->profile_.encode_h265.stdProfileIdc;
                         break;
 
+                    case VK_VIDEO_CODEC_OPERATION_ENCODE_AV1_BIT_KHR:
+                        match = match && lhs->profile_.encode_av1.stdProfile == rhs->profile_.encode_av1.stdProfile;
+                        break;
+
                     default:
                         break;
                 }
@@ -196,6 +219,10 @@ class VideoProfileDesc : public std::enable_shared_from_this<VideoProfileDesc> {
                     hc << desc->profile_.decode_h265.stdProfileIdc;
                     break;
                 }
+                case VK_VIDEO_CODEC_OPERATION_DECODE_AV1_BIT_KHR: {
+                    hc << desc->profile_.decode_av1.stdProfile << desc->profile_.decode_av1.filmGrainSupport;
+                    break;
+                }
                 case VK_VIDEO_CODEC_OPERATION_ENCODE_H264_BIT_KHR: {
                     hc << desc->profile_.encode_h264.stdProfileIdc;
                     break;
@@ -204,6 +231,10 @@ class VideoProfileDesc : public std::enable_shared_from_this<VideoProfileDesc> {
                     hc << desc->profile_.encode_h265.stdProfileIdc;
                     break;
                 }
+                case VK_VIDEO_CODEC_OPERATION_ENCODE_AV1_BIT_KHR: {
+                    hc << desc->profile_.encode_av1.stdProfile;
+                    break;
+                }
                 default:
                     break;
             }
@@ -238,10 +269,13 @@ class VideoProfileDesc : public std::enable_shared_from_this<VideoProfileDesc> {
     VkPhysicalDevice physical_device_;
     Profile profile_;
     Capabilities capabilities_;
+    SupportedQuantizationMapTexelSizes quant_delta_map_texel_sizes_;
+    SupportedQuantizationMapTexelSizes emphasis_map_texel_sizes_;
     Cache *cache_;
 
     bool InitProfile(VkVideoProfileInfoKHR const *profile);
     void InitCapabilities(VkPhysicalDevice physical_device);
+    void InitQuantizationMapFormats(VkPhysicalDevice physical_device);
 };
 
 class VideoPictureResource {
@@ -254,7 +288,7 @@ class VideoPictureResource {
     VkExtent2D coded_extent;
 
     VideoPictureResource();
-    VideoPictureResource(ValidationStateTracker const *dev_data, VkVideoPictureResourceInfoKHR const &res);
+    VideoPictureResource(const ValidationStateTracker &dev_data, VkVideoPictureResourceInfoKHR const &res);
 
     operator bool() const { return image_view_state != nullptr; }
 
@@ -282,6 +316,9 @@ class VideoPictureResource {
         }
     };
 
+    VkOffset3D GetEffectiveImageOffset(const vvl::VideoSession &vs_state) const;
+    VkExtent3D GetEffectiveImageExtent(const vvl::VideoSession &vs_state) const;
+
   private:
     VkImageSubresourceRange GetImageSubresourceRange(ImageView const *image_view_state, uint32_t layer);
 };
@@ -341,7 +378,7 @@ struct VideoReferenceSlot {
 
     VideoReferenceSlot() : index(-1), picture_id(), resource() {}
 
-    VideoReferenceSlot(ValidationStateTracker const *dev_data, VideoProfileDesc const &profile,
+    VideoReferenceSlot(const ValidationStateTracker &dev_data, VideoProfileDesc const &profile,
                        VkVideoReferenceSlotInfoKHR const &slot, bool has_picture_id = true)
         : index(slot.slotIndex),
           picture_id(has_picture_id ? VideoPictureID(profile, slot) : VideoPictureID()),
@@ -356,6 +393,7 @@ struct VideoEncodeRateControlLayerState {
     union {
         VkVideoEncodeH264RateControlLayerInfoKHR h264;
         VkVideoEncodeH265RateControlLayerInfoKHR h265;
+        VkVideoEncodeAV1RateControlLayerInfoKHR av1;
     };
 
     VideoEncodeRateControlLayerState(VkVideoCodecOperationFlagBitsKHR op = VK_VIDEO_CODEC_OPERATION_NONE_KHR,
@@ -375,6 +413,12 @@ struct VideoEncodeRateControlLayerState {
                 break;
             }
 
+            case VK_VIDEO_CODEC_OPERATION_ENCODE_AV1_BIT_KHR: {
+                auto av1_info = vku::FindStructInPNextChain<VkVideoEncodeAV1RateControlLayerInfoKHR>(info);
+                av1 = av1_info ? *av1_info : vku::InitStructHelper();
+                break;
+            }
+
             default:
                 assert(false);
                 break;
@@ -388,6 +432,7 @@ struct VideoEncodeRateControlState {
     union {
         VkVideoEncodeH264RateControlInfoKHR h264;
         VkVideoEncodeH265RateControlInfoKHR h265;
+        VkVideoEncodeAV1RateControlInfoKHR av1;
     };
     std::vector<VideoEncodeRateControlLayerState> layers;
 
@@ -415,6 +460,12 @@ struct VideoEncodeRateControlState {
                 break;
             }
 
+            case VK_VIDEO_CODEC_OPERATION_ENCODE_AV1_BIT_KHR: {
+                auto av1_info = vku::FindStructInPNextChain<VkVideoEncodeAV1RateControlInfoKHR>(info);
+                av1 = av1_info ? *av1_info : vku::InitStructHelper();
+                break;
+            }
+
             default:
                 break;
         }
@@ -438,13 +489,27 @@ class VideoSessionDeviceState {
           encode_() {}
 
     bool IsInitialized() const { return initialized_; }
-    bool IsSlotActive(int32_t slot_index) const { return is_active_[slot_index]; }
+    bool IsSlotActive(int32_t slot_index) const {
+        if (slot_index < 0 || static_cast<uint32_t>(slot_index) >= is_active_.size()) {
+            // Out-of-bounds slot index
+            return false;
+        }
+        return is_active_[slot_index];
+    }
 
     bool IsSlotPicture(int32_t slot_index, const VideoPictureResource &res) const {
+        if (slot_index < 0 || static_cast<uint32_t>(slot_index) >= all_pictures_.size()) {
+            // Out-of-bounds slot index
+            return false;
+        }
         return all_pictures_[slot_index].find(res) != all_pictures_[slot_index].end();
     }
 
     bool IsSlotPicture(int32_t slot_index, const VideoPictureID &picture_id, const VideoPictureResource &res) const {
+        if (slot_index < 0 || static_cast<uint32_t>(slot_index) >= pictures_per_id_.size()) {
+            // Out-of-bounds slot index
+            return false;
+        }
         auto it = pictures_per_id_[slot_index].find(picture_id);
         return it != pictures_per_id_[slot_index].end() && it->second == res;
     }
@@ -462,8 +527,8 @@ class VideoSessionDeviceState {
         encode_.rate_control_state = rate_control_state;
     }
 
-    bool ValidateRateControlState(const ValidationStateTracker *dev_data, const VideoSession *vs_state,
-                                  const safe_VkVideoBeginCodingInfoKHR &begin_info) const;
+    bool ValidateRateControlState(const ValidationStateTracker &dev_data, const VideoSession *vs_state,
+                                  const vku::safe_VkVideoBeginCodingInfoKHR &begin_info, const Location &loc) const;
 
   private:
     bool initialized_;
@@ -485,7 +550,8 @@ class VideoSession : public StateObject {
     };
     using MemoryBindingMap = unordered_map<uint32_t, MemoryBindingInfo>;
 
-    const safe_VkVideoSessionCreateInfoKHR create_info;
+    const vku::safe_VkVideoSessionCreateInfoKHR safe_create_info;
+    const VkVideoSessionCreateInfoKHR &create_info;
     std::shared_ptr<const VideoProfileDesc> profile;
     bool memory_binding_count_queried;
     uint32_t memory_bindings_queried;
@@ -500,10 +566,10 @@ class VideoSession : public StateObject {
         VideoSessionDeviceState &state_;
     };
 
-    VideoSession(ValidationStateTracker *dev_data, VkVideoSessionKHR vs, VkVideoSessionCreateInfoKHR const *pCreateInfo,
+    VideoSession(const ValidationStateTracker &dev_data, VkVideoSessionKHR handle, VkVideoSessionCreateInfoKHR const *pCreateInfo,
                  std::shared_ptr<const VideoProfileDesc> &&profile_desc);
 
-    VkVideoSessionKHR videoSession() const { return handle_.Cast<VkVideoSessionKHR>(); }
+    VkVideoSessionKHR VkHandle() const { return handle_.Cast<VkVideoSessionKHR>(); }
     bool IsDecode() const { return profile->IsDecode(); }
     bool IsEncode() const { return profile->IsEncode(); }
     VkVideoCodecOperationFlagBitsKHR GetCodecOp() const { return profile->GetCodecOp(); }
@@ -544,7 +610,7 @@ class VideoSession : public StateObject {
     bool ReferenceSetupRequested(VkVideoEncodeInfoKHR const &encode_info) const;
 
   private:
-    MemoryBindingMap GetMemoryBindings(ValidationStateTracker *dev_data, VkVideoSessionKHR vs);
+    MemoryBindingMap GetMemoryBindings(const ValidationStateTracker &dev_data, VkVideoSessionKHR vs);
 
     MemoryBindingMap memory_bindings_;
     uint32_t unbound_memory_binding_count_;
@@ -578,14 +644,20 @@ class VideoSessionParameters : public StateObject {
         uint32_t pps_capacity;
     };
 
+    struct AV1Parameters {
+        std::unique_ptr<StdVideoAV1SequenceHeader> seq_header;
+    };
+
     struct Data {
         uint32_t update_sequence_counter;
         H264Parameters h264;
         H265Parameters h265;
+        AV1Parameters av1;
     };
 
     struct EncodeConfig {
         uint32_t quality_level{0};
+        VkExtent2D quantization_map_texel_size{0, 0};
     };
 
     struct Config {
@@ -669,18 +741,21 @@ class VideoSessionParameters : public StateObject {
             }
         }
 
+        const StdVideoAV1SequenceHeader *GetAV1SequenceHeader() const { return data_->av1.seq_header.get(); }
+
       private:
         std::unique_lock<std::mutex> lock_;
         const Data *data_;
     };
 
-    const safe_VkVideoSessionParametersCreateInfoKHR create_info;
+    const vku::safe_VkVideoSessionParametersCreateInfoKHR safe_create_info;
+    const VkVideoSessionParametersCreateInfoKHR &create_info;
     std::shared_ptr<const VideoSession> vs_state;
 
-    VideoSessionParameters(VkVideoSessionParametersKHR vsp, VkVideoSessionParametersCreateInfoKHR const *pCreateInfo,
+    VideoSessionParameters(VkVideoSessionParametersKHR handle, VkVideoSessionParametersCreateInfoKHR const *pCreateInfo,
                            std::shared_ptr<VideoSession> &&vsstate, std::shared_ptr<VideoSessionParameters> &&vsp_template);
 
-    VkVideoSessionParametersKHR videoSessionParameters() const { return handle_.Cast<VkVideoSessionParametersKHR>(); }
+    VkVideoSessionParametersKHR VkHandle() const { return handle_.Cast<VkVideoSessionParametersKHR>(); }
 
     bool IsDecode() const { return vs_state->IsDecode(); }
     bool IsEncode() const { return vs_state->IsEncode(); }
@@ -688,6 +763,7 @@ class VideoSessionParameters : public StateObject {
     VkVideoCodecOperationFlagBitsKHR GetCodecOp() const { return vs_state->GetCodecOp(); }
 
     uint32_t GetEncodeQualityLevel() const { return config_.encode.quality_level; }
+    VkExtent2D GetEncodeQuantizationMapTexelSize() const { return config_.encode.quantization_map_texel_size; }
 
     void Update(VkVideoSessionParametersUpdateInfoKHR const *info);
 
@@ -710,7 +786,7 @@ class VideoSessionParameters : public StateObject {
     void AddEncodeH265(VkVideoEncodeH265SessionParametersAddInfoKHR const *info);
 };
 
-using VideoSessionUpdateList = std::vector<std::function<bool(const ValidationStateTracker *dev_data, const VideoSession *vs_state,
+using VideoSessionUpdateList = std::vector<std::function<bool(const ValidationStateTracker &dev_data, const VideoSession *vs_state,
                                                               VideoSessionDeviceState &dev_state, bool do_validate)>>;
 using VideoSessionUpdateMap = unordered_map<VkVideoSessionKHR, VideoSessionUpdateList>;
 
