@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (C) 2015-2023 Google Inc.
+/* Copyright (c) 2015-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (C) 2015-2024 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,13 +17,14 @@
  */
 
 #include "stateless/stateless_validation.h"
+#include "generated/dispatch_functions.h"
 
 bool StatelessValidation::manual_PreCallValidateCmdSetViewportWithCount(VkCommandBuffer commandBuffer, uint32_t viewportCount,
                                                                         const VkViewport *pViewports,
                                                                         const ErrorObject &error_obj) const {
     bool skip = false;
 
-    if (!physical_device_features.multiViewport) {
+    if (!enabled_features.multiViewport) {
         if (viewportCount != 1) {
             skip |= LogError("VUID-vkCmdSetViewportWithCount-viewportCount-03395", commandBuffer,
                              error_obj.location.dot(Field::viewportCount),
@@ -35,7 +36,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetViewportWithCount(VkComman
                              error_obj.location.dot(Field::viewportCount),
                              "(%" PRIu32
                              ") must "
-                             "not be greater than VkPhysicalDeviceLimits::maxViewports (=%" PRIu32 ").",
+                             "not be greater than VkPhysicalDeviceLimits::maxViewports (%" PRIu32 ").",
                              viewportCount, device_limits.maxViewports);
         }
     }
@@ -54,7 +55,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetScissorWithCount(VkCommand
                                                                        const VkRect2D *pScissors,
                                                                        const ErrorObject &error_obj) const {
     bool skip = false;
-    if (!physical_device_features.multiViewport) {
+    if (!enabled_features.multiViewport) {
         if (scissorCount != 1) {
             skip |= LogError("VUID-vkCmdSetScissorWithCount-scissorCount-03398", commandBuffer,
                              error_obj.location.dot(Field::scissorCount),
@@ -120,8 +121,6 @@ bool StatelessValidation::manual_PreCallValidateCmdSetVertexInputEXT(
     const VkVertexInputBindingDescription2EXT *pVertexBindingDescriptions, uint32_t vertexAttributeDescriptionCount,
     const VkVertexInputAttributeDescription2EXT *pVertexAttributeDescriptions, const ErrorObject &error_obj) const {
     bool skip = false;
-    const auto *vertex_attribute_divisor_features =
-        vku::FindStructInPNextChain<VkPhysicalDeviceVertexAttributeDivisorFeaturesEXT>(device_createinfo_pnext);
 
     if (vertexBindingDescriptionCount > device_limits.maxVertexInputBindings) {
         skip |= LogError("VUID-vkCmdSetVertexInputEXT-vertexBindingDescriptionCount-04791", commandBuffer,
@@ -195,8 +194,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetVertexInputEXT(
                              pVertexBindingDescriptions[binding].stride, device_limits.maxVertexInputBindingStride);
         }
 
-        if (pVertexBindingDescriptions[binding].divisor == 0 &&
-            (!vertex_attribute_divisor_features || !vertex_attribute_divisor_features->vertexAttributeInstanceRateZeroDivisor)) {
+        if (pVertexBindingDescriptions[binding].divisor == 0 && (!enabled_features.vertexAttributeInstanceRateZeroDivisor)) {
             skip |=
                 LogError("VUID-VkVertexInputBindingDescription2EXT-divisor-04798", commandBuffer, binding_loc.dot(Field::divisor),
                          "is zero but "
@@ -204,7 +202,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetVertexInputEXT(
         }
 
         if (pVertexBindingDescriptions[binding].divisor > 1) {
-            if (!vertex_attribute_divisor_features || !vertex_attribute_divisor_features->vertexAttributeInstanceRateDivisor) {
+            if (!enabled_features.vertexAttributeInstanceRateDivisor) {
                 skip |= LogError("VUID-VkVertexInputBindingDescription2EXT-divisor-04799", commandBuffer,
                                  binding_loc.dot(Field::divisor),
                                  "is %" PRIu32
@@ -259,8 +257,8 @@ bool StatelessValidation::manual_PreCallValidateCmdSetVertexInputEXT(
         if ((properties.bufferFeatures & VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT) == 0) {
             skip |=
                 LogError("VUID-VkVertexInputAttributeDescription2EXT-format-04805", commandBuffer, attribute_loc.dot(Field::format),
-                         "(%s) is not a "
-                         "VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT supported format. (supported bufferFeatures: %s)",
+                         "(%s) doesn't support VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT.\n"
+                         "(supported bufferFeatures: %s)",
                          string_VkFormat(pVertexAttributeDescriptions[attribute].format),
                          string_VkFormatFeatureFlags2(properties.bufferFeatures).c_str());
         }
@@ -285,7 +283,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetDiscardRectangleEXT(VkComm
             static_cast<int64_t>(pDiscardRectangles[i].offset.x) + static_cast<int64_t>(pDiscardRectangles[i].extent.width);
         if (x_sum > std::numeric_limits<int32_t>::max()) {
             skip |= LogError("VUID-vkCmdSetDiscardRectangleEXT-offset-00588", commandBuffer, loc,
-                             "offset.x (%" PRId32 ") + extent.width (%" PRIu32 ") is %" PRIi64 ") which will overflow int32_t.",
+                             "offset.x (%" PRId32 ") + extent.width (%" PRIu32 ") is %" PRIi64 " which will overflow int32_t.",
                              pDiscardRectangles[i].offset.x, pDiscardRectangles[i].extent.width, x_sum);
         }
 
@@ -293,7 +291,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetDiscardRectangleEXT(VkComm
             static_cast<int64_t>(pDiscardRectangles[i].offset.y) + static_cast<int64_t>(pDiscardRectangles[i].extent.height);
         if (y_sum > std::numeric_limits<int32_t>::max()) {
             skip |= LogError("VUID-vkCmdSetDiscardRectangleEXT-offset-00589", commandBuffer, loc,
-                             "offset.y (%" PRId32 ") + extent.height (%" PRIu32 ") is %" PRIi64 ") which will overflow int32_t.",
+                             "offset.y (%" PRId32 ") + extent.height (%" PRIu32 ") is %" PRIi64 " which will overflow int32_t.",
                              pDiscardRectangles[i].offset.y, pDiscardRectangles[i].extent.height, y_sum);
         }
     }
@@ -343,7 +341,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetExclusiveScissorNV(VkComma
                                                                          const ErrorObject &error_obj) const {
     bool skip = false;
 
-    if (!physical_device_features.multiViewport) {
+    if (!enabled_features.multiViewport) {
         if (firstExclusiveScissor != 0) {
             skip |= LogError("VUID-vkCmdSetExclusiveScissorNV-firstExclusiveScissor-02035", commandBuffer,
                              error_obj.location.dot(Field::firstExclusiveScissor),
@@ -358,8 +356,8 @@ bool StatelessValidation::manual_PreCallValidateCmdSetExclusiveScissorNV(VkComma
         const uint64_t sum = static_cast<uint64_t>(firstExclusiveScissor) + static_cast<uint64_t>(exclusiveScissorCount);
         if (sum > device_limits.maxViewports) {
             skip |= LogError("VUID-vkCmdSetExclusiveScissorNV-firstExclusiveScissor-02034", commandBuffer, error_obj.location,
-                             "firstExclusiveScissor + exclusiveScissorCount (=%" PRIu32 " + %" PRIu32 " = %" PRIu64
-                             ") is greater than VkPhysicalDeviceLimits::maxViewports (=%" PRIu32 ").",
+                             "firstExclusiveScissor (%" PRIu32 ") + exclusiveScissorCount (%" PRIu32 ") is %" PRIu64
+                             " which is greater than VkPhysicalDeviceLimits::maxViewports (%" PRIu32 ").",
                              firstExclusiveScissor, exclusiveScissorCount, sum, device_limits.maxViewports);
         }
     }
@@ -406,8 +404,8 @@ bool StatelessValidation::manual_PreCallValidateCmdSetViewportWScalingNV(VkComma
     const uint64_t sum = static_cast<uint64_t>(firstViewport) + static_cast<uint64_t>(viewportCount);
     if ((sum < 1) || (sum > device_limits.maxViewports)) {
         skip |= LogError("VUID-vkCmdSetViewportWScalingNV-firstViewport-01324", commandBuffer, error_obj.location,
-                         "firstViewport + viewportCount (=%" PRIu32 " + %" PRIu32 " = %" PRIu64
-                         ") must be between 1 and VkPhysicalDeviceLimits::maxViewports (=%" PRIu32 "), inculsive.",
+                         "firstViewport (%" PRIu32 ") + viewportCount (%" PRIu32 ") is %" PRIu64
+                         ", which must be between 1 and VkPhysicalDeviceLimits::maxViewports (%" PRIu32 "), inculsive.",
                          firstViewport, viewportCount, sum, device_limits.maxViewports);
     }
 
@@ -419,7 +417,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetViewportShadingRatePalette
     const VkShadingRatePaletteNV *pShadingRatePalettes, const ErrorObject &error_obj) const {
     bool skip = false;
 
-    if (!physical_device_features.multiViewport) {
+    if (!enabled_features.multiViewport) {
         if (firstViewport != 0) {
             skip |= LogError("VUID-vkCmdSetViewportShadingRatePaletteNV-firstViewport-02068", commandBuffer,
                              error_obj.location.dot(Field::firstViewport),
@@ -435,8 +433,8 @@ bool StatelessValidation::manual_PreCallValidateCmdSetViewportShadingRatePalette
     const uint64_t sum = static_cast<uint64_t>(firstViewport) + static_cast<uint64_t>(viewportCount);
     if (sum > device_limits.maxViewports) {
         skip |= LogError("VUID-vkCmdSetViewportShadingRatePaletteNV-firstViewport-02067", commandBuffer, error_obj.location,
-                         "firstViewport + viewportCount (=%" PRIu32 " + %" PRIu32 " = %" PRIu64
-                         ") is greater than VkPhysicalDeviceLimits::maxViewports (=%" PRIu32 ").",
+                         "firstViewport (%" PRIu32 ") + viewportCount (%" PRIu32 ") is %" PRIu64
+                         ") which is greater than VkPhysicalDeviceLimits::maxViewports (%" PRIu32 ").",
                          firstViewport, viewportCount, sum, device_limits.maxViewports);
     }
 
@@ -457,7 +455,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetCoarseSampleOrderNV(VkComm
     }
 
     for (uint32_t order_i = 0; order_i < customSampleOrderCount; ++order_i) {
-        skip |= ValidateCoarseSampleOrderCustomNV(&pCustomSampleOrders[order_i],
+        skip |= ValidateCoarseSampleOrderCustomNV(pCustomSampleOrders[order_i],
                                                   error_obj.location.dot(Field::pCustomSampleOrders, order_i));
     }
 
@@ -469,7 +467,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetViewport(VkCommandBuffer c
                                                                const ErrorObject &error_obj) const {
     bool skip = false;
 
-    if (!physical_device_features.multiViewport) {
+    if (!enabled_features.multiViewport) {
         if (firstViewport != 0) {
             skip |=
                 LogError("VUID-vkCmdSetViewport-firstViewport-01224", commandBuffer, error_obj.location.dot(Field::firstViewport),
@@ -500,12 +498,28 @@ bool StatelessValidation::manual_PreCallValidateCmdSetViewport(VkCommandBuffer c
     return skip;
 }
 
+bool StatelessValidation::manual_PreCallValidateCmdSetDepthClampRangeEXT(VkCommandBuffer commandBuffer,
+                                                                         VkDepthClampModeEXT depthClampMode,
+                                                                         const VkDepthClampRangeEXT *pDepthClampRange,
+                                                                         const ErrorObject &error_obj) const {
+    bool skip = false;
+    if (depthClampMode == VK_DEPTH_CLAMP_MODE_USER_DEFINED_RANGE_EXT) {
+        if (!pDepthClampRange) {
+            skip |= LogError("VUID-vkCmdSetDepthClampRangeEXT-pDepthClampRange-09647", device,
+                             error_obj.location.dot(Field::pDepthClampRange), "is NULL.");
+        } else {
+            skip |= ValidateDepthClampRange(*pDepthClampRange, error_obj.location.dot(Field::pDepthClampRange));
+        }
+    }
+    return skip;
+}
+
 bool StatelessValidation::manual_PreCallValidateCmdSetScissor(VkCommandBuffer commandBuffer, uint32_t firstScissor,
                                                               uint32_t scissorCount, const VkRect2D *pScissors,
                                                               const ErrorObject &error_obj) const {
     bool skip = false;
 
-    if (!physical_device_features.multiViewport) {
+    if (!enabled_features.multiViewport) {
         if (firstScissor != 0) {
             skip |= LogError("VUID-vkCmdSetScissor-firstScissor-00593", commandBuffer, error_obj.location.dot(Field::firstScissor),
                              "is %" PRIu32 " but the multiViewport feature was not enabled.", firstScissor);
@@ -562,7 +576,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetLineWidth(VkCommandBuffer
                                                                 const ErrorObject &error_obj) const {
     bool skip = false;
 
-    if (!physical_device_features.wideLines && (lineWidth != 1.0f)) {
+    if (!enabled_features.wideLines && (lineWidth != 1.0f)) {
         skip |= LogError("VUID-vkCmdSetLineWidth-lineWidth-00788", commandBuffer, error_obj.location.dot(Field::lineWidth),
                          "is %f (not 1.0), but wideLines was not enabled.", lineWidth);
     }
@@ -570,13 +584,12 @@ bool StatelessValidation::manual_PreCallValidateCmdSetLineWidth(VkCommandBuffer
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetLineStippleEXT(VkCommandBuffer commandBuffer, uint32_t lineStippleFactor,
-                                                                     uint16_t lineStipplePattern,
-                                                                     const ErrorObject &error_obj) const {
+bool StatelessValidation::manual_PreCallValidateCmdSetLineStipple(VkCommandBuffer commandBuffer, uint32_t lineStippleFactor,
+                                                                  uint16_t lineStipplePattern, const ErrorObject &error_obj) const {
     bool skip = false;
 
     if (lineStippleFactor < 1 || lineStippleFactor > 256) {
-        skip |= LogError("VUID-vkCmdSetLineStippleEXT-lineStippleFactor-02776", commandBuffer,
+        skip |= LogError("VUID-vkCmdSetLineStipple-lineStippleFactor-02776", commandBuffer,
                          error_obj.location.dot(Field::lineStippleFactor), "%" PRIu32 " is not in [1,256].", lineStippleFactor);
     }
 
