@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (C) 2015-2023 Google Inc.
+/* Copyright (c) 2015-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (C) 2015-2024 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,15 +18,18 @@
 
 #include "stateless/stateless_validation.h"
 #include "generated/enum_flag_bits.h"
+#include "generated/dispatch_functions.h"
 
 bool StatelessValidation::manual_PreCallValidateGetMemoryFdKHR(VkDevice device, const VkMemoryGetFdInfoKHR *pGetFdInfo, int *pFd,
                                                                const ErrorObject &error_obj) const {
     constexpr auto allowed_types = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT | VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT;
     bool skip = false;
     if (0 == (pGetFdInfo->handleType & allowed_types)) {
-        skip |= LogError("VUID-VkMemoryGetFdInfoKHR-handleType-00672", pGetFdInfo->memory, error_obj.location,
-                         "handle type %s is not one of the supported handle types.",
-                         string_VkExternalMemoryHandleTypeFlagBits(pGetFdInfo->handleType));
+        skip |= LogError("VUID-VkMemoryGetFdInfoKHR-handleType-00672", pGetFdInfo->memory,
+                         error_obj.location.dot(Field::pGetFdInfo).dot(Field::handleType),
+                         "(%s) is not one of the supported handle types (%s).",
+                         string_VkExternalMemoryHandleTypeFlagBits(pGetFdInfo->handleType),
+                         string_VkExternalMemoryHandleTypeFlags(allowed_types).c_str());
     }
     return skip;
 }
@@ -37,12 +40,12 @@ bool StatelessValidation::manual_PreCallValidateGetMemoryFdPropertiesKHR(VkDevic
                                                                          const ErrorObject &error_obj) const {
     bool skip = false;
     if (fd < 0) {
-        skip |= LogError("VUID-vkGetMemoryFdPropertiesKHR-fd-00673", device, error_obj.location,
-                         "fd handle (%d) is not a valid POSIX file descriptor.", fd);
+        skip |= LogError("VUID-vkGetMemoryFdPropertiesKHR-fd-00673", device, error_obj.location.dot(Field::fd),
+                         "handle (%d) is not a valid POSIX file descriptor.", fd);
     }
     if (handleType == VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT) {
-        skip |= LogError("VUID-vkGetMemoryFdPropertiesKHR-handleType-00674", device, error_obj.location,
-                         "opaque handle type %s is not allowed.", string_VkExternalMemoryHandleTypeFlagBits(handleType));
+        skip |= LogError("VUID-vkGetMemoryFdPropertiesKHR-handleType-00674", device, error_obj.location.dot(Field::handleType),
+                         "(VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT) is not allowed.");
     }
     return skip;
 }
@@ -96,8 +99,8 @@ bool StatelessValidation::manual_PreCallValidateImportSemaphoreFdKHR(VkDevice de
         skip |= LogError("VUID-VkImportSemaphoreFdInfoKHR-handleType-07307", pImportSemaphoreFdInfo->semaphore,
                          info_loc.dot(Field::handleType),
                          "is VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_SYNC_FD_BIT so"
-                         " VK_SEMAPHORE_IMPORT_TEMPORARY_BIT must be set, but flags is 0x%x",
-                         pImportSemaphoreFdInfo->flags);
+                         " VK_SEMAPHORE_IMPORT_TEMPORARY_BIT must be set, but flags is %s",
+                         string_VkSemaphoreImportFlags(pImportSemaphoreFdInfo->flags).c_str());
     }
     return skip;
 }
@@ -123,8 +126,8 @@ bool StatelessValidation::manual_PreCallValidateImportFenceFdKHR(VkDevice device
         (pImportFenceFdInfo->flags & VK_FENCE_IMPORT_TEMPORARY_BIT) == 0) {
         skip |= LogError("VUID-VkImportFenceFdInfoKHR-handleType-07306", pImportFenceFdInfo->fence, info_loc.dot(Field::handleType),
                          "is VK_EXTERNAL_FENCE_HANDLE_TYPE_SYNC_FD_BIT so"
-                         " VK_FENCE_IMPORT_TEMPORARY_BIT must be set, but flags is 0x%x",
-                         pImportFenceFdInfo->flags);
+                         " VK_FENCE_IMPORT_TEMPORARY_BIT must be set, but flags is %s",
+                         string_VkFenceImportFlags(pImportFenceFdInfo->flags).c_str());
     }
     return skip;
 }
@@ -164,9 +167,11 @@ bool StatelessValidation::manual_PreCallValidateGetMemoryWin32HandleKHR(VkDevice
         VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT | VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_TEXTURE_KMT_BIT;
     bool skip = false;
     if ((pGetWin32HandleInfo->handleType & (nt_handles | global_share_handles)) == 0) {
-        skip |= LogError("VUID-VkMemoryGetWin32HandleInfoKHR-handleType-00664", pGetWin32HandleInfo->memory, error_obj.location,
-                         "handle type %s is not one of the supported handle types.",
-                         string_VkExternalMemoryHandleTypeFlagBits(pGetWin32HandleInfo->handleType));
+        skip |= LogError("VUID-VkMemoryGetWin32HandleInfoKHR-handleType-00664", pGetWin32HandleInfo->memory,
+                         error_obj.location.dot(Field::pGetWin32HandleInfo).dot(Field::handleType),
+                         "(%s) is not one of the supported handle types (%s).",
+                         string_VkExternalMemoryHandleTypeFlagBits(pGetWin32HandleInfo->handleType),
+                         string_VkExternalMemoryHandleTypeFlags(nt_handles | global_share_handles).c_str());
     }
     return skip;
 }
@@ -278,24 +283,6 @@ bool StatelessValidation::ExportMetalObjectsPNextUtil(VkExportMetalObjectTypeFla
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateExportMetalObjectsEXT(VkDevice device,
-                                                                      VkExportMetalObjectsInfoEXT *pMetalObjectsInfo,
-                                                                      const ErrorObject &error_obj) const {
-    bool skip = false;
-
-    static_assert(AllVkExportMetalObjectTypeFlagBitsEXT == 0x3F, "Add new ExportMetalObjects support to VVL!");
-
-    constexpr std::array allowed_structs = {
-        VK_STRUCTURE_TYPE_EXPORT_METAL_BUFFER_INFO_EXT,       VK_STRUCTURE_TYPE_EXPORT_METAL_COMMAND_QUEUE_INFO_EXT,
-        VK_STRUCTURE_TYPE_EXPORT_METAL_DEVICE_INFO_EXT,       VK_STRUCTURE_TYPE_EXPORT_METAL_IO_SURFACE_INFO_EXT,
-        VK_STRUCTURE_TYPE_EXPORT_METAL_SHARED_EVENT_INFO_EXT, VK_STRUCTURE_TYPE_EXPORT_METAL_TEXTURE_INFO_EXT,
-    };
-    skip |=
-        ValidateStructPnext(error_obj.location.dot(Field::pMetalObjectsInfo), pMetalObjectsInfo->pNext, allowed_structs.size(),
-                            allowed_structs.data(), GeneratedVulkanHeaderVersion, "VUID-VkExportMetalObjectsInfoEXT-pNext-pNext",
-                            "VUID-VkExportMetalObjectsInfoEXT-sType-unique", false, true);
-    return skip;
-}
 #endif  // VK_USE_PLATFORM_METAL_EXT
 
 namespace {
@@ -380,19 +367,19 @@ ExternalOperationsInfo GetExternalOperationsInfo(const void *pNext) {
 }
 }  // namespace
 
-bool StatelessValidation::ValidateAllocateMemoryExternal(VkDevice device, const VkMemoryAllocateInfo *pAllocateInfo,
+bool StatelessValidation::ValidateAllocateMemoryExternal(VkDevice device, const VkMemoryAllocateInfo &allocate_info,
                                                          VkMemoryAllocateFlags flags, const Location &allocate_info_loc) const {
     bool skip = false;
 
     // Used to remove platform ifdef logic below
-    const ExternalOperationsInfo ext = GetExternalOperationsInfo(pAllocateInfo->pNext);
+    const ExternalOperationsInfo ext = GetExternalOperationsInfo(allocate_info.pNext);
 
-    if (!ext.has_export && ext.total_import_ops == 0 && pAllocateInfo->allocationSize == 0) {
+    if (!ext.has_export && ext.total_import_ops == 0 && allocate_info.allocationSize == 0) {
         skip |= LogError("VUID-VkMemoryAllocateInfo-allocationSize-07897", device, allocate_info_loc.dot(Field::allocationSize),
                          "is 0.");
     }
 
-    auto opaque_alloc_info = vku::FindStructInPNextChain<VkMemoryOpaqueCaptureAddressAllocateInfo>(pAllocateInfo->pNext);
+    auto opaque_alloc_info = vku::FindStructInPNextChain<VkMemoryOpaqueCaptureAddressAllocateInfo>(allocate_info.pNext);
     if (opaque_alloc_info && opaque_alloc_info->opaqueCaptureAddress != 0) {
         const Location address_loc =
             allocate_info_loc.pNext(Struct::VkMemoryOpaqueCaptureAddressAllocateInfo, Field::opaqueCaptureAddress);
@@ -473,11 +460,11 @@ bool StatelessValidation::ValidateAllocateMemoryExternal(VkDevice device, const
                              host_pointer, phys_dev_ext_props.external_memory_host_props.minImportedHostPointerAlignment);
         }
 
-        if (SafeModulo(pAllocateInfo->allocationSize,
+        if (SafeModulo(allocate_info.allocationSize,
                        phys_dev_ext_props.external_memory_host_props.minImportedHostPointerAlignment) != 0) {
             skip |= LogError("VUID-VkMemoryAllocateInfo-allocationSize-01745", device, allocate_info_loc.dot(Field::allocationSize),
                              "(%" PRIuLEAST64 ") is not a multiple of minImportedHostPointerAlignment (%" PRIuLEAST64 ")",
-                             pAllocateInfo->allocationSize,
+                             allocate_info.allocationSize,
                              phys_dev_ext_props.external_memory_host_props.minImportedHostPointerAlignment);
         }
 
@@ -486,15 +473,16 @@ bool StatelessValidation::ValidateAllocateMemoryExternal(VkDevice device, const
             VkMemoryHostPointerPropertiesEXT host_pointer_props = vku::InitStructHelper();
             DispatchGetMemoryHostPointerPropertiesEXT(device, ext.import_info_host_pointer->handleType,
                                                       ext.import_info_host_pointer->pHostPointer, &host_pointer_props);
-            if (((1 << pAllocateInfo->memoryTypeIndex) & host_pointer_props.memoryTypeBits) == 0) {
-                skip |= LogError("VUID-VkMemoryAllocateInfo-memoryTypeIndex-01744", device,
-                                 allocate_info_loc.dot(Field::memoryTypeIndex),
-                                 "is %" PRIu32 " but VkMemoryHostPointerPropertiesEXT::memoryTypeBits is 0x%" PRIx32 ".",
-                                 pAllocateInfo->memoryTypeIndex, host_pointer_props.memoryTypeBits);
+            if (((1 << allocate_info.memoryTypeIndex) & host_pointer_props.memoryTypeBits) == 0) {
+                skip |= LogError(
+                    "VUID-VkMemoryAllocateInfo-memoryTypeIndex-01744", device, allocate_info_loc.dot(Field::memoryTypeIndex),
+                    "is %" PRIu32 " but VkMemoryHostPointerPropertiesEXT::memoryTypeBits is 0x%" PRIx32 " with handleType %s.",
+                    allocate_info.memoryTypeIndex, host_pointer_props.memoryTypeBits,
+                    string_VkExternalMemoryHandleTypeFlagBits(ext.import_info_host_pointer->handleType));
             }
         }
 
-        auto dedicated_allocate_info = vku::FindStructInPNextChain<VkMemoryDedicatedAllocateInfo>(pAllocateInfo->pNext);
+        auto dedicated_allocate_info = vku::FindStructInPNextChain<VkMemoryDedicatedAllocateInfo>(allocate_info.pNext);
         if (dedicated_allocate_info) {
             if (dedicated_allocate_info->buffer != VK_NULL_HANDLE) {
                 skip |= LogError("VUID-VkMemoryAllocateInfo-pNext-02806", device,
@@ -511,10 +499,9 @@ bool StatelessValidation::ValidateAllocateMemoryExternal(VkDevice device, const
     }
 
 #ifdef VK_USE_PLATFORM_METAL_EXT
-    skip |=
-        ExportMetalObjectsPNextUtil(VK_EXPORT_METAL_OBJECT_TYPE_METAL_BUFFER_BIT_EXT, "VUID-VkMemoryAllocateInfo-pNext-06780",
-                                    allocate_info_loc, "VK_EXPORT_METAL_OBJECT_TYPE_METAL_BUFFER_BIT_EXT", pAllocateInfo->pNext);
+    skip |= ExportMetalObjectsPNextUtil(VK_EXPORT_METAL_OBJECT_TYPE_METAL_BUFFER_BIT_EXT, "VUID-VkMemoryAllocateInfo-pNext-06780",
+                                        allocate_info_loc, "VK_EXPORT_METAL_OBJECT_TYPE_METAL_BUFFER_BIT_EXT", allocate_info.pNext);
 #endif  // VK_USE_PLATFORM_METAL_EXT
 
     return skip;
-}
\ No newline at end of file
+}
