@@ -18,6 +18,7 @@
 
 #include "stateless/stateless_validation.h"
 #include "generated/enum_flag_bits.h"
+#include "generated/dispatch_functions.h"
 
 bool StatelessValidation::manual_PreCallValidateAcquireNextImageKHR(VkDevice device, VkSwapchainKHR swapchain, uint64_t timeout,
                                                                     VkSemaphore semaphore, VkFence fence, uint32_t *pImageIndex,
@@ -70,16 +71,16 @@ bool StatelessValidation::ValidateSwapchainCreateInfo(const VkSwapchainCreateInf
     // Validate VK_KHR_image_format_list VkImageFormatListCreateInfo
     const auto format_list_info = vku::FindStructInPNextChain<VkImageFormatListCreateInfo>(create_info.pNext);
     if (format_list_info) {
-        const uint32_t viewFormatCount = format_list_info->viewFormatCount;
-        if (((create_info.flags & VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR) == 0) && (viewFormatCount > 1)) {
+        const uint32_t view_format_count = format_list_info->viewFormatCount;
+        if (((create_info.flags & VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR) == 0) && (view_format_count > 1)) {
             skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-04100", device,
                              loc.pNext(Struct::VkImageFormatListCreateInfo, Field::viewFormatCount),
                              "is %" PRIu32 " but flag (%s) does not includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR.",
-                             viewFormatCount, string_VkImageCreateFlags(create_info.flags).c_str());
+                             view_format_count, string_VkImageCreateFlags(create_info.flags).c_str());
         }
 
         // Using the first format, compare the rest of the formats against it that they are compatible
-        for (uint32_t i = 1; i < viewFormatCount; i++) {
+        for (uint32_t i = 1; i < view_format_count; i++) {
             if (vkuFormatCompatibilityClass(format_list_info->pViewFormats[0]) !=
                 vkuFormatCompatibilityClass(format_list_info->pViewFormats[i])) {
                 skip |= LogError("VUID-VkSwapchainCreateInfoKHR-pNext-04099", device,
@@ -95,8 +96,8 @@ bool StatelessValidation::ValidateSwapchainCreateInfo(const VkSwapchainCreateInf
     if ((create_info.flags & VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR) != 0) {
         if (format_list_info == nullptr) {
             skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-03168", device, loc.dot(Field::flags),
-                             "includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR but the pNext does not contain "
-                             "VkImageFormatListCreateInfo.");
+                             "includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR but the pNext chain does not contain an instance "
+                             "of VkImageFormatListCreateInfo.");
         } else if (format_list_info->viewFormatCount == 0) {
             skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-03168", device, loc.dot(Field::flags),
                              "includes VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR but %s is zero.",
@@ -118,6 +119,38 @@ bool StatelessValidation::ValidateSwapchainCreateInfo(const VkSwapchainCreateInf
             }
         }
     }
+
+    if (create_info.presentMode == VK_PRESENT_MODE_FIFO_LATEST_READY_EXT && !enabled_features.presentModeFifoLatestReady) {
+        skip |=
+            LogError("VUID-VkSwapchainCreateInfoKHR-presentModeFifoLatestReady-10161", device, loc.dot(Field::presentMode),
+                     "is %s, but feature presentModeFifoLatestReady is not enabled", string_VkPresentModeKHR(create_info.presentMode));
+    }
+
+    if (create_info.flags & VK_SWAPCHAIN_CREATE_PROTECTED_BIT_KHR) {
+        const bool is_required_ext_supported =
+            SupportedByPdev(physical_device, vvl::Extension::_VK_KHR_surface_protected_capabilities);
+        const bool is_required_ext_enabled = IsExtEnabled(instance_extensions.vk_khr_surface_protected_capabilities);
+
+        if (is_required_ext_supported && !is_required_ext_enabled) {
+            skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-03187", device, loc.dot(Field::flags),
+                             "contains VK_SWAPCHAIN_CREATE_PROTECTED_BIT_KHR, but extension VK_KHR_surface_protected_capabilities "
+                             "was not enabled and surface capabilities VkSurfaceProtectedCapabilitiesKHR were not queried.");
+        } else if (is_required_ext_enabled) {
+            VkPhysicalDeviceSurfaceInfo2KHR surface_info = vku::InitStructHelper();
+            surface_info.surface = create_info.surface;
+            VkSurfaceProtectedCapabilitiesKHR surface_protected_capabilities = vku::InitStructHelper();
+            VkSurfaceCapabilities2KHR surface_capabilities = vku::InitStructHelper(&surface_protected_capabilities);
+            const VkResult result =
+                DispatchGetPhysicalDeviceSurfaceCapabilities2KHR(physical_device, &surface_info, &surface_capabilities);
+
+            if (result == VK_SUCCESS && !surface_protected_capabilities.supportsProtected) {
+                skip |= LogError("VUID-VkSwapchainCreateInfoKHR-flags-03187", device, loc.dot(Field::flags),
+                                 "contains VK_SWAPCHAIN_CREATE_PROTECTED_BIT_KHR but the surface "
+                                 "capabilities does not have VkSurfaceProtectedCapabilitiesKHR.supportsProtected set to VK_TRUE.");
+            }
+        }
+    }
+
     return skip;
 }
 
@@ -146,31 +179,24 @@ bool StatelessValidation::manual_PreCallValidateCreateSharedSwapchainsKHR(VkDevi
 bool StatelessValidation::manual_PreCallValidateQueuePresentKHR(VkQueue queue, const VkPresentInfoKHR *pPresentInfo,
                                                                 const ErrorObject &error_obj) const {
     bool skip = false;
+    if (!pPresentInfo) return skip;
 
-    if (pPresentInfo && pPresentInfo->pNext) {
-        const auto *present_regions = vku::FindStructInPNextChain<VkPresentRegionsKHR>(pPresentInfo->pNext);
-        if (present_regions) {
-            if (present_regions->swapchainCount != pPresentInfo->swapchainCount) {
-                skip |= LogError("VUID-VkPresentRegionsKHR-swapchainCount-01260", device,
-                                 error_obj.location.pNext(Struct::VkPresentRegionsKHR, Field::swapchainCount),
-                                 "(%" PRIu32 ") is not equal to %s (%" PRIu32 ").", present_regions->swapchainCount,
-                                 error_obj.location.dot(Field::pPresentInfo).dot(Field::swapchainCount).Fields().c_str(),
-                                 pPresentInfo->swapchainCount);
-            }
-            skip |= ValidateStructPnext(error_obj.location.pNext(Struct::VkPresentRegionsKHR), present_regions->pNext, 0, nullptr,
-                                        GeneratedVulkanHeaderVersion, "VUID-VkPresentInfoKHR-pNext-pNext",
-                                        "VUID-VkPresentInfoKHR-sType-unique");
+    if (const auto *present_regions = vku::FindStructInPNextChain<VkPresentRegionsKHR>(pPresentInfo->pNext)) {
+        if (present_regions->swapchainCount != pPresentInfo->swapchainCount) {
+            skip |= LogError("VUID-VkPresentRegionsKHR-swapchainCount-01260", device,
+                             error_obj.location.pNext(Struct::VkPresentRegionsKHR, Field::swapchainCount),
+                             "(%" PRIu32 ") is not equal to %s (%" PRIu32 ").", present_regions->swapchainCount,
+                             error_obj.location.dot(Field::pPresentInfo).dot(Field::swapchainCount).Fields().c_str(),
+                             pPresentInfo->swapchainCount);
         }
     }
 
-    if (pPresentInfo) {
-        for (uint32_t i = 0; i < pPresentInfo->swapchainCount; ++i) {
-            for (uint32_t j = i + 1; j < pPresentInfo->swapchainCount; ++j) {
-                if (pPresentInfo->pSwapchains[i] == pPresentInfo->pSwapchains[j]) {
-                    skip |= LogError("VUID-VkPresentInfoKHR-pSwapchain-09231", device, error_obj.location.dot(Field::pSwapchain),
-                                     "at index %" PRIu32 " and index %" PRIu32 " are both %s.", i, j,
-                                     FormatHandle(pPresentInfo->pSwapchains[i]).c_str());
-                }
+    for (uint32_t i = 0; i < pPresentInfo->swapchainCount; ++i) {
+        for (uint32_t j = i + 1; j < pPresentInfo->swapchainCount; ++j) {
+            if (pPresentInfo->pSwapchains[i] == pPresentInfo->pSwapchains[j]) {
+                skip |= LogError("VUID-VkPresentInfoKHR-pSwapchain-09231", device, error_obj.location.dot(Field::pSwapchain),
+                                 "[%" PRIu32 "] and pSwapchain[%" PRIu32 "] are both %s.", i, j,
+                                 FormatHandle(pPresentInfo->pSwapchains[i]).c_str());
             }
         }
     }
@@ -294,6 +320,15 @@ bool StatelessValidation::manual_PreCallValidateGetPhysicalDeviceSurfaceFormats2
                          error_obj.location.dot(Field::pSurfaceInfo).dot(Field::surface),
                          "is VK_NULL_HANDLE and VK_GOOGLE_surfaceless_query is not enabled.");
     }
+    if (pSurfaceFormats) {
+        for (uint32_t i = 0; i < *pSurfaceFormatCount; ++i) {
+            if (vku::FindStructInPNextChain<VkImageCompressionPropertiesEXT>(pSurfaceFormats[i].pNext) &&
+                !enabled_features.imageCompressionControlSwapchain) {
+                skip |= LogError("VUID-VkSurfaceFormat2KHR-pNext-06750", device, error_obj.location.dot(Field::pNext),
+                                 "contains VkImageCompressionPropertiesEXT, but imageCompressionControlSwapchain is not enabled");
+            }
+        }
+    }
     return skip;
 }
 
@@ -348,7 +383,8 @@ bool StatelessValidation::PreCallValidateGetDeviceGroupSurfacePresentModes2EXT(V
         "VUID-vkGetDeviceGroupSurfacePresentModes2EXT-pSurfaceInfo-parameter", "VUID-VkPhysicalDeviceSurfaceInfo2KHR-sType-sType");
     if (pSurfaceInfo != NULL) {
         constexpr std::array allowed_structs = {VK_STRUCTURE_TYPE_SURFACE_FULL_SCREEN_EXCLUSIVE_INFO_EXT,
-                                                VK_STRUCTURE_TYPE_SURFACE_FULL_SCREEN_EXCLUSIVE_WIN32_INFO_EXT};
+                                                VK_STRUCTURE_TYPE_SURFACE_FULL_SCREEN_EXCLUSIVE_WIN32_INFO_EXT,
+                                                VK_STRUCTURE_TYPE_SURFACE_PRESENT_MODE_EXT};
 
         skip |= ValidateStructPnext(error_obj.location.dot(Field::pSurfaceInfo), pSurfaceInfo->pNext, allowed_structs.size(),
                                     allowed_structs.data(), GeneratedVulkanHeaderVersion,
