@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2019-2023 Valve Corporation
- * Copyright (c) 2019-2023 LunarG, Inc.
+ * Copyright (c) 2019-2024 Valve Corporation
+ * Copyright (c) 2019-2024 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,6 +19,11 @@
 #include "sync/sync_renderpass.h"
 #include "sync/sync_access_context.h"
 #include "sync/sync_commandbuffer.h"
+#include "sync/sync_image.h"
+
+#include "state_tracker/buffer_state.h"
+#include "state_tracker/cmd_buffer_state.h"
+#include "state_tracker/render_pass_state.h"
 
 #include "sync/sync_validation.h"
 
@@ -210,7 +215,7 @@ SyncOpBarriers::SyncOpBarriers(vvl::Func command, const SyncValidator &sync_stat
 }
 
 SyncOpBarriers::SyncOpBarriers(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags, uint32_t event_count,
-                               const VkDependencyInfoKHR *dep_infos)
+                               const VkDependencyInfo *dep_infos)
     : SyncOpBase(command), barriers_(event_count) {
     for (uint32_t i = 0; i < event_count; i++) {
         const auto &dep_info = dep_infos[i];
@@ -240,7 +245,7 @@ SyncOpPipelineBarrier::SyncOpPipelineBarrier(vvl::Func command, const SyncValida
                      pImageMemoryBarriers) {}
 
 SyncOpPipelineBarrier::SyncOpPipelineBarrier(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags,
-                                             const VkDependencyInfoKHR &dep_info)
+                                             const VkDependencyInfo &dep_info)
     : SyncOpBarriers(command, sync_state, queue_flags, 1, &dep_info) {}
 
 bool SyncOpPipelineBarrier::Validate(const CommandBufferAccessContext &cb_context) const {
@@ -267,11 +272,9 @@ bool SyncOpPipelineBarrier::Validate(const CommandBufferAccessContext &cb_contex
             // PHASE1 TODO -- add tag information to log msg when useful.
             const Location loc(command_);
             const auto &sync_state = cb_context.GetSyncState();
-            const auto image_handle = image_state->image();
-            skip |= sync_state.LogError(string_SyncHazardVUID(hazard.Hazard()), image_handle, loc,
-                                        "Hazard %s for image barrier %" PRIu32 " %s. Access info %s.",
-                                        string_SyncHazard(hazard.Hazard()), image_barrier.index,
-                                        sync_state.FormatHandle(image_handle).c_str(), cb_context.FormatHazard(hazard).c_str());
+            const auto error =
+                sync_state.error_messages_.PipelineBarrierError(hazard, cb_context, image_barrier.index, *image_state);
+            skip |= sync_state.SyncError(hazard.Hazard(), image_state->Handle(), loc, error);
         }
     }
     return skip;
@@ -336,10 +339,10 @@ ResourceUsageTag SyncOpPipelineBarrier::Record(CommandBufferAccessContext *cb_co
     const auto tag = cb_context->NextCommandTag(command_);
     for (const auto &barrier_set : barriers_) {
         for (const auto &buffer_barrier : barrier_set.buffer_memory_barriers) {
-            cb_context->AddHandle(tag, buffer_barrier.buffer->Handle());
+            cb_context->AddCommandHandle(tag, buffer_barrier.buffer->Handle());
         }
         for (const auto &image_barrier : barrier_set.image_memory_barriers) {
-            cb_context->AddHandle(tag, image_barrier.image->Handle());
+            cb_context->AddCommandHandle(tag, image_barrier.image->Handle());
         }
     }
     ReplayRecord(*cb_context, tag);
@@ -447,7 +450,7 @@ void SyncOpBarriers::BarrierSet::MakeImageMemoryBarriers(const SyncValidator &sy
         const auto &barrier = barriers[index];
         auto image = sync_state.Get<ImageState>(barrier.image);
         if (image) {
-            auto subresource_range = NormalizeSubresourceRange(image->createInfo, barrier.subresourceRange);
+            auto subresource_range = NormalizeSubresourceRange(image->create_info, barrier.subresourceRange);
             const SyncBarrier sync_barrier(barrier, src, dst);
             image_memory_barriers.emplace_back(image, index, sync_barrier, barrier.oldLayout, barrier.newLayout, subresource_range);
         } else {
@@ -467,7 +470,7 @@ void SyncOpBarriers::BarrierSet::MakeImageMemoryBarriers(const SyncValidator &sy
         auto dst = SyncExecScope::MakeDst(queue_flags, barrier.dstStageMask);
         auto image = sync_state.Get<ImageState>(barrier.image);
         if (image) {
-            auto subresource_range = NormalizeSubresourceRange(image->createInfo, barrier.subresourceRange);
+            auto subresource_range = NormalizeSubresourceRange(image->create_info, barrier.subresourceRange);
             const SyncBarrier sync_barrier(barrier, src, dst);
             image_memory_barriers.emplace_back(image, index, sync_barrier, barrier.oldLayout, barrier.newLayout, subresource_range);
         } else {
@@ -490,7 +493,7 @@ SyncOpWaitEvents::SyncOpWaitEvents(vvl::Func command, const SyncValidator &sync_
 }
 
 SyncOpWaitEvents::SyncOpWaitEvents(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags,
-                                   uint32_t eventCount, const VkEvent *pEvents, const VkDependencyInfoKHR *pDependencyInfo)
+                                   uint32_t eventCount, const VkEvent *pEvents, const VkDependencyInfo *pDependencyInfo)
     : SyncOpBarriers(command, sync_state, queue_flags, eventCount, pDependencyInfo) {
     MakeEventsList(sync_state, eventCount, pEvents);
     assert(events_.size() == barriers_.size());  // Just so nobody gets clever and decides to cull the event or barrier arrays
@@ -501,7 +504,7 @@ const char *const SyncOpWaitEvents::kIgnored = "Wait operation is ignored for th
 bool SyncOpWaitEvents::Validate(const CommandBufferAccessContext &cb_context) const {
     bool skip = false;
     const auto &sync_state = cb_context.GetSyncState();
-    const auto command_buffer_handle = cb_context.GetCBState().commandBuffer();
+    const VkCommandBuffer command_buffer_handle = cb_context.GetCBState().VkHandle();
 
     // This is only interesting at record and not replay (Execute/Submit) time.
     for (size_t barrier_set_index = 0; barrier_set_index < barriers_.size(); barrier_set_index++) {
@@ -541,8 +544,8 @@ bool SyncOpWaitEvents::DoValidate(const CommandExecutionContext &exec_context, c
     const auto &sync_state = exec_context.GetSyncState();
     const QueueId queue_id = exec_context.GetQueueId();
 
-    VkPipelineStageFlags2KHR event_stage_masks = 0U;
-    VkPipelineStageFlags2KHR barrier_mask_params = 0U;
+    VkPipelineStageFlags2 event_stage_masks = 0U;
+    VkPipelineStageFlags2 barrier_mask_params = 0U;
     bool events_not_found = false;
     const auto *events_context = exec_context.GetCurrentEventsContext();
     assert(events_context);
@@ -564,7 +567,7 @@ bool SyncOpWaitEvents::DoValidate(const CommandExecutionContext &exec_context, c
         // For replay calls, don't revalidate "same command buffer" events
         if (sync_event->last_command_tag >= base_tag) continue;
 
-        const auto event_handle = sync_event->event->event();
+        const VkEvent event_handle = sync_event->event->VkHandle();
         // TODO add "destroyed" checks
 
         if (sync_event->first_scope) {
@@ -643,11 +646,9 @@ bool SyncOpWaitEvents::DoValidate(const CommandExecutionContext &exec_context, c
                     *image_state, subresource_range, sync_event->scope.exec_scope, src_access_scope, queue_id,
                     sync_event->FirstScope(), sync_event->first_scope_tag, AccessContext::DetectOptions::kDetectAll);
                 if (hazard.IsHazard()) {
-                    skip |= sync_state.LogError(string_SyncHazardVUID(hazard.Hazard()), image_state->image(), loc,
-                                                "Hazard %s for image barrier %" PRIu32 " %s. Access info %s.",
-                                                string_SyncHazard(hazard.Hazard()), image_memory_barrier.index,
-                                                sync_state.FormatHandle(image_state->image()).c_str(),
-                                                exec_context.FormatHazard(hazard).c_str());
+                    const auto error =
+                        sync_state.error_messages_.WaitEventsError(hazard, exec_context, image_memory_barrier.index, *image_state);
+                    skip |= sync_state.SyncError(hazard.Hazard(), image_state->Handle(), loc, error);
                     break;
                 }
             }
@@ -658,7 +659,7 @@ bool SyncOpWaitEvents::DoValidate(const CommandExecutionContext &exec_context, c
     }
 
     // Note that we can't check for HOST in pEvents as we don't track that set event type
-    const auto extra_stage_bits = (barrier_mask_params & ~VK_PIPELINE_STAGE_2_HOST_BIT_KHR) & ~event_stage_masks;
+    const auto extra_stage_bits = (barrier_mask_params & ~VK_PIPELINE_STAGE_2_HOST_BIT) & ~event_stage_masks;
     if (extra_stage_bits) {
         // Issue error message that event waited for is not in wait events scope
         // NOTE: This isn't exactly the right VUID for WaitEvents2, but it's as close as we currently have support for
@@ -792,7 +793,7 @@ void SyncOpWaitEvents::MakeEventsList(const SyncValidator &sync_state, uint32_t
 }
 
 SyncOpResetEvent::SyncOpResetEvent(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags, VkEvent event,
-                                   VkPipelineStageFlags2KHR stageMask)
+                                   VkPipelineStageFlags2 stageMask)
     : SyncOpBase(command), event_(sync_state.Get<vvl::Event>(event)), exec_scope_(SyncExecScope::MakeSrc(queue_flags, stageMask)) {}
 
 bool SyncOpResetEvent::Validate(const CommandBufferAccessContext &cb_context) const {
@@ -831,15 +832,18 @@ bool SyncOpResetEvent::DoValidate(const CommandExecutionContext &exec_context, c
                 vuid = "SYNC-vkCmdResetEvent-missingbarrier-wait";
                 break;
             }
+            case vvl::Func::Empty:
+            case vvl::Func::vkCmdResetEvent:
+            case vvl::Func::vkCmdResetEvent2KHR:
+            case vvl::Func::vkCmdResetEvent2:
+                break;  // Valid, but nothing to do
             default:
-                // The only other valid last command that wasn't one.
-                assert((sync_event->last_command == vvl::Func::Empty) || (sync_event->last_command == vvl::Func::vkCmdResetEvent) ||
-                       (sync_event->last_command == vvl::Func::vkCmdResetEvent2KHR));
+                assert(false);
                 break;
         }
         if (vuid) {
             const Location loc(command_);
-            skip |= sync_state.LogError(vuid, event_->event(), loc, message, sync_state.FormatHandle(event_->event()).c_str(),
+            skip |= sync_state.LogError(vuid, event_->Handle(), loc, message, sync_state.FormatHandle(event_->Handle()).c_str(),
                                         CmdName(), vvl::String(sync_event->last_command));
         }
     }
@@ -872,7 +876,7 @@ void SyncOpResetEvent::ReplayRecord(CommandExecutionContext &exec_context, Resou
 }
 
 SyncOpSetEvent::SyncOpSetEvent(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags, VkEvent event,
-                               VkPipelineStageFlags2KHR stageMask, const AccessContext *access_context)
+                               VkPipelineStageFlags2 stageMask, const AccessContext *access_context)
     : SyncOpBase(command),
       event_(sync_state.Get<vvl::Event>(event)),
       recorded_context_(),
@@ -888,12 +892,12 @@ SyncOpSetEvent::SyncOpSetEvent(vvl::Func command, const SyncValidator &sync_stat
 }
 
 SyncOpSetEvent::SyncOpSetEvent(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags, VkEvent event,
-                               const VkDependencyInfoKHR &dep_info, const AccessContext *access_context)
+                               const VkDependencyInfo &dep_info, const AccessContext *access_context)
     : SyncOpBase(command),
       event_(sync_state.Get<vvl::Event>(event)),
       recorded_context_(),
       src_exec_scope_(SyncExecScope::MakeSrc(queue_flags, sync_utils::GetGlobalStageMasks(dep_info).src)),
-      dep_info_(new safe_VkDependencyInfo(&dep_info)) {
+      dep_info_(new vku::safe_VkDependencyInfo(&dep_info)) {
     if (access_context) {
         recorded_context_ = std::make_shared<const AccessContext>(*access_context);
     }
@@ -961,7 +965,7 @@ bool SyncOpSetEvent::DoValidate(const CommandExecutionContext &exec_context, con
             std::string vuid("SYNC-");
             vuid.append(CmdName()).append(vuid_stem);
             skip |=
-                sync_state.LogError(vuid.c_str(), event_->event(), loc, message, sync_state.FormatHandle(event_->event()).c_str(),
+                sync_state.LogError(vuid.c_str(), event_->Handle(), loc, message, sync_state.FormatHandle(event_->Handle()).c_str(),
                                     CmdName(), vvl::String(sync_event->last_command));
         }
     }
@@ -1033,7 +1037,7 @@ SyncOpBeginRenderPass::SyncOpBeginRenderPass(vvl::Func command, const SyncValida
     : SyncOpBase(command), rp_context_(nullptr) {
     if (pRenderPassBegin) {
         rp_state_ = sync_state.Get<vvl::RenderPass>(pRenderPassBegin->renderPass);
-        renderpass_begin_info_ = safe_VkRenderPassBeginInfo(pRenderPassBegin);
+        renderpass_begin_info_ = vku::safe_VkRenderPassBeginInfo(pRenderPassBegin);
         auto fb_state = sync_state.Get<vvl::Framebuffer>(pRenderPassBegin->framebuffer);
         if (fb_state) {
             shared_attachments_ = sync_state.GetAttachmentViews(*renderpass_begin_info_.ptr(), *fb_state);
@@ -1045,7 +1049,7 @@ SyncOpBeginRenderPass::SyncOpBeginRenderPass(vvl::Func command, const SyncValida
             }
         }
         if (pSubpassBeginInfo) {
-            subpass_begin_info_ = safe_VkSubpassBeginInfo(pSubpassBeginInfo);
+            subpass_begin_info_ = vku::safe_VkSubpassBeginInfo(pSubpassBeginInfo);
         }
     }
 }
@@ -1067,7 +1071,7 @@ bool SyncOpBeginRenderPass::Validate(const CommandBufferAccessContext &cb_contex
                                cb_context.GetCurrentAccessContext());
 
     // Validate attachment operations
-    if (attachments_.size() == 0) return skip;
+    if (attachments_.empty()) return skip;
     const auto &render_area = renderpass_begin_info_.renderArea;
 
     // Since the isn't a valid RenderPassAccessContext until Record, needs to create the view/generator list... we could limit this
@@ -1101,9 +1105,11 @@ ResourceUsageTag SyncOpBeginRenderPass::Record(CommandBufferAccessContext *cb_co
 }
 
 bool SyncOpBeginRenderPass::ReplayValidate(ReplayState &replay, ResourceUsageTag recorded_tag) const {
-    // Need to update the exec_contexts state (which for RenderPass operations *must* be a QueueBatchContext, as
-    // render pass operations are not allowed in secondary command buffers.
-    replay.BeginRenderPassReplaySetup(*this);
+    CommandExecutionContext &exec_context = replay.GetExecutionContext();
+    // can't be kExecuted, this operation is not allowed in secondary command buffers
+    assert(exec_context.Type() == CommandExecutionContext::kSubmitted);
+    auto &batch_context = static_cast<QueueBatchContext &>(exec_context);
+    batch_context.BeginRenderPassReplaySetup(replay, *this);
 
     // Only the layout transitions happen at the replay tag, loadOp's happen at a subsequent tag
     ResourceUsageRange first_use_range = {recorded_tag, recorded_tag + 1};
@@ -1130,7 +1136,7 @@ bool SyncOpNextSubpass::Validate(const CommandBufferAccessContext &cb_context) c
     const auto *renderpass_context = cb_context.GetCurrentRenderPassContext();
     if (!renderpass_context) return skip;
 
-    skip |= renderpass_context->ValidateNextSubpass(cb_context.GetExecutionContext(), command_);
+    skip |= renderpass_context->ValidateNextSubpass(cb_context, command_);
     return skip;
 }
 
@@ -1140,7 +1146,11 @@ ResourceUsageTag SyncOpNextSubpass::Record(CommandBufferAccessContext *cb_contex
 
 bool SyncOpNextSubpass::ReplayValidate(ReplayState &replay, ResourceUsageTag recorded_tag) const {
     // Any store/resolve operations happen before the NextSubpass tag so we can advance to the next subpass state
-    replay.NextSubpassReplaySetup();
+    CommandExecutionContext &exec_context = replay.GetExecutionContext();
+    // can't be kExecuted, this operation is not allowed in secondary command buffers
+    assert(exec_context.Type() == CommandExecutionContext::kSubmitted);
+    auto &batch_context = static_cast<QueueBatchContext &>(exec_context);
+    batch_context.NextSubpassReplaySetup(replay);
 
     // Only the layout transitions happen at the replay tag, loadOp's happen at a subsequent tag
     ResourceUsageRange first_use_range = {recorded_tag, recorded_tag + 1};
@@ -1163,7 +1173,7 @@ bool SyncOpEndRenderPass::Validate(const CommandBufferAccessContext &cb_context)
     const auto *renderpass_context = cb_context.GetCurrentRenderPassContext();
 
     if (!renderpass_context) return skip;
-    skip |= renderpass_context->ValidateEndRenderPass(cb_context.GetExecutionContext(), command_);
+    skip |= renderpass_context->ValidateEndRenderPass(cb_context, command_);
     return skip;
 }
 
@@ -1175,10 +1185,15 @@ bool SyncOpEndRenderPass::ReplayValidate(ReplayState &replay, ResourceUsageTag r
     // Any store/resolve operations happen before the EndRenderPass tag so we can ignore them
     // Only the layout transitions happen at the replay tag
     ResourceUsageRange first_use_range = {recorded_tag, recorded_tag + 1};
-    bool skip = replay.DetectFirstUseHazard(first_use_range);
+    bool skip = false;
+    skip |= replay.DetectFirstUseHazard(first_use_range);
 
-    // We can cleanup here as the recorded tag represents the final layout transition (which is the last operation or the RP
-    replay.EndRenderPassReplayCleanup();
+    // We can cleanup here as the recorded tag represents the final layout transition (which is the last operation or the RP)
+    CommandExecutionContext &exec_context = replay.GetExecutionContext();
+    // can't be kExecuted, this operation is not allowed in secondary command buffers
+    assert(exec_context.Type() == CommandExecutionContext::kSubmitted);
+    auto &batch_context = static_cast<QueueBatchContext &>(exec_context);
+    batch_context.EndRenderPassReplayCleanup(replay);
 
     return skip;
 }
@@ -1186,20 +1201,8 @@ bool SyncOpEndRenderPass::ReplayValidate(ReplayState &replay, ResourceUsageTag r
 void SyncOpEndRenderPass::ReplayRecord(CommandExecutionContext &exec_context, ResourceUsageTag exec_tag) const {}
 
 ReplayState::ReplayState(CommandExecutionContext &exec_context, const CommandBufferAccessContext &recorded_context,
-                         const ErrorObject &error_obj, uint32_t index)
-    : exec_context_(exec_context),
-      recorded_context_(recorded_context),
-      error_obj_(error_obj),
-      index_(index),
-      base_tag_(exec_context.GetTagLimit()) {}
-
-void ReplayState::BeginRenderPassReplaySetup(const SyncOpBeginRenderPass &begin_op) {
-    exec_context_.BeginRenderPassReplaySetup(*this, begin_op);
-}
-
-void ReplayState::NextSubpassReplaySetup() { exec_context_.NextSubpassReplaySetup(*this); }
-
-void ReplayState::EndRenderPassReplayCleanup() { exec_context_.EndRenderPassReplayCleanup(*this); }
+                         const ErrorObject &error_obj, uint32_t index, ResourceUsageTag base_tag)
+    : exec_context_(exec_context), recorded_context_(recorded_context), error_obj_(error_obj), index_(index), base_tag_(base_tag) {}
 
 AccessContext *ReplayState::ReplayStateRenderPassBegin(VkQueueFlags queue_flags, const SyncOpBeginRenderPass &begin_op,
                                                        const AccessContext &external_context) {
@@ -1220,22 +1223,18 @@ const AccessContext *ReplayState::GetRecordedAccessContext() const {
 bool ReplayState::DetectFirstUseHazard(const ResourceUsageRange &first_use_range) const {
     bool skip = false;
     if (first_use_range.non_empty()) {
-        HazardResult hazard;
         // We're allowing for the Replay(Validate|Record) to modify the exec_context (e.g. for Renderpass operations), so
         // we need to fetch the current access context each time
-        hazard = GetRecordedAccessContext()->DetectFirstUseHazard(exec_context_.GetQueueId(), first_use_range,
-                                                                  *exec_context_.GetCurrentAccessContext());
+        const HazardResult hazard = GetRecordedAccessContext()->DetectFirstUseHazard(exec_context_.GetQueueId(), first_use_range,
+                                                                                     *exec_context_.GetCurrentAccessContext());
 
         if (hazard.IsHazard()) {
             const SyncValidator &sync_state = exec_context_.GetSyncState();
             const auto handle = exec_context_.Handle();
-            const auto recorded_handle = recorded_context_.GetCBState().commandBuffer();
-            skip = sync_state.LogError(
-                string_SyncHazardVUID(hazard.Hazard()), handle, error_obj_.location,
-                "Hazard %s for entry %" PRIu32 ", %s, %s access info %s. Access info %s.", string_SyncHazard(hazard.Hazard()),
-                index_, sync_state.FormatHandle(recorded_handle).c_str(), exec_context_.ExecutionTypeString(),
-                recorded_context_.FormatUsage(exec_context_.ExecutionUsageString(), *hazard.RecordedAccess()).c_str(),
-                exec_context_.FormatHazard(hazard).c_str());
+            const VkCommandBuffer recorded_handle = recorded_context_.GetCBState().VkHandle();
+            const auto error =
+                sync_state.error_messages_.FirstUseError(hazard, exec_context_, recorded_context_, index_, recorded_handle);
+            skip |= sync_state.SyncError(hazard.Hazard(), handle, error_obj_.location, error);
         }
     }
     return skip;
@@ -1340,6 +1339,12 @@ void SyncEventsContext::AddReferencedTags(ResourceUsageTagSet &referenced) const
         }
     }
 }
+
+SyncEventState::SyncEventState(const SyncEventState::EventPointer &event_state) : SyncEventState() {
+    event = event_state;
+    destroyed = (event.get() == nullptr) || event_state->Destroyed();
+}
+
 void SyncEventState::ResetFirstScope() {
     first_scope.reset();
     scope = SyncExecScope();
@@ -1347,7 +1352,7 @@ void SyncEventState::ResetFirstScope() {
 }
 
 // Keep the "ignore this event" logic in same place for ValidateWait and RecordWait to use
-SyncEventState::IgnoreReason SyncEventState::IsIgnoredByWait(vvl::Func command, VkPipelineStageFlags2KHR srcStageMask) const {
+SyncEventState::IgnoreReason SyncEventState::IsIgnoredByWait(vvl::Func command, VkPipelineStageFlags2 srcStageMask) const {
     IgnoreReason reason = NotIgnored;
 
     if ((vvl::Func::vkCmdWaitEvents2KHR == command || vvl::Func::vkCmdWaitEvents2 == command) &&
@@ -1359,7 +1364,7 @@ SyncEventState::IgnoreReason SyncEventState::IsIgnoredByWait(vvl::Func command,
     } else if (unsynchronized_set != vvl::Func::Empty) {
         reason = SetRace;
     } else if (first_scope) {
-        const VkPipelineStageFlags2KHR missing_bits = scope.mask_param & ~srcStageMask;
+        const VkPipelineStageFlags2 missing_bits = scope.mask_param & ~srcStageMask;
         // Note it is the "not missing bits" path that is the only "NotIgnored" path
         if (missing_bits) reason = MissingStageBits;
     } else {
@@ -1369,7 +1374,7 @@ SyncEventState::IgnoreReason SyncEventState::IsIgnoredByWait(vvl::Func command,
     return reason;
 }
 
-bool SyncEventState::HasBarrier(VkPipelineStageFlags2KHR stageMask, VkPipelineStageFlags2KHR exec_scope_arg) const {
+bool SyncEventState::HasBarrier(VkPipelineStageFlags2 stageMask, VkPipelineStageFlags2 exec_scope_arg) const {
     return (last_command == vvl::Func::Empty) || (stageMask & VK_PIPELINE_STAGE_ALL_COMMANDS_BIT) || (barriers & exec_scope_arg) ||
            (barriers & VK_PIPELINE_STAGE_ALL_COMMANDS_BIT);
 }
