@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2019-2023 Valve Corporation
- * Copyright (c) 2019-2023 LunarG, Inc.
+ * Copyright (c) 2019-2024 Valve Corporation
+ * Copyright (c) 2019-2024 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,16 +16,21 @@
  */
 #pragma once
 
-#include "state_tracker/buffer_state.h"
-#include "state_tracker/cmd_buffer_state.h"
-
 #include "sync/sync_access_context.h"
+#include <vulkan/utility/vk_safe_struct.hpp>
+#include "error_message/error_location.h"
 
 class CommandBufferAccessContext;
 class CommandExecutionContext;
 class RenderPassAccessContext;
 class ReplayState;
 
+namespace vvl {
+class ImageView;
+class RenderPass;
+class CommandBuffer;
+}  // namespace vvl
+
 using SyncMemoryBarrier = SyncBarrier;
 
 struct SyncEventState {
@@ -35,7 +40,7 @@ struct SyncEventState {
     vvl::Func last_command;             // Only Event commands are valid here.
     ResourceUsageTag last_command_tag;  // Needed to filter replay validation
     vvl::Func unsynchronized_set;
-    VkPipelineStageFlags2KHR barriers;
+    VkPipelineStageFlags2 barriers;
     SyncExecScope scope;
     ResourceUsageTag first_scope_tag;
     bool destroyed;
@@ -54,15 +59,12 @@ struct SyncEventState {
     SyncEventState(const SyncEventState &) = default;
     SyncEventState(SyncEventState &&) = default;
 
-    SyncEventState(const SyncEventState::EventPointer &event_state) : SyncEventState() {
-        event = event_state;
-        destroyed = (event.get() == nullptr) || event_state->Destroyed();
-    }
+    SyncEventState(const SyncEventState::EventPointer &event_state);
 
     void ResetFirstScope();
     const AccessContext::ScopeMap &FirstScope() const { return first_scope->GetAccessStateMap(); }
-    IgnoreReason IsIgnoredByWait(vvl::Func command, VkPipelineStageFlags2KHR srcStageMask) const;
-    bool HasBarrier(VkPipelineStageFlags2KHR stageMask, VkPipelineStageFlags2KHR exec_scope) const;
+    IgnoreReason IsIgnoredByWait(vvl::Func command, VkPipelineStageFlags2 srcStageMask) const;
+    bool HasBarrier(VkPipelineStageFlags2 stageMask, VkPipelineStageFlags2 exec_scope) const;
     void AddReferencedTags(ResourceUsageTagSet &referenced) const;
 };
 
@@ -186,7 +188,7 @@ class SyncOpBarriers : public SyncOpBase {
                    const VkBufferMemoryBarrier *pBufferMemoryBarriers, uint32_t imageMemoryBarrierCount,
                    const VkImageMemoryBarrier *pImageMemoryBarriers);
     SyncOpBarriers(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags, uint32_t event_count,
-                   const VkDependencyInfoKHR *pDependencyInfo);
+                   const VkDependencyInfo *pDependencyInfo);
 
     ~SyncOpBarriers() override = default;
 
@@ -226,7 +228,7 @@ class SyncOpPipelineBarrier : public SyncOpBarriers {
                           const VkBufferMemoryBarrier *pBufferMemoryBarriers, uint32_t imageMemoryBarrierCount,
                           const VkImageMemoryBarrier *pImageMemoryBarriers);
     SyncOpPipelineBarrier(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags,
-                          const VkDependencyInfoKHR &pDependencyInfo);
+                          const VkDependencyInfo &pDependencyInfo);
     ~SyncOpPipelineBarrier() override = default;
 
     bool Validate(const CommandBufferAccessContext &cb_context) const override;
@@ -244,7 +246,7 @@ class SyncOpWaitEvents : public SyncOpBarriers {
                      const VkImageMemoryBarrier *pImageMemoryBarriers);
 
     SyncOpWaitEvents(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags, uint32_t eventCount,
-                     const VkEvent *pEvents, const VkDependencyInfoKHR *pDependencyInfo);
+                     const VkEvent *pEvents, const VkDependencyInfo *pDependencyInfo);
     ~SyncOpWaitEvents() override = default;
 
     bool Validate(const CommandBufferAccessContext &cb_context) const override;
@@ -265,7 +267,7 @@ class SyncOpWaitEvents : public SyncOpBarriers {
 class SyncOpResetEvent : public SyncOpBase {
   public:
     SyncOpResetEvent(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags, VkEvent event,
-                     VkPipelineStageFlags2KHR stageMask);
+                     VkPipelineStageFlags2 stageMask);
     ~SyncOpResetEvent() override = default;
 
     bool Validate(const CommandBufferAccessContext &cb_context) const override;
@@ -282,9 +284,9 @@ class SyncOpResetEvent : public SyncOpBase {
 class SyncOpSetEvent : public SyncOpBase {
   public:
     SyncOpSetEvent(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags, VkEvent event,
-                   VkPipelineStageFlags2KHR stageMask, const AccessContext *access_context);
+                   VkPipelineStageFlags2 stageMask, const AccessContext *access_context);
     SyncOpSetEvent(vvl::Func command, const SyncValidator &sync_state, VkQueueFlags queue_flags, VkEvent event,
-                   const VkDependencyInfoKHR &dep_info, const AccessContext *access_context);
+                   const VkDependencyInfo &dep_info, const AccessContext *access_context);
     ~SyncOpSetEvent() override = default;
 
     bool Validate(const CommandBufferAccessContext &cb_context) const override;
@@ -301,7 +303,7 @@ class SyncOpSetEvent : public SyncOpBase {
     std::shared_ptr<const AccessContext> recorded_context_;
     SyncExecScope src_exec_scope_;
     // Note that the dep info is *not* dehandled, but retained for comparison with a future WaitEvents2
-    std::shared_ptr<safe_VkDependencyInfo> dep_info_;
+    std::shared_ptr<vku::safe_VkDependencyInfo> dep_info_;
 };
 
 class SyncOpBeginRenderPass : public SyncOpBase {
@@ -317,8 +319,8 @@ class SyncOpBeginRenderPass : public SyncOpBase {
     const RenderPassAccessContext *GetRenderPassAccessContext() const { return rp_context_; }
 
   protected:
-    safe_VkRenderPassBeginInfo renderpass_begin_info_;
-    safe_VkSubpassBeginInfo subpass_begin_info_;
+    vku::safe_VkRenderPassBeginInfo renderpass_begin_info_;
+    vku::safe_VkSubpassBeginInfo subpass_begin_info_;
     std::vector<std::shared_ptr<const vvl::ImageView>> shared_attachments_;
     std::vector<const syncval_state::ImageViewState *> attachments_;
     std::shared_ptr<const vvl::RenderPass> rp_state_;
@@ -337,8 +339,8 @@ class SyncOpNextSubpass : public SyncOpBase {
     void ReplayRecord(CommandExecutionContext &exec_context, ResourceUsageTag exec_tag) const override;
 
   protected:
-    safe_VkSubpassBeginInfo subpass_begin_info_;
-    safe_VkSubpassEndInfo subpass_end_info_;
+    vku::safe_VkSubpassBeginInfo subpass_begin_info_;
+    vku::safe_VkSubpassEndInfo subpass_end_info_;
 };
 
 class SyncOpEndRenderPass : public SyncOpBase {
@@ -352,7 +354,7 @@ class SyncOpEndRenderPass : public SyncOpBase {
     void ReplayRecord(CommandExecutionContext &exec_context, ResourceUsageTag exec_tag) const override;
 
   protected:
-    safe_VkSubpassEndInfo subpass_end_info_;
+    vku::safe_VkSubpassEndInfo subpass_end_info_;
 };
 // The barrier operation for pipeline and subpass dependencies`
 struct PipelineBarrierOp {
@@ -428,15 +430,11 @@ class ReplayState {
     bool DetectFirstUseHazard(const ResourceUsageRange &first_use_range) const;
 
     ReplayState(CommandExecutionContext &exec_context, const CommandBufferAccessContext &recorded_context,
-                const ErrorObject &error_object, uint32_t index);
+                const ErrorObject &error_object, uint32_t index, ResourceUsageTag base_tag);
 
     CommandExecutionContext &GetExecutionContext() const { return exec_context_; }
     ResourceUsageTag GetBaseTag() const { return base_tag_; }
 
-    void BeginRenderPassReplaySetup(const SyncOpBeginRenderPass &begin_op);
-    void NextSubpassReplaySetup();
-    void EndRenderPassReplayCleanup();
-
     AccessContext *ReplayStateRenderPassBegin(VkQueueFlags queue_flags, const SyncOpBeginRenderPass &begin_op,
                                               const AccessContext &external_context);
     AccessContext *ReplayStateRenderPassNext();
