@@ -96,14 +96,14 @@ def EnableCaching() -> None:
 class APISpecific:
     # Version object factory method
     @staticmethod
-    def createApiVersion(targetApiName: str, name: str, number: str) -> Version:
+    def createApiVersion(targetApiName: str, name: str) -> Version:
         match targetApiName:
 
             # Vulkan specific API version creation
             case 'vulkan':
                 nameApi = name.replace('VK_', 'VK_API_')
                 nameString = f'"{name}"'
-                return Version(name, nameString, nameApi, number)
+                return Version(name, nameString, nameApi)
 
 
 # This Generator Option is used across all generators.
@@ -305,6 +305,11 @@ class BaseGenerator(OutputGenerator):
             for required in dict:
                 for group in dict[required]:
                     for structName in dict[required][group]:
+                        isAlias = structName in self.structAliasMap
+                        structName = self.structAliasMap[structName] if isAlias else structName
+                        # An EXT struct can alias a KHR struct,
+                        # that in turns aliaes a core struct
+                        # => Try to propagate aliasing, it can safely result in a no-op
                         isAlias = structName in self.structAliasMap
                         structName = self.structAliasMap[structName] if isAlias else structName
                         if structName in self.vk.structs:
@@ -402,7 +407,7 @@ class BaseGenerator(OutputGenerator):
         else: # version
             number = interface.get('number')
             if number != '1.0':
-                self.currentVersion = APISpecific.createApiVersion(self.targetApiName, name, number)
+                self.currentVersion = APISpecific.createApiVersion(self.targetApiName, name)
                 self.vk.versions[name] = self.currentVersion
 
     def endFeature(self):
