@@ -1,6 +1,6 @@
 #!/usr/bin/python3 -i
 #
-# Copyright (c) 2021-2023 The Khronos Group Inc.
+# Copyright (c) 2021-2024 The Khronos Group Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@ import os
 import re
 import json
 from generators.base_generator import BaseGenerator
+from generators.generator_utils import IsNonVulkanSprivCapability
 
 #
 # Generate SPIR-V grammar helper for SPIR-V opcodes, enums, etc
@@ -32,8 +33,6 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
         self.opnames = []
         self.atomicsOps = []
         self.groupOps = []
-        self.debugOps = []
-        self.annotationOps = []
         self.imageGatherOps = []
         self.imageSampleOps = []
         self.imageFetchOps = []
@@ -47,6 +46,7 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
         self.cooperativeMatrixList = []
         self.hasType = []
         self.hasResult = []
+        self.provisionalList = []
         # Need range to be large as largest possible operand index
         # This is done to make it easier to group switch case of same value
         self.imageOperandsParamCount = [[] for i in range(3)]
@@ -65,11 +65,9 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
 
     def addToStringList(self, operandKind, kind, list, ignoreList = []):
         if operandKind['kind'] == kind:
-            values = [] # prevent alias from being duplicatd
             for enum in operandKind['enumerants']:
-                if enum['value'] not in values and enum['enumerant'] not in ignoreList:
+                if enum['enumerant'] not in ignoreList:
                     list.append(enum['enumerant'])
-                    values.append(enum['value'])
 
     #
     # Takes the SPIR-V Grammar JSON and parses it
@@ -83,18 +81,6 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
             instructions = data['instructions']
             operandKinds = data['operand_kinds']
 
-            # Build list from json of all capabilities that are only for kernel
-            # This needs to be done before loop instructions
-            kernelCapability = ['Kernel']
-            # some SPV_INTEL_* are not allowed in Vulkan and are just adding unused opcodes
-            # TODO bring in vk.xml to cross check valid extensions/capabilities instead of starting another hardcoded list
-            kernelCapability.append('ArbitraryPrecisionIntegersINTEL')
-            kernelCapability.append('ArbitraryPrecisionFixedPointINTEL')
-            kernelCapability.append('ArbitraryPrecisionFloatingPointINTEL')
-            kernelCapability.append('SubgroupAvcMotionEstimationINTEL')
-            kernelCapability.append('SubgroupAvcMotionEstimationIntraINTEL')
-            kernelCapability.append('SubgroupAvcMotionEstimationChromaINTEL')
-
             for operandKind in operandKinds:
                 kind = operandKind['kind']
                 category = operandKind['category']
@@ -110,17 +96,11 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 elif category == 'BitEnum':
                     self.kindBitEnum.append(kind)
 
-                if kind == 'Capability':
-                    for enum in operandKind['enumerants']:
-                        if 'capabilities' in enum and len(enum['capabilities']) == 1 and enum['capabilities'][0] == 'Kernel':
-                            kernelCapability.append(enum['enumerant'])
-                elif kind == 'ImageOperands':
-                    values = [] # prevent alias from being duplicatd
+                if kind == 'ImageOperands':
                     for enum in operandKind['enumerants']:
                         count = 0  if 'parameters' not in enum else len(enum['parameters'])
-                        if enum['value'] not in values:
-                            self.imageOperandsParamCount[count].append(enum['enumerant'])
-                            values.append(enum['value'])
+                        self.imageOperandsParamCount[count].append(enum['enumerant'])
+
                 self.addToStringList(operandKind, 'StorageClass', self.storageClassList)
                 self.addToStringList(operandKind, 'ExecutionModel', self.executionModelList)
                 self.addToStringList(operandKind, 'ExecutionMode', self.executionModeList)
@@ -129,33 +109,22 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 self.addToStringList(operandKind, 'Dim', self.dimList)
                 self.addToStringList(operandKind, 'CooperativeMatrixOperands', self.cooperativeMatrixList, ['NoneKHR'])
 
-            # Currently no alias field, so need to manually track alias commands
-            # https://github.com/KhronosGroup/SPIRV-Headers/issues/109
-            aliasOpname = [
-                'OpSDotKHR',
-                'OpUDotKHR',
-                'OpSUDotKHR',
-                'OpSDotAccSatKHR',
-                'OpUDotAccSatKHR',
-                'OpSUDotAccSatKHR',
-                'OpReportIntersectionKHR',
-                'OpTypeAccelerationStructureNV',
-                'OpDemoteToHelperInvocationEXT',
-                'OpDecorateStringGOOGLE',
-                'OpMemberDecorateStringGOOGLE',
-            ]
+                if 'enumerants' in operandKind:
+                    for enum in operandKind['enumerants']:
+                        if 'provisional' in enum:
+                            self.provisionalList.append(enum['enumerant'])
 
             for instruction in instructions:
                 opname = instruction['opname']
                 opcode = instruction['opcode']
 
-                if opname in aliasOpname:
-                    continue
+                if 'provisional' in instruction:
+                    self.provisionalList.append(opname)
 
                 if 'capabilities' in instruction:
                     notSupported = True
                     for capability in instruction['capabilities']:
-                        if capability not in kernelCapability:
+                        if not IsNonVulkanSprivCapability(capability):
                             notSupported = False
                             break
                     if notSupported:
@@ -177,10 +146,6 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                     self.atomicsOps.append(opname)
                 if instruction['class'] == 'Non-Uniform':
                     self.groupOps.append(opname)
-                if instruction['class'] == 'Debug':
-                    self.debugOps.append(opname)
-                if instruction['class'] == 'Annotation':
-                    self.annotationOps.append(opname)
                 if re.search("OpImage.*Gather", opname) is not None:
                     self.imageGatherOps.append(opname)
                 if re.search("OpImageFetch.*", opname) is not None:
@@ -188,7 +153,9 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 if re.search("OpImageSample.*", opname) is not None:
                     self.imageSampleOps.append(opname)
                 if re.search("OpType.*", opname) is not None:
-                    self.typeOps.append(opname)
+                    # Currently this is for GPU-AV which doesn't supporrt provisional extensions
+                    if opname not in self.provisionalList:
+                        self.typeOps.append(opname)
                 if 'operands' in instruction:
                     for index, operand in enumerate(instruction['operands']):
                         kind = operand['kind']
@@ -206,22 +173,14 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
 
                             operands = self.opcodes[opcode]['operands']
                             if kind in self.kindId:
-                                if opname in ['OpLoopMerge', 'OpSelectionMerge', 'OpBranch']:
-                                    operands.append('Label')
-                                elif opname == 'OpBranchConditional' and 'Condition' not in operand['name']:
-                                    operands.append('Label')
-                                else:
-                                    operands.append('Id')
+                                operands.append('Id')
                             elif kind in self.kindLiteral:
                                 if kind == 'LiteralString':
                                     operands.append('LiteralString')
                                 else:
                                     operands.append('Literal')
                             elif kind in self.kindComposite:
-                                if opname == 'OpSwitch':
-                                    operands.append('Label')
-                                else:
-                                    operands.append('Composite')
+                                operands.append('Composite')
                             elif kind in self.kindValueEnum:
                                 operands.append('ValueEnum')
                             elif kind in self.kindBitEnum:
@@ -261,6 +220,22 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                         self.imageAccessOperand[imageRef].append(opname)
                     elif sampledImageRef != 0:
                         self.imageAccessOperand[sampledImageRef].append(opname)
+                # exceptions that don't fit the OpImage naming
+                if opname == 'OpFragmentFetchAMD' or opname == 'OpFragmentMaskFetchAMD':
+                    self.imageAccessOperand[3].append(opname)
+
+                # We want to manually mark "Label" if an ID is used for Control Flow
+                # It is easier to manage the few cases here then complex the operand logic above
+                if opname == 'OpLoopMerge':
+                    self.opcodes[opcode]['operands'] = ['Label', 'Label', 'BitEnum']
+                if opname == 'OpSelectionMerge':
+                    self.opcodes[opcode]['operands'] = ['Label', 'BitEnum']
+                if opname == 'OpBranch':
+                    self.opcodes[opcode]['operands'] = ['Label']
+                if opname == 'OpBranchConditional':
+                    self.opcodes[opcode]['operands'] = ['Id', 'Label', 'Label', 'Literal']
+                if opname == 'OpSwitch':
+                    self.opcodes[opcode]['operands'] = ['Id', 'Label', 'Label']
 
     def generate(self):
         self.write(f'''// *** THIS FILE IS GENERATED - DO NOT EDIT ***
@@ -268,7 +243,7 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
 
             /***************************************************************************
             *
-            * Copyright (c) 2021-2023 The Khronos Group Inc.
+            * Copyright (c) 2021-2024 The Khronos Group Inc.
             *
             * Licensed under the Apache License, Version 2.0 (the "License");
             * you may not use this file except in compliance with the License.
@@ -316,12 +291,17 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
             std::string string_SpvCooperativeMatrixOperands(uint32_t mask);
             ''')
 
-        hasTypeCase = "\n".join([f"        case spv::{f}:" for f in self.hasType])
-        hasResultCase = "\n".join([f"        case spv::{f}:" for f in self.hasResult])
+        hasTypeCase = "\n".join([f"        case spv::{f}:" for f in self.hasType if f not in self.provisionalList])
+        hasTypeCaseProvisional = "\n".join([f"        case spv::{f}:" for f in self.hasType if f in self.provisionalList])
+        hasResultCase = "\n".join([f"        case spv::{f}:" for f in self.hasResult if f not in self.provisionalList])
+        hasResultCaseProvisional = "\n".join([f"        case spv::{f}:" for f in self.hasResult if f in self.provisionalList])
         out.append(f'''
             static constexpr bool OpcodeHasType(uint32_t opcode) {{
                 switch (opcode) {{
             {hasTypeCase}
+            #ifdef VK_ENABLE_BETA_EXTENSIONS
+            {hasTypeCaseProvisional}
+            #endif
                         return true;
                     default:
                         return false;
@@ -331,6 +311,9 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
             static constexpr bool OpcodeHasResult(uint32_t opcode) {{
                 switch (opcode) {{
             {hasResultCase}
+            #ifdef VK_ENABLE_BETA_EXTENSIONS
+            {hasResultCaseProvisional}
+            #endif
                         return true;
                     default:
                         return false;
@@ -341,8 +324,6 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
         # \n is not allowed in f-string until 3.12
         atomicCase = "\n".join([f"        case spv::{f}:" for f in self.atomicsOps])
         groupCase = "\n".join([f"        case spv::{f}:" for f in self.groupOps])
-        debugCase = "\n".join([f"        case spv::{f}:" for f in self.debugOps])
-        annotationCase = "\n".join([f"        case spv::{f}:" for f in self.annotationOps])
         out.append(f'''
             // Any non supported operation will be covered with other VUs
             static constexpr bool AtomicOperation(uint32_t opcode) {{
@@ -363,24 +344,6 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                         return false;
                 }}
             }}
-
-           static constexpr  bool DebugOperation(uint32_t opcode) {{
-                switch (opcode) {{
-            {debugCase}
-                        return true;
-                    default:
-                        return false;
-                }}
-            }}
-
-            static constexpr bool AnnotationOperation(uint32_t opcode) {{
-                switch (opcode) {{
-            {annotationCase}
-                        return true;
-                    default:
-                        return false;
-                }}
-            }}
             ''')
 
         imageGatherOpsCase = "\n".join([f"        case spv::{f}:" for f in self.imageGatherOps])
@@ -536,6 +499,7 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
 
         out.append('''
             enum class OperandKind {
+                Invalid = 0,
                 Id,
                 Label, // Id but for Control Flow
                 Literal,
@@ -566,7 +530,11 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 switch(opcode) {{
             {"".join([f"""        case spv::{x}:
                         return "{x}";
-            """ for x in self.opnames])}
+            """ for x in self.opnames if x not in self.provisionalList])}
+#ifdef VK_ENABLE_BETA_EXTENSIONS
+            {"".join([f"""        case spv::{x}:
+                        return "{x}";
+            """ for x in self.opnames if x in self.provisionalList])}#endif
                     default:
                         return "Unknown Opcode";
                 }}
@@ -576,7 +544,11 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 switch(storage_class) {{
             {"".join([f"""        case spv::StorageClass{x}:
                         return "{x}";
-            """ for x in self.storageClassList])}
+            """ for x in self.storageClassList if x not in self.provisionalList])}
+#ifdef VK_ENABLE_BETA_EXTENSIONS
+            {"".join([f"""        case spv::StorageClass{x}:
+                        return "{x}";
+            """ for x in self.storageClassList if x in self.provisionalList])}#endif
                     default:
                         return "Unknown Storage Class";
                 }}
@@ -586,7 +558,11 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 switch(execution_model) {{
             {"".join([f"""        case spv::ExecutionModel{x}:
                         return "{x}";
-            """ for x in self.executionModelList])}
+            """ for x in self.executionModelList if x not in self.provisionalList])}
+#ifdef VK_ENABLE_BETA_EXTENSIONS
+            {"".join([f"""        case spv::ExecutionModel{x}:
+                        return "{x}";
+            """ for x in self.executionModelList if x in self.provisionalList])}#endif
                     default:
                         return "Unknown Execution Model";
                 }}
@@ -596,7 +572,11 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 switch(execution_mode) {{
             {"".join([f"""        case spv::ExecutionMode{x}:
                         return "{x}";
-            """ for x in self.executionModeList])}
+            """ for x in self.executionModeList if x not in self.provisionalList])}
+#ifdef VK_ENABLE_BETA_EXTENSIONS
+            {"".join([f"""        case spv::ExecutionMode{x}:
+                        return "{x}";
+            """ for x in self.executionModeList if x in self.provisionalList])}#endif
                     default:
                         return "Unknown Execution Mode";
                 }}
@@ -606,7 +586,11 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 switch(decoration) {{
             {"".join([f"""        case spv::Decoration{x}:
                         return "{x}";
-            """ for x in self.decorationList])}
+            """ for x in self.decorationList if x not in self.provisionalList])}
+#ifdef VK_ENABLE_BETA_EXTENSIONS
+            {"".join([f"""        case spv::Decoration{x}:
+                        return "{x}";
+            """ for x in self.decorationList if x in self.provisionalList])}#endif
                     default:
                         return "Unknown Decoration";
                 }}
@@ -616,7 +600,11 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 switch(built_in) {{
             {"".join([f"""        case spv::BuiltIn{x}:
                         return "{x}";
-            """ for x in self.builtInList])}
+            """ for x in self.builtInList if x not in self.provisionalList])}
+#ifdef VK_ENABLE_BETA_EXTENSIONS
+            {"".join([f"""        case spv::BuiltIn{x}:
+                        return "{x}";
+            """ for x in self.builtInList if x in self.provisionalList])}#endif
                     default:
                         return "Unknown BuiltIn";
                 }}
@@ -626,7 +614,7 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 switch(dim) {{
             {"".join([f"""        case spv::Dim{x}:
                         return "{x}";
-            """ for x in self.dimList])}
+            """ for x in self.dimList if x not in self.provisionalList])}
                     default:
                         return "Unknown Dim";
                 }}
@@ -635,10 +623,10 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
             static const char* string_SpvCooperativeMatrixOperandsMask(spv::CooperativeMatrixOperandsMask mask) {{
                 switch(mask) {{
                     case spv::CooperativeMatrixOperandsMaskNone:
-                        return "None";
+                        return "NoneKHR";
             {"".join([f"""        case spv::CooperativeMatrixOperands{x}Mask:
                         return "{x}";
-            """ for x in self.cooperativeMatrixList])}
+            """ for x in self.cooperativeMatrixList if x not in self.provisionalList])}
                     default:
                         return "Unknown CooperativeMatrixOperandsMask";
                 }}
@@ -665,6 +653,8 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 // clang-format off\n''')
         for info in self.opcodes.values():
             opname = info['opname']
+            if opname in self.provisionalList:
+                continue # Currently this is for GPU-AV which doesn't supporrt provisional extensions
             kinds = ", ".join([f"OperandKind::{f}" for f in info['operands']])
             out.append(f'        {{spv::{opname}, {{{{{kinds}}}}}}},\n')
         out.append('''    }; // clang-format on
