@@ -1,6 +1,6 @@
 #!/usr/bin/python3 -i
 #
-# Copyright (c) 2020-2023 The Khronos Group Inc.
+# Copyright (c) 2020-2024 The Khronos Group Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,32 +16,32 @@
 
 import sys
 import os
+import json
 from generators.vulkan_object import SpirvEnables
 from generators.base_generator import BaseGenerator
+from generators.generator_utils import IsNonVulkanSprivCapability
 
 #
 # Generate SPIR-V validation for SPIR-V extensions and capabilities
 class SpirvValidationHelperOutputGenerator(BaseGenerator):
-    def __init__(self):
+    def __init__(self, grammar):
         BaseGenerator.__init__(self)
 
         # Sometimes the Vulkan-Headers XML will mention new SPIR-V capability or extensions
-        # That require an update of the SPIRV-Headers which might not be ready to pull in.
-        # These 2 arrays SHOULD be empty when possible and when the SPIR-V Headers are updated these
-        # should be attempted to be cleared
-        self.capabilityExcludeList = [
-            'ClusterCullingShadingHUAWEI',
-            'ShaderEnqueueAMDX',
-            'TextureBlockMatch2QCOM',
-        ]
-
-        # There are some enums that share the same value in the SPIR-V header.
-        # This array remove the duplicate to not print out, usually due to being the older value given
-        self.capabilityAliasList = [
-          'ShaderViewportIndexLayerNV',
-          'ShadingRateNV',
-          'FragmentBarycentricNV',
-        ]
+        # that require an update of the SPIRV-Headers which might not be ready to pull in.
+        # Get the list of safe enum values to use from the SPIR-V grammar
+        self.capabilityList = []
+        self.provisionalList = []
+        with open(grammar) as grammar_file:
+            grammar_dict = json.load(grammar_file)
+        for kind in grammar_dict['operand_kinds']:
+            if kind['kind'] == 'Capability':
+                for enum in kind['enumerants']:
+                    if not IsNonVulkanSprivCapability(enum['enumerant']):
+                        self.capabilityList.append(enum['enumerant'])
+                    if 'provisional' in enum:
+                        self.provisionalList.append(enum['enumerant'])
+                break
 
         # Promoted features structure in state_tracker.cpp are put in the VkPhysicalDeviceVulkan*Features structs
         # but the XML can still list them. This list all promoted structs to ignore since they are aliased.
@@ -79,6 +79,7 @@ class SpirvValidationHelperOutputGenerator(BaseGenerator):
         self.propertyMap = {
             'VkPhysicalDeviceVulkan11Properties' : 'phys_dev_props_core11',
             'VkPhysicalDeviceVulkan12Properties' : 'phys_dev_props_core12',
+            'VkPhysicalDeviceVulkan13Properties' : 'phys_dev_props_core13', # Not used, but left for grep searching
         }
 
     #
@@ -120,7 +121,7 @@ class SpirvValidationHelperOutputGenerator(BaseGenerator):
 
             /***************************************************************************
             *
-            * Copyright (c) 2020-2023 The Khronos Group Inc.
+            * Copyright (c) 2020-2024 The Khronos Group Inc.
             *
             * Licensed under the Apache License, Version 2.0 (the "License");
             * you may not use this file except in compliance with the License.
@@ -146,8 +147,7 @@ class SpirvValidationHelperOutputGenerator(BaseGenerator):
             #include <functional>
             #include <spirv/unified1/spirv.hpp>
             #include "vk_extension_helper.h"
-            #include "state_tracker/shader_module.h"
-            #include "state_tracker/device_state.h"
+            #include "state_tracker/shader_instruction.h"
             #include "core_checks/core_validation.h"
             ''')
 
@@ -182,36 +182,49 @@ class SpirvValidationHelperOutputGenerator(BaseGenerator):
 
         #
         # Build the struct with all the requirments for the spirv capabilities
+        out.append('const std::unordered_multimap<uint32_t, RequiredSpirvInfo>& GetSpirvCapabilites() {\n')
         out.append('// clang-format off\n')
-        out.append('static const std::unordered_multimap<uint32_t, RequiredSpirvInfo> spirvCapabilities = {\n')
+        out.append('    static const std::unordered_multimap<uint32_t, RequiredSpirvInfo> spirv_capabilities = {')
         for spirv in [x for x in self.vk.spirv if x.capability]:
             for enable in [x for x in spirv.enable if x.struct is None or x.struct not in self.promotedFeatures]:
-                if spirv.name in self.capabilityExcludeList:
-                    out.append('    // Not found in current SPIR-V Headers\n    //')
-                out.append(f'    {{spv::Capability{spirv.name}, {self.createMapValue(spirv.name, enable, False)}}},\n')
-        out.append('};\n')
+                if spirv.name not in self.capabilityList:
+                    out.append('\n        // Not found in current SPIR-V Headers\n        // ')
+                elif spirv.name in self.provisionalList:
+                    out.append('\n#ifdef VK_ENABLE_BETA_EXTENSIONS\n        ')
+                else:
+                    out.append('\n        ')
+                out.append(f'{{spv::Capability{spirv.name}, {self.createMapValue(spirv.name, enable, False)}}},')
+
+                out.append('\n#endif') if spirv.name in self.provisionalList else None
+        out.append('\n    };\n')
         out.append('// clang-format on\n')
+        out.append('    return spirv_capabilities;\n')
+        out.append('};\n')
         out.append('\n')
 
         #
         # Build the struct with all the requirments for the spirv extensions
+        out.append('const std::unordered_multimap<std::string_view, RequiredSpirvInfo>& GetSpirvExtensions() {\n')
         out.append('// clang-format off\n')
-        out.append('static const std::unordered_multimap<std::string_view, RequiredSpirvInfo> spirvExtensions = {\n')
+        out.append('    static const std::unordered_multimap<std::string_view, RequiredSpirvInfo> spirv_extensions = {')
         for spirv in [x for x in self.vk.spirv if x.extension]:
             for enable in spirv.enable:
-                out.append(f'    {{"{spirv.name}", {self.createMapValue(spirv.name, enable, True)}}},\n')
-        out.append('};\n')
+                out.append(f'\n        {{"{spirv.name}", {self.createMapValue(spirv.name, enable, True)}}},')
+        out.append('\n    };\n')
         out.append('// clang-format on\n')
+        out.append('    return spirv_extensions;\n')
+        out.append('}\n')
         out.append('\n')
 
         #
         # Creates the Enum string helpers for better error messages. Same idea of vk_enum_string_helper.h but for SPIR-V
         out.append('static inline const char* string_SpvCapability(uint32_t input_value) {\n')
         out.append('    switch ((spv::Capability)input_value) {\n')
-        for spirv in [x for x in self.vk.spirv if x.capability]:
-            if (spirv.name not in self.capabilityAliasList) and (spirv.name not in self.capabilityExcludeList):
-                out.append(f'         case spv::Capability{spirv.name}:\n')
-                out.append(f'            return "{spirv.name}";\n')
+        for name in self.capabilityList:
+            out.append('#ifdef VK_ENABLE_BETA_EXTENSIONS\n') if name in self.provisionalList else None
+            out.append(f'         case spv::Capability{name}:\n')
+            out.append(f'            return "{name}";\n')
+            out.append('#endif\n') if name in self.provisionalList else None
         out.append('        default:\n')
         out.append('            return \"Unhandled OpCapability\";\n')
         out.append('    };\n')
@@ -240,7 +253,7 @@ class SpirvValidationHelperOutputGenerator(BaseGenerator):
 static inline const char* SpvCapabilityRequirements(uint32_t capability) {
     static const vvl::unordered_map<uint32_t, std::string_view> table {
 ''')
-        for spirv in [x for x in self.vk.spirv if x.capability and x.name not in self.capabilityExcludeList]:
+        for spirv in [x for x in self.vk.spirv if x.capability and x.name in self.capabilityList]:
             requirment = ''
             for index, enable in enumerate([x for x in spirv.enable if x.struct is None or x.struct not in self.promotedFeatures]):
                 requirment += ' OR ' if (index != 0) else ''
@@ -252,7 +265,10 @@ static inline const char* SpvCapabilityRequirements(uint32_t capability) {
                     requirment += enable.extension
                 elif enable.property is not None:
                     requirment += f'({enable.property}::{enable.member} == {enable.value})'
+
+            out.append('#ifdef VK_ENABLE_BETA_EXTENSIONS\n') if spirv.name in self.provisionalList else None
             out.append(f'    {{spv::Capability{spirv.name}, "{requirment}"}},\n')
+            out.append('#endif  // VK_ENABLE_BETA_EXTENSIONS\n') if spirv.name in self.provisionalList else None
         out.append('''    };
 
     // VUs before catch unknown capabilities
@@ -264,27 +280,26 @@ static inline const char* SpvCapabilityRequirements(uint32_t capability) {
 
         out.append('''
 // clang-format off
-static inline const char* SpvExtensionRequirments(std::string_view extension) {
-    static const vvl::unordered_map<std::string_view, std::string_view> table {
+static inline std::string SpvExtensionRequirments(std::string_view extension) {
+    static const vvl::unordered_map<std::string_view, vvl::Requirements> table {
 ''')
         for spirv in [x for x in self.vk.spirv if x.extension]:
             requirment = ''
             for index, enable in enumerate(spirv.enable):
-                requirment += ' OR ' if (index != 0) else ''
+                requirment += ', ' if (index != 0) else ''
                 if enable.version is not None:
-                    requirment += enable.version
-                elif enable.feature is not None:
-                    requirment += f'{enable.struct}::{enable.feature}'
+                    requirment += f'{{vvl::Version::_{enable.version}}}'
                 elif enable.extension is not None:
-                    requirment += enable.extension
-                elif enable.property is not None:
-                    requirment += f'({enable.property}::{enable.member} == {enable.value})'
-            out.append(f'    {{"{spirv.name}", "{requirment}"}},\n')
+                    requirment += f'{{vvl::Extension::_{enable.extension}}}'
+                elif enable.feature is not None or enable.property is not None:
+                    print("Need to add support for feature/properties in spirv extensions")
+                    sys.exit(1)
+            out.append(f'    {{"{spirv.name}", {{{requirment}}}}},\n')
         out.append('''    };
 
     // VUs before catch unknown extensions
     const auto entry = table.find(extension);
-    return entry->second.data();
+    return String(entry->second);
 }
 // clang-format on
 ''')
@@ -299,12 +314,13 @@ static inline const char* SpvExtensionRequirments(std::string_view extension) {
         #
         # The main function to validate all the extensions and capabilities
         out.append('''
-            bool CoreChecks::ValidateShaderCapabilitiesAndExtensions(const spirv::Instruction &insn, const bool pipeline, const Location& loc) const {
+            bool CoreChecks::ValidateShaderCapabilitiesAndExtensions(const spirv::Instruction &insn,  const Location& loc) const {
                 bool skip = false;
+                const bool pipeline = loc.function != vvl::Func::vkCreateShadersEXT;
 
                 if (insn.Opcode() == spv::OpCapability) {
                     // All capabilities are generated so if it is not in the list it is not supported by Vulkan
-                    if (spirvCapabilities.count(insn.Word(1)) == 0) {
+                    if (GetSpirvCapabilites().count(insn.Word(1)) == 0) {
                         const char *vuid = pipeline ? "VUID-VkShaderModuleCreateInfo-pCode-08739" : "VUID-VkShaderCreateInfoEXT-pCode-08739";
                         skip |= LogError(vuid, device, loc,
                             "SPIR-V has Capability (%s) declared, but this is not supported by Vulkan.", string_SpvCapability(insn.Word(1)));
@@ -314,7 +330,7 @@ static inline const char* SpvExtensionRequirments(std::string_view extension) {
                     // Each capability has one or more requirements to check
                     // Only one item has to be satisfied and an error only occurs
                     // when all are not satisfied
-                    auto caps = spirvCapabilities.equal_range(insn.Word(1));
+                    auto caps = GetSpirvCapabilites().equal_range(insn.Word(1));
                     bool has_support = false;
                     for (auto it = caps.first; (it != caps.second) && (has_support == false); ++it) {
                         if (it->second.version) {
@@ -381,7 +397,7 @@ static inline const char* SpvExtensionRequirments(std::string_view extension) {
                 std::string extension_name = insn.GetAsString(1);
 
                 if (0 == extension_name.compare(0, spv_prefix.size(), spv_prefix)) {
-                    if (spirvExtensions.count(extension_name) == 0) {
+                    if (GetSpirvExtensions().count(extension_name) == 0) {
                         const char *vuid = pipeline ? "VUID-VkShaderModuleCreateInfo-pCode-08741" : "VUID-VkShaderCreateInfoEXT-pCode-08741";
                         skip |= LogError(vuid, device,loc,
                             "SPIR-V Extension %s was declared, but that is not supported by Vulkan.", extension_name.c_str());
@@ -399,7 +415,7 @@ static inline const char* SpvExtensionRequirments(std::string_view extension) {
                 // Each SPIR-V Extension has one or more requirements to check
                 // Only one item has to be satisfied and an error only occurs
                 // when all are not satisfied
-                auto ext = spirvExtensions.equal_range(extension_name);
+                auto ext = GetSpirvExtensions().equal_range(extension_name);
                 bool has_support = false;
                 for (auto it = ext.first; (it != ext.second) && (has_support == false); ++it) {
                     if (it->second.version) {
@@ -420,7 +436,7 @@ static inline const char* SpvExtensionRequirments(std::string_view extension) {
                 if (has_support == false) {
                     const char *vuid = pipeline ? "VUID-VkShaderModuleCreateInfo-pCode-08742" : "VUID-VkShaderCreateInfoEXT-pCode-08742";
                     skip |= LogError(vuid, device, loc,
-                        "SPIR-V Extension %s was declared, but one of the following requirements is required (%s).", extension_name.c_str(), SpvExtensionRequirments(extension_name));
+                        "SPIR-V Extension %s was declared, but one of the following requirements is required (%s).", extension_name.c_str(), SpvExtensionRequirments(extension_name).c_str());
                 }
             } //spv::OpExtension
             return skip;
