@@ -14,9 +14,10 @@
 
 #include "../framework/layer_validation_tests.h"
 
+class NegativeDebugExtensions : public VkLayerTest {};
+
 TEST_F(NegativeDebugExtensions, DebugMarkerName) {
     TEST_DESCRIPTION("Ensure debug marker object names are printed in debug report output");
-
     AddRequiredExtensions(VK_EXT_DEBUG_REPORT_EXTENSION_NAME);
     AddRequiredExtensions(VK_EXT_DEBUG_MARKER_EXTENSION_NAME);
     RETURN_IF_SKIP(Init());
@@ -25,46 +26,35 @@ TEST_F(NegativeDebugExtensions, DebugMarkerName) {
         GTEST_SKIP() << "Skipping object naming test with MockICD.";
     }
 
-    VkBuffer buffer;
-    VkDeviceMemory memory_1, memory_2;
     std::string memory_name = "memory_name";
 
     VkBufferCreateInfo buffer_create_info = vku::InitStructHelper();
     buffer_create_info.usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT;
     buffer_create_info.size = 1;
-
-    vk::CreateBuffer(device(), &buffer_create_info, nullptr, &buffer);
+    vkt::Buffer buffer(*m_device, buffer_create_info, vkt::no_mem);
 
     VkMemoryRequirements memRequirements;
-    vk::GetBufferMemoryRequirements(device(), buffer, &memRequirements);
+    vk::GetBufferMemoryRequirements(device(), buffer.handle(), &memRequirements);
 
     VkMemoryAllocateInfo memory_allocate_info = vku::InitStructHelper();
     memory_allocate_info.allocationSize = memRequirements.size;
     memory_allocate_info.memoryTypeIndex = 0;
 
-    vk::AllocateMemory(device(), &memory_allocate_info, nullptr, &memory_1);
-    vk::AllocateMemory(device(), &memory_allocate_info, nullptr, &memory_2);
-
+    vkt::DeviceMemory memory_1(*m_device, memory_allocate_info);
+    vkt::DeviceMemory memory_2(*m_device, memory_allocate_info);
     VkDebugMarkerObjectNameInfoEXT name_info = vku::InitStructHelper();
-    name_info.object = (uint64_t)memory_2;
+    name_info.object = (uint64_t)memory_2.handle();
     name_info.objectType = VK_DEBUG_REPORT_OBJECT_TYPE_DEVICE_MEMORY_EXT;
     name_info.pObjectName = memory_name.c_str();
     vk::DebugMarkerSetObjectNameEXT(device(), &name_info);
 
-    vk::BindBufferMemory(device(), buffer, memory_1, 0);
+    vk::BindBufferMemory(device(), buffer.handle(), memory_1.handle(), 0);
 
     // Test core_validation layer
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, memory_name);
-    vk::BindBufferMemory(device(), buffer, memory_2, 0);
+    m_errorMonitor->SetDesiredError(memory_name.c_str());
+    vk::BindBufferMemory(device(), buffer.handle(), memory_2.handle(), 0);
     m_errorMonitor->VerifyFound();
 
-    vk::FreeMemory(device(), memory_1, nullptr);
-    memory_1 = VK_NULL_HANDLE;
-    vk::FreeMemory(device(), memory_2, nullptr);
-    memory_2 = VK_NULL_HANDLE;
-    vk::DestroyBuffer(device(), buffer, nullptr);
-    buffer = VK_NULL_HANDLE;
-
     VkCommandBuffer commandBuffer;
     std::string commandBuffer_name = "command_buffer_name";
     VkCommandPoolCreateInfo pool_create_info = vku::InitStructHelper();
@@ -92,16 +82,52 @@ TEST_F(NegativeDebugExtensions, DebugMarkerName) {
     const VkRect2D scissors[] = {scissor, scissor};
 
     // Test parameter_validation layer
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, commandBuffer_name);
+    m_errorMonitor->SetDesiredError(commandBuffer_name.c_str());
     vk::CmdSetScissor(commandBuffer, 0, 1, scissors);
     m_errorMonitor->VerifyFound();
 
     // Test object_tracker layer
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, commandBuffer_name);
+    m_errorMonitor->SetDesiredError(commandBuffer_name.c_str());
     vk::FreeCommandBuffers(device(), command_pool_2.handle(), 1, &commandBuffer);
     m_errorMonitor->VerifyFound();
 }
 
+TEST_F(NegativeDebugExtensions, DebugMarkerSetObject) {
+    AddRequiredExtensions(VK_EXT_DEBUG_REPORT_EXTENSION_NAME);
+    AddRequiredExtensions(VK_EXT_DEBUG_MARKER_EXTENSION_NAME);
+    RETURN_IF_SKIP(Init());
+
+    if (IsPlatformMockICD()) {
+        GTEST_SKIP() << "Skipping object naming test with MockICD.";
+    }
+
+    std::string memory_name = "memory_name";
+
+    VkMemoryAllocateInfo memory_allocate_info = vku::InitStructHelper();
+    memory_allocate_info.allocationSize = 64;
+    memory_allocate_info.memoryTypeIndex = 0;
+    vkt::DeviceMemory memory(*m_device, memory_allocate_info);
+
+    VkDebugMarkerObjectNameInfoEXT name_info = vku::InitStructHelper();
+    name_info.object = (uint64_t)VK_NULL_HANDLE;
+    name_info.objectType = VK_DEBUG_REPORT_OBJECT_TYPE_DEVICE_MEMORY_EXT;
+    name_info.pObjectName = memory_name.c_str();
+    m_errorMonitor->SetDesiredError("VUID-VkDebugMarkerObjectNameInfoEXT-object-01491");
+    vk::DebugMarkerSetObjectNameEXT(device(), &name_info);
+    m_errorMonitor->VerifyFound();
+
+    name_info.object = (uint64_t)memory.handle();
+    name_info.objectType = VK_DEBUG_REPORT_OBJECT_TYPE_UNKNOWN_EXT;
+    m_errorMonitor->SetDesiredError("VUID-VkDebugMarkerObjectNameInfoEXT-objectType-01490");
+    vk::DebugMarkerSetObjectNameEXT(device(), &name_info);
+    m_errorMonitor->VerifyFound();
+
+    name_info.objectType = VK_DEBUG_REPORT_OBJECT_TYPE_QUEUE_EXT;
+    m_errorMonitor->SetDesiredError("VUID-VkDebugMarkerObjectNameInfoEXT-object-01492");
+    vk::DebugMarkerSetObjectNameEXT(device(), &name_info);
+    m_errorMonitor->VerifyFound();
+}
+
 TEST_F(NegativeDebugExtensions, DebugUtilsName) {
     TEST_DESCRIPTION("Ensure debug utils object names are printed in debug messenger output");
 
@@ -128,45 +154,42 @@ TEST_F(NegativeDebugExtensions, DebugUtilsName) {
     VkDebugUtilsMessengerEXT my_messenger = VK_NULL_HANDLE;
     vk::CreateDebugUtilsMessengerEXT(instance(), &callback_create_info, nullptr, &my_messenger);
 
-    VkBuffer buffer;
-    VkDeviceMemory memory_1, memory_2;
     std::string memory_name = "memory_name";
 
     VkBufferCreateInfo buffer_create_info = vku::InitStructHelper();
     buffer_create_info.usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT;
     buffer_create_info.size = 1;
-
-    vk::CreateBuffer(device(), &buffer_create_info, nullptr, &buffer);
+    vkt::Buffer buffer(*m_device, buffer_create_info, vkt::no_mem);
 
     VkMemoryRequirements memRequirements;
-    vk::GetBufferMemoryRequirements(device(), buffer, &memRequirements);
+    vk::GetBufferMemoryRequirements(device(), buffer.handle(), &memRequirements);
 
     VkMemoryAllocateInfo memory_allocate_info = vku::InitStructHelper();
     memory_allocate_info.allocationSize = memRequirements.size;
     memory_allocate_info.memoryTypeIndex = 0;
 
-    vk::AllocateMemory(device(), &memory_allocate_info, nullptr, &memory_1);
-    vk::AllocateMemory(device(), &memory_allocate_info, nullptr, &memory_2);
+    vkt::DeviceMemory memory_1(*m_device, memory_allocate_info);
+    vkt::DeviceMemory memory_2(*m_device, memory_allocate_info);
 
     VkDebugUtilsObjectNameInfoEXT name_info = vku::InitStructHelper();
     name_info.objectType = VK_OBJECT_TYPE_DEVICE_MEMORY;
     name_info.pObjectName = memory_name.c_str();
 
     // Pass in bad handle make sure ObjectTracker catches it
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkDebugUtilsObjectNameInfoEXT-objectType-02590");
+    m_errorMonitor->SetDesiredError("VUID-VkDebugUtilsObjectNameInfoEXT-objectType-02590");
     name_info.objectHandle = (uint64_t)0xcadecade;
     vk::SetDebugUtilsObjectNameEXT(device(), &name_info);
     m_errorMonitor->VerifyFound();
 
     // Pass in null handle
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkSetDebugUtilsObjectNameEXT-pNameInfo-02588");
+    m_errorMonitor->SetDesiredError("VUID-vkSetDebugUtilsObjectNameEXT-pNameInfo-02588");
     name_info.objectHandle = 0;
     vk::SetDebugUtilsObjectNameEXT(device(), &name_info);
     m_errorMonitor->VerifyFound();
 
     // Pass in 'unknown' object type and see if parameter validation catches it
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkSetDebugUtilsObjectNameEXT-pNameInfo-02587");
-    name_info.objectHandle = (uint64_t)memory_2;
+    m_errorMonitor->SetDesiredError("VUID-vkSetDebugUtilsObjectNameEXT-pNameInfo-02587");
+    name_info.objectHandle = (uint64_t)memory_2.handle();
     name_info.objectType = VK_OBJECT_TYPE_UNKNOWN;
     vk::SetDebugUtilsObjectNameEXT(device(), &name_info);
     m_errorMonitor->VerifyFound();
@@ -174,20 +197,13 @@ TEST_F(NegativeDebugExtensions, DebugUtilsName) {
     name_info.objectType = VK_OBJECT_TYPE_DEVICE_MEMORY;
     vk::SetDebugUtilsObjectNameEXT(device(), &name_info);
 
-    vk::BindBufferMemory(device(), buffer, memory_1, 0);
+    vk::BindBufferMemory(device(), buffer.handle(), memory_1.handle(), 0);
 
     // Test core_validation layer
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, memory_name);
-    vk::BindBufferMemory(device(), buffer, memory_2, 0);
+    m_errorMonitor->SetDesiredError(memory_name.c_str());
+    vk::BindBufferMemory(device(), buffer.handle(), memory_2.handle(), 0);
     m_errorMonitor->VerifyFound();
 
-    vk::FreeMemory(device(), memory_1, nullptr);
-    memory_1 = VK_NULL_HANDLE;
-    vk::FreeMemory(device(), memory_2, nullptr);
-    memory_2 = VK_NULL_HANDLE;
-    vk::DestroyBuffer(device(), buffer, nullptr);
-    buffer = VK_NULL_HANDLE;
-
     VkCommandBuffer commandBuffer;
     std::string commandBuffer_name = "command_buffer_name";
     VkCommandPoolCreateInfo pool_create_info = vku::InitStructHelper();
@@ -233,7 +249,7 @@ TEST_F(NegativeDebugExtensions, DebugUtilsName) {
 
     vk::CmdInsertDebugUtilsLabelEXT(commandBuffer, &command_label);
     // Test parameter_validation layer
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, commandBuffer_name);
+    m_errorMonitor->SetDesiredError(commandBuffer_name.c_str());
     vk::CmdSetScissor(commandBuffer, 0, 1, scissors);
     m_errorMonitor->VerifyFound();
 
@@ -243,13 +259,50 @@ TEST_F(NegativeDebugExtensions, DebugUtilsName) {
     }
 
     // Test object_tracker layer
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, commandBuffer_name);
+    m_errorMonitor->SetDesiredError(commandBuffer_name.c_str());
     vk::FreeCommandBuffers(device(), command_pool_2.handle(), 1, &commandBuffer);
     m_errorMonitor->VerifyFound();
 
     vk::DestroyDebugUtilsMessengerEXT(instance(), my_messenger, nullptr);
 }
 
+TEST_F(NegativeDebugExtensions, DebugMarkerSetUtils) {
+    AddRequiredExtensions(VK_EXT_DEBUG_UTILS_EXTENSION_NAME);
+    AddRequiredExtensions(VK_EXT_DEBUG_MARKER_EXTENSION_NAME);
+    RETURN_IF_SKIP(Init());
+
+    if (IsPlatformMockICD()) {
+        GTEST_SKIP() << "Skipping object naming test with MockICD.";
+    }
+
+    VkMemoryAllocateInfo memory_allocate_info = vku::InitStructHelper();
+    memory_allocate_info.allocationSize = 64;
+    memory_allocate_info.memoryTypeIndex = 0;
+    vkt::DeviceMemory memory(*m_device, memory_allocate_info);
+
+    int tags[3] = {1, 2, 3};
+    VkDebugMarkerObjectTagInfoEXT name_info = vku::InitStructHelper();
+    name_info.object = (uint64_t)VK_NULL_HANDLE;
+    name_info.objectType = VK_DEBUG_REPORT_OBJECT_TYPE_DEVICE_MEMORY_EXT;
+    name_info.tagName = 1;
+    name_info.tagSize = 4;
+    name_info.pTag = tags;
+    m_errorMonitor->SetDesiredError("VUID-VkDebugMarkerObjectTagInfoEXT-object-01494");
+    vk::DebugMarkerSetObjectTagEXT(device(), &name_info);
+    m_errorMonitor->VerifyFound();
+
+    name_info.object = (uint64_t)memory.handle();
+    name_info.objectType = VK_DEBUG_REPORT_OBJECT_TYPE_UNKNOWN_EXT;
+    m_errorMonitor->SetDesiredError("VUID-VkDebugMarkerObjectTagInfoEXT-objectType-01493");
+    vk::DebugMarkerSetObjectTagEXT(device(), &name_info);
+    m_errorMonitor->VerifyFound();
+
+    name_info.objectType = VK_DEBUG_REPORT_OBJECT_TYPE_QUEUE_EXT;
+    m_errorMonitor->SetDesiredError("VUID-VkDebugMarkerObjectTagInfoEXT-object-01495");
+    vk::DebugMarkerSetObjectTagEXT(device(), &name_info);
+    m_errorMonitor->VerifyFound();
+}
+
 TEST_F(NegativeDebugExtensions, DebugUtilsParameterFlags) {
     AddRequiredExtensions(VK_EXT_DEBUG_UTILS_EXTENSION_NAME);
     RETURN_IF_SKIP(Init());
@@ -267,7 +320,7 @@ TEST_F(NegativeDebugExtensions, DebugUtilsParameterFlags) {
     callback_create_info.pfnUserCallback = DebugUtilsCallback;
     callback_create_info.pUserData = &callback_data;
     VkDebugUtilsMessengerEXT my_messenger = VK_NULL_HANDLE;
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkDebugUtilsMessengerCreateInfoEXT-messageSeverity-requiredbitmask");
+    m_errorMonitor->SetDesiredError("VUID-VkDebugUtilsMessengerCreateInfoEXT-messageSeverity-requiredbitmask");
     vk::CreateDebugUtilsMessengerEXT(instance(), &callback_create_info, nullptr, &my_messenger);
     m_errorMonitor->VerifyFound();
 }
@@ -326,7 +379,7 @@ TEST_F(NegativeDebugExtensions, SetDebugUtilsObjectSecondDevice) {
         GTEST_SKIP() << "Skipping object naming test with MockICD.";
     }
 
-    auto features = m_device->phy().features();
+    auto features = m_device->Physical().Features();
     vkt::Device second_device(gpu_, m_device_extension_names, &features, nullptr);
 
     DebugUtilsLabelCheckData callback_data;
@@ -349,9 +402,9 @@ TEST_F(NegativeDebugExtensions, SetDebugUtilsObjectSecondDevice) {
 
     VkDebugUtilsObjectNameInfoEXT name_info = vku::InitStructHelper();
     name_info.objectType = VK_OBJECT_TYPE_DEVICE;
-    name_info.objectHandle = (uint64_t)second_device.device();
+    name_info.objectHandle = (uint64_t)second_device.handle();
     name_info.pObjectName = object_name;
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-vkSetDebugUtilsObjectNameEXT-pNameInfo-07874");
+    m_errorMonitor->SetDesiredError("VUID-vkSetDebugUtilsObjectNameEXT-pNameInfo-07874");
     vk::SetDebugUtilsObjectNameEXT(device(), &name_info);
     m_errorMonitor->VerifyFound();
 
@@ -392,9 +445,143 @@ TEST_F(NegativeDebugExtensions, SetDebugUtilsObjectDestroyedHandle) {
     name_info.objectType = VK_OBJECT_TYPE_SAMPLER;
     name_info.objectHandle = bad_handle;
     name_info.pObjectName = object_name;
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkDebugUtilsObjectNameInfoEXT-objectType-02590");
+    m_errorMonitor->SetDesiredError("VUID-VkDebugUtilsObjectNameInfoEXT-objectType-02590");
     vk::SetDebugUtilsObjectNameEXT(device(), &name_info);
     m_errorMonitor->VerifyFound();
 
     vk::DestroyDebugUtilsMessengerEXT(instance(), my_messenger, nullptr);
-}
\ No newline at end of file
+}
+
+TEST_F(NegativeDebugExtensions, DebugLabelPrimaryCommandBuffer) {
+    TEST_DESCRIPTION("Test primary command buffer debug labels which are validated at submit time.");
+    AddRequiredExtensions(VK_EXT_DEBUG_UTILS_EXTENSION_NAME);
+    RETURN_IF_SKIP(Init());
+
+    m_command_buffer.Begin();
+    vk::CmdEndDebugUtilsLabelEXT(m_command_buffer);
+    m_command_buffer.End();
+
+    m_errorMonitor->SetDesiredError("VUID-vkCmdEndDebugUtilsLabelEXT-commandBuffer-01912");
+    m_default_queue->Submit(m_command_buffer);
+    m_errorMonitor->VerifyFound();
+    m_default_queue->Wait();
+}
+
+TEST_F(NegativeDebugExtensions, DebugLabelPrimaryCommandBuffer2) {
+    TEST_DESCRIPTION("Test primary command buffer debug labels which are validated at submit time.");
+    AddRequiredExtensions(VK_EXT_DEBUG_UTILS_EXTENSION_NAME);
+    RETURN_IF_SKIP(Init());
+
+    VkDebugUtilsLabelEXT label = vku::InitStructHelper();
+    label.pLabelName = "regionA";
+    vkt::CommandBuffer cb0(*m_device, m_command_pool);
+    cb0.Begin();
+    vk::CmdBeginDebugUtilsLabelEXT(cb0, &label);
+    cb0.End();
+    m_default_queue->Submit(cb0);
+
+    vkt::CommandBuffer cb1(*m_device, m_command_pool);
+    cb1.Begin();
+    vk::CmdEndDebugUtilsLabelEXT(cb1);
+    vk::CmdEndDebugUtilsLabelEXT(cb1);
+    cb1.End();
+    m_errorMonitor->SetDesiredError("VUID-vkCmdEndDebugUtilsLabelEXT-commandBuffer-01912");
+    m_default_queue->Submit(cb1);
+    m_errorMonitor->VerifyFound();
+    m_default_queue->Wait();
+}
+
+TEST_F(NegativeDebugExtensions, DebugLabelPrimaryCommandBuffer3) {
+    TEST_DESCRIPTION("Test primary command buffer debug labels which are validated at submit time.");
+    AddRequiredExtensions(VK_EXT_DEBUG_UTILS_EXTENSION_NAME);
+    RETURN_IF_SKIP(Init());
+
+    VkDebugUtilsLabelEXT label = vku::InitStructHelper();
+    label.pLabelName = "regionA";
+    vkt::CommandBuffer cb0(*m_device, m_command_pool);
+    cb0.Begin();
+    vk::CmdBeginDebugUtilsLabelEXT(cb0, &label);
+    vk::CmdEndDebugUtilsLabelEXT(cb0);
+    cb0.End();
+
+    vkt::CommandBuffer cb1(*m_device, m_command_pool);
+    label.pLabelName = "regionB";
+    cb1.Begin();
+    vk::CmdBeginDebugUtilsLabelEXT(cb1, &label);
+    vk::CmdEndDebugUtilsLabelEXT(cb1);
+    vk::CmdEndDebugUtilsLabelEXT(cb1);
+    cb1.End();
+    m_errorMonitor->SetDesiredError("VUID-vkCmdEndDebugUtilsLabelEXT-commandBuffer-01912");
+    std::array cbs = {&cb0, &cb1};
+    m_default_queue->Submit(cbs);
+    m_errorMonitor->VerifyFound();
+    m_default_queue->Wait();
+}
+
+TEST_F(NegativeDebugExtensions, DebugLabelSecondaryCommandBuffer) {
+    TEST_DESCRIPTION("Test secondary command buffer debug labels which are validated at recording time.");
+    AddRequiredExtensions(VK_EXT_DEBUG_UTILS_EXTENSION_NAME);
+    RETURN_IF_SKIP(Init());
+
+    vkt::CommandBuffer cb(*m_device, m_command_pool, VK_COMMAND_BUFFER_LEVEL_SECONDARY);
+    cb.Begin();
+    m_errorMonitor->SetDesiredError("VUID-vkCmdEndDebugUtilsLabelEXT-commandBuffer-01913");
+    vk::CmdEndDebugUtilsLabelEXT(cb);
+    m_errorMonitor->VerifyFound();
+    cb.End();
+}
+
+TEST_F(NegativeDebugExtensions, SwapchainImagesDebugMarker) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/7977");
+    SetTargetApiVersion(VK_API_VERSION_1_1);
+    AddRequiredExtensions(VK_EXT_DEBUG_MARKER_EXTENSION_NAME);
+    AddSurfaceExtension();
+    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitSurface());
+
+    SurfaceInformation info = GetSwapchainInfo(m_surface.Handle());
+    InitSwapchainInfo();
+
+    VkSwapchainCreateInfoKHR swapchain_create_info = vku::InitStructHelper();
+    swapchain_create_info.surface = m_surface.Handle();
+    swapchain_create_info.minImageCount = info.surface_capabilities.minImageCount;
+    swapchain_create_info.imageFormat = info.surface_formats[0].format;
+    swapchain_create_info.imageColorSpace = info.surface_formats[0].colorSpace;
+    swapchain_create_info.imageExtent = {info.surface_capabilities.minImageExtent.width,
+                                         info.surface_capabilities.minImageExtent.height};
+    swapchain_create_info.imageArrayLayers = 1;
+    swapchain_create_info.imageUsage = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
+    swapchain_create_info.imageSharingMode = VK_SHARING_MODE_EXCLUSIVE;
+    swapchain_create_info.preTransform = VK_SURFACE_TRANSFORM_IDENTITY_BIT_KHR;
+    swapchain_create_info.compositeAlpha = info.surface_composite_alpha;
+    swapchain_create_info.presentMode = info.surface_non_shared_present_mode;
+    swapchain_create_info.clipped = VK_FALSE;
+    swapchain_create_info.oldSwapchain = VK_NULL_HANDLE;
+
+    vkt::Swapchain swapchain(*m_device, swapchain_create_info);
+    const auto images = swapchain.GetImages();
+
+    {
+        VkDebugMarkerObjectNameInfoEXT name_info = vku::InitStructHelper();
+        name_info.object = (uint64_t)images[0];
+        name_info.objectType = VK_DEBUG_REPORT_OBJECT_TYPE_EVENT_EXT;
+        std::string image_name = "swapchain [0]";
+        name_info.pObjectName = image_name.c_str();
+        m_errorMonitor->SetDesiredError("VUID-VkDebugMarkerObjectNameInfoEXT-object-01492");
+        vk::DebugMarkerSetObjectNameEXT(device(), &name_info);
+        m_errorMonitor->VerifyFound();
+    }
+
+    {
+        int tags[2] = {1, 2};
+        VkDebugMarkerObjectTagInfoEXT name_info = vku::InitStructHelper();
+        name_info.object = (uint64_t)images[0];
+        name_info.objectType = VK_DEBUG_REPORT_OBJECT_TYPE_EVENT_EXT;
+        name_info.tagName = 1;
+        name_info.tagSize = 2;
+        name_info.pTag = tags;
+        m_errorMonitor->SetDesiredError("VUID-VkDebugMarkerObjectTagInfoEXT-object-01495");
+        vk::DebugMarkerSetObjectTagEXT(device(), &name_info);
+        m_errorMonitor->VerifyFound();
+    }
+}
