@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (c) 2015-2023 Google, Inc.
+ * Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (c) 2015-2025 Google, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -12,11 +12,12 @@
  */
 
 #include "../framework/layer_validation_tests.h"
-#include "generated/vk_safe_struct.h"
 #include "generated/vk_extension_helper.h"
 
 #include <cstdlib>
 
+class VkPositiveLayerTest : public VkLayerTest {};
+
 TEST_F(VkPositiveLayerTest, StatelessValidationDisable) {
     TEST_DESCRIPTION("Specify a non-zero value for a reserved parameter with stateless validation disabled");
 
@@ -39,12 +40,12 @@ TEST_F(VkPositiveLayerTest, Maintenance1Tests) {
 
     AddRequiredExtensions(VK_KHR_MAINTENANCE_1_EXTENSION_NAME);
     RETURN_IF_SKIP(Init());
-    vkt::CommandBuffer cmd_buf(m_device, m_commandPool);
-    cmd_buf.begin();
+    vkt::CommandBuffer cmd_buf(*m_device, m_command_pool);
+    cmd_buf.Begin();
     // Set Negative height, should give error if Maintenance 1 is not enabled
     VkViewport viewport = {0, 0, 16, -16, 0, 1};
     vk::CmdSetViewport(cmd_buf.handle(), 0, 1, &viewport);
-    cmd_buf.end();
+    cmd_buf.End();
 }
 
 TEST_F(VkPositiveLayerTest, ValidStructPNext) {
@@ -56,19 +57,13 @@ TEST_F(VkPositiveLayerTest, ValidStructPNext) {
     VkDedicatedAllocationBufferCreateInfoNV dedicated_buffer_create_info = vku::InitStructHelper();
     dedicated_buffer_create_info.dedicatedAllocation = VK_TRUE;
 
-    uint32_t queue_family_index = 0;
     VkBufferCreateInfo buffer_create_info = vku::InitStructHelper(&dedicated_buffer_create_info);
     buffer_create_info.size = 1024;
     buffer_create_info.usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT;
-    buffer_create_info.queueFamilyIndexCount = 1;
-    buffer_create_info.pQueueFamilyIndices = &queue_family_index;
-
-    VkBuffer buffer;
-    VkResult err = vk::CreateBuffer(m_device->device(), &buffer_create_info, NULL, &buffer);
-    ASSERT_EQ(VK_SUCCESS, err);
+    vkt::Buffer buffer(*m_device, buffer_create_info, vkt::no_mem);
 
     VkMemoryRequirements memory_reqs;
-    vk::GetBufferMemoryRequirements(m_device->device(), buffer, &memory_reqs);
+    vk::GetBufferMemoryRequirements(device(), buffer, &memory_reqs);
 
     VkDedicatedAllocationMemoryAllocateInfoNV dedicated_memory_info = vku::InitStructHelper();
     dedicated_memory_info.buffer = buffer;
@@ -78,18 +73,11 @@ TEST_F(VkPositiveLayerTest, ValidStructPNext) {
     memory_info.allocationSize = memory_reqs.size;
 
     bool pass;
-    pass = m_device->phy().set_memory_type(memory_reqs.memoryTypeBits, &memory_info, 0);
+    pass = m_device->Physical().SetMemoryType(memory_reqs.memoryTypeBits, &memory_info, 0);
     ASSERT_TRUE(pass);
 
-    VkDeviceMemory buffer_memory;
-    err = vk::AllocateMemory(m_device->device(), &memory_info, NULL, &buffer_memory);
-    ASSERT_EQ(VK_SUCCESS, err);
-
-    err = vk::BindBufferMemory(m_device->device(), buffer, buffer_memory, 0);
-    ASSERT_EQ(VK_SUCCESS, err);
-
-    vk::DestroyBuffer(m_device->device(), buffer, NULL);
-    vk::FreeMemory(m_device->device(), buffer_memory, NULL);
+    vkt::DeviceMemory buffer_memory(*m_device, memory_info);
+    vk::BindBufferMemory(device(), buffer, buffer_memory, 0);
 }
 
 TEST_F(VkPositiveLayerTest, DeviceIDPropertiesExtensions) {
@@ -101,17 +89,16 @@ TEST_F(VkPositiveLayerTest, DeviceIDPropertiesExtensions) {
     RETURN_IF_SKIP(InitFramework());
 
     if (DeviceValidationVersion() != VK_API_VERSION_1_0) {
-        GTEST_SKIP() << "Tests for 1.0 only";
+        GTEST_SKIP() << "Test's for 1.0 only";
     }
 
-    VkPhysicalDeviceIDProperties id_props =  vku::InitStructHelper();
-    VkPhysicalDeviceFeatures2 features2 = vku::InitStructHelper(&id_props);
-    vk::GetPhysicalDeviceFeatures2KHR(gpu(), &features2);
+    VkPhysicalDeviceIDProperties id_props = vku::InitStructHelper();
+    VkPhysicalDeviceProperties2 props2 = vku::InitStructHelper(&id_props);
+    vk::GetPhysicalDeviceProperties2KHR(Gpu(), &props2);
 }
 
 TEST_F(VkPositiveLayerTest, ParameterLayerFeatures2Capture) {
     TEST_DESCRIPTION("Ensure parameter_validation_layer correctly captures physical device features");
-
     AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
     RETURN_IF_SKIP(InitFramework());
 
@@ -119,14 +106,20 @@ TEST_F(VkPositiveLayerTest, ParameterLayerFeatures2Capture) {
 
     VkPhysicalDeviceFeatures2 features2 = vku::InitStructHelper();
     GetPhysicalDeviceFeatures2(features2);
+    if (!features2.features.samplerAnisotropy) {
+        GTEST_SKIP() << "samplerAnisotropy feature not supported";
+    }
+    if (!features2.features.pipelineStatisticsQuery) {
+        GTEST_SKIP() << "pipelineStatisticsQuery feature not supported";
+    }
 
     // We're not creating a valid m_device, but the phy wrapper is useful
-    vkt::PhysicalDevice physical_device(gpu());
+    vkt::PhysicalDevice physical_device(Gpu());
     vkt::QueueCreateInfoArray queue_info(physical_device.queue_properties_);
     // Only request creation with queuefamilies that have at least one queue
     std::vector<VkDeviceQueueCreateInfo> create_queue_infos;
-    auto qci = queue_info.data();
-    for (uint32_t i = 0; i < queue_info.size(); ++i) {
+    auto qci = queue_info.Data();
+    for (uint32_t i = 0; i < queue_info.Size(); ++i) {
         if (qci[i].queueCount) {
             create_queue_infos.push_back(qci[i]);
         }
@@ -143,10 +136,10 @@ TEST_F(VkPositiveLayerTest, ParameterLayerFeatures2Capture) {
     dev_info.pEnabledFeatures = nullptr;
 
     VkDevice device;
-    err = vk::CreateDevice(gpu(), &dev_info, nullptr, &device);
+    err = vk::CreateDevice(Gpu(), &dev_info, nullptr, &device);
     ASSERT_EQ(VK_SUCCESS, err);
 
-    if (features2.features.samplerAnisotropy) {
+    {
         // Test that the parameter layer is caching the features correctly using CreateSampler
         VkSamplerCreateInfo sampler_ci = SafeSaneSamplerCreateInfo();
         // If the features were not captured correctly, this should cause an error
@@ -157,21 +150,17 @@ TEST_F(VkPositiveLayerTest, ParameterLayerFeatures2Capture) {
         err = vk::CreateSampler(device, &sampler_ci, nullptr, &sampler);
         ASSERT_EQ(VK_SUCCESS, err);
         vk::DestroySampler(device, sampler, nullptr);
-    } else {
-        printf("Feature samplerAnisotropy not enabled;  parameter_layer check skipped.\n");
     }
 
     // Verify the core validation layer has captured the physical device features by creating a a query pool.
-    if (features2.features.pipelineStatisticsQuery) {
+    {
         VkQueryPool query_pool;
-        VkQueryPoolCreateInfo qpci = vkt::QueryPool::create_info(VK_QUERY_TYPE_PIPELINE_STATISTICS, 1);
+        VkQueryPoolCreateInfo qpci = vkt::QueryPool::CreateInfo(VK_QUERY_TYPE_PIPELINE_STATISTICS, 1);
         qpci.pipelineStatistics = VK_QUERY_PIPELINE_STATISTIC_INPUT_ASSEMBLY_VERTICES_BIT;
         err = vk::CreateQueryPool(device, &qpci, nullptr, &query_pool);
         ASSERT_EQ(VK_SUCCESS, err);
 
         vk::DestroyQueryPool(device, query_pool, nullptr);
-    } else {
-        printf("Feature pipelineStatisticsQuery not enabled;  core_validation_layer check skipped.\n");
     }
 
     vk::DestroyDevice(device, nullptr);
@@ -195,7 +184,7 @@ TEST_F(VkPositiveLayerTest, ModifyPnext) {
     shading.maxFragmentShadingRateInvocationCount = static_cast<VkSampleCountFlagBits>(0);
     VkPhysicalDeviceProperties2 props = vku::InitStructHelper(&shading);
 
-    vk::GetPhysicalDeviceProperties2(gpu(), &props);
+    vk::GetPhysicalDeviceProperties2(Gpu(), &props);
 }
 
 TEST_F(VkPositiveLayerTest, UseFirstQueueUnqueried) {
@@ -214,7 +203,7 @@ TEST_F(VkPositiveLayerTest, UseFirstQueueUnqueried) {
     device_ci.pQueueCreateInfos = &queue_ci;
 
     VkDevice test_device;
-    vk::CreateDevice(gpu(), &device_ci, nullptr, &test_device);
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &test_device);
 
     vk::DestroyDevice(test_device, nullptr);
 }
@@ -225,7 +214,8 @@ TEST_F(VkPositiveLayerTest, GetDevProcAddrNullPtr) {
     TEST_DESCRIPTION("Call GetDeviceProcAddr on an enabled instance extension expecting nullptr");
     AddRequiredExtensions(VK_KHR_SURFACE_EXTENSION_NAME);
     RETURN_IF_SKIP(Init());
-    auto fpDestroySurface = (PFN_vkCreateValidationCacheEXT)vk::GetDeviceProcAddr(m_device->device(), "vkDestroySurfaceKHR");
+    m_errorMonitor->SetAllowedFailureMsg("WARNING-vkGetDeviceProcAddr-device");
+    auto fpDestroySurface = (PFN_vkCreateValidationCacheEXT)vk::GetDeviceProcAddr(device(), "vkDestroySurfaceKHR");
     if (fpDestroySurface) {
         m_errorMonitor->SetError("Null was expected!");
     }
@@ -235,8 +225,8 @@ TEST_F(VkPositiveLayerTest, GetDevProcAddrExtensions) {
     TEST_DESCRIPTION("Call GetDeviceProcAddr with and without extension enabled");
     SetTargetApiVersion(VK_API_VERSION_1_1);
     RETURN_IF_SKIP(Init());
-    auto vkTrimCommandPool = vk::GetDeviceProcAddr(m_device->device(), "vkTrimCommandPool");
-    auto vkTrimCommandPoolKHR = vk::GetDeviceProcAddr(m_device->device(), "vkTrimCommandPoolKHR");
+    auto vkTrimCommandPool = vk::GetDeviceProcAddr(device(), "vkTrimCommandPool");
+    auto vkTrimCommandPoolKHR = vk::GetDeviceProcAddr(device(), "vkTrimCommandPoolKHR");
     if (nullptr == vkTrimCommandPool) m_errorMonitor->SetError("Unexpected null pointer");
     if (nullptr != vkTrimCommandPoolKHR) m_errorMonitor->SetError("Didn't receive expected null pointer");
 
@@ -254,7 +244,7 @@ TEST_F(VkPositiveLayerTest, GetDevProcAddrExtensions) {
     device_ci.pQueueCreateInfos = &queue_ci;
 
     VkDevice device;
-    vk::CreateDevice(gpu(), &device_ci, NULL, &device);
+    vk::CreateDevice(Gpu(), &device_ci, NULL, &device);
 
     vkTrimCommandPoolKHR = vk::GetDeviceProcAddr(device, "vkTrimCommandPoolKHR");
     if (nullptr == vkTrimCommandPoolKHR) m_errorMonitor->SetError("Unexpected null pointer");
@@ -278,14 +268,14 @@ TEST_F(VkPositiveLayerTest, Vulkan12FeaturesBufferDeviceAddress) {
     RETURN_IF_SKIP(InitState(nullptr, &features2));
 
     VkMemoryAllocateFlagsInfo alloc_flags = vku::InitStructHelper();
-    alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR;
+    alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT;
     vkt::Buffer buffer(*m_device, 1024, VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
                        &alloc_flags);
-    (void)buffer.address();
+    (void)buffer.Address();
 
     // Also verify that we don't get the KHR extension address without enabling the KHR extension
     auto vkGetBufferDeviceAddressKHR =
-        (PFN_vkGetBufferDeviceAddressKHR)vk::GetDeviceProcAddr(m_device->device(), "vkGetBufferDeviceAddressKHR");
+        (PFN_vkGetBufferDeviceAddressKHR)vk::GetDeviceProcAddr(device(), "vkGetBufferDeviceAddressKHR");
     if (nullptr != vkGetBufferDeviceAddressKHR) m_errorMonitor->SetError("Didn't receive expected null pointer");
 }
 
@@ -362,51 +352,6 @@ TEST_F(VkPositiveLayerTest, ExtensionXmlDependsLogic) {
     RETURN_IF_SKIP(InitState());
 }
 
-// https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/5112
-TEST_F(VkPositiveLayerTest, SafeVoidPointerCopies) {
-    TEST_DESCRIPTION("Ensure valid deep copy of pData / dataSize combination structures");
-
-    // safe_VkSpecializationInfo, constructor
-    {
-        std::vector<std::byte> data(20, std::byte{0b11110000});
-
-        VkSpecializationInfo info = {};
-        info.dataSize = size32(data);
-        info.pData = data.data();
-
-        safe_VkSpecializationInfo safe(&info);
-
-        ASSERT_TRUE(safe.pData != info.pData);
-        ASSERT_TRUE(safe.dataSize == info.dataSize);
-
-        data.clear();  // Invalidate any references, pointers, or iterators referring to contained elements.
-
-        auto copied_bytes = reinterpret_cast<const std::byte *>(safe.pData);
-        ASSERT_TRUE(copied_bytes[19] == std::byte{0b11110000});
-    }
-
-    // safe_VkPipelineExecutableInternalRepresentationKHR, initialize
-    {
-        std::vector<std::byte> data(11, std::byte{0b01001001});
-
-        VkPipelineExecutableInternalRepresentationKHR info = {};
-        info.dataSize = size32(data);
-        info.pData = data.data();
-
-        safe_VkPipelineExecutableInternalRepresentationKHR safe;
-
-        safe.initialize(&info);
-
-        ASSERT_TRUE(safe.dataSize == info.dataSize);
-        ASSERT_TRUE(safe.pData != info.pData);
-
-        data.clear();  // Invalidate any references, pointers, or iterators referring to contained elements.
-
-        auto copied_bytes = reinterpret_cast<const std::byte *>(safe.pData);
-        ASSERT_TRUE(copied_bytes[10] == std::byte{0b01001001});
-    }
-}
-
 TEST_F(VkPositiveLayerTest, FormatProperties3FromProfiles) {
     // https://github.com/KhronosGroup/Vulkan-Profiles/pull/392
     TEST_DESCRIPTION("Make sure VkFormatProperties3KHR is overwritten correctly in Profiles layer");
@@ -415,8 +360,8 @@ TEST_F(VkPositiveLayerTest, FormatProperties3FromProfiles) {
     RETURN_IF_SKIP(Init());
     VkFormatProperties3KHR fmt_props_3 = vku::InitStructHelper();
     VkFormatProperties2 fmt_props = vku::InitStructHelper(&fmt_props_3);
-    vk::GetPhysicalDeviceFormatProperties2(gpu(), VK_FORMAT_R8_UNORM, &fmt_props);
-    vk::GetPhysicalDeviceFormatProperties2(gpu(), VK_FORMAT_R8G8B8A8_UNORM, &fmt_props);
+    vk::GetPhysicalDeviceFormatProperties2(Gpu(), VK_FORMAT_R8_UNORM, &fmt_props);
+    vk::GetPhysicalDeviceFormatProperties2(Gpu(), VK_FORMAT_R8G8B8A8_UNORM, &fmt_props);
 }
 
 TEST_F(VkPositiveLayerTest, GDPAWithMultiCmdExt) {
@@ -440,7 +385,7 @@ TEST_F(VkPositiveLayerTest, UseInteractionApi1) {
     }
 
     VkDeviceGroupPresentCapabilitiesKHR device_group_present_caps = vku::InitStructHelper();
-    vk::GetDeviceGroupPresentCapabilitiesKHR(m_device->device(), &device_group_present_caps);
+    vk::GetDeviceGroupPresentCapabilitiesKHR(device(), &device_group_present_caps);
 }
 
 TEST_F(VkPositiveLayerTest, UseInteractionApi2) {
@@ -457,7 +402,7 @@ TEST_F(VkPositiveLayerTest, UseInteractionApi2) {
     }
 
     VkDeviceGroupPresentCapabilitiesKHR device_group_present_caps = vku::InitStructHelper();
-    vk::GetDeviceGroupPresentCapabilitiesKHR(m_device->device(), &device_group_present_caps);
+    vk::GetDeviceGroupPresentCapabilitiesKHR(device(), &device_group_present_caps);
 }
 
 TEST_F(VkPositiveLayerTest, ExtensionExpressions) {
@@ -467,22 +412,15 @@ TEST_F(VkPositiveLayerTest, ExtensionExpressions) {
     SetTargetApiVersion(VK_API_VERSION_1_1);
     AddRequiredExtensions(VK_KHR_CREATE_RENDERPASS_2_EXTENSION_NAME);
     AddRequiredExtensions(VK_KHR_FRAGMENT_SHADING_RATE_EXTENSION_NAME);
-    RETURN_IF_SKIP(InitFramework());
-
-    VkPhysicalDeviceFragmentShadingRateFeaturesKHR fsr_features = vku::InitStructHelper();
-    GetPhysicalDeviceFeatures2(fsr_features);
-    if (!fsr_features.pipelineFragmentShadingRate) {
-        GTEST_SKIP() << "VkPhysicalDeviceFragmentShadingRateFeaturesKHR::pipelineFragmentShadingRate not supported";
-    }
-
-    RETURN_IF_SKIP(InitState(nullptr, &fsr_features));
+    AddRequiredFeature(vkt::Feature::pipelineFragmentShadingRate);
+    RETURN_IF_SKIP(Init());
 
     VkExtent2D fragment_size = {1, 1};
     std::array combiner_ops = {VK_FRAGMENT_SHADING_RATE_COMBINER_OP_KEEP_KHR, VK_FRAGMENT_SHADING_RATE_COMBINER_OP_KEEP_KHR};
 
-    m_commandBuffer->begin();
-    vk::CmdSetFragmentShadingRateKHR(*m_commandBuffer, &fragment_size, combiner_ops.data());
-    m_commandBuffer->end();
+    m_command_buffer.Begin();
+    vk::CmdSetFragmentShadingRateKHR(m_command_buffer, &fragment_size, combiner_ops.data());
+    m_command_buffer.End();
 }
 
 TEST_F(VkPositiveLayerTest, AllowedDuplicateStype) {
@@ -514,9 +452,9 @@ TEST_F(VkPositiveLayerTest, ExtensionsInCreateInstance) {
     //       So, this test will only catch an erroneous extension _if_ run on HW/a driver that crashes in this use
     //       case.
 
-    for (const auto &ext : InstanceExtensions::get_info_map()) {
+    for (const auto &ext : InstanceExtensions::GetInfoMap()) {
         // Add all "real" instance extensions
-        if (InstanceExtensionSupported(ext.first.c_str())) {
+        if (InstanceExtensionSupported(String(ext.first))) {
             bool version_required = false;
             for (const auto &req : ext.second.requirements) {
                 std::string name(req.name);
@@ -526,7 +464,7 @@ TEST_F(VkPositiveLayerTest, ExtensionsInCreateInstance) {
                 }
             }
             if (!version_required) {
-                m_instance_extension_names.emplace_back(ext.first.c_str());
+                m_instance_extension_names.emplace_back(String(ext.first));
             }
         }
     }
@@ -534,83 +472,6 @@ TEST_F(VkPositiveLayerTest, ExtensionsInCreateInstance) {
     RETURN_IF_SKIP(InitFramework());
 }
 
-TEST_F(VkPositiveLayerTest, CustomSafePNextCopy) {
-    TEST_DESCRIPTION("Check passing custom data down the pNext chain for safe struct construction");
-
-    // This tests an additional "copy_state" parameter in the SafePNextCopy function that allows "customizing" safe_* struct
-    // construction.. This is required for structs such as VkPipelineRenderingCreateInfo (which extend VkGraphicsPipelineCreateInfo)
-    // whose members must be partially ignored depending on the graphics sub-state present.
-
-    VkFormat format = VK_FORMAT_B8G8R8A8_UNORM;
-    VkPipelineRenderingCreateInfo pri = vku::InitStructHelper();
-    pri.colorAttachmentCount = 1;
-    pri.pColorAttachmentFormats = &format;
-
-    bool ignore_default_construction = true;
-    PNextCopyState copy_state = {
-        [&ignore_default_construction](VkBaseOutStructure *safe_struct, const VkBaseOutStructure *in_struct) -> bool {
-            if (ignore_default_construction) {
-                auto tmp = reinterpret_cast<safe_VkPipelineRenderingCreateInfo *>(safe_struct);
-                tmp->colorAttachmentCount = 0;
-                tmp->pColorAttachmentFormats = nullptr;
-                return true;
-            }
-            return false;
-        },
-    };
-
-    {
-        VkGraphicsPipelineCreateInfo gpci = vku::InitStructHelper(&pri);
-        safe_VkGraphicsPipelineCreateInfo safe_gpci(&gpci, false, false, &copy_state);
-
-        auto safe_pri = reinterpret_cast<const safe_VkPipelineRenderingCreateInfo *>(safe_gpci.pNext);
-        // Ensure original input struct was not modified
-        ASSERT_EQ(pri.colorAttachmentCount, 1);
-        ASSERT_EQ(pri.pColorAttachmentFormats, &format);
-
-        // Ensure safe struct was modified
-        ASSERT_EQ(safe_pri->colorAttachmentCount, 0);
-        ASSERT_EQ(safe_pri->pColorAttachmentFormats, nullptr);
-    }
-
-    // Ensure PNextCopyState::init is also applied when there is more than one element in the pNext chain
-    {
-        VkGraphicsPipelineLibraryCreateInfoEXT gpl_info = vku::InitStructHelper(&pri);
-        VkGraphicsPipelineCreateInfo gpci = vku::InitStructHelper(&gpl_info);
-
-        safe_VkGraphicsPipelineCreateInfo safe_gpci(&gpci, false, false, &copy_state);
-
-        auto safe_gpl_info = reinterpret_cast<const safe_VkGraphicsPipelineLibraryCreateInfoEXT *>(safe_gpci.pNext);
-        auto safe_pri = reinterpret_cast<const safe_VkPipelineRenderingCreateInfo *>(safe_gpl_info->pNext);
-        // Ensure original input struct was not modified
-        ASSERT_EQ(pri.colorAttachmentCount, 1);
-        ASSERT_EQ(pri.pColorAttachmentFormats, &format);
-
-        // Ensure safe struct was modified
-        ASSERT_EQ(safe_pri->colorAttachmentCount, 0);
-        ASSERT_EQ(safe_pri->pColorAttachmentFormats, nullptr);
-    }
-
-    // Check that signaling to use the default constructor works
-    {
-        pri.colorAttachmentCount = 1;
-        pri.pColorAttachmentFormats = &format;
-
-        ignore_default_construction = false;
-        VkGraphicsPipelineCreateInfo gpci = vku::InitStructHelper(&pri);
-        safe_VkGraphicsPipelineCreateInfo safe_gpci(&gpci, false, false, &copy_state);
-
-        auto safe_pri = reinterpret_cast<const safe_VkPipelineRenderingCreateInfo *>(safe_gpci.pNext);
-        // Ensure original input struct was not modified
-        ASSERT_EQ(pri.colorAttachmentCount, 1);
-        ASSERT_EQ(pri.pColorAttachmentFormats, &format);
-
-        // Ensure safe struct was modified
-        ASSERT_EQ(safe_pri->colorAttachmentCount, 1);
-        ASSERT_EQ(*safe_pri->pColorAttachmentFormats, format);
-    }
-}
-
 TEST_F(VkPositiveLayerTest, ExclusiveScissorVersionCount) {
     TEST_DESCRIPTION("Test using vkCmdSetExclusiveScissorEnableNV.");
 
@@ -634,10 +495,10 @@ TEST_F(VkPositiveLayerTest, ExclusiveScissorVersionCount) {
     }
     RETURN_IF_SKIP(InitState());
 
-    m_commandBuffer->begin();
+    m_command_buffer.Begin();
     VkBool32 exclusiveScissorEnable = VK_TRUE;
-    vk::CmdSetExclusiveScissorEnableNV(m_commandBuffer->handle(), 0u, 1u, &exclusiveScissorEnable);
-    m_commandBuffer->end();
+    vk::CmdSetExclusiveScissorEnableNV(m_command_buffer.handle(), 0u, 1u, &exclusiveScissorEnable);
+    m_command_buffer.End();
 }
 
 TEST_F(VkPositiveLayerTest, GetCalibratedTimestamps) {
@@ -647,12 +508,12 @@ TEST_F(VkPositiveLayerTest, GetCalibratedTimestamps) {
     RETURN_IF_SKIP(Init());
 
     uint32_t count = 0;
-    vk::GetPhysicalDeviceCalibrateableTimeDomainsEXT(gpu(), &count, nullptr);
+    vk::GetPhysicalDeviceCalibrateableTimeDomainsEXT(Gpu(), &count, nullptr);
     if (count < 2) {
         GTEST_SKIP() << "only 1 TimeDomain supported";
     }
     std::vector<VkTimeDomainEXT> time_domains(count);
-    vk::GetPhysicalDeviceCalibrateableTimeDomainsEXT(gpu(), &count, time_domains.data());
+    vk::GetPhysicalDeviceCalibrateableTimeDomainsEXT(Gpu(), &count, time_domains.data());
 
     VkCalibratedTimestampInfoEXT timestamp_infos[2];
     timestamp_infos[0] = vku::InitStructHelper();
@@ -672,12 +533,12 @@ TEST_F(VkPositiveLayerTest, GetCalibratedTimestampsKHR) {
     RETURN_IF_SKIP(Init());
 
     uint32_t count = 0;
-    vk::GetPhysicalDeviceCalibrateableTimeDomainsKHR(gpu(), &count, nullptr);
+    vk::GetPhysicalDeviceCalibrateableTimeDomainsKHR(Gpu(), &count, nullptr);
     if (count < 2) {
         GTEST_SKIP() << "only 1 TimeDomain supported";
     }
     std::vector<VkTimeDomainKHR> time_domains(count);
-    vk::GetPhysicalDeviceCalibrateableTimeDomainsKHR(gpu(), &count, time_domains.data());
+    vk::GetPhysicalDeviceCalibrateableTimeDomainsKHR(Gpu(), &count, time_domains.data());
 
     VkCalibratedTimestampInfoKHR timestamp_infos[2];
     timestamp_infos[0] = vku::InitStructHelper();
@@ -689,3 +550,161 @@ TEST_F(VkPositiveLayerTest, GetCalibratedTimestampsKHR) {
     uint64_t max_deviation;
     vk::GetCalibratedTimestampsKHR(device(), 2, timestamp_infos, timestamps, &max_deviation);
 }
+
+TEST_F(VkPositiveLayerTest, ExtensionPhysicalDeviceFeatureEXT) {
+    TEST_DESCRIPTION("VkPhysicalDeviceGlobalPriorityQueryFeaturesKHR has an EXT and KHR extension that can enable it");
+    AddRequiredExtensions(VK_EXT_GLOBAL_PRIORITY_QUERY_EXTENSION_NAME);
+    RETURN_IF_SKIP(InitFramework());
+    VkPhysicalDeviceGlobalPriorityQueryFeaturesKHR query_feature = vku::InitStructHelper();
+    query_feature.globalPriorityQuery = VK_TRUE;
+    RETURN_IF_SKIP(InitState(nullptr, &query_feature));
+}
+
+TEST_F(VkPositiveLayerTest, ExtensionPhysicalDeviceFeatureKHR) {
+    TEST_DESCRIPTION("VkPhysicalDeviceGlobalPriorityQueryFeaturesKHR has an EXT and KHR extension that can enable it");
+    AddRequiredExtensions(VK_KHR_GLOBAL_PRIORITY_EXTENSION_NAME);
+    RETURN_IF_SKIP(InitFramework());
+    VkPhysicalDeviceGlobalPriorityQueryFeaturesKHR query_feature = vku::InitStructHelper();
+    query_feature.globalPriorityQuery = VK_TRUE;
+    RETURN_IF_SKIP(InitState(nullptr, &query_feature));
+}
+
+TEST_F(VkPositiveLayerTest, NoExtensionFromInstanceFunction) {
+    TEST_DESCRIPTION("Valid because we instance functions don't know which device it needs");
+    RETURN_IF_SKIP(Init());
+    VkFormatProperties format_properties;
+    // need VK_KHR_sampler_ycbcr_conversion if it was a device function
+    vk::GetPhysicalDeviceFormatProperties(Gpu(), VK_FORMAT_B16G16R16G16_422_UNORM, &format_properties);
+}
+
+TEST_F(VkPositiveLayerTest, InstanceExtensionsCallingDeviceStruct0) {
+    TEST_DESCRIPTION(
+        "Use VkImageFormatListCreateInfo with VkPhysicalDeviceImageFormatInfo2 if VK_KHR_image_format_list is available");
+    SetTargetApiVersion(VK_API_VERSION_1_0);
+    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
+    RETURN_IF_SKIP(Init());
+    if (!DeviceExtensionSupported(Gpu(), nullptr, VK_KHR_IMAGE_FORMAT_LIST_EXTENSION_NAME)) {
+        GTEST_SKIP() << "VK_KHR_image_format_list is not supported";
+    }
+    const VkFormat format = VK_FORMAT_R8G8B8A8_UNORM;
+
+    // Requires device extension (VK_KHR_image_format_list)
+    VkImageFormatListCreateInfo format_list_info = vku::InitStructHelper();
+    format_list_info.viewFormatCount = 1;
+    format_list_info.pViewFormats = &format;
+
+    VkPhysicalDeviceImageFormatInfo2 image_format_info = vku::InitStructHelper(&format_list_info);
+    image_format_info.type = VK_IMAGE_TYPE_2D;
+    image_format_info.format = format;
+    image_format_info.tiling = VK_IMAGE_TILING_OPTIMAL;
+    image_format_info.usage = VK_IMAGE_USAGE_TRANSFER_SRC_BIT;
+
+    VkImageFormatProperties2 image_format_properties = vku::InitStructHelper();
+    // Requires instance extension (VK_KHR_get_physical_device_properties2)
+    vk::GetPhysicalDeviceImageFormatProperties2KHR(Gpu(), &image_format_info, &image_format_properties);
+}
+
+TEST_F(VkPositiveLayerTest, InstanceExtensionsCallingDeviceStruct1) {
+    TEST_DESCRIPTION(
+        "Use VkBufferUsageFlags2CreateInfo with VkPhysicalDeviceExternalBufferInfo if VK_KHR_maintenance5 is available");
+    SetTargetApiVersion(VK_API_VERSION_1_1);
+    RETURN_IF_SKIP(Init());
+    if (!DeviceExtensionSupported(Gpu(), nullptr, VK_KHR_MAINTENANCE_5_EXTENSION_NAME)) {
+        GTEST_SKIP() << "VK_KHR_maintenance5 is not supported";
+    }
+
+#ifdef _WIN32
+    const auto handle_type = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_BIT;
+#else
+    const auto handle_type = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT;
+#endif
+
+    // Requires device extension (VK_KHR_maintenance5)
+    VkBufferUsageFlags2CreateInfo bufferUsageFlags2 = vku::InitStructHelper();
+    bufferUsageFlags2.usage = VK_BUFFER_USAGE_2_TRANSFER_SRC_BIT;
+
+    VkPhysicalDeviceExternalBufferInfo externalBufferInfo = vku::InitStructHelper(&bufferUsageFlags2);
+    externalBufferInfo.handleType = handle_type;
+    VkExternalBufferProperties externalBufferProperties = vku::InitStructHelper();
+    // Instance extension promoted to VK_VERSION_1_1
+    vk::GetPhysicalDeviceExternalBufferProperties(Gpu(), &externalBufferInfo, &externalBufferProperties);
+}
+
+TEST_F(VkPositiveLayerTest, TimelineSemaphoreWithVulkan11) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/8308");
+    SetTargetApiVersion(VK_API_VERSION_1_1);
+    AddRequiredExtensions(VK_NV_LOW_LATENCY_2_EXTENSION_NAME);
+    AddRequiredExtensions(VK_KHR_TIMELINE_SEMAPHORE_EXTENSION_NAME);
+    RETURN_IF_SKIP(Init());
+}
+
+TEST_F(VkPositiveLayerTest, UnrecognizedEnumOutOfRange) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/8367");
+    RETURN_IF_SKIP(Init());
+    if (!DeviceExtensionSupported(Gpu(), nullptr, VK_KHR_MAINTENANCE_5_EXTENSION_NAME)) {
+        GTEST_SKIP() << "VK_KHR_maintenance5 is not supported";
+    }
+    VkFormatProperties format_properties;
+    vk::GetPhysicalDeviceFormatProperties(Gpu(), static_cast<VkFormat>(8000), &format_properties);
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(VkPositiveLayerTest, UnrecognizedEnumOutOfRange2) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/8367");
+    SetTargetApiVersion(VK_API_VERSION_1_1);
+    RETURN_IF_SKIP(Init());
+    if (!DeviceExtensionSupported(Gpu(), nullptr, VK_KHR_MAINTENANCE_5_EXTENSION_NAME)) {
+        GTEST_SKIP() << "VK_KHR_maintenance5 is not supported";
+    }
+    VkFormatProperties2 format_properties = vku::InitStructHelper();
+    vk::GetPhysicalDeviceFormatProperties2(Gpu(), static_cast<VkFormat>(8000), &format_properties);
+}
+
+TEST_F(VkPositiveLayerTest, UnrecognizedFlagOutOfRange) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/8367");
+    RETURN_IF_SKIP(Init());
+    if (!DeviceExtensionSupported(Gpu(), nullptr, VK_KHR_MAINTENANCE_5_EXTENSION_NAME)) {
+        GTEST_SKIP() << "VK_KHR_maintenance5 is not supported";
+    }
+
+    VkImageFormatProperties format_properties;
+    vk::GetPhysicalDeviceImageFormatProperties(Gpu(), VK_FORMAT_R8G8B8A8_UNORM, VK_IMAGE_TYPE_1D, VK_IMAGE_TILING_OPTIMAL,
+                                               static_cast<VkImageUsageFlags>(0xffffffff), 0, &format_properties);
+}
+
+TEST_F(VkPositiveLayerTest, UnrecognizedFlagOutOfRange2) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/8367");
+    SetTargetApiVersion(VK_API_VERSION_1_1);
+    RETURN_IF_SKIP(Init());
+    if (!DeviceExtensionSupported(Gpu(), nullptr, VK_KHR_MAINTENANCE_5_EXTENSION_NAME)) {
+        GTEST_SKIP() << "VK_KHR_maintenance5 is not supported";
+    }
+
+    VkImageFormatProperties2 format_properties = vku::InitStructHelper();
+    VkPhysicalDeviceImageFormatInfo2 format_info = vku::InitStructHelper();
+    format_info.format = VK_FORMAT_R8G8B8A8_UNORM;
+    format_info.flags = 0;
+    format_info.tiling = VK_IMAGE_TILING_OPTIMAL;
+    format_info.type = VK_IMAGE_TYPE_1D;
+    format_info.usage = static_cast<VkImageUsageFlags>(0xffffffff);
+    vk::GetPhysicalDeviceImageFormatProperties2(Gpu(), &format_info, &format_properties);
+}
+
+TEST_F(VkPositiveLayerTest, PhysicalDeviceLayeredApiVulkanProperties) {
+    SetTargetApiVersion(VK_API_VERSION_1_2);
+    RETURN_IF_SKIP(InitFramework());
+    // Don't enable maintenance7 because we are doing this at an instance level
+    // just make sure the extension is there
+    if (!DeviceExtensionSupported(VK_KHR_MAINTENANCE_7_EXTENSION_NAME)) {
+        GTEST_SKIP() << "Did not find the required device extensions";
+    }
+    RETURN_IF_SKIP(InitState());
+
+    VkPhysicalDeviceLayeredApiPropertiesKHR api_props = vku::InitStructHelper();
+    VkPhysicalDeviceLayeredApiPropertiesListKHR api_prop_lists = vku::InitStructHelper();
+    api_prop_lists.layeredApiCount = 1;
+    api_prop_lists.pLayeredApis = &api_props;
+
+    VkPhysicalDeviceProperties2 phys_dev_props_2 = vku::InitStructHelper(&api_prop_lists);
+    vk::GetPhysicalDeviceProperties2(Gpu(), &phys_dev_props_2);
+}
\ No newline at end of file
