@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2015-2023 The Khronos Group Inc.
- * Copyright (c) 2015-2023 Valve Corporation
- * Copyright (c) 2015-2023 LunarG, Inc.
- * Copyright (c) 2015-2023 Google, Inc.
+ * Copyright (c) 2015-2024 The Khronos Group Inc.
+ * Copyright (c) 2015-2024 Valve Corporation
+ * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (c) 2015-2024 Google, Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,24 +12,19 @@
  *     http://www.apache.org/licenses/LICENSE-2.0
  */
 
-#include "utils/cast_utils.h"
 #include "../framework/layer_validation_tests.h"
 #include "../framework/pipeline_helper.h"
 
-TEST_F(NegativeShaderCooperativeMatrix, KHRSpecInfo) {
+class NegativeShaderCooperativeMatrix : public CooperativeMatrixTest {};
+
+TEST_F(NegativeShaderCooperativeMatrix, SpecInfo) {
     TEST_DESCRIPTION("Test VK_KHR_cooperative_matrix.");
 
     SetTargetApiVersion(VK_API_VERSION_1_3);
 
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_COOPERATIVE_MATRIX_EXTENSION_NAME);
     AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
-    // glslang will generate OpCapability VulkanMemoryModel and need entension enabled
-    AddRequiredExtensions(VK_KHR_VULKAN_MEMORY_MODEL_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::shaderFloat16);
-    AddRequiredFeature(vkt::Feature::cooperativeMatrix);
-    AddRequiredFeature(vkt::Feature::vulkanMemoryModel);
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
 
     std::vector<VkDescriptorSetLayoutBinding> bindings(0);
     const vkt::DescriptorSetLayout dsl(*m_device, bindings);
@@ -75,31 +70,27 @@ TEST_F(NegativeShaderCooperativeMatrix, KHRSpecInfo) {
     CreateComputePipelineHelper pipe(*this);
     pipe.cs_ =
         std::make_unique<VkShaderObj>(this, csSource, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_GLSL, &specInfo);
-    pipe.InitState();
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkPipelineShaderStageCreateInfo-pSpecializationInfo-06849");
+    m_errorMonitor->SetDesiredError("VUID-VkPipelineShaderStageCreateInfo-pSpecializationInfo-06849");
     pipe.CreateComputePipeline();
     m_errorMonitor->VerifyFound();
 }
 
-TEST_F(NegativeShaderCooperativeMatrix, KHRUnsupportedStage) {
+TEST_F(NegativeShaderCooperativeMatrix, UnsupportedStageUint32) {
     TEST_DESCRIPTION("Test error using cooperative matrix in unsupported stage");
 
     SetTargetApiVersion(VK_API_VERSION_1_3);
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
+    InitRenderTarget();
 
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_COOPERATIVE_MATRIX_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_VULKAN_MEMORY_MODEL_EXTENSION_NAME);
-    AddRequiredFeature(vkt::Feature::cooperativeMatrix);
-    AddRequiredFeature(vkt::Feature::vulkanMemoryModel);
-    RETURN_IF_SKIP(Init());
+    if (!HasValidProperty(VK_SCOPE_SUBGROUP_KHR, 16, 16, 16, VK_COMPONENT_TYPE_UINT32_KHR)) {
+        GTEST_SKIP() << "Valid Property not found";
+    }
 
     VkPhysicalDeviceCooperativeMatrixPropertiesKHR props = vku::InitStructHelper();
     GetPhysicalDeviceProperties2(props);
-
     if ((props.cooperativeMatrixSupportedStages & VK_SHADER_STAGE_VERTEX_BIT) != 0) {
         GTEST_SKIP() << "Cannot execute test due to vertex stage expected to be unsupported";
     }
-    InitRenderTarget();
 
     char const *vtSource = R"glsl(
         #version 450
@@ -117,44 +108,75 @@ TEST_F(NegativeShaderCooperativeMatrix, KHRUnsupportedStage) {
     )glsl";
 
     CreatePipelineHelper pipe(*this);
-    pipe.vs_ = std::make_unique<VkShaderObj>(this, vtSource, VK_SHADER_STAGE_VERTEX_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_GLSL);
+    pipe.vs_ = std::make_unique<VkShaderObj>(this, vtSource, VK_SHADER_STAGE_VERTEX_BIT);
     pipe.shader_stages_ = {pipe.vs_->GetStageCreateInfo(), pipe.fs_->GetStageCreateInfo()};
-    pipe.InitState();
-    m_errorMonitor->SetAllowedFailureMsg("VUID-VkShaderModuleCreateInfo-pCode-08739");
-    // Ignore messages that types and sizes are unsupported by implementation
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08975");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-KSize-08977");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08979");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08981");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08976");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08978");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08980");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08982");
 
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-RuntimeSpirv-cooperativeMatrixSupportedStages-08985");
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-cooperativeMatrixSupportedStages-08985");
     pipe.CreateGraphicsPipeline();
 
     m_errorMonitor->VerifyFound();
 }
 
-TEST_F(NegativeShaderCooperativeMatrix, KHRParametersMatchProperties) {
+TEST_F(NegativeShaderCooperativeMatrix, UnsupportedStageFloat16) {
+    TEST_DESCRIPTION("Test error using cooperative matrix in unsupported stage");
+
+    SetTargetApiVersion(VK_API_VERSION_1_3);
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
+    InitRenderTarget();
+
+    if (!HasValidProperty(VK_SCOPE_SUBGROUP_KHR, 8, 8, 16, VK_COMPONENT_TYPE_FLOAT16_KHR)) {
+        GTEST_SKIP() << "Valid Property not found";
+    }
+
+    VkPhysicalDeviceCooperativeMatrixPropertiesKHR props = vku::InitStructHelper();
+    GetPhysicalDeviceProperties2(props);
+    if ((props.cooperativeMatrixSupportedStages & VK_SHADER_STAGE_VERTEX_BIT) != 0) {
+        GTEST_SKIP() << "Cannot execute test due to vertex stage expected to be unsupported";
+    }
+
+    char const *vtSource = R"glsl(
+        #version 450
+        #pragma use_vulkan_memory_model
+        #extension GL_KHR_cooperative_matrix : enable
+        #extension GL_KHR_shader_subgroup_basic : enable
+        #extension GL_KHR_memory_scope_semantics : enable
+        #extension GL_EXT_shader_explicit_arithmetic_types : enable
+        void main() {
+            coopmat<float16_t, gl_ScopeSubgroup, 8, 16, gl_MatrixUseA> A;
+            coopmat<float16_t, gl_ScopeSubgroup, 16, 8, gl_MatrixUseB> B;
+            coopmat<float16_t, gl_ScopeSubgroup, 8, 8, gl_MatrixUseAccumulator> C;
+            coopmat<float16_t, gl_ScopeSubgroup, 8, 8, gl_MatrixUseAccumulator> D = coopMatMulAdd(A, B, C);
+       }
+    )glsl";
+
+    CreatePipelineHelper pipe(*this);
+    pipe.vs_ = std::make_unique<VkShaderObj>(this, vtSource, VK_SHADER_STAGE_VERTEX_BIT);
+    pipe.shader_stages_ = {pipe.vs_->GetStageCreateInfo(), pipe.fs_->GetStageCreateInfo()};
+
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-cooperativeMatrixSupportedStages-08985");
+    pipe.CreateGraphicsPipeline();
+
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(NegativeShaderCooperativeMatrix, ParametersMatchProperties) {
     TEST_DESCRIPTION("Test that parameters match one of the matrices in any of the supported VkCooperativeMatrixPropertiesKHR");
 
     SetTargetApiVersion(VK_API_VERSION_1_3);
 
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_COOPERATIVE_MATRIX_EXTENSION_NAME);
     AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
-    // glslang will generate OpCapability VulkanMemoryModel and need entension enabled
-    AddRequiredExtensions(VK_KHR_VULKAN_MEMORY_MODEL_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::shaderFloat16);
-    AddRequiredFeature(vkt::Feature::cooperativeMatrix);
-    AddRequiredFeature(vkt::Feature::vulkanMemoryModel);
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
 
-    std::vector<VkDescriptorSetLayoutBinding> bindings(0);
-    const vkt::DescriptorSetLayout dsl(*m_device, bindings);
-    const vkt::PipelineLayout pl(*m_device, {&dsl});
+    VkPhysicalDeviceCooperativeMatrixPropertiesKHR props = vku::InitStructHelper();
+    VkPhysicalDeviceVulkan11Properties props11 = vku::InitStructHelper(&props);
+    GetPhysicalDeviceProperties2(props11);
+    if ((props.cooperativeMatrixSupportedStages & VK_SHADER_STAGE_COMPUTE_BIT) == 0) {
+        GTEST_SKIP() << "Compute stage is not supported";
+    }
+    if (props11.subgroupSize > 32) {
+        GTEST_SKIP() << "local_size_x (32) is not a multiple of subgroupSize";
+    }
 
     // Tests are assume that Float16 3*5 is not available
     char const *csSource = R"glsl(
@@ -171,29 +193,25 @@ TEST_F(NegativeShaderCooperativeMatrix, KHRParametersMatchProperties) {
     )glsl";
 
     CreateComputePipelineHelper pipe(*this);
-    pipe.cs_ = std::make_unique<VkShaderObj>(this, csSource, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_GLSL, nullptr);
-    pipe.InitState();
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-RuntimeSpirv-OpTypeCooperativeMatrixKHR-08974");
+    pipe.cs_ = std::make_unique<VkShaderObj>(this, csSource, VK_SHADER_STAGE_COMPUTE_BIT);
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-OpTypeCooperativeMatrixKHR-10163");
     pipe.CreateComputePipeline();
     m_errorMonitor->VerifyFound();
 }
 
-TEST_F(NegativeShaderCooperativeMatrix, KHRDimXMultipleSubgroupSize) {
+TEST_F(NegativeShaderCooperativeMatrix, DimXMultipleSubgroupSize) {
     TEST_DESCRIPTION("Local workgroup size in the X dimension of the pipeline multiple of subgroupSize");
 
     SetTargetApiVersion(VK_API_VERSION_1_3);
 
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_COOPERATIVE_MATRIX_EXTENSION_NAME);
     AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
-    // glslang will generate OpCapability VulkanMemoryModel and need entension enabled
-    AddRequiredExtensions(VK_KHR_VULKAN_MEMORY_MODEL_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::shaderFloat16);
-    AddRequiredFeature(vkt::Feature::cooperativeMatrix);
-    AddRequiredFeature(vkt::Feature::vulkanMemoryModel);
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
+
+    if (!HasValidProperty(VK_SCOPE_SUBGROUP_KHR, 16, 16, 16, VK_COMPONENT_TYPE_FLOAT16_KHR)) {
+        GTEST_SKIP() << "Valid Property not found";
+    }
 
-    // Tests are assume that Float16 3*5 is not available
     char const *csSource = R"glsl(
         #version 450
         #pragma use_vulkan_memory_model
@@ -204,10 +222,10 @@ TEST_F(NegativeShaderCooperativeMatrix, KHRDimXMultipleSubgroupSize) {
         #extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
         layout(local_size_x_id = 0, local_size_y = 1, local_size_z = 1) in;
         void main() {
-            coopmat<uint32_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseA> A;
-            coopmat<uint32_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseB> B;
-            coopmat<uint32_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseAccumulator> C;
-            coopmat<uint32_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseAccumulator> D = coopMatMulAdd(A, B, C);
+            coopmat<float16_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseA> A;
+            coopmat<float16_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseB> B;
+            coopmat<float16_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseAccumulator> C;
+            coopmat<float16_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseAccumulator> D = coopMatMulAdd(A, B, C);
         }
     )glsl";
 
@@ -227,37 +245,86 @@ TEST_F(NegativeShaderCooperativeMatrix, KHRDimXMultipleSubgroupSize) {
     CreateComputePipelineHelper pipe(*this);
     pipe.cs_ =
         std::make_unique<VkShaderObj>(this, csSource, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_GLSL, &specInfo);
-    pipe.InitState();
-    pipe.pipeline_layout_ = vkt::PipelineLayout(*m_device, {});
-    // Ignore messages that types and sizes are unsupported by implementation
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08975");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-KSize-08977");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08979");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08981");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08976");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08978");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08980");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08982");
 
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkPipelineShaderStageCreateInfo-module-08987");
+    m_errorMonitor->SetDesiredError("VUID-VkPipelineShaderStageCreateInfo-module-08987", 3);
     pipe.CreateComputePipeline();
     m_errorMonitor->VerifyFound();
 }
 
-TEST_F(NegativeShaderCooperativeMatrix, KHRSameScope) {
+TEST_F(NegativeShaderCooperativeMatrix, DimXMultipleSubgroupSizeWorkgroupScope) {
+    TEST_DESCRIPTION(
+        "Local workgroup size in the X dimension of the pipeline multiple of subgroupSize and less than or equal to "
+        "cooperativeMatrixWorkgroupScopeMaxWorkgroupSize");
+
+    SetTargetApiVersion(VK_API_VERSION_1_3);
+
+    AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
+    AddRequiredExtensions(VK_NV_COOPERATIVE_MATRIX_2_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::shaderFloat16);
+    AddRequiredFeature(vkt::Feature::cooperativeMatrixWorkgroupScope);
+    AddRequiredFeature(vkt::Feature::cooperativeMatrixFlexibleDimensions);
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
+
+    if (!HasValidProperty(VK_SCOPE_WORKGROUP_KHR, 32, 32, 32, VK_COMPONENT_TYPE_FLOAT16_KHR)) {
+        GTEST_SKIP() << "Valid Property not found";
+    }
+
+    char const *csSource = R"glsl(
+        #version 450
+        #pragma use_vulkan_memory_model
+        #extension GL_KHR_cooperative_matrix : enable
+        #extension GL_KHR_shader_subgroup_basic : enable
+        #extension GL_KHR_memory_scope_semantics : enable
+        #extension GL_EXT_shader_explicit_arithmetic_types : enable
+        #extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
+        layout(local_size_x_id = 0, local_size_y = 1, local_size_z = 1) in;
+        void main() {
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseA> A;
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseB> B;
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseAccumulator> C;
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseAccumulator> D = coopMatMulAdd(A, B, C);
+        }
+    )glsl";
+
+    VkPhysicalDeviceCooperativeMatrix2PropertiesNV props2 = vku::InitStructHelper();
+    GetPhysicalDeviceProperties2(props2);
+
+    const uint32_t specData[] = {
+        props2.cooperativeMatrixWorkgroupScopeMaxWorkgroupSize + 1,
+    };
+    const VkSpecializationMapEntry entries[] = {
+        {0, sizeof(uint32_t) * 0, sizeof(uint32_t)},
+    };
+    const VkSpecializationInfo specInfo = {
+        1,
+        entries,
+        sizeof(specData),
+        specData,
+    };
+
+    CreateComputePipelineHelper pipe(*this);
+    pipe.cs_ =
+        std::make_unique<VkShaderObj>(this, csSource, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_GLSL, &specInfo);
+
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-cooperativeMatrixFlexibleDimensions-10165", 3);
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-cooperativeMatrixFlexibleDimensions-10166");
+    m_errorMonitor->SetDesiredError("VUID-VkPipelineShaderStageCreateInfo-module-10169", 6);
+    pipe.CreateComputePipeline();
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(NegativeShaderCooperativeMatrix, SameScope) {
     TEST_DESCRIPTION("In OpCooperativeMatrixMulAddKHR all matrices should have same scope");
 
     SetTargetApiVersion(VK_API_VERSION_1_3);
 
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_COOPERATIVE_MATRIX_EXTENSION_NAME);
     AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
-    // glslang will generate OpCapability VulkanMemoryModel and need entension enabled
-    AddRequiredExtensions(VK_KHR_VULKAN_MEMORY_MODEL_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::shaderFloat16);
-    AddRequiredFeature(vkt::Feature::cooperativeMatrix);
-    AddRequiredFeature(vkt::Feature::vulkanMemoryModel);
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
+
+    if (!HasValidProperty(VK_SCOPE_SUBGROUP_KHR, 16, 16, 16, VK_COMPONENT_TYPE_FLOAT16_KHR)) {
+        GTEST_SKIP() << "Valid Property not found";
+    }
 
     char const *csSource = R"glsl(
         #version 450
@@ -271,10 +338,10 @@ TEST_F(NegativeShaderCooperativeMatrix, KHRSameScope) {
         layout(constant_id = 1) const uint scope1 = gl_ScopeSubgroup;
         layout(local_size_x = 64) in;
         void main() {
-            coopmat<float32_t, scope0, 16, 16, gl_MatrixUseA> A;
-            coopmat<float32_t, scope1, 16, 16, gl_MatrixUseB> B;
-            coopmat<float32_t, scope0, 16, 16, gl_MatrixUseAccumulator> C;
-            coopmat<float32_t, scope0, 16, 16, gl_MatrixUseAccumulator> D = coopMatMulAdd(A, B, C);
+            coopmat<float16_t, scope0, 16, 16, gl_MatrixUseA> A;
+            coopmat<float16_t, scope1, 16, 16, gl_MatrixUseB> B;
+            coopmat<float16_t, scope0, 16, 16, gl_MatrixUseAccumulator> C;
+            coopmat<float16_t, scope0, 16, 16, gl_MatrixUseAccumulator> D = coopMatMulAdd(A, B, C);
         }
     )glsl";
 
@@ -296,164 +363,245 @@ TEST_F(NegativeShaderCooperativeMatrix, KHRSameScope) {
     CreateComputePipelineHelper pipe(*this);
     pipe.cs_ =
         std::make_unique<VkShaderObj>(this, csSource, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_GLSL, &specInfo);
-    pipe.InitState();
-    pipe.pipeline_layout_ = vkt::PipelineLayout(*m_device, {});
-    // Ignore messages that types and sizes are unsupported by implementation
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08975");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-KSize-08977");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08979");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08981");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08976");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08978");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08980");
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08982");
 
     // SPIR-V code is expected to be bad after specialization, due to scopes are different
+    // Need to ignore the spirv-val
     m_errorMonitor->SetAllowedFailureMsg("VUID-VkPipelineShaderStageCreateInfo-pSpecializationInfo-06849");
 
-    // Expect gl_ScopeInvocation will not be found in the implementation
-    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpTypeCooperativeMatrixKHR-08974");
-
-    m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-RuntimeSpirv-scope-08984");
+    // The scope will be invalid
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-10060");
+    // Expect gl_ScopeInvocation will not be found in the implementation since it is not allowed in Vulkan
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-OpTypeCooperativeMatrixKHR-10163");
     pipe.CreateComputePipeline();
     m_errorMonitor->VerifyFound();
 }
 
-// Tests fail in CI from failing spirv-as
-// Need to rewrite tests without subtests logic
-TEST_F(NegativeShaderCooperativeMatrix, DISABLED_MatchSizeWithProperties) {
+TEST_F(NegativeShaderCooperativeMatrix, WorkgroupScope) {
+    TEST_DESCRIPTION("Workgroup scope requires cooperativeMatrixWorkgroupScope");
+
+    SetTargetApiVersion(VK_API_VERSION_1_3);
+
+    AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
+    AddRequiredExtensions(VK_NV_COOPERATIVE_MATRIX_2_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::shaderFloat16);
+    AddRequiredFeature(vkt::Feature::cooperativeMatrixFlexibleDimensions);
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
+
+    if (!HasValidProperty(VK_SCOPE_WORKGROUP_KHR, 32, 32, 32, VK_COMPONENT_TYPE_FLOAT16_KHR)) {
+        GTEST_SKIP() << "Valid Property not found";
+    }
+
+    char const *csSource = R"glsl(
+        #version 450
+        #pragma use_vulkan_memory_model
+        #extension GL_KHR_cooperative_matrix : enable
+        #extension GL_KHR_shader_subgroup_basic : enable
+        #extension GL_KHR_memory_scope_semantics : enable
+        #extension GL_EXT_shader_explicit_arithmetic_types : enable
+        #extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
+        layout(local_size_x = 64) in;
+        void main() {
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseA> A;
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseB> B;
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseAccumulator> C;
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseAccumulator> D = coopMatMulAdd(A, B, C);
+        }
+    )glsl";
+
+    CreateComputePipelineHelper pipe(*this);
+    pipe.cs_ = std::make_unique<VkShaderObj>(this, csSource, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_GLSL);
+
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-cooperativeMatrixWorkgroupScope-10164", 3);
+    pipe.CreateComputePipeline();
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(NegativeShaderCooperativeMatrix, WorkgroupScopeMaxDimensions) {
+    TEST_DESCRIPTION("Matrix dimensions must be less than or equal to cooperativeMatrixFlexibleDimensionsMaxDimension");
+
+    SetTargetApiVersion(VK_API_VERSION_1_3);
+
+    AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
+    AddRequiredExtensions(VK_NV_COOPERATIVE_MATRIX_2_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::shaderFloat16);
+    AddRequiredFeature(vkt::Feature::cooperativeMatrixFlexibleDimensions);
+    AddRequiredFeature(vkt::Feature::cooperativeMatrixWorkgroupScope);
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
+
+    if (!HasValidProperty(VK_SCOPE_WORKGROUP_KHR, 32, 32, 32, VK_COMPONENT_TYPE_FLOAT16_KHR)) {
+        GTEST_SKIP() << "Valid Property not found";
+    }
+
+    char const *csSource = R"glsl(
+        #version 450
+        #pragma use_vulkan_memory_model
+        #extension GL_KHR_cooperative_matrix : enable
+        #extension GL_KHR_shader_subgroup_basic : enable
+        #extension GL_KHR_memory_scope_semantics : enable
+        #extension GL_EXT_shader_explicit_arithmetic_types : enable
+        #extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
+        layout(constant_id = 0) const uint dim = 32;
+        layout(local_size_x = 64) in;
+        void main() {
+            coopmat<float16_t, gl_ScopeWorkgroup, dim, dim, gl_MatrixUseA> A;
+            coopmat<float16_t, gl_ScopeWorkgroup, dim, dim, gl_MatrixUseB> B;
+            coopmat<float16_t, gl_ScopeWorkgroup, dim, dim, gl_MatrixUseAccumulator> C;
+            coopmat<float16_t, gl_ScopeWorkgroup, dim, dim, gl_MatrixUseAccumulator> D = coopMatMulAdd(A, B, C);
+        }
+    )glsl";
+
+    VkPhysicalDeviceCooperativeMatrix2PropertiesNV props2 = vku::InitStructHelper();
+    GetPhysicalDeviceProperties2(props2);
+    uint32_t dim = props2.cooperativeMatrixFlexibleDimensionsMaxDimension;
+    dim /= 32;
+    dim++;
+    dim *= 32;
+
+    const uint32_t specData[] = {
+        dim,
+    };
+    const VkSpecializationMapEntry entries[] = {
+        {0, sizeof(uint32_t) * 0, sizeof(uint32_t)},
+    };
+    const VkSpecializationInfo specInfo = {
+        1,
+        entries,
+        sizeof(specData),
+        specData,
+    };
+
+    CreateComputePipelineHelper pipe(*this);
+    pipe.cs_ =
+        std::make_unique<VkShaderObj>(this, csSource, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_GLSL, &specInfo);
+
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-cooperativeMatrixFlexibleDimensionsMaxDimension-10167", 3);
+    pipe.CreateComputePipeline();
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(NegativeShaderCooperativeMatrix, WorkgroupScopeMaxSharedMemory) {
+    TEST_DESCRIPTION("cooperativeMatrixWorkgroupScopeReservedSharedMemory limit");
+
+    SetTargetApiVersion(VK_API_VERSION_1_3);
+
+    AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
+    AddRequiredExtensions(VK_NV_COOPERATIVE_MATRIX_2_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::shaderInt8);
+    AddRequiredFeature(vkt::Feature::shaderFloat16);
+    AddRequiredFeature(vkt::Feature::cooperativeMatrixFlexibleDimensions);
+    AddRequiredFeature(vkt::Feature::cooperativeMatrixWorkgroupScope);
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
+
+    if (!HasValidProperty(VK_SCOPE_WORKGROUP_KHR, 32, 32, 32, VK_COMPONENT_TYPE_FLOAT16_KHR)) {
+        GTEST_SKIP() << "Valid Property not found";
+    }
+
+    char const *csSource = R"glsl(
+        #version 450
+        #pragma use_vulkan_memory_model
+        #extension GL_KHR_cooperative_matrix : enable
+        #extension GL_KHR_shader_subgroup_basic : enable
+        #extension GL_KHR_memory_scope_semantics : enable
+        #extension GL_EXT_shader_explicit_arithmetic_types : enable
+        #extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
+        layout(constant_id = 0) const uint dim = 32;
+        shared uint8_t sh[dim];
+        layout(local_size_x = 64) in;
+        void main() {
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseA> A;
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseB> B;
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseAccumulator> C;
+            coopmat<float16_t, gl_ScopeWorkgroup, 32, 32, gl_MatrixUseAccumulator> D = coopMatMulAdd(A, B, C);
+        }
+    )glsl";
+
+    VkPhysicalDeviceCooperativeMatrix2PropertiesNV props2 = vku::InitStructHelper();
+    GetPhysicalDeviceProperties2(props2);
+
+    uint32_t shmem_size =
+        m_device->Physical().limits_.maxComputeSharedMemorySize - props2.cooperativeMatrixWorkgroupScopeReservedSharedMemory + 1;
+
+    const uint32_t specData[] = {
+        shmem_size,
+    };
+    const VkSpecializationMapEntry entries[] = {
+        {0, sizeof(uint32_t) * 0, sizeof(uint32_t)},
+    };
+    const VkSpecializationInfo specInfo = {
+        1,
+        entries,
+        sizeof(specData),
+        specData,
+    };
+
+    CreateComputePipelineHelper pipe(*this);
+    pipe.cs_ =
+        std::make_unique<VkShaderObj>(this, csSource, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_GLSL, &specInfo);
+
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-maxComputeSharedMemorySize-10168");
+    pipe.CreateComputePipeline();
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(NegativeShaderCooperativeMatrix, MatchSizeWithProperties) {
     TEST_DESCRIPTION("Check size match properties");
 
     SetTargetApiVersion(VK_API_VERSION_1_3);
 
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_COOPERATIVE_MATRIX_EXTENSION_NAME);
     AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
-    // glslang will generate OpCapability VulkanMemoryModel and need entension enabled
-    AddRequiredExtensions(VK_KHR_VULKAN_MEMORY_MODEL_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::shaderFloat16);
-    AddRequiredFeature(vkt::Feature::cooperativeMatrix);
-    AddRequiredFeature(vkt::Feature::vulkanMemoryModel);
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
 
-    // OpExtension "SPV_KHR_storage_buffer_storage_class"
-    const std::string csSourceTemplate = R"glsl(
-        OpCapability Shader
-        OpCapability VulkanMemoryModel
-        OpCapability CooperativeMatrixKHR
-        OpExtension "SPV_KHR_cooperative_matrix"
-        OpExtension "SPV_KHR_vulkan_memory_model"
-        %import = OpExtInstImport "GLSL.std.450"
-        OpMemoryModel Logical Vulkan
-        OpEntryPoint GLCompute %main "main"
-        OpExecutionMode %main LocalSize 64 1 1
-        OpDecorate %wgs BuiltIn WorkgroupSize
-        %void  = OpTypeVoid
-        %func  = OpTypeFunction %void
-        %sint  = OpTypeInt 32 1
-        %uint  = OpTypeInt 32 0
-        %uvec3 = OpTypeVector %uint 3
-        %c_3u  = OpConstant %uint 3
-        %c_64u = OpConstant %uint 64
-        %wgs   = OpConstantComposite %uvec3 %c64_u %one %one
+    VkPhysicalDeviceCooperativeMatrixPropertiesKHR props = vku::InitStructHelper();
+    GetPhysicalDeviceProperties2(props);
+    if ((props.cooperativeMatrixSupportedStages & VK_SHADER_STAGE_COMPUTE_BIT) == 0) {
+        GTEST_SKIP() << "Compute stage is not supported";
+    }
 
-        %rows  = OpConstant %uint 16
+    if (HasValidProperty(VK_SCOPE_SUBGROUP_KHR, 8, 8, 16, VK_COMPONENT_TYPE_FLOAT16_KHR)) {
+        GTEST_SKIP() << "Valid Property found, need invalid to test";
+    }
 
-        %A_col = OpConstant %uint $A_VAL
-        %A_use = OpConstant %uint 0
-        %A_t   = OpTypeCooperativeMatrixKHR %sint %c_3u %rows %A_col %A_use
-        %A_ptr = OpTypePointer Private %A_t
-        %A_var = OpVariable %A_ptr Private
-
-        %B_col = OpConstant %uint $B_VAL
-        %B_use = OpConstant %uint 1
-        %B_t   = OpTypeCooperativeMatrixKHR %sint %c_3u %rows %B_col %B_use
-        %B_ptr = OpTypePointer Private %B_t
-        %B_var = OpVariable %B_ptr Private
-
-        %C_col = OpConstant %uint $C_VAL
-        %C_use = OpConstant %uint 2
-        %C_t   = OpTypeCooperativeMatrixKHR %sint %c_3u %rows %C_col %C_use
-        %C_ptr = OpTypePointer Private %C_t
-        %C_var = OpVariable %C_ptr Private
-
-        %R_col = OpConstant %uint $R_VAL
-        %R_use = OpConstant %uint 2
-        %R_t   = OpTypeCooperativeMatrixKHR %sint %c_3u %rows %R_col %R_use
-        %R_ptr = OpTypePointer Private %R_t
-        %R_var = OpVariable %R_ptr Private
-
-        %main  = OpFunction %void None %func
-        %body  = OpLabel
-        %A     = OpLoad %A_t %A_var
-        %B     = OpLoad %B_t %B_var
-        %C     = OpLoad %C_t %C_var
-        %R     = OpCooperativeMatrixMulAddKHR %R_t %A %B %C MatrixASignedComponents|MatrixBSignedComponents|MatrixCSignedComponents|MatrixResultSignedComponents
-        OpStore %R_var %R
-        OpReturn
-        OpFunctionEnd
+    char const *source = R"glsl(
+        #version 450
+        #pragma use_vulkan_memory_model
+        #extension GL_KHR_cooperative_matrix : enable
+        #extension GL_KHR_shader_subgroup_basic : enable
+        #extension GL_KHR_memory_scope_semantics : enable
+        #extension GL_EXT_shader_explicit_arithmetic_types : enable
+        layout(local_size_x = 32) in;
+        void main() {
+            coopmat<float16_t, gl_ScopeSubgroup, 8, 16, gl_MatrixUseA> A;
+            coopmat<float16_t, gl_ScopeSubgroup, 16, 8, gl_MatrixUseB> B;
+            coopmat<float16_t, gl_ScopeSubgroup, 8, 8, gl_MatrixUseAccumulator> C;
+            coopmat<float16_t, gl_ScopeSubgroup, 8, 8, gl_MatrixUseAccumulator> D = coopMatMulAdd(A, B, C);
+       }
     )glsl";
 
-    const struct {
-        const char *var;
-        const char *name;
-    } subtests[] = {
-        {"$A_VAL", "VUID-RuntimeSpirv-MSize-08975"},
-        {"$B_VAL", "VUID-RuntimeSpirv-KSize-08977"},
-        {"$C_VAL", "VUID-RuntimeSpirv-MSize-08979"},
-        {"$R_VAL", "VUID-RuntimeSpirv-MSize-08981"},
-    };
-    auto replace = [](std::string &str, const std::string &from, const std::string &to) {
-        size_t pos;
-        while ((pos = str.find(from)) != std::string::npos) str.replace(pos, from.length(), to);
-    };
+    // There is no way to avoid this message
+    m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpTypeCooperativeMatrixKHR-10163");
 
-    for (const auto &x : subtests) {
-        std::string css(csSourceTemplate);
-
-        for (const auto &y : subtests) {
-            if (x.var == y.var) {
-                // Set expected message
-                replace(css, y.var, "17");
-                m_errorMonitor->SetDesiredFailureMsg(kErrorBit, y.name);
-            } else {
-                // Ignore messages that types and sizes are unsupported by implementation
-                replace(css, y.var, "16");
-                m_errorMonitor->SetAllowedFailureMsg(y.name);
-            }
-        }
-
-        // Allow SPIR-V tests to be accepted as Vertex Shader, while it essentially Compute Shader
-        m_errorMonitor->SetAllowedFailureMsg("VUID-VkShaderModuleCreateInfo-pCode-08737");
-
-        // There is no way to avoid this message
-        m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpTypeCooperativeMatrixKHR-08974");
-
-        CreateComputePipelineHelper pipe(*this);
-        pipe.cs_ = std::make_unique<VkShaderObj>(this, css.c_str(), VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0,
-                                                 SPV_SOURCE_ASM, nullptr);
-        pipe.InitState();
-        pipe.CreateComputePipeline();
-        m_errorMonitor->VerifyFound();
-    }
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-10060");
+    CreateComputePipelineHelper pipe(*this);
+    pipe.cs_ = std::make_unique<VkShaderObj>(this, source, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_2);
+    pipe.CreateComputePipeline();
+    m_errorMonitor->VerifyFound();
 }
 
-// Tests fail in CI from failing spirv-as
-// Need to rewrite tests without subtests logic
-TEST_F(NegativeShaderCooperativeMatrix, DISABLED_KHRSignedCheck) {
+TEST_F(NegativeShaderCooperativeMatrix, SignedCheck) {
     TEST_DESCRIPTION("Test that if component type of is signed check that appropriate MatrixSignedComponents is present");
 
     SetTargetApiVersion(VK_API_VERSION_1_3);
 
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_COOPERATIVE_MATRIX_EXTENSION_NAME);
     AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
-    // glslang will generate OpCapability VulkanMemoryModel and need entension enabled
-    AddRequiredExtensions(VK_KHR_VULKAN_MEMORY_MODEL_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::shaderFloat16);
-    AddRequiredFeature(vkt::Feature::cooperativeMatrix);
-    AddRequiredFeature(vkt::Feature::vulkanMemoryModel);
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
+
+    VkPhysicalDeviceCooperativeMatrixPropertiesKHR props = vku::InitStructHelper();
+    GetPhysicalDeviceProperties2(props);
+    if ((props.cooperativeMatrixSupportedStages & VK_SHADER_STAGE_COMPUTE_BIT) == 0) {
+        GTEST_SKIP() << "Compute stage is not supported";
+    }
 
     // OpExtension "SPV_KHR_storage_buffer_storage_class"
     const std::string csSourceTemplate = R"glsl(
@@ -494,7 +642,7 @@ TEST_F(NegativeShaderCooperativeMatrix, DISABLED_KHRSignedCheck) {
         %18 = OpLoad %15 %17
         %23 = OpLoad %20 %22
         %25 = OpLoad %11 %24
-        %26 = OpCooperativeMatrixMulAddKHR %11 %18 %23 %25 MatrixASignedComponents|MatrixBSignedComponents|MatrixCSignedComponents|MatrixResultSignedComponents
+        %26 = OpCooperativeMatrixMulAddKHR %11 %18 %23 %25 MatrixASignedComponentsKHR|MatrixBSignedComponentsKHR|MatrixCSignedComponentsKHR|MatrixResultSignedComponentsKHR
         OpStore %13 %26
         OpReturn
         OpFunctionEnd
@@ -511,10 +659,10 @@ TEST_F(NegativeShaderCooperativeMatrix, DISABLED_KHRSignedCheck) {
         const char *remove;
         const char *expect;
     } subtests[] = {
-        {"MatrixASignedComponents|", "VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08976"},
-        {"MatrixBSignedComponents|", "VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08978"},
-        {"MatrixCSignedComponents|", "VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08980"},
-        {"|MatrixResultSignedComponents", "VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-08982"},
+        {"MatrixASignedComponentsKHR|", "VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-10060"},
+        {"MatrixBSignedComponentsKHR|", "VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-10060"},
+        {"MatrixCSignedComponentsKHR|", "VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-10060"},
+        {"|MatrixResultSignedComponentsKHR", "VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-10060"},
     };
 
     for (const auto &x: subtests) {
@@ -522,14 +670,12 @@ TEST_F(NegativeShaderCooperativeMatrix, DISABLED_KHRSignedCheck) {
         const char *css = csSourceStr.c_str();
         CreateComputePipelineHelper pipe(*this);
 
-        pipe.cs_ =
-            std::make_unique<VkShaderObj>(this, css, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_ASM, nullptr);
-        pipe.InitState();
+        pipe.cs_ = std::make_unique<VkShaderObj>(this, css, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_0, SPV_SOURCE_ASM);
 
         for (const auto &y : subtests) {
             if (x.remove == y.remove) {
                 // Set expected message
-                m_errorMonitor->SetDesiredFailureMsg(kErrorBit, y.expect);
+                m_errorMonitor->SetDesiredError(y.expect);
             } else {
                 // Ignore messages that types and sizes are unsupported by implementation
                 m_errorMonitor->SetAllowedFailureMsg(y.expect);
@@ -537,10 +683,11 @@ TEST_F(NegativeShaderCooperativeMatrix, DISABLED_KHRSignedCheck) {
         }
 
         // Ignore messages that types and sizes are unsupported by implementation
-        m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08975");
-        m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-KSize-08977");
-        m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08979");
-        m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MSize-08981");
+        m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-10060");
+        m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-10060");
+        m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-10060");
+        m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpCooperativeMatrixMulAddKHR-10060");
+        m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-OpTypeCooperativeMatrixKHR-10163");
 
         pipe.CreateComputePipeline();
 
